/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              RX32S10xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : RX32S10xx
* File         : RX32S10xx.h
* By           : RX_SD_Team
* Description  : 
*********************************************************************************************************
*/

#ifndef __RX32S10xx_H__
#define __RX32S10xx_H__

#ifdef __cplusplus
extern "C" {
#endif



/*
* @brief  General Bool State
*/
typedef enum {DISABLE = 0, ENABLE = !DISABLE}   FunctionalState;        /* 用于使能/非使能            */
typedef enum {RESET   = 0, SET    = !RESET  }   FlagStatus, ITStatus;   /* 用于状态或者中断标志置1置0 */
typedef enum {FALSE   = 0, TRUE   = !FALSE  }   Bool;                   /* 用于函数返回               */

#define SET(n,c)		((n)|=(c))
#define CLR(n,c)		((n)&=~(c))

#define SET_BIT(REG, BIT)     ((REG) |= (BIT))

#define CLEAR_BIT(REG, BIT)   ((REG) &= ~(BIT))

#define READ_BIT(REG, BIT)    ((REG) & (BIT))

#define CLEAR_REG(REG)        ((REG) = (0x0))

#define WRITE_REG(REG, VAL)   ((REG) = (VAL))

#define READ_REG(REG)         ((REG))

#define MODIFY_REG(REG, CLEARMASK, SETMASK)  WRITE_REG((REG), (((READ_REG(REG)) & (~(CLEARMASK))) | (SETMASK)))

#define M8(adr)     (*((volatile uint8_t *)  (adr)))
#define M16(adr)    (*((volatile uint16_t *) (adr)))
#define M32(adr)    (*((volatile uint32_t *) (adr)))


#define BIT0			((uint32_t)0x00000001)
#define BIT1			((uint32_t)0x00000002)
#define BIT2			((uint32_t)0x00000004)
#define BIT3			((uint32_t)0x00000008)
#define BIT4			((uint32_t)0x00000010)
#define BIT5			((uint32_t)0x00000020)
#define BIT6			((uint32_t)0x00000040)
#define BIT7			((uint32_t)0x00000080)
#define BIT8			((uint32_t)0x00000100)
#define BIT9			((uint32_t)0x00000200)
#define BIT10			((uint32_t)0x00000400)
#define BIT11			((uint32_t)0x00000800)
#define BIT12			((uint32_t)0x00001000)
#define BIT13			((uint32_t)0x00002000)
#define BIT14			((uint32_t)0x00004000)
#define BIT15			((uint32_t)0x00008000)
#define BIT16			((uint32_t)0x00010000)
#define BIT17			((uint32_t)0x00020000)
#define BIT18			((uint32_t)0x00040000)
#define BIT19			((uint32_t)0x00080000)
#define BIT20			((uint32_t)0x00100000)
#define BIT21			((uint32_t)0x00200000)
#define BIT22			((uint32_t)0x00400000)
#define BIT23			((uint32_t)0x00800000)
#define BIT24			((uint32_t)0x01000000)
#define BIT25			((uint32_t)0x02000000)
#define BIT26			((uint32_t)0x04000000)
#define BIT27			((uint32_t)0x08000000)
#define BIT28			((uint32_t)0x10000000)
#define BIT29			((uint32_t)0x20000000)
#define BIT30			((uint32_t)0x40000000)
#define BIT31			((uint32_t)0x80000000)

/*
**********************************************************************************************************
*                                       Interrupt Number Definition
**********************************************************************************************************
*/
typedef enum IRQn
{
/*****************************  Cortex-M0 Processor Exceptions Numbers  *********************************/
    NonMaskableInt_IRQn         = -14,      /*!< 2 Non Maskable Interrupt  CMU                    */
    HardFault_IRQn              = -13,      /*!< 3 Cortex-M0 Hard Fault Interrupt                 */
    SVCall_IRQn                 = -5,       /*!< 11 Cortex-M0 SV Call Interrupt                   */
    PendSV_IRQn                 = -2,       /*!< 14 Cortex-M0 Pend SV Interrupt                   */
    SysTick_IRQn                = -1,       /*!< 15 Cortex-M0 System Tick Interrupt               */

/*****************************  HT6xxx Specific Interrupt Numbers  **************************************/
    PMU_IRQn                    = 0,        /*!< PMU Interrupt                                    */
			SADC_IRQn									= 1,		/* SAR ADC Interrupt */				
    EXTI0_IRQn                  = 2,        /*!< External Interrupt 0 Interrupt                   */
    EXTI1_IRQn                  = 3,        /*!< External Interrupt 1 Interrupt                   */
    EXTI2_IRQn                  = 4,        /*!< External Interrupt 2 Interrupt                   */
    EXTI3_IRQn                  = 5,        /*!< External Interrupt 3 Interrupt                   */
    EXTI4_IRQn                  = 6,        /*!< External Interrupt 4 Interrupt                   */
    EXTI5_IRQn                  = 7,        /*!< External Interrupt 5 Interrupt                   */
    EXTI6_IRQn                  = 8,        /*!< External Interrupt 6 Interrupt                   */
    UART0_IRQn                  = 9,        /*!< UART0 Interrupt                                  */
    UART1_IRQn                  = 10,       /*!< UART1 Interrupt                                  */
    CMP1_IRQn										= 11,       /*!< CMP1 Interrupt                                  */
    CMP2_IRQn										= 12,       /*!< CMP2 Interrupt                                  */
    CMP3_IRQn										= 13,       /*!< CMP3 Interrupt                                  */
    ME_IRQn											= 14,       /*!< ME Interrupt                                  */
    TIMER_0_IRQn                = 15,       /*!< Timer0 Interrupt                                 */
    TIMER_1_IRQn                = 16,       /*!< Timer1 Interrupt                                 */
    TIMER_2_IRQn                = 17,       /*!< Timer2 Interrupt                                 */
    TIMER_3_IRQn                = 18,       /*!< Timer3 Interrupt                                 */
    RTC_IRQn                    = 20,       /*!< RTC Interrupt                                    */
    I2C_IRQn                    = 21,       /*!< I2C Converter Interrupt                          */
    SPI0_IRQn                   = 22,       /*!< SPI0 Interrupt                                   */
    SelfTestF_IRQn              = 24,       /*!< SelfTestF Interrupt                              */
		CMP4_IRQn										= 25,       /*!< CMP4 Interrupt                                  */
		CMP5_IRQn										= 26,       /*!< CMP5 Interrupt                                  */
    TIM8_IRQn                		= 27,       /*!< TIM8 Interrupt                                 */
    EXTI7_IRQn                  = 28,       /*!< External Interrupt 7 Interrupt                   */
    EXTI8_IRQn                  = 29,       /*!< External Interrupt 8 Interrupt                   */
    DMA_IRQn                    = 31,       /*!< DMA Interrupt                                    */
} IRQn_Type;


/*
**********************************************************************************************************
*                                   Processor and Core Peripheral Section
**********************************************************************************************************
*/

/********************  Configuration of the Cortex-M0 Processor and Core Peripherals  *******************/
#define __MPU_PRESENT             0         /*!< MPU present or not                               */
#define __NVIC_PRIO_BITS          2         /*!< Number of Bits used for Priority Levels          */
#define __Vendor_SysTickConfig    0         /*!< Set to 1 if different SysTick Config is used     */
                                            /* end of group HT60xx_CMSIS                          */

#include "core_cm0.h"                       /* Cortex-M0 processor and core peripherals           */
#include <stdint.h>                         /* standard types definitions                         */


/*
**********************************************************************************************************
*                                 Device Specific Peripheral Registers structures
**********************************************************************************************************
*/
#if defined ( __CC_ARM   )
#pragma anon_unions
#endif

/*****************************************  Timer Control Block  *****************************************/
/*
* @brief  定时器寄存器定义
*/
typedef struct
{
    __IO uint32_t TMRCON;                   /*!< Offset: 0x000 定时器控制寄存器      (R/W) */
    __IO uint32_t TMRDIV;                   /*!< Offset: 0x004 预分频寄存器          (R/W) */
    __IO uint32_t TMRPRD;                   /*!< Offset: 0x008 周期寄存器            (R/W) */
    __IO uint32_t TMRCAP;                   /*!< Offset: 0x00C 捕获数据寄存器        (R/W) */
    __IO uint32_t TMRCNT;                   /*!< Offset: 0x010 计数寄存器            (R/W) */
    __IO uint32_t TMRCMP;                   /*!< Offset: 0x014 比较寄存器            (R/W) */
    __IO uint32_t TMRIE;                    /*!< Offset: 0x018 定时器中断使能寄存器  (R/W) */
    __IO uint32_t TMRIF;                    /*!< Offset: 0x01C 定时器中断标志寄存器  (R/W) */
}TMR_TypeDef;                           /* end of group RT_TMR_TypeDef                 */


/*****************************************  UART Control Block  ******************************************/
/*
* @brief  UART寄存器定义
*/
typedef struct
{
    __IO uint32_t MODESEL;                  /*!< Offset: 0x000 UART功能选择寄存器      (R/W) */
    __IO uint32_t UARTCON;                  /*!< Offset: 0x004 串口功能配置寄存器           (R/W) */
    __IO uint32_t Reserved[1];              
    __IO uint32_t SREL;                     /*!< Offset: 0x00C 串口波特率发生器重载值   (R/W) */
    __IO uint32_t SBUF;                     /*!< Offset: 0x010 串口数据缓冲寄存器      (R/W) */
    __IO uint32_t UARTSTA;                  /*!< Offset: 0x014 串口状态寄存器               (R/W) */          
}UART_TypeDef;                          /* end of group RT_UART_TypeDef */


/*****************************************  I2C Control Block  *******************************************/
/*
* @brief  I2C寄存器定义
*/
typedef struct
{
    __IO uint32_t I2CDAT;                   /*!< Offset: 0x000 I2C数据寄存器 (R/W) */
    __IO uint32_t I2CADR;                   /*!< Offset: 0x004 I2C地址寄存器 (R/W) */
    __IO uint32_t I2CCON;                   /*!< Offset: 0x008 I2C控制寄存器 (R/W) */
    __IO uint32_t I2CSTA;                   /*!< Offset: 0x00C I2C状态寄存器 (R/W) */
}I2C_TypeDef;                           /* end of group RT_I2C_TypeDef         */


/*****************************************  SPI Control Block  *******************************************/
/*
* @brief  SPI寄存器定义
*/
typedef struct
{
	__IO uint32_t SPICON;                   /*!< Offset: 0x000 SPI控制寄存器         (R/W) */
	__IO uint32_t SPISTA;                   /*!< Offset: 0x004 SPI状态寄存器         (R/W) */
	__IO uint32_t SPIDAT;                   /*!< Offset: 0x008 SPI数据寄存器         (R/W) */
	__IO uint32_t SPISSN;                   /*!< Offset: 0x00C SPI从机选择寄存器     (R/W) */
	__IO uint32_t Reserved[1];
	__IO uint32_t CSDLY;                    /*!< Offset: 0x014 SPI cs/delay控制寄存器(R/W) */
}SPI_TypeDef;                           /* end of group RT_SPI_TypeDef             */


/*****************************************  RTC Control Block  *******************************************/
/*
* @brief  RTC寄存器定义
*/
typedef struct
{
	__IO uint32_t RTCCON;                   /*!< Offset: 0x000 RTC控制寄存器           (R/W) */
	__IO uint32_t RTCIE;                    /*!< Offset: 0x004 RTC中断使能寄存器       (R/W) */
	__IO uint32_t RTCIF;                    /*!< Offset: 0x008 RTC中断标志寄存器       (R/W) */
	__IO uint32_t ALMR;                     /*!< Offset: 0x00C 闹钟寄存器              (R/W) */
	__IO uint32_t RTCTMR1;                  /*!< Offset: 0x010 RTC定时器1计数设置      (R/W) */
	__IO uint32_t RTCTMR2;                  /*!< Offset: 0x014 RTC定时器2计数设置      (R/W) */
	__IO uint32_t SECR;                     /*!< Offset: 0x018 秒寄存器(写保护)        (R/W) */
	__IO uint32_t MINR;                     /*!< Offset: 0x01C 分寄存器(写保护)        (R/W) */
	__IO uint32_t HOURR;                    /*!< Offset: 0x020 时寄存器(写保护)        (R/W) */
	__IO uint32_t DAYR;                     /*!< Offset: 0x024 日寄存器(写保护)        (R/W) */
	__IO uint32_t MONTHR;                   /*!< Offset: 0x028 月寄存器(写保护)        (R/W) */
	__IO uint32_t YEARR;                    /*!< Offset: 0x02C 年寄存器(写保护)        (R/W) */
	__IO uint32_t WEEKR;                    /*!< Offset: 0x030 周寄存器(写保护)        (R/W) */
	__IO uint32_t Reserved[2];
	__IO uint32_t RTCRD;                    /*!< Offset: 0x03C RTC读控制寄存器         (R/W) */
	__IO uint32_t RTCWR;                    /*!< Offset: 0x040 RTC写控制寄存器         (R/W) */
		
}RTC_TypeDef;                           /* end of group RT_RTC_TypeDef                   */

/*****************************************  PMU Control Block  *******************************************/
/*
* @brief  PMU寄存器定义
*/
typedef struct
{
    __IO uint32_t PMUCON;                   /*!< Offset: 0x000 PMU配置寄存器 (R/W)              */
    __IO uint32_t VDETCFG;                  /*!< Offset: 0x004 电源检测阈值配置寄存器 (R/W)     */
    __IO uint32_t VDETPCFG;                 /*!< Offset: 0x008 电源检测时间周期配置寄存器 (R/W) */
    __IO uint32_t PMUIE;                    /*!< Offset: 0x00C PMU中断使能寄存器 (R/W)          */
    __IO uint32_t PMUIF;                    /*!< Offset: 0x010 PMU中断标志寄存器 (R/W)          */
    __I  uint32_t PMUSTA;                   /*!< Offset: 0x014 PMU状态指示寄存器 (R)            */
    __IO uint32_t WAKEIF;                   /*!< Offset: 0x018 唤醒源标志寄存器 (R/W)           */                 
    __I  uint32_t Reserved[5];
    __IO uint32_t RSTSTA;                   /*!< Offset: 0x030 复位标志寄存器 (R/W)             */
}PMU_TypeDef;   

/*****************************************  CMU Control Block  *******************************************/
/*
* @brief  CMU寄存器定义
*/
typedef struct
{
	__IO uint32_t WPREG;                    /*!< Offset: 0x000 写保护控制寄存器          (R/W)*/                    
	__IO uint32_t SYSCLKCFG;                /*!< Offset: 0x004 系统时钟配置寄存器（写保护） (R/W)*/                  
	__IO uint32_t JTAGSTA;                  /*!< Offset: 0x008 芯片调试状态指示寄存器           (R/W)*/                          
	__IO uint32_t LRCADJ;                   /*!< Offset: 0x00C LRC时钟调整寄存器 （写保护) (R/W)*/                    
	__IO uint32_t HRCADJ;                   /*!< Offset: 0x010 HRC时钟调整寄存器（写保护） (R/W)*/                  
	__IO uint32_t HRCDIV;                   /*!< Offset: 0x014 HRC时钟分频寄存器（写保护）           (R/W)*/                          
	__IO uint32_t CLKSTA;                   /*!< Offset: 0x018 时钟状态寄存器（只读）          (R/W)*/
	__IO uint32_t SYSCLKDIV;                /*!< Offset: 0x01C 系统时钟分频寄存器（写保护） (R/W)*/
	__I  uint32_t Reserved1[1];
	__IO uint32_t CLKOUTSEL;                /*!< Offset: 0x024 CLKOUT时钟选择寄存器（写保护）           (R/W)*/
	__IO uint32_t CLKOUTDIV;                /*!< Offset: 0x028 CLKOUT时钟分频寄存器（写保护）          (R/W)*/
	__IO uint32_t CLKCTRL0;                 /*!< Offset: 0x02C 内部模块使能寄存器0（写保护） (R/W)*/
	__IO uint32_t CLKCTRL1;                 /*!< Offset: 0x030 内部模块使能寄存器1（写保护）           (R/W)*/
	__IO uint32_t FLASHCON;                 /*!< Offset: 0x034 Flash控制寄存器（写保护）    (R/W)*/
  __IO uint32_t FLASHLOCK;                /*!< Offset: 0x038 Flash锁定寄存器               (R/W)*/
	__IO uint32_t PREFETCH;                 /*!< Offset: 0x03C 指令预取使能寄存器           (R/W)*/
	__I  uint32_t Reserved2[4];
	__IO uint32_t INFOLOCK;                 /*!< Offset: 0x050 Info Flash锁定寄存器(不开放给用户) (R/W)  */
  __IO uint32_t PORRSTSET;                /*!< Offset: 0x054 POR/LBOR复位寄存器(不开放给用户) (R/W)    */
	__I  uint32_t Reserved3[107];
	__IO uint32_t MULTFUNCFG;               /*!< Offset: 0x204 多功能配置寄存器(写保护) (R/W)*/
	__IO uint32_t LFCLKCFG;                 /*!< Offset: 0x208 低频时钟源配置寄存器（写保护）           (R/W)*/
  __I  uint32_t Reserved4[829];
	__IO uint32_t CHIPID;                   /*!< Offset: 0xF00 芯片ID寄存器          (R/W)*/
  __I  uint32_t Reserved5[3];
	__IO uint32_t PLLLOCK_PRE;              /*!< Offset: 0xF10 PLL锁定条件设置寄存器 (R/W)*/
	__I  uint32_t Reserved6[3];
	__IO uint32_t C_PLL;                    /*!< Offset: 0xF20 PLL时钟频率产生控制寄存器           (R/W)*/
	__I  uint32_t Reserved7[3];
	__IO uint32_t FLTCTR;                   /*!< Offset: 0xF30 时钟滤波控制寄存器 (R/W)*/

}CMU_TypeDef;                           /* end of group RT_RCC_TypeDef                    */


/*****************************************  WDT Control Block  *******************************************/
/*
* @brief  WDT寄存器定义
*/
typedef struct
{
	__I	 uint32_t Reserved[1];
	__IO uint32_t WDTCLR;                   /*!< Offset: 0x004 看门狗喂狗与时间配置寄存器 (R/W) */
	__I  uint32_t WDTCNT;                   /*!< Offset: 0x008 看门狗计数寄存器（只读）   (R)   */
}WDT_TypeDef;                           /* end of group RT_WDT_TypeDef                      */


/*****************************************  GPIO Control Block  ******************************************/
/*
* @brief GPIO寄存器定义
*/
typedef struct
{
	__IO uint32_t IOCFG;                    /*!< Offset: 0x000 端口功能配置寄存器1(写保护)     (R/W)  */
	__IO uint32_t AFCFG1;                   /*!< Offset: 0x004 端口复用功能配置寄存器1(写保护) (R/W)  */
	__IO uint32_t PTDIR;                    /*!< Offset: 0x008 端口方向配置寄存器              (R/W)  */
	__IO uint32_t PTUP;                     /*!< Offset: 0x00C 端口上拉配置寄存器              (R/W)  */
	__IO uint32_t PTDAT;                    /*!< Offset: 0x010 端口数据寄存器                  (R/W)  */
	__O  uint32_t PTSET;                    /*!< Offset: 0x014 端口设置寄存器(只写)            ( /W)  */
	__O  uint32_t PTCLR;                    /*!< Offset: 0x018 端口复位寄存器(只写)            ( /W)  */
	__O  uint32_t PTTOG;                    /*!< Offset: 0x01C 端口翻转寄存器(只写)            ( /W)  */
	__IO uint32_t PTOD;                     /*!< Offset: 0x020 端口Open Drain功能配置寄存器    (R/W)  */
	__IO uint32_t FILT;                     /*!< Offset: 0x024 GPIO 输入模拟滤波控制寄存器     (R/W)  */
	__IO uint32_t HIIPM;                    /*!< Offset: 0x028 端口高阻控制寄存器              (R/W)  */
	__IO uint32_t NEWAFEN;                  /*!< Offset: 0x02C 新增复用功能生效控制寄存器      (R/W)  */
	__IO uint32_t AFCFG2;                   /*!< Offset: 0x030 端口复用功能配置寄存器2(写保护) (R/W)  */
}GPIO_TypeDef;                          /* end of group RT_GPIO_TypeDef                           */


/*****************************************  INT Control Block  *******************************************/
/*
* @brief INT中断寄存器定义
*/
typedef struct
{
    __IO uint32_t EXTIE;                    /*!< Offset: 0x000 外部中断边沿配置寄存器 (R/W) */
    __IO uint32_t EXTIF;                    /*!< Offset: 0x004 外部中断标志寄存器     (R/W) */
    __IO uint32_t PINFLT;                   /*!< Offset: 0x008 引脚数字滤波使能寄存器 (R/W) */
    __IO uint32_t Reserved[1];
    __IO uint32_t EXTIE2;                   /*!< Offset: 0x010 外部中断边沿配置寄存器2(R/W) */
    __IO uint32_t EXTIF2;                   /*!< Offset: 0x014 外部中断标志寄存器2    (R/W) */
    __IO uint32_t PINFLT2;                  /*!< Offset: 0x018 引脚数字滤波使能寄存器2(R/W) */
}INT_TypeDef;                           /* end of group RT_INT_TypeDef                  */


/*****************************************  DMA Control Block  *******************************************/
/*
* @brief DMA寄存器定义
*/

typedef struct
{
    __IO uint32_t DMAIE;                     /*!< Offset: 0x000 中断使能寄存器 (R/W)          */
    __IO uint32_t DMAIF;                     /*!< Offset: 0x004 通道中断标志 (R/W)            */
    __IO uint32_t CHNSTA;                    /*!< Offset: 0x008 通道0状态寄存器 (R/W)         */	
}DMA_TypeDef;                           /* end of group RT_DMA_TypeDef                    */

typedef struct
{
    __IO uint32_t CHNCTL;                   /*!< Offset: 0x000 通道控制寄存器 (R/W)       */
    __IO uint32_t CHNSRC;                   /*!< Offset: 0x004 通道源地址寄存器 (R/W)     */
    __IO uint32_t CHNTAR;                   /*!< Offset: 0x008 通道目的地址寄存器 (R/W)   */
    __IO uint32_t CHNCNT;                   /*!< Offset: 0x00C 通道传输数量设置寄存器(R/W)*/
    __IO uint32_t CHNTCCNT;                 /*!< Offset: 0x010 通道传输完成数据个数 (R/W) */
    __IO uint32_t CHNBULKNUM;               /*!< Offset: 0x014 通道块传输内个数设置 (R/W) */
}DMA_Channel_TypeDef;                   /* end of group RT_DMA_Channel_TypeDef        */

/*****************************************  CRC Control Block  *******************************************/
/*
* @brief CRC寄存器定义
*/
typedef struct
{
	__IO uint32_t CRCCON;						  /*!< Offset: 0x000 CRC控制寄存器 (R/W)		 */
	union
	{
		__I  uint32_t CRCDAT;					  /*!< Offset: 0x004 CRC数据寄存器  (R/W)		 */
		__O  uint32_t CRCDAT_DWORD; 			  /*!<	  Reserved								*/
		__O  uint16_t CRCDAT_WORD;				  /*!<	  Reserved								*/
		__O  uint16_t Reserved_WORD;			  /*!<	  Reserved								*/
		__O  uint8_t  CRCDAT_BYTE;
		__O  uint8_t  Reserved_BYTE[3];
	};
	__IO uint32_t CRCINIT;					 /*!< Offset: 0x008 CRC初始化种子寄存器(R/W)	*/
}CRC_TypeDef;							 /* end of group RT_CRC_TypeDef 				*/

/*****************************************  SCR Control Block  *******************************************/
/*
* @brief SCR寄存器定义
*/
typedef struct
{
    __IO uint32_t SCR;                       /*!< Offset: 0x010 系统控制寄存器 (R/W)        */
}SCR_TypeDef;                            /* end of group RT_SCR_TypeDef                 */

/*****************************************  DIV Control Block  *******************************************/
/*
* @brief DIV寄存器定义
*/
typedef struct
{
    __IO uint32_t CR;                    /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t DD;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t DI;                    /*!< Offset: 0x008    (R/W)        */
	  __IO uint32_t DQ;                    /*!< Offset: 0x00C    (R/W)        */
	  __IO uint32_t DR;                    /*!< Offset: 0x010    (R/W)        */
}DIV_TypeDef;                            /* end of group RT_DIV_TypeDef     */

/*****************************************  SQRT Control Block  *******************************************/
/*
* @brief SQRT寄存器定义
*/
typedef struct
{
    __IO uint32_t CR;                    /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t DI;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t DO;                    /*!< Offset: 0x008    (R/W)        */
}SQRT_TypeDef;                            /* end of group RT_SQRT_TypeDef    */

/*****************************************  PID Control Block  *******************************************/
/*
* @brief PID寄存器定义
*/
typedef struct
{
    __IO uint32_t CR;                     /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t REF;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t FB;                     /*!< Offset: 0x008    (R/W)        */
	  __IO uint32_t OUT;                    /*!< Offset: 0x00C    (R/W)        */
	  __IO uint32_t ERR;                    /*!< Offset: 0x010    (R/W)        */
    __IO uint32_t INTG;                   /*!< Offset: 0x014    (R/W)        */
	  __IO uint32_t KPG;                    /*!< Offset: 0x018    (R/W)        */
	  __IO uint32_t KIG;                    /*!< Offset: 0x01C    (R/W)        */
	  __IO uint32_t KDG;                    /*!< Offset: 0x020    (R/W)        */
	  __IO uint32_t KIMULP;                 /*!< Offset: 0x024    (R/W)        */
    __IO uint32_t KDIV;                    /*!< Offset: 0x028    (R/W)        */
	  __IO uint32_t INTGLIM;                /*!< Offset: 0x02C    (R/W)        */
	  __IO uint32_t OUTLIM;                 /*!< Offset: 0x030    (R/W)        */
}PID_TypeDef;                             /* end of group RT_PID_TypeDef     */


/*****************************************  ME Control Block  *******************************************/
/*
* @brief ME寄存器定义
*/
typedef struct
{
    __IO uint32_t CR;                    /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t Ia;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t Ib;                    /*!< Offset: 0x008    (R/W)        */
	  __IO uint32_t Ic;                    /*!< Offset: 0x00C    (R/W)        */
	  __IO uint32_t Ialpha;                /*!< Offset: 0x010    (R/W)        */
    __IO uint32_t Ibeta;                 /*!< Offset: 0x014    (R/W)        */
	  __IO uint32_t Iq;                    /*!< Offset: 0x018    (R/W)        */
	  __IO uint32_t Id;                    /*!< Offset: 0x01C    (R/W)        */
	  __IO uint32_t Vq;                    /*!< Offset: 0x020    (R/W)        */
	  __IO uint32_t Vd;                    /*!< Offset: 0x024    (R/W)        */
    __IO uint32_t Valpha;                /*!< Offset: 0x028    (R/W)        */
	  __IO uint32_t Vbeta;                 /*!< Offset: 0x02C    (R/W)        */
	  __IO uint32_t Angle;                 /*!< Offset: 0x030    (R/W)        */
	  __IO uint32_t Cos;                   /*!< Offset: 0x034    (R/W)        */
	  __IO uint32_t Sin;                   /*!< Offset: 0x038    (R/W)        */
	  __IO uint32_t IPDSin;                /*!< Offset: 0x03C    (R/W)        */
   	__IO uint32_t Kslid;                 /*!< Offset: 0x040    (R/W)        */
	  __IO uint32_t Kslf;                  /*!< Offset: 0x044    (R/W)        */
	  __IO uint32_t KF;                    /*!< Offset: 0x048    (R/W)        */
	  __IO uint32_t KG;                    /*!< Offset: 0x04C    (R/W)        */
	  __IO uint32_t E0;                    /*!< Offset: 0x050    (R/W)        */
	  __IO uint32_t Ealpha;                /*!< Offset: 0x054    (R/W)        */
	  __IO uint32_t Ebeta;                 /*!< Offset: 0x058    (R/W)        */
	  __IO uint32_t Zalpha;                /*!< Offset: 0x05C    (R/W)        */
    __IO uint32_t Zbeta;                 /*!< Offset: 0x060    (R/W)        */
	  __IO uint32_t IalphaError;           /*!< Offset: 0x064    (R/W)        */
    __IO uint32_t IbetaError;            /*!< Offset: 0x068    (R/W)        */
	  __IO uint32_t EstIalpha;             /*!< Offset: 0x06C    (R/W)        */
	  __IO uint32_t EstIbeta;              /*!< Offset: 0x070    (R/W)        */
	  __IO uint32_t SMOTheta;              /*!< Offset: 0x074    (R/W)        */
	  __IO uint32_t SMOIalpha;             /*!< Offset: 0x078    (R/W)        */
	  __IO uint32_t SMOIbeta;              /*!< Offset: 0x07C    (R/W)        */
   	__IO uint32_t BEMFA;                 /*!< Offset: 0x080    (R/W)        */
	  __IO uint32_t BEMFB;                 /*!< Offset: 0x084    (R/W)        */
	  __IO uint32_t IPDTheta;              /*!< Offset: 0x088    (R/W)        */
	__I	 uint32_t reserved[1];                    
	  __IO uint32_t SVZone;                /*!< Offset: 0x090    (R/W)        */
	  __IO uint32_t FOVM;                  /*!< Offset: 0x094    (R/W)        */
	  __IO uint32_t TPWM;                  /*!< Offset: 0x098    (R/W)        */
	  __IO uint32_t LSMIN;                 /*!< Offset: 0x09C    (R/W)        */
    __IO uint32_t PDCH1;                 /*!< Offset: 0x0A0    (R/W)        */
	  __IO uint32_t PDCH2;                 /*!< Offset: 0x0A4    (R/W)        */
	  __IO uint32_t PDCH3;                 /*!< Offset: 0x0A8    (R/W)        */
	__I  uint32_t reserved2[1];
		__IO uint32_t IER;                   /*!< Offset: 0x0B0    (R/W)        */
	__IO uint32_t IFR;                   /*!< Offset: 0x0B4    (R/W)        */
	__I  uint32_t reserved3[2];	
	__IO uint32_t MCYC;                    /*!< Offset: 0x0C0    (R/W)        */
}ME_TypeDef;                            /* end of group RT_ME_TypeDef      */

/*****************************************  OPA Control Block  *******************************************/
/*
* @brief OPA寄存器定义
*/
typedef struct
{
    __IO uint32_t OPA_CR;                     /*!< Offset: 0x000 控制寄存器       (R/W) */
	__IO uint32_t OPA_CR2;                    /*!< Offset: 0x004 控制寄存器       (R/W) */
}OPA_TypeDef;                             /* end of group RT_OPA_TypeDef            */

/*****************************************  CMP Control Block  *******************************************/
/*
* @brief CMP1寄存器定义
*/
typedef struct
{
	__IO uint32_t CR1; 				  /*!< Offset: 0x000 控制寄存器	   (R/W) */
	__IO uint32_t CR2;					/* Offset: 0x004 (R/W) */
	__IO uint32_t CAL;					/* Offset: 0x008 (R/W) */
}CMP_TypeDef;	 


/*****************************************  ADC Control Block  ******************************************/
/*
* @brief ADC寄存器定义
*/
typedef struct
{
    __IO uint32_t SADCCON;                  /*!< Offset: 0x000 SADC控制寄存器          (R/W)  */
    __IO uint32_t SADCCLKCFG;               /*!< Offset: 0x004 SADC时钟配置寄存器      (R/W)  */
    __IO uint32_t SADCDIFCFG;               /*!< Offset: 0x008 SADC差分输入配置寄存器  (R/W)  */
    __IO uint32_t SADCSTR;                  /*!< Offset: 0x00C SADC序列转换控制寄存器  (R/W)  */
    __IO uint32_t SADCIE;                   /*!< Offset: 0x010 SADC中断控制寄存器      (R/W)  */
    __IO uint32_t SADCIF;                   /*!< Offset: 0x014 SADC中断标志寄存器      (R/W)  */
    __IO uint32_t SADCSAMP;                 /*!< Offset: 0x018 SADC采样时间配置寄存器  (R/W)  */
    __IO uint32_t SADCSAMP2;                /*!< Offset: 0x01C SADC采样时间配置寄存器2     (R/W)  */
    __IO uint32_t SADCTRGLEN;               /*!< Offset: 0x020 SADC触发源及序列长度寄存器 (R/W)  */
	__IO uint32_t SADCSEQCFG;               /*!< Offset: 0x024 SADC序列通道配置寄存器     (R/W)  */
	__I  uint32_t reserved0[1];                   
	__IO uint32_t SADCDAT;                  /*!< Offset: 0x02C SADC通用数据寄存器    (R)  */
	__IO  uint32_t SADC0DAT;                /*!< Offset: 0x030 SADC0序列数据寄存器 (R)  */
	__IO  uint32_t SADC1DAT;                /*!< Offset: 0x034 SADC1序列数据寄存器 (R)  */
	__IO  uint32_t SADC2DAT;                /*!< Offset: 0x038 SADC2序列数据寄存器 (R)  */
	__IO  uint32_t SADC3DAT;                /*!< Offset: 0x03C SADC3序列数据寄存器 (R)  */
	__IO  uint32_t SADC4DAT;                /*!< Offset: 0x040 SADC4序列数据寄存器 (R)  */
	__IO  uint32_t SADC5DAT;                /*!< Offset: 0x044 SADC5序列数据寄存器 (R)  */
	__IO  uint32_t SADC6DAT;                /*!< Offset: 0x048 SADC6序列数据寄存器 (R)  */
	__IO  uint32_t SADC7DAT;                /*!< Offset: 0x04C SADC7序列数据寄存器 (R)  */
	__I  uint32_t reserved1[8];    
	__IO uint32_t SADC_CALCH;               /*!< Offset: 0x070                     (R/W)  */
	__IO uint32_t SADC_CALDAT0;             /*!< Offset: 0x074                     (R/W)  */
	__IO uint32_t SADC_CALDAT1;             /*!< Offset: 0x078                     (R/W)  */
	__IO uint32_t SADC_CALDAT2;             /*!< Offset: 0x07C                     (R/W)  */
	__IO uint32_t SADC_CALDAT3;             /*!< Offset: 0x080                     (R/W)  */
	__IO uint32_t SADC_CALDAT4;             /*!< Offset: 0x084                     (R/W)  */
	__IO uint32_t TPSOffset;                /*!< Offset: 0x088                     (R/W)  */
	__IO uint32_t SADCCRV;                  /*!< Offset: 0x08C                     (R/W)  */
	__IO uint32_t SADCTAD;                  /*!< Offset: 0x090                     (R/W)  */
	__IO uint32_t SADCDVS;                  /*!< Offset: 0x094                     (R/W)  */	
}ADC_TypeDef;                           /* end of group RT_ADC_TypeDef               */

/*****************************************  TIM8 Control Block  *******************************************/
/*
* @brief TIM8寄存器定义
*/
typedef struct
{
	__IO uint32_t CR1;                  /*!< Offset: 0x000 控制寄存器   (R/W) */
	__IO uint32_t CR2;                  /*!< Offset: 0x004 控制寄存器2  (R/W) */
	__IO uint32_t SMCR;                 /*!< Offset: 0x008 控制寄存器   (R/W) */
	__IO uint32_t DIER;                 /*!< Offset: 0x00C 控制寄存器   (R/W) */
	__IO uint32_t SR;                   /*!< Offset: 0x010 状态寄存器   (R/W) */
	__IO uint32_t EGR;                  /*!< Offset: 0x014 控制寄存器   (R/W) */
	__IO uint32_t CCMR1;                /*!< Offset: 0x018 控制寄存器   (R/W) */
	__IO uint32_t CCMR2;                /*!< Offset: 0x01C 控制寄存器   (R/W) */
	__IO uint32_t CCER;                 /*!< Offset: 0x020 捕获/比较使能寄存器(R/W)*/
	__IO uint32_t CNT;                  /*!< Offset: 0x024 计数器             (R/W)*/
	__IO uint32_t PSC;                  /*!< Offset: 0x028 预分频器           (R/W)*/
	__IO uint32_t ARR;                  /*!< Offset: 0x02C 自动重装载寄存器   (R/W)*/
	__IO uint32_t RCR;                  /*!< Offset: 0x030 重复计数寄存器     (R/W)*/
	__IO uint32_t CCR1;                 /*!< Offset: 0x034 捕获/比较寄存器1   (R/W)*/
	__IO uint32_t CCR2;                 /*!< Offset: 0x038 捕获/比较寄存器2   (R/W)*/
	__IO uint32_t CCR3;                 /*!< Offset: 0x03C 捕获/比较寄存器3   (R/W)*/
	__IO uint32_t CCR4;                 /*!< Offset: 0x040 捕获/比较寄存器4   (R/W)*/
	__IO uint32_t BDTR;                 /*!< Offset: 0x044 刹车和死区寄存器   (R/W)*/
	__IO uint32_t DCR;                  /*!< Offset: 0x048 DMA控制寄存器      (R/W)*/
	__IO uint32_t DMAR;                 /*!< Offset: 0x04C 连续模式的DMA地址  (R/W)*/
	__IO uint32_t CCR5;                 /*!< Offset: 0x050 捕获/比较寄存器5   (R/W)*/
	__IO uint32_t CCMR3;                /*!< Offset: 0x054 捕获/比较模式寄存器3  (R/W)*/
}TIM8_TypeDef;                      /* end of group RT_TIM8_TypeDef               */

/*****************************************  Info Control Block  ******************************************/


/*
**********************************************************************************************************
*                                        Peripheral memory map
*                                           Unit of Byte
**********************************************************************************************************
*/

/*  Base Address  */
#define RT_FLASH_BASE        (0x00000000UL)
#define RT_RAM_BASE          (0x20000000UL)
#define RT_APB0_BASE         (0x40000000UL)
#define RT_AHB_BASE          (0x40020000UL)

#define RT_FLASH_SIZE        (0x00008000UL)
#define RT_RAM_SIZE          (0x00001000UL)
#define RT_APB0_SIZE         (0x00020000UL)
#define RT_AHB_SIZE          (0x00010000UL)

/*  APB0 Peripherals Address  */
#define RT_TMR0_BASE         (RT_APB0_BASE + 0x01000)
#define RT_TMR1_BASE         (RT_APB0_BASE + 0x02000)
#define RT_TMR2_BASE         (RT_APB0_BASE + 0x03000)
#define RT_TMR3_BASE         (RT_APB0_BASE + 0x04000)

#define RT_UART0_BASE        (RT_APB0_BASE + 0x05000)
#define RT_UART1_BASE        (RT_APB0_BASE + 0x06000)
#define RT_TIM8_BASE         (RT_APB0_BASE + 0x07000)

#define RT_I2C_BASE          (RT_APB0_BASE + 0x0A000)
#define RT_SPI_BASE          (RT_APB0_BASE + 0x0B000)

#define RT_RTC_BASE          (RT_APB0_BASE + 0x0C000)
#define RT_OPA_BASE          (RT_APB0_BASE + 0x0D000)
#define RT_CMP1_BASE         (RT_APB0_BASE + 0x0E000)
#define RT_CMP2_BASE         (RT_APB0_BASE + 0x0E010)
#define RT_CMP3_BASE         (RT_APB0_BASE + 0x0E020)
#define RT_CMP4_BASE         (RT_APB0_BASE + 0x0E030)
#define RT_CMP5_BASE         (RT_APB0_BASE + 0x0E040)

#define RT_ADC_BASE          (RT_APB0_BASE + 0x0E100)

#define RT_CMU_BASE          (RT_APB0_BASE + 0x0F000)
#define RT_PMU_BASE          (RT_APB0_BASE + 0x0F400)

#define RT_WDT_BASE          (RT_APB0_BASE + 0x10000)
#define RT_GPIO_BASE         (RT_APB0_BASE + 0x11000)
#define RT_INT_BASE          (RT_APB0_BASE + 0x11800)

/*  AHB Peripherals Address  */
#define RT_DMA_BASE          (RT_AHB_BASE )
#define RT_DMA_Channel0_BASE (RT_AHB_BASE + 0x0000C)
#define RT_DMA_Channel1_BASE (RT_AHB_BASE + 0x00024)
#define RT_DMA_Channel2_BASE (RT_AHB_BASE + 0x0003C)

#define RT_CRC_BASE          (RT_AHB_BASE + 0x01000)

#define RT_DIV_BASE          (RT_AHB_BASE + 0x02000)

#define RT_SQRT_BASE         (RT_AHB_BASE + 0x03000)

#define RT_PID1_BASE         (RT_AHB_BASE + 0x04000)
#define RT_PID2_BASE         (RT_AHB_BASE + 0x05000)
#define RT_PID3_BASE         (RT_AHB_BASE + 0x06000)

#define RT_ME_BASE           (RT_AHB_BASE + 0x08000)

/*  GPIO Address  */
#define RT_GPIOA_BASE        (RT_GPIO_BASE + 0x00000)
#define RT_GPIOB_BASE        (RT_GPIO_BASE + 0x00100)
#define RT_GPIOC_BASE        (RT_GPIO_BASE + 0x00200)
#define RT_GPIOD_BASE        (RT_GPIO_BASE + 0x00300)

#define RT_INFO_BASE         (RT_FLASH_BASE + 0x00040000)
#define RT_INFO_SIZE         (0x00000400UL)
#define RT_INFO_BANK         (0x00000400UL)

/*
**********************************************************************************************************
*                                        Peripheral declaration
**********************************************************************************************************
*/
#define TMR0              ((TMR_TypeDef    *) RT_TMR0_BASE )
#define TMR1              ((TMR_TypeDef    *) RT_TMR1_BASE )
#define TMR2              ((TMR_TypeDef    *) RT_TMR2_BASE )
#define TMR3              ((TMR_TypeDef    *) RT_TMR3_BASE )

#define UART0             ((UART_TypeDef   *) RT_UART0_BASE)
#define UART1             ((UART_TypeDef   *) RT_UART1_BASE)

#define TIM8              ((TIM8_TypeDef   *) RT_TIM8_BASE )

#define I2C               ((I2C_TypeDef    *) RT_I2C_BASE  )
#define SPI0              ((SPI_TypeDef    *) RT_SPI_BASE  )

#define RTC               ((RTC_TypeDef    *) RT_RTC_BASE  )

#define OPA               ((OPA_TypeDef    *) RT_OPA_BASE  )
#define CMP1              ((CMP_TypeDef   *) RT_CMP1_BASE )
#define CMP2              ((CMP_TypeDef   *) RT_CMP2_BASE )
#define CMP3              ((CMP_TypeDef   *) RT_CMP3_BASE )
#define CMP4              ((CMP_TypeDef   *) RT_CMP4_BASE )
#define CMP5              ((CMP_TypeDef   *) RT_CMP5_BASE )

#define ADC               ((ADC_TypeDef    *) RT_ADC_BASE  )
#define CMU               ((CMU_TypeDef    *) RT_CMU_BASE  )

#define PMU               ((PMU_TypeDef    *) RT_PMU_BASE  )

#define WDT               ((WDT_TypeDef    *) RT_WDT_BASE  )
#define GPIO              ((GPIO_TypeDef   *) RT_GPIO_BASE )
#define INT               ((INT_TypeDef    *) RT_INT_BASE  )

#define DMA               ((DMA_TypeDef    *) RT_DMA_BASE  )
#define DMA_Channel0      ((DMA_Channel_TypeDef*) RT_DMA_Channel0_BASE  )
#define DMA_Channel1      ((DMA_Channel_TypeDef*) RT_DMA_Channel1_BASE  )
#define DMA_Channel2      ((DMA_Channel_TypeDef*) RT_DMA_Channel2_BASE  )

#define CRC               ((CRC_TypeDef    *) RT_CRC_BASE  )
#define DIV               ((DIV_TypeDef    *) RT_DIV_BASE  )
#define SQRT              ((SQRT_TypeDef   *) RT_SQRT_BASE )

#define PID1              ((PID_TypeDef    *) RT_PID1_BASE )
#define PID2              ((PID_TypeDef    *) RT_PID2_BASE )
#define PID3              ((PID_TypeDef    *) RT_PID3_BASE )

#define ME                ((ME_TypeDef     *) RT_ME_BASE   )

#define GPIOA             ((GPIO_TypeDef   *) RT_GPIOA_BASE)
#define GPIOB             ((GPIO_TypeDef   *) RT_GPIOB_BASE)
#define GPIOC             ((GPIO_TypeDef   *) RT_GPIOC_BASE)
#define GPIOD             ((GPIO_TypeDef   *) RT_GPIOD_BASE)



/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of Timer Module
**********************************************************************************************************
*/
// TMR
#define  TMR_TMRCON_mask                            ((uint32_t)0x80ff)
#define  TMR_TMRDIV_mask                            ((uint32_t)0xffff)  
#define  TMR_TMRPRD_mask                            ((uint32_t)0xffff)
#define  TMR_TMRCAP_mask                            ((uint32_t)0xffff) 
#define  TMR_TMRCNT_mask                            ((uint32_t)0xffff)  
#define  TMR_TMRCMP_mask                            ((uint32_t)0xffff)  
#define  TMR_TMRCMP_mask                            ((uint32_t)0xffff)  
#define  TMR_TMRIF_mask                             ((uint32_t)0x000f) 


// UART
#define  UART_MODESEL_mask                          ((uint32_t)0x0001) 
#define  UART_UARTCON_mask                          ((uint32_t)0x87ff)
#define  UART_SREL_mask                             ((uint32_t)0xffff) 
#define  UART_SBUF_mask                             ((uint32_t)0x00ff)
#define  UART_UARTSTA_mask                          ((uint32_t)0x001f) 


// TIM8
#define  TIM8_CR1_mask                              ((uint32_t)0x03FF) 
#define  TIM8_CR2_mask                              ((uint32_t)0xFFFD)           /*!< 控制寄存器2 */
#define  TIM8_SMCR_mask                             ((uint32_t)0xFFF7)
#define  TIM8_DIER_mask                             ((uint32_t)0x1FFFF)
#define  TIM8_SR_mask                               ((uint32_t)0x7EFF)
#define  TIM8_EGR_mask                              ((uint32_t)0x01FF)
#define  TIM8_CCMR1_mask                            ((uint32_t)0xFFFF)
#define  TIM8_CCMR2_mask                            ((uint32_t)0xFFFF)
#define  TIM8_CCER_mask                             ((uint32_t)0xFFFF)
#define  TIM8_CNT_mask                              ((uint32_t)0xFFFF)
#define  TIM8_PSC_mask                              ((uint32_t)0xFFFF)           /*!< 预分频器 */
#define  TIM8_ARR_mask                              ((uint32_t)0xFFFF)           /*!< 自动重装载寄存器 */
#define  TIM8_RCR_mask                              ((uint32_t)0x00FF)           /*!< 重复计数寄存器 */
#define  TIM8_CCR1_mask                             ((uint32_t)0xFFFF)           /*!< 捕获/比较寄存器1 */
#define  TIM8_CCR2_mask                             ((uint32_t)0xFFFF)           /*!< 捕获/比较寄存器2 */
#define  TIM8_CCR3_mask                             ((uint32_t)0xFFFF)           /*!< 捕获/比较寄存器3 */
#define  TIM8_CCR4_mask                             ((uint32_t)0xFFFF)           /*!< 捕获/比较寄存器4 */
#define  TIM8_BDTR_mask                             ((uint32_t)0xFFFF)
#define  TIM8_DCR_mask                              ((uint32_t)0x1F1F)
#define  TIM8_CCR5_mask                             ((uint32_t)0xFFFF)
#define  TIM8_CCMR3_mask                            ((uint32_t)0xFFFF)

// I2C
#define  I2C_I2CDAT_mask                            ((uint32_t)0x00ff)  
#define  I2C_I2CADR_mask                            ((uint32_t)0x00ff) 
#define  I2C_I2CCON_mask                            ((uint32_t)0x7fff) 
#define  I2C_I2CSTA_mask                            ((uint32_t)0x00f8)
// SPI
#define  SPI_SPICON_mask                            ((uint32_t)0x00ff)
#define  SPI_SPISTA_mask                            ((uint32_t)0x000f) 
#define  SPI_SPIDAT_mask                            ((uint32_t)0x00ff)
#define  SPI_SPISSN_mask                            ((uint32_t)0x00ff)
#define  SPI_CSDLY_mask                             ((uint32_t)0x00ff) 
// RTC
#define  RTC_RTCCON_mask                            ((uint32_t)0x806f) 
#define  RTC_RTCIE_mask                             ((uint32_t)0x00ff) 
#define  RTC_RTCIF_mask                             ((uint32_t)0x00ff) 
#define  RTC_ALMR_mask                              ((uint32_t)0x1f3f)   
#define  RTC_RTCTMR1_mask                           ((uint32_t)0xffff)
#define  RTC_RTCTMR2_mask                           ((uint32_t)0xffff) 
#define  RTC_SECR_mask                              ((uint32_t)0x003f)           /*!<RTC秒寄存器           */
#define  RTC_MINR_mask                              ((uint32_t)0x003f)           /*!<RTC分钟寄存器         */
#define  RTC_HOURR_mask                             ((uint32_t)0x001f)           /*!<RTC小时寄存器         */
#define  RTC_DAYR_mask                              ((uint32_t)0x001f)           /*!<RTC日寄存器           */
#define  RTC_MONTHR_mask                            ((uint32_t)0x000f)           /*!<RTC月寄存器           */
#define  RTC_YEARR_mask                             ((uint32_t)0x007f)           /*!<RTC年寄存器           */
#define  RTC_WEEKR_mask                             ((uint32_t)0x0007)           /*!<RTC周寄存器           */ 
#define  RTC_RTCRD_mask                             ((uint32_t)0x0001)           /*!<RTC读控制位         */
#define  RTC_RTCWR_mask                             ((uint32_t)0x0001)           /*!<RTC写控制位         */

// PMU
#define  PMU_PMUCON_mask                            ((uint32_t)0x8013) 
#define  PMU_VDETCFG_mask                           ((uint32_t)0xc1ff)
#define  PMU_VDETPCFG_mask                          ((uint32_t)0x001f)
#define  PMU_PMUIE_mask                             ((uint32_t)0x0003)
#define  PMU_PMUIF_mask                             ((uint32_t)0x0003)
#define  PMU_PMUSTA_mask                            ((uint32_t)0x0003)
#define  PMU_WAKEIF_mask                            ((uint32_t)0xffffffff) 
#define  PMU_PDTFLT_mask                            ((uint32_t)0x000f) 
#define  PMU_RSTSTA_mask                            ((uint32_t)0xffff) 

// CMU
#define  CMU_SYSCLKCFG_mask                         ((uint32_t)0x0087)
#define  CMU_JTAGSTA_mask                           ((uint32_t)0x0001)  
#define  CMU_LRCADJ_mask                            ((uint32_t)0x00ff)
#define  CMU_HRCADJ_mask                            ((uint32_t)0x007f) 
#define  CMU_HRCDIV_mask                            ((uint32_t)0x0003) 
#define  CMU_CLKSTA_mask                            ((uint32_t)0x017c) 
#define  CMU_SYSCLKDIV_mask                         ((uint32_t)0x000f)
#define  CMU_CLKOUTSEL_mask                         ((uint32_t)0x0007)
#define  CMU_CLKOUTDIV_mask                         ((uint32_t)0x000f)
#define  CMU_CLKCTRL0_mask                          ((uint32_t)0xFEBB7C)
#define  CMU_CLKCTRL1_mask                          ((uint32_t)0x1C403F)  
#define  CMU_CHIPID_mask                            ((uint32_t)0xFFFF)
#define  CMU_PLLLOCK_mask                           ((uint32_t)0x0033)
#define  CMU_MULTFUNCFG_mask                        ((uint32_t)0x8003)
#define  RCC_LFCLKCFG_mask                          ((uint32_t)0x8005)
#define  CMU_C_PLL_mask                             ((uint32_t)0x8017)
#define  CMU_FLTCTR_mask                            ((uint32_t)0x00FF)


// WDT
#define  WDT_WDTCLR_mask                            ((uint32_t)0xffff)
#define  WDT_WDTCNT_mask                            ((uint32_t)0xffff)   
// GPIO
#define  GPIO_IOCFG_mask                            ((uint32_t)0xffff)
#define  GPIO_AFCFG1_mask                           ((uint32_t)0xffff)  
#define  GPIO_AFCFG2_mask                           ((uint32_t)0xffff)
#define  GPIO_PTDIR_mask                            ((uint32_t)0xffff) 
#define  GPIO_PTUP_mask                             ((uint32_t)0xffff)
#define  GPIO_PTDAT_mask                            ((uint32_t)0xffff)
#define  GPIO_PTSET_mask                            ((uint32_t)0xffff)
#define  GPIO_PTCLR_mask                            ((uint32_t)0xffff)
#define  GPIO_PTTOG_mask                            ((uint32_t)0xffff)
#define  GPIO_PTOD_mask                             ((uint32_t)0xffff)
#define  GPIO_FILT_mask                             ((uint32_t)0xffff)  
#define  GPIO_HIIPM_mask                            ((uint32_t)0xffff) 
#define  GPIO_NEWAFEN_mask                          ((uint32_t)0xffff)
// EXTI
#define  INT_EXTIE_mask                             ((uint32_t)0x7f7f)  
#define  INT_EXTIF_mask                             ((uint32_t)0x7f7f) 
#define  INT_PINFLT_mask                            ((uint32_t)0x037f) 
#define  INT_EXTIE2_mask                            ((uint32_t)0x0007<<16)
#define  INT_EXTIF2_mask                            ((uint32_t)0x0303<<16)
#define  INT_PINFLT2_mask                           ((uint32_t)0x0007<<16) 
// DMA
#define  DMA_DMAIE_mask                             ((uint32_t)0x0777)  
#define  DMA_DMAIF_mask                             ((uint32_t)0x0777) 
#define  DMA_CHNSTA_mask                            ((uint32_t)0x0007)
#define  DMA_CHNCTL_mask                            ((uint32_t)0x7fff) 
#define  DMA_CHNSRC_mask                            ((uint32_t)0xffffffff)
#define  DMA_CHNTAR_mask                            ((uint32_t)0xffffffff)
#define  DMA_CHNCNT_mask                            ((uint32_t)0xffff)
#define  DMA_CHNTCCNT_mask                          ((uint32_t)0xffff) 
#define  DMA_CHNBULKNUM_mask                        ((uint32_t)0xffff)

// CRC
#define  CRC_CRCCON_mask                            ((uint32_t)0x0077) 
#define  CRC_CRCDAT_mask                            ((uint32_t)0xffffffff)  
#define  CRC_CRCINIT_mask                           ((uint32_t)0xffffffff) 
// DIV
#define  DIV_CR_mask                                ((uint32_t)0x001f)
#define  DIV_DD_mask                                ((uint32_t)0xffffffff) 
#define  DIV_DI_mask                                ((uint32_t)0xffffffff)
#define  DIV_DQ_mask                                ((uint32_t)0xffffffff)
#define  DIV_DR_mask                                ((uint32_t)0xffffffff) 
// OPA
#define  OPA_CR_mask                                ((uint32_t)0x01ff)
#define  OPA_CR2_mask                               ((uint32_t)0x0f00) 
// CMP
#define  CMP_CR1_mask                               ((uint32_t)0xf7ff)
#define  CMP_CR2_mask                               ((uint32_t)0x07ff)
#define  CMP_CAL_mask                               ((uint32_t)0x0F3F)
// SQRT
#define  SQRT_CR_mask                               ((uint32_t)0x0006)
#define  SQRT_DI_mask                               ((uint32_t)0xFFFFFFFF)
#define  SQRT_DO_mask                               ((uint32_t)0xFFFF)  
// PID
#define  PID_CR_mask                                ((uint32_t)0x000F)  
#define  PID_REF_mask                               ((uint32_t)0xFFFF) 
#define  PID_FB_mask                                ((uint32_t)0xFFFF) 
#define  PID_OUT_mask                               ((uint32_t)0xFFFF)
#define  PID_ERR_mask                               ((uint32_t)0xFFFF) 
#define  PID_INTG_mask                              ((uint32_t)0xFFFFFFFF)
#define  PID_KPG_mask                               ((uint32_t)0xFFFF)
#define  PID_KIG_mask                               ((uint32_t)0xFFFF)
#define  PID_KDG_mask                               ((uint32_t)0xFFFF)
#define  PID_KIMULP_mask                            ((uint32_t)0xFFFF)
#define  PID_PIDDIV_mask                            ((uint32_t)0x0FFF)
#define  PID_OUTLIM_mask                            ((uint32_t)0xFFFF)           /*!<   */
// ME
#define  ME_CR_mask                                 ((uint32_t)0x1FFFF)
#define  ME_Ia_mask                                 ((uint32_t)0xFFFF)
#define  ME_Ib_mask                                 ((uint32_t)0xFFFF) 
#define  ME_Ic_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Ialpha_mask                             ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Ibeta_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Iq_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Id_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Vq_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Vd_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Valpha_mask                             ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Vbeta_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Angle_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Cos_mask                                ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Sin_mask                                ((uint32_t)0xFFFF)           /*!<  */
#define  ME_IPDSin_mask                             ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Kslid_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Kslf_mask                               ((uint32_t)0xFFFF)           /*!<  */
#define  ME_KF_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_KG_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_E0_mask                                 ((uint32_t)0xFFFF)           /*!<  */
#define  ME_Ealpha_mask                             ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_Ebeta_mask                              ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_Zalpha_mask                             ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_Zbeta_mask                              ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_IalphaError_mask                        ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_IbetaError_mask                         ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_EstIalpha_mask                          ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_EstIbeta_mask                           ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_SMOTheta_mask                           ((uint32_t)0xFFFF)           /*!<  */
#define  ME_SMOIalpha_mask                          ((uint32_t)0xFFFF)           /*!<  */
#define  ME_SMOIbeta_mask                           ((uint32_t)0xFFFF)           /*!<  */
#define  ME_BEMFA_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_BEMFB_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_IPDTheta_mask                           ((uint32_t)0xFFFF)           /*!<  */
#define  ME_SVZone_mask                             ((uint32_t)0x00FF)           /*!<  */
#define  ME_FOVM_mask                               ((uint32_t)0xFFFF)           /*!<  */
#define  ME_TPWM_mask                               ((uint32_t)0xFFFF)           /*!<  */
#define  ME_LSMIN_mask                              ((uint32_t)0xFFFF)           /*!<  */
#define  ME_PDCH1_mask                              ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_PDCH2_mask                              ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_PDCH3_mask                              ((uint32_t)0xFFFFFFFF)           /*!<  */
#define  ME_IER_mask                                ((uint32_t)0x1f71)
#define  ME_IFR_mask                                ((uint32_t)0x1f71)
// ADC
#define  ADC_SADCCON_mask                           ((uint32_t)0xFF3F)
#define  ADC_SADCCLKCFG_mask                        ((uint32_t)0x010f)
#define  ADC_SADCDIFCFG_mask                        ((uint32_t)0x07f3)
#define  ADC_SADCSTR_mask                           ((uint32_t)0x0001)
#define  ADC_SADCIE_mask                            ((uint32_t)0x0103)
#define  ADC_SADCIF_mask                            ((uint32_t)0x0103)
#define  ADC_SADCSAMP_mask                          ((uint32_t)0xffffffff)
#define  ADC_SADCSAMP2_mask                         ((uint32_t)0xffffffff)
#define  ADC_SADCTRGLEN_mask                        ((uint32_t)0x073f)
#define  ADC_SADCSEQCFG_mask                        ((uint32_t)0xffffffff)
#define  ADC_SADCDAT_mask                           ((uint32_t)0x0fff)
#define  ADC_SADC_CALCH_mask                        ((uint32_t)0x0f13) 
#define  ADC_SADC_CALDAT_mask                      ((uint32_t)0x0fff)
#define  ADC_SADC_TPSOffset_mask                    ((uint32_t)0x0fff)
#define  ADC_SADCCRV_mask                           ((uint32_t)0x000f)



#ifdef __cplusplus
}
#endif

#endif
