/*
*********************************************************************************************************
*                                              rx32s10xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : rx32s10xx
* File         : system_rx32s10xx.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s10xx
*********************************************************************************************************
*/


#include "rx32s10xx.h"
#include "system_rx32s10xx.h"
#include "main.h"

//#define SYSCLK_HSE	FREQ_HSE
//#define SYSCLK_HRC  FREQ_HRC
#define SYSCLK_PLL	((uint32_t)FREQ_FSYS_Mhz*1000000)
/*
*********************************************************************************************************
*                                           本地宏/结构体
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             本地变量
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                           本地函数申明
*********************************************************************************************************
*/
#ifdef SYSCLK_HSE
uint32_t SystemCoreClock = SYSCLK_HSE;
#elif defined SYSCLK_PLL
uint32_t SystemCoreClock = SYSCLK_PLL;
#else
uint32_t SystemCoreClock = SYSCLK_HRC;
#endif

static void SetFsysClock(void);
static void LoadTrimCode(void);
/*
*********************************************************************************************************
*                                        INITIAL THE SYSTERM
*
* 函数说明：此函数在main()之前被调用
*
* 入口参数：无
*
* 返回参数：无
*
*********************************************************************************************************
*/
void SystemInit (void)
{
	SetFsysClock();
	LoadTrimCode();
}

static void SetFsysClock(void)
{

#ifdef SYSCLK_HSE
	//------------------------- HRC
	CMU_enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
	CMU_set_HRC22M(0); // HRC/2
	CMU_set_HRCDIV(CMU_HRCDIV_1); // HRC_DIV1
	//------------------------- HSE
	GPIOA->IOCFG |= (0x03<<2);
	GPIOA->AFCFG1 &= ~(0x0F<<4);     //第一复用

	CLR(GPIOA->PTUP,(0x03<<2));
	SET(GPIOA->PTOD,(0x03<<2));
	
	CMU_enable_CLKCTRL0(CMU_CLKCTRL0_HSEEN);
	while((CMU->CLKSTA & CMU_CLKSTA_HSELOCK) == 0);					//Wait HSE_LOCK 
	// 啟動HSE停振檢測
	CMU_disable_CLKCTRL0(CMU_CLKCTRL0_HSEDETEN); 
  //------------------------- SYS
	CMU_set_CPUCLK_DIV(CMU_CPUCLKDIV_1);  // CPU_DIV1
	CMU_set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HSE);
	
#elif defined SYSCLK_PLL
	//------------------------- HRC
	CMU_enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
	CMU_set_HRC22M(0); // HRC/2
	CMU_set_HRCDIV(CMU_HRCDIV_1); // HRC_DIV1
  //------------------------- PLL
  CMU_set_PLLSEL(0);     // PLL source HRC
	CMU_set_CPLL_GAIN(3);  // x8
	CMU_set_CPLL_DIV(0);   // /2
	CMU_set_SEL20M(1);         //SEL_20M  1:pll=42M  0:21M
	CMU_open_PLL();	
  //------------------------- SYS
	CMU->PREFETCH = 1;  // enable prefetch
	CMU_set_CPUCLK_DIV(CMU_CPUCLKDIV_1);  // CPU_DIV1
	CMU_set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_PLL);

#else
	//------------------------- HRC
	CMU_enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
	CMU_set_HRC22M(0); // HRC/2
	CMU_set_HRCDIV(CMU_HRCDIV_1); // HRC_DIV1
  //------------------------- SYS
	CMU_set_CPUCLK_DIV(CMU_CPUCLKDIV_1);  // CPU_DIV1
	CMU_set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HRC);
	
#endif	
}

static void LoadTrimCode(void)
{
	if(M32(INFO_HRCADJ_ADR) == ~M32(INFO_HRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->HRCADJ = M32(INFO_HRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  
  if(M32(INFO_LRCADJ_ADR) == ~M32(INFO_LRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->LRCADJ = M32(INFO_LRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  
  if((M16(INFO_CMP1PADJ_ADR) == ~M16(INFO_CMP1PADJ_ADR+2))
    &&(M16(INFO_CMP1NADJ_ADR) == ~M16(INFO_CMP1NADJ_ADR+2)))
  {
    CMP1->CAL = (M16(INFO_CMP1NADJ_ADR)<<8)|M16(INFO_CMP1PADJ_ADR);
  }
  
  if((M16(INFO_CMP2PADJ_ADR) == ~M16(INFO_CMP2PADJ_ADR+2))
    &&(M16(INFO_CMP2NADJ_ADR) == ~M16(INFO_CMP2NADJ_ADR+2)))
  {
    CMP2->CAL = (M16(INFO_CMP2NADJ_ADR)<<8)|M16(INFO_CMP2PADJ_ADR);
  }
  
  if((M16(INFO_CMP3PADJ_ADR) == ~M16(INFO_CMP3PADJ_ADR+2))
    &&(M16(INFO_CMP3NADJ_ADR) == ~M16(INFO_CMP3NADJ_ADR+2)))
  {
    CMP3->CAL = (M16(INFO_CMP3NADJ_ADR)<<8)|M16(INFO_CMP3PADJ_ADR);
  }
  
  if((M16(INFO_CMP4PADJ_ADR) == ~M16(INFO_CMP4PADJ_ADR+2))
    &&(M16(INFO_CMP4NADJ_ADR) == ~M16(INFO_CMP4NADJ_ADR+2)))
  {
    CMP4->CAL = (M16(INFO_CMP4NADJ_ADR)<<8)|M16(INFO_CMP4PADJ_ADR);
  }
  
  if((M16(INFO_CMP5PADJ_ADR) == ~M16(INFO_CMP5PADJ_ADR+2))
    &&(M16(INFO_CMP5NADJ_ADR) == ~M16(INFO_CMP5NADJ_ADR+2)))
  {
    CMP5->CAL = (M16(INFO_CMP5NADJ_ADR)<<8)|M16(INFO_CMP5PADJ_ADR);
  }
}
/*
*********************************************************************************************************
*                                        IAR 上电变量是否初始化
*
* 函数说明：只有IAR编译器时才会使用（修改返回值就能决定初始化选项）
*
* 入口参数：无
*
* 返回参数：返回值“0”代表RAM不初始化，返回值“1”代表RAM初始化
*
*********************************************************************************************************
*/
#if defined ( __ICCARM__ )

int __low_level_init(void)
{
    return 1;
}
#endif
