/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : system_rx32s11xx.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/


#include "rx32s11xx.h"
#include "system_rx32s11xx.h"
#include "rx32s11xx_cmu.h"
#include "rx32s11xx_adc.h"
#include "rx32s11xx_gpio.h"

//#define SYSCLK_HSE FREQ_HSE
#define SYSCLK_HRC FREQ_HRC

/*
*********************************************************************************************************
*                                          local macro/Structure
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                        local Function declaration
*********************************************************************************************************
*/
#ifdef SYSCLK_HSE
uint32_t SystemCoreClock = SYSCLK_HSE;
#elif defined SYSCLK_HRC
uint32_t SystemCoreClock = SYSCLK_HRC;
#endif

static void SetFsysClock(void);
static void LoadTrimCode(void);
/*
*********************************************************************************************************
*                                        INITIAL THE SYSTERM
*
*********************************************************************************************************
*/
void SystemInit(void)
{
	SetFsysClock();
	LoadTrimCode();
}

static void SetFsysClock(void)
{
#ifdef SYSCLK_HSE
  GPIO_InitTypeDef GPIO_InitStruct = {0};  
  //GPIO Configuration
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Pull  = GPIO_FLOATING;
  GPIO_InitStruct.AF    = AF_FUNCTION1;
  GPIO_InitStruct.Pin   = GPIO_PIN_7;//PD7_AF1:OSC_IN
  GPIO_Init(GPIOD, &GPIO_InitStruct);
  GPIO_InitStruct.Pin   = GPIO_PIN_8;//PD8_AF1:OSC_OUT
  GPIO_Init(GPIOD, &GPIO_InitStruct);
  //Enable HSE
  CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HSELOCKEN);//HSE_LOCK Flag is the real HSE status
  CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HSEEN);
  while( (CMU->CLKSTA & CMU_CLKSTA_HSELOCK)==0 );
  CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HSEDETEN);
  //Select Sysclk Source to HSE
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HSE);    //Fsys = 40MHz   ensure that HSE can start oscillation before config
//  //Disable HRC
//  CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  //Set CPUCLK_DIV
  CMU_Set_SYSCLKDIV(CMU_SYSCLKDIV_1);          //Fcpu = 40MHz
  //Set SADCCLK_DIV
  SADC_Set_ClkPrescaler(SADC,SADC_SADCCLK_DIV3);//Fadc = 13.33MHz
  
#elif defined SYSCLK_HRC
  //Enable HRC
	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  //Select Sysclk Source to HRC
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HRC);    //Fsys = 42MHz
  //Set CPUCLK_DIV
  CMU_Set_SYSCLKDIV(CMU_SYSCLKDIV_1);          //Fcpu = 42MHz
  //Set SADCCLK_DIV
  SADC_Set_ClkPrescaler(SADC,SADC_SADCCLK_DIV3);//Fadc = 14MHz
#endif	
}

static void LoadTrimCode(void)
{
	if(M32(INFO_HRCADJ_ADR) == ~M32(INFO_HRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->HRCADJ = M32(INFO_HRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  if(M32(INFO_LRCADJ_ADR) == ~M32(INFO_LRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->LRCADJ = M32(INFO_LRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  if((M16(INFO_CMP4PADJ_ADR) == ~M16(INFO_CMP4PADJ_ADR+2))
    &&(M16(INFO_CMP4NADJ_ADR) == ~M16(INFO_CMP4NADJ_ADR+2)))
  {
    CMP4->CAL = (M16(INFO_CMP4NADJ_ADR)<<8)|M16(INFO_CMP4PADJ_ADR);
  }
  if((M16(INFO_CMP5PADJ_ADR) == ~M16(INFO_CMP5PADJ_ADR+2))
    &&(M16(INFO_CMP5NADJ_ADR) == ~M16(INFO_CMP5NADJ_ADR+2)))
  {
    CMP5->CAL = (M16(INFO_CMP5NADJ_ADR)<<8)|M16(INFO_CMP5PADJ_ADR);
  }
}

/*
*********************************************************************************************************
*   ONLY  IAR-MDK
*********************************************************************************************************
*/
#if defined ( __ICCARM__ )

int __low_level_init(void)
{
    return 1;
}

#endif
