/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s30xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s30xx
* File         : rx32s30xx.h
* By           : RX_SD_Team
* Description  : 
*********************************************************************************************************
*/

#ifndef __rx32s30xx_H__
#define __rx32s30xx_H__

#ifdef __cplusplus
extern "C" {
#endif



/*
* @brief  General Bool State
*/
typedef enum {DISABLE = 0, ENABLE = !DISABLE}   FunctionalState;        /* enable or disable            */
typedef enum {RESET   = 0, SET    = !RESET  }   FlagStatus, ITStatus;   /* Status or Interrupt Flag     */
typedef enum {FALSE   = 0, TRUE   = !FALSE  }   Bool;                   /* function return              */

#define SET(n,c)		((n)|=(c))
#define CLR(n,c)		((n)&=~(c))

#define SET_BIT(REG, BIT)     ((REG) |= (BIT))

#define CLEAR_BIT(REG, BIT)   ((REG) &= ~(BIT))

#define READ_BIT(REG, BIT)    ((REG) & (BIT))

#define CLEAR_REG(REG)        ((REG) = (0x0))

#define WRITE_REG(REG, VAL)   ((REG) = (VAL))

#define READ_REG(REG)         ((REG))

#define MODIFY_REG(REG, CLEARMASK, SETMASK)  WRITE_REG((REG), (((READ_REG(REG)) & (~(CLEARMASK))) | (SETMASK)))

#define M8(adr)     (*((volatile uint8_t *)  (adr)))
#define M16(adr)    (*((volatile uint16_t *) (adr)))
#define M32(adr)    (*((volatile uint32_t *) (adr)))

#define BIT0			((uint32_t)0x00000001)
#define BIT1			((uint32_t)0x00000002)
#define BIT2			((uint32_t)0x00000004)
#define BIT3			((uint32_t)0x00000008)
#define BIT4			((uint32_t)0x00000010)
#define BIT5			((uint32_t)0x00000020)
#define BIT6			((uint32_t)0x00000040)
#define BIT7			((uint32_t)0x00000080)
#define BIT8			((uint32_t)0x00000100)
#define BIT9			((uint32_t)0x00000200)
#define BIT10			((uint32_t)0x00000400)
#define BIT11			((uint32_t)0x00000800)
#define BIT12			((uint32_t)0x00001000)
#define BIT13			((uint32_t)0x00002000)
#define BIT14			((uint32_t)0x00004000)
#define BIT15			((uint32_t)0x00008000)
#define BIT16			((uint32_t)0x00010000)
#define BIT17			((uint32_t)0x00020000)
#define BIT18			((uint32_t)0x00040000)
#define BIT19			((uint32_t)0x00080000)
#define BIT20			((uint32_t)0x00100000)
#define BIT21			((uint32_t)0x00200000)
#define BIT22			((uint32_t)0x00400000)
#define BIT23			((uint32_t)0x00800000)
#define BIT24			((uint32_t)0x01000000)
#define BIT25			((uint32_t)0x02000000)
#define BIT26			((uint32_t)0x04000000)
#define BIT27			((uint32_t)0x08000000)
#define BIT28			((uint32_t)0x10000000)
#define BIT29			((uint32_t)0x20000000)
#define BIT30			((uint32_t)0x40000000)
#define BIT31			((uint32_t)0x80000000)

/*
**********************************************************************************************************
*                                       Interrupt Number Definition
**********************************************************************************************************
*/
typedef enum IRQn
{
/*****************************  Cortex-M3 Processor Exceptions Numbers  *********************************/
    NonMaskableInt_IRQn         = -14,      /*!< 2 Non Maskable Interrupt  CMU                    */
    HardFault_IRQn              = -13,      /*!< 3 Cortex-M3 Hard Fault Interrupt                 */
    SVCall_IRQn                 = -5,       /*!< 11 Cortex-M3 SV Call Interrupt                   */
    PendSV_IRQn                 = -2,       /*!< 14 Cortex-M3 Pend SV Interrupt                   */
    SysTick_IRQn                = -1,       /*!< 15 Cortex-M3 System Tick Interrupt               */

/***************************** rx32s30 Specific Interrupt Numbers  **************************************/
    PMU_IRQn                    = 0,        /*!< PMU Interrupt                                    */
	  SADC_IRQn									  = 1,		    /*!< SAR ADC Interrupt                                */				
    EXTI0_IRQn                  = 2,        /*!< External Interrupt 0 Interrupt                   */
    EXTI1_IRQn                  = 3,        /*!< External Interrupt 1 Interrupt                   */
    EXTI2_IRQn                  = 4,        /*!< External Interrupt 2 Interrupt                   */
    EXTI3_IRQn                  = 5,        /*!< External Interrupt 3 Interrupt                   */
    EXTI4_IRQn                  = 6,        /*!< External Interrupt 4 Interrupt                   */
    EXTI5_IRQn                  = 7,        /*!< External Interrupt 5 Interrupt                   */
    EXTI6_IRQn                  = 8,        /*!< External Interrupt 6 Interrupt                   */
    UART0_IRQn                  = 9,        /*!< UART0 Interrupt                                  */
    UART1_IRQn                  = 10,       /*!< UART1 Interrupt                                  */
    ME_IRQn											= 14,       /*!< ME Interrupt                                     */
    TMR0_IRQn                   = 15,       /*!< Timer0 Interrupt                                 */
    TMR1_IRQn                   = 16,       /*!< Timer1 Interrupt                                 */
    TMR2_IRQn                   = 17,       /*!< Timer2 Interrupt                                 */
    TMR3_IRQn                   = 18,       /*!< Timer3 Interrupt                                 */
    RTC_IRQn                    = 20,       /*!< RTC Interrupt                                    */
    I2C_IRQn                    = 21,       /*!< I2C Converter Interrupt                          */
    SPI_IRQn                    = 22,       /*!< SPI0 Interrupt                                   */
    CMP4_IRQn										= 25,       /*!< CMP4 Interrupt                                   */
  	CMP5_IRQn										= 26,       /*!< CMP5 Interrupt                                   */
    TIM8_IRQn                		= 27,       /*!< TIM8 Interrupt                                   */
    EXTI7_IRQn                  = 28,       /*!< External Interrupt 7 Interrupt                   */
    EXTI8_IRQn                  = 29,       /*!< External Interrupt 8 Interrupt                   */
    EXTI9_IRQn                  = 30,       /*!< External Interrupt 9 Interrupt                   */
    DMA_IRQn                    = 31,       /*!< DMA Interrupt                                    */
} IRQn_Type;

/*
**********************************************************************************************************
*                                   Processor and Core Peripheral Section
**********************************************************************************************************
*/

/********************  Configuration of the Cortex-M3 Processor and Core Peripherals  *******************/
#define __MPU_PRESENT             0         /*!< MPU present or not                               */
#define __NVIC_PRIO_BITS          2         /*!< Number of Bits used for Priority Levels          */
#define __Vendor_SysTickConfig    0         /*!< Set to 1 if different SysTick Config is used     */
                                            /* end of group rx32s30_CMSIS                         */

#include "core_cm3.h"                       /* Cortex-M3 processor and core peripherals           */
#include <stdint.h>                         /* standard types definitions                         */

/*
**********************************************************************************************************
*                                 Device Specific Peripheral Registers structures
**********************************************************************************************************
*/
#if defined ( __CC_ARM   )
#pragma anon_unions
#endif

/*****************************************  Timer Control Block  *****************************************/
/**
* @brief TMR Register Definition
*/
typedef struct
{
    __IO uint32_t TMRCON;                   /*!< Offset: 0x000 Timer control register           (R/W) */
    __IO uint32_t TMRDIV;                   /*!< Offset: 0x004 Pre-dividing register            (R/W) */
    __IO uint32_t TMRPRD;                   /*!< Offset: 0x008 period register                  (R/W) */
    __IO uint32_t TMRCAP;                   /*!< Offset: 0x00C capture data register            (R/W) */
    __IO uint32_t TMRCNT;                   /*!< Offset: 0x010 count register                   (R/W) */
    __IO uint32_t TMRCMP;                   /*!< Offset: 0x014 compare register                 (R/W) */
    __IO uint32_t TMRIE;                    /*!< Offset: 0x018 Timer Interrupt enable register  (R/W) */
    __IO uint32_t TMRIF;                    /*!< Offset: 0x01C Timer Interrupt Flag register    (R/W) */
	  __IO uint32_t CCMR;                     /*!< Offset: 0x020 Timer Ecoder interface mode      (R/W) */
} TMR_TypeDef;                           /* end of group TMR_TypeDef                 */


/*****************************************  UART Control Block  ******************************************/
/**
* @brief UART Register Definition
*/
typedef struct
{
    __IO uint32_t Reserved1[1];             /*!< Offset: 0x000 reserve */
    __IO uint32_t UARTCON;                  /*!< Offset: 0x004 Uart function configuration register      (R/W) */
    __IO uint32_t Reserved2[1];              
    __IO uint32_t SREL;                     /*!< Offset: 0x00C reload value of Uart baud rate generator   (R/W) */
    __IO uint32_t SBUF;                     /*!< Offset: 0x010 Uart data buffer register                  (R/W) */
    __IO uint32_t UARTSTA;                  /*!< Offset: 0x014 Uart Status register                       (R/W) */          
} UART_TypeDef;                          /* end of group UART_TypeDef */


/*****************************************  I2C Control Block  *******************************************/
/**
* @brief I2C Register Definition
*/
typedef struct
{
    __IO uint32_t I2CDAT;                   /*!< Offset: 0x000 I2C data register    (R/W) */
    __IO uint32_t I2CADR;                   /*!< Offset: 0x004 I2C address register (R/W) */
    __IO uint32_t I2CCON;                   /*!< Offset: 0x008 I2C control register (R/W) */
    __IO uint32_t I2CSTA;                   /*!< Offset: 0x00C I2C status register  (R/W) */
} I2C_TypeDef;                           /* end of group I2C_TypeDef         */


/*****************************************  SPI Control Block  *******************************************/
/**
* @brief SPI Register Definition
*/
typedef struct
{
	__IO uint32_t SPICON;                   /*!< Offset: 0x000 SPI control register        (R/W) */
	__IO uint32_t SPISTA;                   /*!< Offset: 0x004 SPI status register         (R/W) */
	__IO uint32_t SPIDAT;                   /*!< Offset: 0x008 SPI data register           (R/W) */
	__IO uint32_t SPISSN;                   /*!< Offset: 0x00C SPI slave selection register (R/W) */
	__IO uint32_t Reserved[1];              /*!< Offset: 0x0010 reserve                     (R/W) */
	__IO uint32_t CSDLY;                    /*!< Offset: 0x014 SPI cs/delay control register(R/W) */
} SPI_TypeDef;                         /* end of group SPI_TypeDef             */


/*****************************************  RTC Control Block  *******************************************/
/**
* @brief RTC Register Definition
*/
typedef struct
{
	__IO uint32_t RTCCON;                   /*!< Offset: 0x000 RTC control register               (R/W) */
	__IO uint32_t RTCIE;                    /*!< Offset: 0x004 RTC Interrupt enable register      (R/W) */
	__IO uint32_t RTCIF;                    /*!< Offset: 0x008 RTC Interrupt Flag register        (R/W) */
	__IO uint32_t Reserved[2];              /*!< Offset: 0x00C-0x0010 reserve             (R/W) */
	__IO uint32_t RTCTMR2;                  /*!< Offset: 0x014 set RTC Timer2 count       (R/W) */
} RTC_TypeDef;                         /* end of group RTC_TypeDef                   */

/*****************************************  PMU Control Block  *******************************************/
/**
* @brief PMU Register Definition
*/
typedef struct
{
    __IO uint32_t PMUCON;                   /*!< Offset: 0x000 PMU configuration register                             (R/W) */
    __IO uint32_t VDETCFG;                  /*!< Offset: 0x004 source detection threshold configuration register      (R/W) */
    __IO uint32_t VDETPCFG;                 /*!< Offset: 0x008 source detection time priority  configuration register (R/W) */
    __IO uint32_t PMUIE;                    /*!< Offset: 0x00C PMU Interrupt enable register                          (R/W) */
    __IO uint32_t PMUIF;                    /*!< Offset: 0x010 PMU Interrupt Flag register                            (R/W) */
    __I  uint32_t PMUSTA;                   /*!< Offset: 0x014 PMU status indicator register                          (R)   */
    __IO uint32_t WAKEIF;                   /*!< Offset: 0x018 wake up source Flag register                           (R/W) */                 
    __I  uint32_t Reserved[5];
    __IO uint32_t RSTSTA;                   /*!< Offset: 0x030 Reset Flag register                                    (R/W) */
} PMU_TypeDef;   

/*****************************************  CMU Control Block  *******************************************/
/**
* @brief CMU Register Definition
*/
typedef struct
{ 
	__IO uint32_t WPREG;                    /*!< Offset: 0x000 write protect control register                       (R/W)*/                    
	__IO uint32_t SYSCLKCFG;                /*!< Offset: 0x004 SysTick  configuration register(write protect)       (R/W)*/                  
	__IO uint32_t JTAGSTA;                  /*!< Offset: 0x008 chip debug status instruction register               (R/W)*/                          
	__IO uint32_t LRCADJ;                   /*!< Offset: 0x00C LRC timer regulation register(write protect)         (R/W)*/                    
	__IO uint32_t HRCADJ;                   /*!< Offset: 0x010 HRC timer regulation register(write protect)         (R/W)*/                  
	__IO uint32_t HRCDIV;                   /*!< Offset: 0x014 HRC timer frequency division register(write protect) (R/W)*/                          
	__IO uint32_t CLKSTA;                   /*!< Offset: 0x018 timer status register(read only)                     (R/W)*/
	__IO uint32_t SYSCLKDIV;                /*!< Offset: 0x01C SysTick frequency division register(write protect)   (R/W)*/
	__I  uint32_t Reserved1[1];
	__IO uint32_t CLKOUTSEL;                /*!< Offset: 0x024 CLKOUT timer select register(write protect)          (R/W)*/
	__IO uint32_t CLKOUTDIV;                /*!< Offset: 0x028 CLKOUT timer frequency division register(write protect)(R/W)*/
	__IO uint32_t CLKCTRL0;                 /*!< Offset: 0x02C internal module enable register0(write protect)        (R/W)*/
	__IO uint32_t CLKCTRL1;                 /*!< Offset: 0x030 internal module enable register1(write protect)        (R/W)*/
	__IO uint32_t FLASHCON;                 /*!< Offset: 0x034 Flash control register(write protect)                  (R/W)*/
  __IO uint32_t FLASHLOCK;                /*!< Offset: 0x038 Flash lock register                                    (R/W)*/
	__IO uint32_t CPUCFG;                   /*!< Offset: 0x03C instruction prefetch enable register                   (R/W)*/
	__I  uint32_t Reserved2[4];
	__IO uint32_t INFOLOCK;                 /*!< Offset: 0x050 Info Flash lock register(Not open to users)            (R/W)*/
  __IO uint32_t PORRSTSET;                /*!< Offset: 0x054 POR/LBOR reset egister(Not open to users)              (R/W)*/
	__I  uint32_t Reserved3[107];
	__IO uint32_t MULTFUNCFG;               /*!< Offset: 0x204 moltifunction configuration register(write protect)    (R/W)*/
	__IO uint32_t LFCLKCFG;                 /*!< Offset: 0x208 low frequency timer source configuration register(write protect) (R/W)*/
  __I  uint32_t Reserved4[829];
	__IO uint32_t CHIPID;                   /*!< Offset: 0xF00 chip ID register                                        (R/W)*/
  __I  uint32_t Reserved5[3];
	__IO uint32_t PLLLOCK_PRE;              /*!< Offset: 0xF10 set PLL lock condition register                         (R/W)*/
	__I  uint32_t Reserved6[3];
	__IO uint32_t C_PLL;                    /*!< Offset: 0xF20 PLL timer frequency production control register         (R/W)*/
	__I  uint32_t Reserved7[3];
	__IO uint32_t FLTCTR;                   /*!< Offset: 0xF30 timer filter control register                           (R/W)*/

} CMU_TypeDef;                           /* end of group RCC_TypeDef                    */


/*****************************************  WDT Control Block  *******************************************/
/**
* @brief WDT Register Definition
*/
typedef struct
{
	__I	 uint32_t Reserved[1];
	__IO uint32_t WDTCLR;                   /*!< Offset: 0x004 dog .WDT and time configuration register (R/W) */
	__I  uint32_t WDTCNT;                   /*!< Offset: 0x008 dog .WDT count register(read only)       (R)   */
} WDT_TypeDef;                         /* end of group WDT_TypeDef                      */


/*****************************************  GPIO Control Block  ******************************************/
/**
* @brief GPIO Register Definition
*/
typedef struct
{
	__IO uint32_t IOCFG;                    /*!< Offset: 0x000 GPIO function configuration register 1(write protect)              (R/W)  */
	__IO uint32_t AFCFG1;                   /*!< Offset: 0x004 GPIO multiplexing function configuration register 1(write protect) (R/W)  */
	__IO uint32_t PTDIR;                    /*!< Offset: 0x008 GPIO direction configuration register                              (R/W)  */
	__IO uint32_t PUPDN;                    /*!< Offset: 0x00C GPIO pull and down configuration register                          (R/W)  */
	__IO uint32_t PTDAT;                    /*!< Offset: 0x010 GPIO data register                                                 (R/W)  */
	__O  uint32_t PTSET;                    /*!< Offset: 0x014 GPIO set register(write only)                                      ( /W)  */
	__O  uint32_t PTCLR;                    /*!< Offset: 0x018 GPIO reset register(write only)                                    ( /W)  */
	__O  uint32_t PTTOG;                    /*!< Offset: 0x01C GPIO flip register(write only)                                     ( /W)  */
	__IO uint32_t PTOD;                     /*!< Offset: 0x020 GPIO Open Drain function configuration register                    (R/W)  */
	__IO uint32_t Reserved1[1];             /*!< Offset: 0x024 reserve                                                            (R/W)  */
	__IO uint32_t HIIPM;                    /*!< Offset: 0x028 GPIO high resistance control register                              (R/W)  */
	__IO uint32_t NEWAFEN;                  /*!< Offset: 0x02C append multiplexing function effective control register            (R/W)  */
	__IO uint32_t AFCFG2;                   /*!< Offset: 0x030 GPIO multiplexing function configuration register 2(write protect) (R/W)  */
} GPIO_TypeDef;                          /* end of group GPIO_TypeDef                           */


/*****************************************  INT Control Block  *******************************************/
/**
* @brief INT Register Definition
*/
typedef struct
{
    __IO uint32_t EXTIE;                    /*!< Offset: 0x000 External Interrupt edge configuration register (R/W) */
    __IO uint32_t EXTIF;                    /*!< Offset: 0x004 External Interrupt edge Flag register          (R/W) */
    __IO uint32_t PINFLTEN;                 /*!< Offset: 0x008 Pin digital filter enable register             (R/W) */
    __IO uint32_t RXFLTEN;                  /*!< Offset: 0x00C UART_RX Pin digital filter enable register     (R/W) */
    __IO uint32_t EXTIE2;                   /*!< Offset: 0x010 External Interrupt edge configuration register 2(R/W)*/
    __IO uint32_t EXTIF2;                   /*!< Offset: 0x014 External Interrupt edge Flag register 2        (R/W) */
    __IO uint32_t Reserved[1];              /*!< Offset: 0x018 */
    __IO uint32_t FLTSEL;                   /*!< Offset: 0x01C External Interrupt filter select register       (R/W)*/
} INT_TypeDef;                           /* end of group INT_TypeDef                  */

/*****************************************  DMA Control Block  *******************************************/
/**
* @brief DMA Register Definition
*/

typedef struct
{
    __IO uint32_t DMAIE;                     /*!< Offset: 0x000 Interrupt enable register   (R/W)          */
    __IO uint32_t DMAIF;                     /*!< Offset: 0x004 channel Interrupt Flag      (R/W)          */
    __IO uint32_t CHNSTA;                    /*!< Offset: 0x008 channel 0 status register   (R/W)          */	
} DMA_TypeDef;                           /* end of group DMA_TypeDef                    */

typedef struct
{
    __IO uint32_t CHNCTL;                   /*!< Offset: 0x000 channel control register               (R/W) */
    __IO uint32_t CHNSRC;                   /*!< Offset: 0x004 channel source address register        (R/W) */
    __IO uint32_t CHNTAR;                   /*!< Offset: 0x008 channel destination address register   (R/W) */
    __IO uint32_t CHNCNT;                   /*!< Offset: 0x00C set channel conmunicate amount register(R/W) */
    __IO uint32_t CHNTCCNT;                 /*!< Offset: 0x010 number of complete channel conmunicate (R/W) */
    __IO uint32_t CHNBULKNUM;               /*!< Offset: 0x014 set the number of channel conmunicate  (R/W) */
} DMA_Channel_TypeDef;                   /* end of group DMA_Channel_TypeDef        */

/*****************************************  CRC Control Block  *******************************************/
/**
* @brief CRC Register Definition
*/
typedef struct
{
	__IO uint32_t CRCCON;						  /*!< Offset: 0x000 CRC control register (R/W)		 */
	union
	{
		__I  uint32_t CRCDAT;					  /*!< Offset: 0x004 CRC data register  (R/W)		 */
		__O  uint32_t CRCDAT_DWORD; 			  /*!<	  Reserved								*/
		__O  uint16_t CRCDAT_WORD;				  /*!<	  Reserved								*/
		__O  uint16_t Reserved_WORD;			  /*!<	  Reserved								*/
		__O  uint8_t  CRCDAT_BYTE;
		__O  uint8_t  Reserved_BYTE[3];
	};
	__IO uint32_t CRCINIT;					 /*!< Offset: 0x008 CRC initial register(R/W)	*/
} CRC_TypeDef;							     /* end of group CRC_TypeDef 				*/

/*****************************************  SCR Control Block  *******************************************/
/**
* @brief SCR Register Definition
*/
typedef struct
{
    __IO uint32_t SCR;                       /*!< Offset: 0x010 System control register (R/W)        */
} SCR_TypeDef;                            /* end of group SCR_TypeDef                 */

/*****************************************  SQRT Control Block  *******************************************/
/**
* @brief SQRT Register Definition
*/
typedef struct
{
    __IO uint32_t CR;                    /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t DI;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t DO;                    /*!< Offset: 0x008    (R/W)        */
} SQRT_TypeDef;                       /* end of group SQRT_TypeDef    */

/*****************************************  PID Control Block  *******************************************/
/**
* @brief PID Register Definition
*/
typedef struct
{
    __IO uint32_t CR;                     /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t REF;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t FB;                     /*!< Offset: 0x008    (R/W)        */
	  __IO uint32_t OUT;                    /*!< Offset: 0x00C    (R/W)        */
	  __IO uint32_t ERR;                    /*!< Offset: 0x010    (R/W)        */
    __IO uint32_t INTG;                   /*!< Offset: 0x014    (R/W)        */
	  __IO uint32_t KPG;                    /*!< Offset: 0x018    (R/W)        */
	  __IO uint32_t KIG;                    /*!< Offset: 0x01C    (R/W)        */
	  __IO uint32_t KDG;                    /*!< Offset: 0x020    (R/W)        */
	  __IO uint32_t KIMULP;                 /*!< Offset: 0x024    (R/W)        */
    __IO uint32_t KDIV;                   /*!< Offset: 0x028    (R/W)        */
	  __IO uint32_t INTGLIM;                /*!< Offset: 0x02C    (R/W)        */
	  __IO uint32_t OUTLIM;                 /*!< Offset: 0x030    (R/W)        */
} PID_TypeDef;                         /* end of group PID_TypeDef     */

/*****************************************  ME Control Block  *******************************************/
/**
* @brief ME Register Definition
*/
typedef struct
{
    __IO uint32_t CR;                    /*!< Offset: 0x000    (R/W)        */
	  __IO uint32_t Ia;                    /*!< Offset: 0x004    (R/W)        */
	  __IO uint32_t Ib;                    /*!< Offset: 0x008    (R/W)        */
	  __IO uint32_t Ic;                    /*!< Offset: 0x00C    (R/W)        */
	  __IO uint32_t Ialpha;                /*!< Offset: 0x010    (R/W)        */
    __IO uint32_t Ibeta;                 /*!< Offset: 0x014    (R/W)        */
	  __IO uint32_t Iq;                    /*!< Offset: 0x018    (R/W)        */
	  __IO uint32_t Id;                    /*!< Offset: 0x01C    (R/W)        */
	  __IO uint32_t Vq;                    /*!< Offset: 0x020    (R/W)        */
	  __IO uint32_t Vd;                    /*!< Offset: 0x024    (R/W)        */
    __IO uint32_t Valpha;                /*!< Offset: 0x028    (R/W)        */
	  __IO uint32_t Vbeta;                 /*!< Offset: 0x02C    (R/W)        */
	  __IO uint32_t Angle;                 /*!< Offset: 0x030    (R/W)        */
	  __IO uint32_t Cos;                   /*!< Offset: 0x034    (R/W)        */
	  __IO uint32_t Sin;                   /*!< Offset: 0x038    (R/W)        */
	  __IO uint32_t IPDSin;                /*!< Offset: 0x03C    (R/W)        */
   	__IO uint32_t Kslid;                 /*!< Offset: 0x040    (R/W)        */
	  __IO uint32_t Kslf;                  /*!< Offset: 0x044    (R/W)        */
	  __IO uint32_t KF;                    /*!< Offset: 0x048    (R/W)        */
	  __IO uint32_t KG;                    /*!< Offset: 0x04C    (R/W)        */
	  __IO uint32_t E0;                    /*!< Offset: 0x050    (R/W)        */
	  __IO uint32_t Ealpha;                /*!< Offset: 0x054    (R/W)        */
	  __IO uint32_t Ebeta;                 /*!< Offset: 0x058    (R/W)        */
	  __IO uint32_t Zalpha;                /*!< Offset: 0x05C    (R/W)        */
    __IO uint32_t Zbeta;                 /*!< Offset: 0x060    (R/W)        */
	  __IO uint32_t IalphaError;           /*!< Offset: 0x064    (R/W)        */
    __IO uint32_t IbetaError;            /*!< Offset: 0x068    (R/W)        */
	  __IO uint32_t EstIalpha;             /*!< Offset: 0x06C    (R/W)        */
	  __IO uint32_t EstIbeta;              /*!< Offset: 0x070    (R/W)        */
	  __IO uint32_t SMOTheta;              /*!< Offset: 0x074    (R/W)        */
	  __IO uint32_t SMOIalpha;             /*!< Offset: 0x078    (R/W)        */
	  __IO uint32_t SMOIbeta;              /*!< Offset: 0x07C    (R/W)        */
   	__IO uint32_t BEMFA;                 /*!< Offset: 0x080    (R/W)        */
	  __IO uint32_t BEMFB;                 /*!< Offset: 0x084    (R/W)        */
	  __IO uint32_t IPDTheta;              /*!< Offset: 0x088    (R/W)        */
	  __I	 uint32_t reserved[1];                    
	  __IO uint32_t SVZone;                /*!< Offset: 0x090    (R/W)        */
	  __IO uint32_t FOVM;                  /*!< Offset: 0x094    (R/W)        */
	  __IO uint32_t TPWM;                  /*!< Offset: 0x098    (R/W)        */
	  __IO uint32_t LSMIN;                 /*!< Offset: 0x09C    (R/W)        */
    __IO uint32_t PDCH1;                 /*!< Offset: 0x0A0    (R/W)        */
	  __IO uint32_t PDCH2;                 /*!< Offset: 0x0A4    (R/W)        */
	  __IO uint32_t PDCH3;                 /*!< Offset: 0x0A8    (R/W)        */
	  __I  uint32_t reserved2[1];
		__IO uint32_t IER;                   /*!< Offset: 0x0B0    (R/W)        */
	  __IO uint32_t IFR;                   /*!< Offset: 0x0B4    (R/W)        */
	  __I  uint32_t reserved3[2];	
	  __IO uint32_t MCYC;                  /*!< Offset: 0x0C0    (R/W)        */
} ME_TypeDef;                         /* end of group ME_TypeDef      */

/*****************************************  OPA Control Block  *******************************************/
/**
* @brief OPA Register Definition
*/
typedef struct
{
  __IO uint32_t OPA_CR;                     /*!< Offset: 0x000 control register       (R/W) */
	__IO uint32_t OPA_CR2;                    /*!< Offset: 0x004 control register       (R/W) */
} OPA_TypeDef;                             /* end of group OPA_TypeDef            */

/*****************************************  CMP Control Block  *******************************************/
/**
* @brief CMP1 Register Definition
*/
typedef struct
{
	__IO uint32_t CR1; 				  /*!< Offset: 0x000 control register	   (R/W) */
	__IO uint32_t CR2;					/* Offset: 0x004 (R/W) */
	__IO uint32_t CAL;					/* Offset: 0x008 (R/W) */
	__I  uint32_t DAT;          /* Offset: 0x00C (R) */
} CMP_TypeDef;	 

/*****************************************  ADC Control Block  ******************************************/
/**
* @brief ADC Register Definition
*/
typedef struct
{
  __IO uint32_t SADCCON;                  /*!< Offset: 0x000        (R/W)  */
 	__I  uint32_t reserved0[2];                   
  __IO uint32_t SADCSTR;                  /*!< Offset: 0x00C        (R/W)  */
  __IO uint32_t SADCIE;                   /*!< Offset: 0x010        (R/W)  */
  __IO uint32_t SADCIF;                   /*!< Offset: 0x014        (R/W)  */
  __IO uint32_t SADCSAMP;                 /*!< Offset: 0x018        (R/W)  */
  __IO uint32_t SADCSAMP2;                /*!< Offset: 0x01C        (R/W)  */
  __IO uint32_t SADCTRGLEN;               /*!< Offset: 0x020        (R/W)  */
	__IO uint32_t SADCSEQCFG;               /*!< Offset: 0x024        (R/W)  */
 	__I  uint32_t reserved1[1];                   
	__IO uint32_t SADCDAT;                  /*!< Offset: 0x02C        (R)    */
	__IO uint32_t SADC0DAT;                 /*!< Offset: 0x030        (R)    */
	__IO uint32_t SADC1DAT;                 /*!< Offset: 0x034        (R)    */
	__IO uint32_t SADC2DAT;                 /*!< Offset: 0x038        (R)    */
	__IO uint32_t SADC3DAT;                 /*!< Offset: 0x03C        (R)    */
	__IO uint32_t SADC4DAT;                 /*!< Offset: 0x040        (R)    */
	__IO uint32_t SADC5DAT;                 /*!< Offset: 0x044        (R)    */
	__IO uint32_t SADC6DAT;                 /*!< Offset: 0x048        (R)    */
	__IO uint32_t SADC7DAT;                 /*!< Offset: 0x04C        (R)    */
	__I  uint32_t reserved2[8];    
	__IO uint32_t CALCON;                   /*!< Offset: 0x070        (R/W)  */
	__IO uint32_t CALDAT0;                  /*!< Offset: 0x074        (R/W)  */
	__IO uint32_t CALDAT1;                  /*!< Offset: 0x078        (R/W)  */
	__IO uint32_t CALDAT2;                  /*!< Offset: 0x07C        (R/W)  */
	__IO uint32_t CALDAT3;                  /*!< Offset: 0x080        (R/W)  */
	__IO uint32_t CALDAT4;                  /*!< Offset: 0x084        (R/W)  */
	__IO uint32_t TPSOffset;                /*!< Offset: 0x088        (R/W)  */
	__I  uint32_t reserved3[1];   
	__IO uint32_t SADCTAD;                  /*!< Offset: 0x090        (R/W)  */
} SADC_TypeDef;                         /* end of group ADC_TypeDef   */

/*****************************************  TIM8 Control Block  *******************************************/
/**
* @brief TIM8 Register Definition
*/
typedef struct
{
	__IO uint32_t CR1;                  /*!< Offset: 0x000 control register   (R/W) */
	__IO uint32_t CR2;                  /*!< Offset: 0x004 control register 2 (R/W) */
	__IO uint32_t SMCR;                 /*!< Offset: 0x008 control register   (R/W) */
	__IO uint32_t DIER;                 /*!< Offset: 0x00C control register   (R/W) */
	__IO uint32_t SR;                   /*!< Offset: 0x010 status register    (R/W) */
	__IO uint32_t EGR;                  /*!< Offset: 0x014 control register   (R/W) */
	__IO uint32_t CCMR1;                /*!< Offset: 0x018 control register   (R/W) */
	__IO uint32_t CCMR2;                /*!< Offset: 0x01C control register   (R/W) */
	__IO uint32_t CCER;                 /*!< Offset: 0x020 capture/compare enable register (R/W)*/
	__IO uint32_t CNT;                  /*!< Offset: 0x024 counter                         (R/W)*/
	__IO uint32_t PSC;                  /*!< Offset: 0x028 Pre-divider                     (R/W)*/
	__IO uint32_t ARR;                  /*!< Offset: 0x02C automatic reload register       (R/W)*/
	__IO uint32_t RCR;                  /*!< Offset: 0x030 repeat count register           (R/W)*/
	__IO uint32_t CCR1;                 /*!< Offset: 0x034 capture/compare register 1      (R/W)*/
	__IO uint32_t CCR2;                 /*!< Offset: 0x038 capture/compare register 2      (R/W)*/
	__IO uint32_t CCR3;                 /*!< Offset: 0x03C capture/compare register 3      (R/W)*/
	__IO uint32_t CCR4;                 /*!< Offset: 0x040 capture/compare register 4      (R/W)*/
	__IO uint32_t BDTR;                 /*!< Offset: 0x044 break and dead register         (R/W)*/
	__IO uint32_t DCR;                  /*!< Offset: 0x048 DMA control register            (R/W)*/
	__IO uint32_t DMAR;                 /*!< Offset: 0x04C DMA address of continuous mode  (R/W)*/
	__IO uint32_t CCR5;                 /*!< Offset: 0x050 capture/compare register 5      (R/W)*/
	__IO uint32_t CCMR3;                /*!< Offset: 0x054 capture/compare mode register 3 (R/W)*/
} TIM_TypeDef;                      /* end of group TIM8_TypeDef               */

/*****************************************  Info Control Block  ******************************************/


/*
**********************************************************************************************************
*                                        Peripheral memory map
*                                           Unit of Byte
**********************************************************************************************************
*/

/*  Base Address  */
#define FLASH_BASE        (0x00000000UL)
#define RAM_BASE          (0x20000000UL)
#define APB0_BASE         (0x40000000UL)
#define AHB_BASE          (0x40020000UL)

#define FLASH_SIZE        (0x00010000UL)
#define RAM_SIZE          (0x00001000UL)
#define APB0_SIZE         (0x00020000UL)
#define AHB_SIZE          (0x00010000UL)

/*  APB0 Peripherals Address  */
#define TMR0_BASE         (APB0_BASE + 0x01000)
#define TMR1_BASE         (APB0_BASE + 0x02000)
#define TMR2_BASE         (APB0_BASE + 0x03000)
#define TMR3_BASE         (APB0_BASE + 0x04000)

#define UART0_BASE        (APB0_BASE + 0x05000)
#define UART1_BASE        (APB0_BASE + 0x06000)
#define TIM8_BASE         (AHB_BASE + 0x07000)

#define I2C_BASE          (APB0_BASE + 0x0A000)
#define SPI_BASE          (APB0_BASE + 0x0B000)

#define RTC_BASE          (APB0_BASE + 0x0C000)
#define OPA_BASE          (APB0_BASE + 0x0D000)
#define CMP1_BASE         (APB0_BASE + 0x0E000)
#define CMP2_BASE         (APB0_BASE + 0x0E010)
#define CMP3_BASE         (APB0_BASE + 0x0E020)
#define CMP4_BASE         (APB0_BASE + 0x0E030)
#define CMP5_BASE         (APB0_BASE + 0x0E040)

#define SADC_BASE         (APB0_BASE + 0x0E100)

#define CMU_BASE          (APB0_BASE + 0x0F000)
#define PMU_BASE          (APB0_BASE + 0x0F400)

#define WDT_BASE          (APB0_BASE + 0x10000)
#define GPIO_BASE         (APB0_BASE + 0x11000)
#define INT_BASE          (APB0_BASE + 0x11800)

/*  AHB Peripherals Address  */
#define DMA_BASE          (AHB_BASE )
#define DMA_Channel0_BASE (AHB_BASE + 0x0000C)
#define DMA_Channel1_BASE (AHB_BASE + 0x00024)
#define DMA_Channel2_BASE (AHB_BASE + 0x0003C)

#define CRC_BASE          (AHB_BASE + 0x01000)
#define SQBASE            (AHB_BASE + 0x03000)

#define PID1_BASE         (AHB_BASE + 0x04000)
#define PID2_BASE         (AHB_BASE + 0x05000)
#define PID3_BASE         (AHB_BASE + 0x06000)

#define ME_BASE           (AHB_BASE + 0x08000)

/*  GPIO Address  */
#define GPIOA_BASE        (GPIO_BASE + 0x00000)
#define GPIOB_BASE        (GPIO_BASE + 0x00100)
#define GPIOC_BASE        (GPIO_BASE + 0x00200)
#define GPIOD_BASE        (GPIO_BASE + 0x00300)

#define INFO_BASE         (FLASH_BASE + 0x00040000)
#define INFO_SIZE         (0x00000400UL)
#define INFO_BANK         (0x00000400UL)

/*
**********************************************************************************************************
*                                        Peripheral declaration
**********************************************************************************************************
*/
#define TMR0              ((TMR_TypeDef    *) TMR0_BASE )
#define TMR1              ((TMR_TypeDef    *) TMR1_BASE )
#define TMR2              ((TMR_TypeDef    *) TMR2_BASE )
#define TMR3              ((TMR_TypeDef    *) TMR3_BASE )

#define UART0             ((UART_TypeDef   *) UART0_BASE)
#define UART1             ((UART_TypeDef   *) UART1_BASE)

#define TIM8              ((TIM_TypeDef   *) TIM8_BASE )
#define I2C               ((I2C_TypeDef    *) I2C_BASE  )
#define SPI               ((SPI_TypeDef    *) SPI_BASE  )
#define RTC               ((RTC_TypeDef    *) RTC_BASE  )
#define OPA               ((OPA_TypeDef    *) OPA_BASE  )

#define CMP1              ((CMP_TypeDef    *) CMP1_BASE )
#define CMP2              ((CMP_TypeDef    *) CMP2_BASE )
#define CMP3              ((CMP_TypeDef    *) CMP3_BASE )
#define CMP4              ((CMP_TypeDef    *) CMP4_BASE )
#define CMP5              ((CMP_TypeDef    *) CMP5_BASE )

#define SADC              ((SADC_TypeDef   *) SADC_BASE )
#define CMU               ((CMU_TypeDef    *) CMU_BASE  )
#define PMU               ((PMU_TypeDef    *) PMU_BASE  )
#define WDT               ((WDT_TypeDef    *) WDT_BASE  )
#define GPIO              ((GPIO_TypeDef   *) GPIO_BASE )
#define INT               ((INT_TypeDef    *) INT_BASE  )

#define DMA               ((DMA_TypeDef    *) DMA_BASE  )
#define DMA_Channel0      ((DMA_Channel_TypeDef*) DMA_Channel0_BASE  )
#define DMA_Channel1      ((DMA_Channel_TypeDef*) DMA_Channel1_BASE  )
#define DMA_Channel2      ((DMA_Channel_TypeDef*) DMA_Channel2_BASE  )

#define CRC               ((CRC_TypeDef    *) CRC_BASE  )
#define SQRT              ((SQRT_TypeDef   *) SQBASE )

#define PID1              ((PID_TypeDef    *) PID1_BASE )
#define PID2              ((PID_TypeDef    *) PID2_BASE )
#define PID3              ((PID_TypeDef    *) PID3_BASE )

#define ME                ((ME_TypeDef     *) ME_BASE   )

#define GPIOA             ((GPIO_TypeDef   *) GPIOA_BASE)
#define GPIOB             ((GPIO_TypeDef   *) GPIOB_BASE)
#define GPIOC             ((GPIO_TypeDef   *) GPIOC_BASE)
#define GPIOD             ((GPIO_TypeDef   *) GPIOD_BASE)

#define INFO              ((INFO_TypeDef   *) (INFO_BASE + 0x104) )    /*!< RTC coefficient stored in Info  */

/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of Timer Module
**********************************************************************************************************
*/
// CRC
#define  CRC_CRCCON_mask                            ((uint32_t)0x0077) 

#ifdef __cplusplus
}
#endif

#endif
