/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s30xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s30xx
* File         : system_rx32s30xx.c
* By           : RX_SD_Team
* Description  : Only rx32s30xx
*********************************************************************************************************
*/


#include "rx32s30xx.h"
#include "system_rx32s30xx.h"
#include "main.h"

//#define SYSCLK_HSE	FREQ_HSE
//#define SYSCLK_HRC  FREQ_HRC
#define SYSCLK_PLL	((uint32_t)FREQ_FSYS_Mhz*1000000)
/*
*********************************************************************************************************
*                                          local macro/Structure
*********************************************************************************************************
*/
/*
*********************************************************************************************************
*                                             local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                        local Function declaration
*********************************************************************************************************
*/

#ifdef SYSCLK_HSE
uint32_t SystemCoreClock = SYSCLK_HSE;
#elif defined SYSCLK_PLL
uint32_t SystemCoreClock = SYSCLK_PLL;
#else
uint32_t SystemCoreClock = SYSCLK_HRC;
#endif


static void SetFsysClock(void);
static void LoadTrimCode(void);
static void GPIO_Reset_Init(GPIO_TypeDef* GPIOx);
/*
*********************************************************************************************************
*                                        INITIAL THE SYSTERM
*
*********************************************************************************************************
*/
void delay(uint32_t nDelay)
{
  uint32_t i;
  while(nDelay--)
  {
    for(i=0;i<16838;i++);
  }
}

void SystemInit(void)
{
	SetFsysClock();
	LoadTrimCode();

	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_PAEN);
	GPIO_Set_HIIPM(GPIOA,GPIO_PIN_2,GPIO_HIIPM_ENABLE);
	CMU_Disable_CLKCTRL1(CMU_CLKCTRL1_PAEN);

	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_PAEN|CMU_CLKCTRL1_PBEN|CMU_CLKCTRL1_PCEN|CMU_CLKCTRL1_PDEN);
	GPIO_Reset_Init(GPIOA);
	GPIO_Reset_Init(GPIOB);
	GPIO_Reset_Init(GPIOC);
	GPIO_Reset_Init(GPIOD);
	CMU_Disable_CLKCTRL1(CMU_CLKCTRL1_PAEN|CMU_CLKCTRL1_PBEN|CMU_CLKCTRL1_PCEN|CMU_CLKCTRL1_PDEN);

}

static void SetFsysClock(void)
{
#ifdef SYSCLK_HSE //Fsys = Fhse
	// ------------------------- HRC
	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
	// ------------------------- HSE
	GPIOA->IOCFG |= (0x03<<2);  //PA2 PA3
	GPIOA->AFCFG1 &= ~(0x0F<<4);//AF1

	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HSEEN);
	delay(20);
	while((CMU->CLKSTA & CMU_CLKSTA_HSELOCK) == 0);//Wait HSE_LOCK 
	CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HSEDETEN); 
  // ------------------------- SYS
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HSE);

#elif defined SYSCLK_PLL //Fsys = Fpll ,note: select HRC or HSE to PLL 
	// ------------------------- HRC
	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
//	// ------------------------- HSE
//	GPIOA->IOCFG |= (0x03<<2);  //PA2 PA3
//	GPIOA->AFCFG1 &= ~(0x0F<<4);//AF1
//	CMU_enable_CLKCTRL0(CMU_CLKCTRL0_HSEEN);
//	delay(20);
//	while((CMU->CLKSTA & CMU_CLKSTA_HSELOCK) == 0);//Wait HSE_LOCK 
//	CMU_disable_CLKCTRL0(CMU_CLKCTRL0_HSEDETEN);
  // ------------------------- PLL
  CMU_Set_PLLSEL(C_PLL_SEL_HRC);     // PLL source HRC
	CMU_Set_CPLL_GAIN(C_PLL_Gain_x8);  // 10.5*8=84MHz	
	CMU_Set_PREFETCH(CMU_CPUCFG_PREFETCH_ENABLE); 
	CMU_Set_LATENCY(CMU_CPUCFG_LATENCY_5T);
  CMU_Set_AEC_Mode(CMU_CPUCFG_AE_ControlMode);
	CMU_Open_PLL();
  // ------------------------- SYS
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_PLL);
	CMU_Set_AHBCLK_PRE(CMU_SYSCLKCFG_AHBPRE_1);// Hclk: 84/1 = 84MHz
  CMU_Set_APBCLK_PRE(CMU_SYSCLKCFG_APBPRE_2);// Pclk: 84/2 = 42MHz
	CMU_Set_ADCCLK_PRE(CMU_SYSCLKCFG_ADCPRE_6);// ADCclk: 84/6 = 14MHz
#else //Fsys = Fhrc
	// ------------------------- HRC
	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  // ------------------------- SYS
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HRC);
#endif  
}

static void LoadTrimCode(void)
{
	if(M32(INFO_HRCADJ_ADR) == ~M32(INFO_HRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->HRCADJ = M32(INFO_HRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  if(M32(INFO_LRCADJ_ADR) == ~M32(INFO_LRCADJ_ADR+4))
  {
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->LRCADJ = M32(INFO_LRCADJ_ADR);
    CMU->WPREG = 0; 
  }
  if((M16(INFO_CMP4PADJ_ADR) == ~M16(INFO_CMP4PADJ_ADR+2))
    &&(M16(INFO_CMP4NADJ_ADR) == ~M16(INFO_CMP4NADJ_ADR+2)))
  {
    CMP4->CAL = (M16(INFO_CMP4NADJ_ADR)<<8)|M16(INFO_CMP4PADJ_ADR);
  }
  if((M16(INFO_CMP5PADJ_ADR) == ~M16(INFO_CMP5PADJ_ADR+2))
    &&(M16(INFO_CMP5NADJ_ADR) == ~M16(INFO_CMP5NADJ_ADR+2)))
  {
    CMP5->CAL = (M16(INFO_CMP5NADJ_ADR)<<8)|M16(INFO_CMP5PADJ_ADR);
  }
}
static void GPIO_Reset_Init(GPIO_TypeDef* GPIOx)
{
	if(GPIOx == GPIOA)
	{
		GPIOx->IOCFG = 0x3;
	}
	else
	{
		GPIOx->IOCFG = 0;
	}
	GPIOx->PTDIR = 0;
	GPIOx->PUPDN = 0xFFFFFFFF;
}

/*
*********************************************************************************************************
*   ONLY  IAR-MDK
*********************************************************************************************************
*/
#if defined ( __ICCARM__ )

int __low_level_init(void)
{
    return 1;
}
#endif
