/*
*********************************************************************************************************
*                                              RX32S50xx
*                                           Library Function
*
*                                   Copyright 2019, RX Tech, Corp.
*                                        All Rights Reserved
*
*
* Project      : RX32S50xx
* File         : RX32S50xx.h
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : 
*********************************************************************************************************
*/
    
#ifndef __RX32S50xx_H
#define __RX32S50xx_H

#ifdef __cplusplus
 extern "C" {
#endif 

typedef enum {DISABLE = 0, ENABLE = !DISABLE}   FunctionalState;        
typedef enum {RESET   = 0, SET    = !RESET  }   FlagStatus, ITStatus;   
typedef enum {FALSE   = 0, TRUE   = !FALSE  }   Bool;                   

#define SET(n,c)		((n)|=(c))
#define CLR(n,c)		((n)&=~(c))

#define SET_BIT(REG, BIT)     ((REG) |= (BIT))

#define CLEAR_BIT(REG, BIT)   ((REG) &= ~(BIT))

#define READ_BIT(REG, BIT)    ((REG) & (BIT))

#define CLEAR_REG(REG)        ((REG) = (0x0))

#define WRITE_REG(REG, VAL)   ((REG) = (VAL))

#define READ_REG(REG)         ((REG))

#define MODIFY_REG(REG, CLEARMASK, SETMASK)  WRITE_REG((REG), (((READ_REG(REG)) & (~(CLEARMASK))) | (SETMASK)))

#define POSITION_VAL(VAL)     (__CLZ(__RBIT(VAL))) 

#define M8(adr)     (*((volatile uint8_t *)  (adr)))
#define M16(adr)    (*((volatile uint16_t *) (adr)))
#define M32(adr)    (*((volatile uint32_t *) (adr)))


#define BIT0			((uint32_t)0x00000001)
#define BIT1			((uint32_t)0x00000002)
#define BIT2			((uint32_t)0x00000004)
#define BIT3			((uint32_t)0x00000008)
#define BIT4			((uint32_t)0x00000010)
#define BIT5			((uint32_t)0x00000020)
#define BIT6			((uint32_t)0x00000040)
#define BIT7			((uint32_t)0x00000080)
#define BIT8			((uint32_t)0x00000100)
#define BIT9			((uint32_t)0x00000200)
#define BIT10			((uint32_t)0x00000400)
#define BIT11			((uint32_t)0x00000800)
#define BIT12			((uint32_t)0x00001000)
#define BIT13			((uint32_t)0x00002000)
#define BIT14			((uint32_t)0x00004000)
#define BIT15			((uint32_t)0x00008000)
#define BIT16			((uint32_t)0x00010000)
#define BIT17			((uint32_t)0x00020000)
#define BIT18			((uint32_t)0x00040000)
#define BIT19			((uint32_t)0x00080000)
#define BIT20			((uint32_t)0x00100000)
#define BIT21			((uint32_t)0x00200000)
#define BIT22			((uint32_t)0x00400000)
#define BIT23			((uint32_t)0x00800000)
#define BIT24			((uint32_t)0x01000000)
#define BIT25			((uint32_t)0x02000000)
#define BIT26			((uint32_t)0x04000000)
#define BIT27			((uint32_t)0x08000000)
#define BIT28			((uint32_t)0x10000000)
#define BIT29			((uint32_t)0x20000000)
#define BIT30			((uint32_t)0x40000000)
#define BIT31			((uint32_t)0x80000000)

/** @addtogroup Configuration_section_for_CMSIS
  * @{
  */
/**
  * @brief Configuration of the Cortex-M4 Processor and Core Peripherals 
 */
#define __CM4_REV                  0x0200U  /*!< Core Revision r2p0                           */
 #define __MPU_PRESENT             0U       /*!< Other devices does not provide an MPU        */
#define __NVIC_PRIO_BITS           4U       /*!< rx32s50 uses 4 Bits for the Priority Levels  */
#define __Vendor_SysTickConfig     0U       /*!< Set to 1 if different SysTick Config is used */

#define __FPU_PRESENT               1        //@brief FPU present

/**
  * @}
  */

/** @addtogroup Peripheral_interrupt_number_definition
  * @{
  */

/**
 * @brief RX32S50xx Interrupt Number Definition, according to the selected device 
 *        in @ref Library_configuration_section 
 */

 /*!< Interrupt Number Definition */
typedef enum
{
/******  Cortex-M4 Processor Exceptions Numbers ***************************************************/
  NonMaskableInt_IRQn         = -14,    /*!< 2 Non Maskable Interrupt                             */
  HardFault_IRQn              = -13,    /*!< 3 Cortex-M4 Hard Fault Interrupt                     */
  MemoryManagement_IRQn       = -12,    /*!< 4 Cortex-M4 Memory Management Interrupt              */
  BusFault_IRQn               = -11,    /*!< 5 Cortex-M4 Bus Fault Interrupt                      */
  UsageFault_IRQn             = -10,    /*!< 6 Cortex-M4 Usage Fault Interrupt                    */
  SVCall_IRQn                 = -5,     /*!< 11 Cortex-M4 SV Call Interrupt                       */
  DebugMonitor_IRQn           = -4,     /*!< 12 Cortex-M4 Debug Monitor Interrupt                 */
  PendSV_IRQn                 = -2,     /*!< 14 Cortex-M4 Pend SV Interrupt                       */
  SysTick_IRQn                = -1,     /*!< 15 Cortex-M4 System Tick Interrupt                   */

/******  RX32S50xx specific Interrupt Numbers *********************************************************/
  WWDG_IRQn                   = 0,      /*!< Window WatchDog Interrupt                            */
  PVD_IRQn                    = 1,      /*!< PVD through EXTI Line detection Interrupt            */
  TAMPER_IRQn                 = 2,      /*!< Tamper Interrupt                                     */
  RTC_IRQn                    = 3,      /*!< RTC global Interrupt                                 */
  FLASH_IRQn                  = 4,      /*!< FLASH global Interrupt                               */
  RCC_IRQn                    = 5,      /*!< RCC global Interrupt                                 */
  EXTI0_IRQn                  = 6,      /*!< EXTI Line0 Interrupt                                 */
  EXTI1_IRQn                  = 7,      /*!< EXTI Line1 Interrupt                                 */
  EXTI2_IRQn                  = 8,      /*!< EXTI Line2 Interrupt                                 */
  EXTI3_IRQn                  = 9,      /*!< EXTI Line3 Interrupt                                 */
  EXTI4_IRQn                  = 10,     /*!< EXTI Line4 Interrupt                                 */
  DMA1_Channel1_IRQn          = 11,     /*!< DMA1 Channel 1 global Interrupt                      */
  DMA1_Channel2_IRQn          = 12,     /*!< DMA1 Channel 2 global Interrupt                      */
  DMA1_Channel3_IRQn          = 13,     /*!< DMA1 Channel 3 global Interrupt                      */
  DMA1_Channel4_IRQn          = 14,     /*!< DMA1 Channel 4 global Interrupt                      */
  DMA1_Channel5_IRQn          = 15,     /*!< DMA1 Channel 5 global Interrupt                      */
  DMA1_Channel6_IRQn          = 16,     /*!< DMA1 Channel 6 global Interrupt                      */
  DMA1_Channel7_IRQn          = 17,     /*!< DMA1 Channel 7 global Interrupt                      */
  ADC1_2_IRQn                 = 18,     /*!< ADC1 and ADC2 global Interrupt                       */
  USB_HP_CAN1_TX_IRQn         = 19,     /*!< USB Device High Priority or CAN1 TX Interrupts       */
  USB_LP_CAN1_RX0_IRQn        = 20,     /*!< USB Device Low Priority or CAN1 RX0 Interrupts       */
  CAN1_RX1_IRQn               = 21,     /*!< CAN1 RX1 Interrupt                                   */
  CAN1_SCE_IRQn               = 22,     /*!< CAN1 SCE Interrupt                                   */
  EXTI9_5_IRQn                = 23,     /*!< External Line[9:5] Interrupts                        */
  TIM1_BRK_IRQn               = 24,     /*!< TIM1 Break Interrupt                                 */
  TIM1_UP_IRQn                = 25,     /*!< TIM1 Update Interrupt                                */
  TIM1_TRG_COM_IRQn           = 26,     /*!< TIM1 Trigger and Commutation Interrupt               */
  TIM1_CC_IRQn                = 27,     /*!< TIM1 Capture Compare Interrupt                       */
  TIM2_IRQn                   = 28,     /*!< TIM2 global Interrupt                                */
  TIM3_IRQn                   = 29,     /*!< TIM3 global Interrupt                                */
  TIM4_IRQn                   = 30,     /*!< TIM4 global Interrupt                                */
  I2C1_EV_IRQn                = 31,     /*!< I2C1 Event Interrupt                                 */
  I2C1_ER_IRQn                = 32,     /*!< I2C1 Error Interrupt                                 */
  I2C2_EV_IRQn                = 33,     /*!< I2C2 Event Interrupt                                 */
  I2C2_ER_IRQn                = 34,     /*!< I2C2 Error Interrupt                                 */
  SPI1_IRQn                   = 35,     /*!< SPI1 global Interrupt                                */
  SPI2_IRQn                   = 36,     /*!< SPI2 global Interrupt                                */
  USART1_IRQn                 = 37,     /*!< USART1 global Interrupt                              */
  USART2_IRQn                 = 38,     /*!< USART2 global Interrupt                              */
  USART3_IRQn                 = 39,     /*!< USART3 global Interrupt                              */
  EXTI15_10_IRQn              = 40,     /*!< External Line[15:10] Interrupts                      */
  RTC_Alarm_IRQn              = 41,     /*!< RTC Alarm through EXTI Line Interrupt                */
	TIM8_BRK_IRQn               = 43,     /*!< TIM8 Break Interrupt                                 */
  TIM8_UP_IRQn                = 44,     /*!< TIM8 Update Interrupt                                */
  TIM8_TRG_COM_IRQn           = 45,     /*!< TIM8 Trigger and Commutation Interrupt               */
  TIM8_CC_IRQn                = 46,     /*!< TIM8 Capture Compare Interrupt                       */
	ADC3_IRQn                   = 47,     /*!< ADC1 global Interrupt                                */
	PID1_6_IRQn                 = 61,     /*!< PID1_6 global Interrupt                              */
	ME1_2_IRQn                  = 62,     /*!< ME1_2 global Interrupt                                  */
	CMP1_2_3_IRQn               = 64,     /*!< CMP1_2_3 global Interrupt                            */
	DMA3_Channel1_IRQn          = 66,     /*!< DMA3 Channel 1 global Interrupt                      */
  DMA3_Channel2_IRQn          = 67,     /*!< DMA3 Channel 2 global Interrupt                      */
  DMA3_Channel3_IRQn          = 68,     /*!< DMA3 Channel 3 global Interrupt                      */
  DMA3_Channel4_IRQn          = 69,     /*!< DMA3 Channel 4 global Interrupt                      */
  DMA3_Channel5_IRQn          = 70,     /*!< DMA3 Channel 5 global Interrupt                      */
  DMA3_Channel6_IRQn          = 71,     /*!< DMA3 Channel 6 global Interrupt                      */
  DMA3_Channel7_IRQn          = 72,     /*!< DMA3 Channel 7 global Interrupt                      */
	DMA3_Channel8_IRQn          = 73,     /*!< DMA3 Channel 8 global Interrupt                      */
	TIM20_BRK_IRQn              = 77,     /*!< TIM20 Break Interrupt                                 */
  TIM20_UP_IRQn               = 78,     /*!< TIM20 Update Interrupt                                */
  TIM20_TRG_COM_IRQn          = 79,     /*!< TIM20 Trigger and Commutation Interrupt               */
  TIM20_CC_IRQn               = 80,     	/*!< TIM20 Capture Compare Interrupt                       */
} IRQn_Type;

/**
  * @}
  */

#include "core_cm4.h"
#include <stdint.h>

/** @addtogroup Peripheral_registers_structures
  * @{
  */   

/** 
  * @brief ADC Registers 
  */
typedef struct
{
  __IO uint32_t SR;       //0x00
  __IO uint32_t CR1;      //0x04
  __IO uint32_t CR2;      //0x08
  __IO uint32_t SMPR1;    //0x0C
  __IO uint32_t RESERVED[1];
  __IO uint32_t JOFR1;    //0x14
  __IO uint32_t JOFR2;    //0x18
  __IO uint32_t JOFR3;    //0x1C
  __IO uint32_t JOFR4;    //0x20
  __IO uint32_t HTR;      //0x24
  __IO uint32_t LTR;      //0x28
  __IO uint32_t SQR1;     //0x2C
  __IO uint32_t SQR2;     //0x30
  __IO uint32_t SQR3;     //0x34
  __IO uint32_t JSQR;     //0x38
  __IO uint32_t JDR1;     //0x3C
  __IO uint32_t JDR2;     //0x40
  __IO uint32_t JDR3;     //0x44
  __IO uint32_t JDR4;     //0x48
  __IO uint32_t DR;       //0x4C
  __IO uint32_t ANA;      //0x50
  __IO uint32_t CAL;      //0x54
} ADC_TypeDef;

typedef struct
{
  __IO uint32_t SR;               /*!< ADC status register,    used for ADC multimode (bits common to several ADC instances). Address offset: ADC1 base address         */
  __IO uint32_t CR1;              /*!< ADC control register 1, used for ADC multimode (bits common to several ADC instances). Address offset: ADC1 base address + 0x04  */
  __IO uint32_t CR2;              /*!< ADC control register 2, used for ADC multimode (bits common to several ADC instances). Address offset: ADC1 base address + 0x08  */
  uint32_t  RESERVED[16];
  __IO uint32_t DR;               /*!< ADC data register,      used for ADC multimode (bits common to several ADC instances). Address offset: ADC1 base address + 0x4C  */
} ADC_Common_TypeDef;

/*
  * @brief PID Registers 
*/
typedef struct
{
  __IO uint32_t CR;       //0x00            
  __IO uint32_t OUT;      //0x04         
  __IO uint32_t REF;      //0x08              
  __IO uint32_t FB;       //0x0C            
  __IO uint32_t ERR;      //0x10             
  __IO uint32_t INTG;     //0x14             
  __IO uint32_t KP;       //0x18             
  __IO uint32_t KI;       //0x1C            
  __IO uint32_t KD;       //0x20              
  __IO uint32_t KIMULP;   //0x24            
  __IO uint32_t INTGLIM;  //0x28              
  __IO uint32_t OUTLIM;   //0x2C            
  __IO uint32_t IER;      //0x30             
  __IO uint32_t IFR;      //0x34             
  __IO uint32_t SR;       //0x38            
} PID_TypeDef;   

/*****************************************  ME Control Block  *******************************************/
/*
* @brief ME Registers 
*/
typedef struct
{
  __IO uint32_t CR;           //0x00         
  __IO uint32_t Ia;           //0x04        
  __IO uint32_t Ib;           //0x08
  __IO uint32_t Angle;        //0x0C                    
  __IO uint32_t Ialpha;       //0x10       
  __IO uint32_t Ibeta;        //0x14       
  __IO uint32_t Iq;           //0x18        
  __IO uint32_t Id;           //0x1C        
  __IO uint32_t Vq;           //0x20        
  __IO uint32_t Vd;           //0x24         
  __IO uint32_t Valpha;       //0x28       
  __IO uint32_t Vbeta;        //0x2C     
  __IO uint32_t Cos;          //0x30
  __IO uint32_t Sin;          //0x34   
  __IO uint32_t SVZone;       //0x38   
  __IO uint32_t PDCH1;        //0x3C      
  __IO uint32_t PDCH2;        //0x40   
  __IO uint32_t PDCH3;        //0x44
  __IO uint32_t Taon;         //0x48     
  __IO uint32_t Tbon;         //0x4C
  __IO uint32_t Tcon;         //0x50    
  __IO uint32_t FOVM;         //0x54  
  __IO uint32_t TPWM;         //0x58    
  __IO uint32_t LSMIN;        //0x5C
  __IO uint32_t IER;          //0x60         
  __IO uint32_t IFR;          //0x64           
  __IO uint32_t SR;           //0x68       
} ME_TypeDef;    

/** 
  * @brief Backup Registers  
  */

typedef struct
{
  uint32_t  RESERVED0;
  __IO uint32_t DR1;     //0x04
  __IO uint32_t DR2;     //0x08
  __IO uint32_t DR3;     //0x0C
  __IO uint32_t DR4;     //0x10
  __IO uint32_t DR5;     //0x14
  __IO uint32_t DR6;     //0x18
  __IO uint32_t DR7;     //0x1C
  __IO uint32_t DR8;     //0x20
  __IO uint32_t DR9;     //0x24
  __IO uint32_t DR10;    //0x28
  __IO uint32_t RTCCR;   //0x2C
  __IO uint32_t CR;      //0x30
  __IO uint32_t CSR;     //0x34
} BKP_TypeDef;
  
/** 
  * @brief Controller Area Network TxMailBox 
  */

typedef struct
{
  __IO uint32_t TIR;        //0x180,0x190,0x1A0
  __IO uint32_t TDTR;				//0x184,0x194,0x1A4
  __IO uint32_t TDLR;				//0x188,0x198,0x1A8
  __IO uint32_t TDHR;				//0x18C,0x19C,0x1AC
} CAN_TxMailBox_TypeDef;

/** 
  * @brief Controller Area Network FIFOMailBox 
  */
  
typedef struct
{
  __IO uint32_t RIR;         //0x1B0,0x1C0
  __IO uint32_t RDTR;				 //0x1B4,0x1C4
  __IO uint32_t RDLR;				 //0x1B8,0x1C8
  __IO uint32_t RDHR;				 //0x1BC,0x1CC
} CAN_FIFOMailBox_TypeDef;

/** 
  * @brief Controller Area Network FilterRegister 
  */
  
typedef struct
{
  __IO uint32_t FR1;
  __IO uint32_t FR2;
} CAN_FilterRegister_TypeDef;

/** 
  * @brief Controller Area Network 
  */
  
typedef struct
{
  __IO uint32_t MCR;                         //0x00
  __IO uint32_t MSR;                         //0x04
  __IO uint32_t TSR;                         //0x08
  __IO uint32_t RF0R;                        //0x0C
  __IO uint32_t RF1R;                        //0x10
  __IO uint32_t IER;                         //0x14
  __IO uint32_t ESR;                         //0x18
  __IO uint32_t BTR;                         //0x1C
  uint32_t  RESERVED0[88];
  CAN_TxMailBox_TypeDef sTxMailBox[3];       
  CAN_FIFOMailBox_TypeDef sFIFOMailBox[2];
  uint32_t  RESERVED1[12];
  __IO uint32_t FMR;												//0x200
  __IO uint32_t FM1R;												//0x204
  uint32_t  RESERVED2;
  __IO uint32_t FS1R;												//0x20C
  uint32_t  RESERVED3;
  __IO uint32_t FFA1R;											//0x214
  uint32_t  RESERVED4;
  __IO uint32_t FA1R;												//0x21C
  uint32_t  RESERVED5[8];
  CAN_FilterRegister_TypeDef sFilterRegister[14];//0x240...0x31C
} CAN_TypeDef;

/** 
  * @brief CRC calculation unit 
  */

typedef struct
{
  __IO uint32_t DR;           /*!< CRC Data register,                           Address offset: 0x00 */
  __IO uint8_t  IDR;          /*!< CRC Independent data register,               Address offset: 0x04 */
  uint8_t       RESERVED0;    /*!< Reserved,                                    Address offset: 0x05 */
  uint16_t      RESERVED1;    /*!< Reserved,                                    Address offset: 0x06 */  
  __IO uint32_t CR;           /*!< CRC Control register,                        Address offset: 0x08 */ 
} CRC_TypeDef;


/** 
  * @brief Debug MCU
  */

typedef struct
{
  __IO uint32_t IDCODE;
  __IO uint32_t CR;
}DBGMCU_TypeDef;

/** 
  * @brief DMA Controller
  */

typedef struct
{
  __IO uint32_t CCR;    //0x08+20*(Channel-1)
  __IO uint32_t CNDTR;  //0x0C+20*(Channel-1)
  __IO uint32_t CPAR;   //0x10+20*(Channel-1)
  __IO uint32_t CMAR;		//0x14+20*(Channel-1)
} DMA_Channel_TypeDef;

typedef struct
{
  __IO uint32_t ISR;   //0x00
  __IO uint32_t IFCR;  //0x04
} DMA_TypeDef;

/** 
  * @brief AMPU Registers
  */
typedef struct
{
  __IO uint32_t OP;								/*!< Option register, Address offset: 0x00 */
  __IO  uint32_t SR;								/*!< Status register, Address offset: 0x04 */
	__IO uint32_t BUF_ID;						/*!< Bufffer Index register, Address offset: 0x08 */
	__IO uint32_t BUF_Data;					/*!< Bufffer Data register, Address offset: 0x0C */
	__IO  uint16_t KEY0_L;						/*!< KEY0 Low 16 Bits Data register, Address offset: 0x10 */
	__IO  uint16_t RESERVED0;
	__IO  uint16_t KEY0_H;						/*!< KEY0 High 16 Bits Data register, Address offset: 0x14 */
	__IO  uint16_t RESERVED1;
	__IO  uint16_t KEY1_L;						/*!< KEY1 Low 16 Bits Data register, Address offset: 0x18 */
	__IO  uint16_t RESERVED2;
	__IO  uint16_t KEY1_H;						/*!< KEY1 High 16 Bits Data register, Address offset: 0x1C */
	__IO  uint16_t RESERVED3;
	__IO  uint16_t KEY2_L;						/*!< KEY2 Low 16 Bits Data register, Address offset: 0x20 */
	__IO  uint16_t RESERVED4;
	__IO  uint16_t KEY2_H;						/*!< KEY2 High 16 Bits Data register, Address offset: 0x24 */
	__IO  uint16_t RESERVED5;
	__IO  uint16_t KEY3_L;						/*!< KEY3 Low 16 Bits Data register, Address offset: 0x28 */
	__IO  uint16_t RESERVED6;
	__IO  uint16_t KEY3_H;						/*!< KEY3 High 16 Bits Data register, Address offset: 0x2C */
	__IO  uint16_t RESERVED7;
	__IO  uint16_t CR;								/*!< Control register, Address offset: 0x30 */
  __IO  uint16_t RESERVED8;
} AMPU_TypeDef;

typedef struct
{
  __IO uint16_t RBADRL;						/*!< Region Base Address Low 16 Bits register, Address offset: 0x00 */
  __IO uint16_t RESERVED0;
	__IO uint16_t RBADRH;						/*!< Region Base Address High 16 Bits register, Address offset: 0x04 */
  __IO uint16_t RESERVED1;
	__IO uint16_t READRL;						/*!< Region End Address Low 16 Bits register, Address offset: 0x08 */
  __IO uint16_t RESERVED2;
	__IO uint16_t READRH;						/*!< Region End Address High 16 Bits register, Address offset: 0x0C */
  __IO uint16_t RESERVED3;
	__IO uint16_t RCSR;							/*!< Region Control/Status register, Address offset: 0x10 */
} AMPU_Region_TypeDef;



/** 
  * @brief External Interrupt/Event Controller
  */

typedef struct
{
  __IO uint32_t IMR;    //0x00
  __IO uint32_t EMR;		//0x04
  __IO uint32_t RTSR;		//0x08
  __IO uint32_t FTSR;		//0x0C
  __IO uint32_t SWIER;	//0x10
  __IO uint32_t PR;			//0x14
} EXTI_TypeDef;

/** 
  * @brief FLASH Registers
  */

typedef struct
{
  __IO uint32_t ACR;      //0x00
  __IO uint32_t KEYR;     //0x04
  __IO uint32_t OPTKEYR;  //0x08
  __IO uint32_t SR;       //0x0C
  __IO uint32_t CR;       //0x10
  __IO uint32_t AR;       //0x14
  __IO uint32_t RESERVED;
  __IO uint32_t OBR;      //0x1C
  __IO uint32_t WRPR;     //0x20
			uint32_t RESERVED1[3];
	__IO uint32_t SYSKEYR;                    // 0x30: System Memory Key register
} FLASH_TypeDef;

/** 
  * @brief Option Bytes Registers
  */
  
typedef struct
{
  __IO uint16_t RDP;
  __IO uint16_t USER;
  __IO uint16_t Data0;
  __IO uint16_t Data1;
  __IO uint16_t WRP0;
  __IO uint16_t WRP1;
  __IO uint16_t WRP2;
  __IO uint16_t WRP3;
} OB_TypeDef;

/** 
  * @brief General Purpose I/O
  */
typedef struct
{
  __IO uint32_t MODER;    //0x00
  __IO uint32_t OTYPER;   //0x04
  __IO uint32_t OSPEEDR;  //0x08
  __IO uint32_t PUPDR;    //0x0C
  __IO uint32_t IDR;      //0x10
  __IO uint32_t ODR;      //0x14
  __IO uint32_t BSRR;     //0x18
	__IO uint32_t LCKR;     //0x1C
	__IO uint32_t AFRL;     //0x20
	__IO uint32_t AFRH;     //0x24
} GPIO_TypeDef;

/** 
  * @brief Alternate Function SYSCFG
  */

typedef struct
{
  uint32_t  RESERVED0[2];//0x00 0x04
  __IO uint32_t EXTICR[4];//0x08 0x0C 0x10 0x14
	__IO uint32_t RESERVED1[3];//0x18 0x1C 0x20
	__IO uint32_t SKR;//0x24
	__IO uint32_t SCSR;//0x28
} SYSCFG_TypeDef;


/** 
  * @brief Inter Integrated Circuit Interface
  */

typedef struct
{
  __IO uint32_t CR1;    //0x00
  __IO uint32_t CR2;    //0x04
  __IO uint32_t OAR1;   //0x08
  __IO uint32_t OAR2;   //0x0C
  __IO uint32_t DR;     //0x10
  __IO uint32_t SR1;    //0x14
  __IO uint32_t SR2;    //0x18
  __IO uint32_t CCR;    //0x1C
  __IO uint32_t TRISE;  //0x20
} I2C_TypeDef;

/** 
  * @brief Independent WATCHDOG
  */

typedef struct
{
  __IO uint32_t KR;           /*!< Key register,                                Address offset: 0x00 */
  __IO uint32_t PR;           /*!< Prescaler register,                          Address offset: 0x04 */
  __IO uint32_t RLR;          /*!< Reload register,                             Address offset: 0x08 */
  __IO uint32_t SR;           /*!< Status register,                             Address offset: 0x0C */
} IWDG_TypeDef;

/** 
  * @brief Power Control
  */

typedef struct
{
  __IO uint32_t CR;    //0x00
  __IO uint32_t CSR;   //0x04
} PWR_TypeDef;

/** 
  * @brief Reset and Clock Control
  */

typedef struct
{
  __IO uint32_t CR;         //0x00
  __IO uint32_t CFGR;       //0x04
  __IO uint32_t CIR;        //0x08
  __IO uint32_t APB2RSTR;   //0x0C
  __IO uint32_t APB1RSTR;   //0x10
  __IO uint32_t AHBENR;     //0x14
  __IO uint32_t APB2ENR;    //0x18
  __IO uint32_t APB1ENR;    //0x1C
  __IO uint32_t BDCR;       //0x20
  __IO uint32_t CSR;        //0x24
	__IO uint32_t AHBRSTR;    //0x28


} RCC_TypeDef;

/** 
  * @brief Real-Time Clock
  */

typedef struct
{
  __IO uint32_t CRH;   //0x00
  __IO uint32_t CRL;   //0x04
  __IO uint32_t PRLH;  //0x08
  __IO uint32_t PRLL;  //0x0C
  __IO uint32_t DIVH;  //0x10
  __IO uint32_t DIVL;  //0x14
  __IO uint32_t CNTH;  //0x18
  __IO uint32_t CNTL;  //0x1C
  __IO uint32_t ALRH;  //0x20
  __IO uint32_t ALRL;  //0x24
} RTC_TypeDef;

/** 
  * @brief SD host Interface
  */

typedef struct
{
  __IO uint32_t POWER;   //0x00
  __IO uint32_t CLKCR;   //0x04
  __IO uint32_t ARG;     //0x08
  __IO uint32_t CMD;     //0x0C
  __I uint32_t RESPCMD;  //0x10
  __I uint32_t RESP1;    //0x14
  __I uint32_t RESP2;    //0x18
  __I uint32_t RESP3;    //0x1C
  __I uint32_t RESP4;    //0x20
  __IO uint32_t DTIMER;  //0x24
  __IO uint32_t DLEN;    //0x28
  __IO uint32_t DCTRL;   //0x2C
  __I uint32_t DCOUNT;   //0x30
  __I uint32_t STA;      //0x34
  __IO uint32_t ICR;     //0x38
  __IO uint32_t MASK;    //0x3C
  uint32_t  RESERVED0[2];
  __I uint32_t FIFOCNT;  //0x48
  uint32_t  RESERVED1[13];
  __IO uint32_t FIFO;    //0x80
} SDIO_TypeDef;

/** 
  * @brief Serial Peripheral Interface
  */

typedef struct
{
  __IO uint32_t CR1;      //0x00
  __IO uint32_t CR2;      //0x04
  __IO uint32_t SR;       //0x08
  __IO uint32_t DR;       //0x0C
  __IO uint32_t CRCPR;    //0x10
  __IO uint32_t RXCRCR;   //0x14
  __IO uint32_t TXCRCR;   //0x18
  __IO uint32_t I2SCFGR;  //0x1C
	__IO uint32_t I2SPR;    //0x20
} SPI_TypeDef;


/** 
  * @brief CMP
  */

typedef struct
{
  __IO uint32_t CR1;             /*!< CMP control register 1,                      Address offset: 0x00 */
  __IO uint32_t CR2;             /*!< CMP control register 2,                      Address offset: 0x04 */
  __IO uint32_t CAL;             /*!< CMP calculation register,                    Address offset: 0x08 */
} CMP_TypeDef;


/** 
  * @brief CRV
  */

typedef struct
{
  __IO uint32_t CR1;                 /*!< CRV control register 1,        Address offset: 0x00 */
  __IO uint32_t CH1VSEL;             /*!< CRV Select 1,                  Address offset: 0x04 */
  __IO uint32_t CH2VSEL;             /*!< CRV Select 2,                  Address offset: 0x08 */
  __IO uint32_t CH3VSEL;             /*!< CRV Select 3,                  Address offset: 0x0C */
  __IO uint32_t VREAL;               /*!< CRV Real V,                    Address offset: 0x10 */
  __IO uint32_t TC;                  /*!< CRV Real V,                    Address offset: 0x14 */

} CRV_TypeDef;

/** 
  * @brief PGA
  */

typedef struct
{
  __IO uint32_t CR1;                 /*!< PGA CR1                           Address offset: 0x00 */
  __IO uint32_t CR2    ;             /*!< PGA CR2                           Address offset: 0x04 */
  __IO uint32_t CR3;                 /*!< PGA CR3                           Address offset: 0x08 */
} PGA_TypeDef;

/**
  * @brief TIM Timers
  */
typedef struct
{
  __IO uint32_t CR1;             /*!< TIM control register 1,                      Address offset: 0x00 */
  __IO uint32_t CR2;             /*!< TIM control register 2,                      Address offset: 0x04 */
  __IO uint32_t SMCR;            /*!< TIM slave Mode Control register,             Address offset: 0x08 */
  __IO uint32_t DIER;            /*!< TIM DMA/interrupt enable register,           Address offset: 0x0C */
  __IO uint32_t SR;              /*!< TIM status register,                         Address offset: 0x10 */
  __IO uint32_t EGR;             /*!< TIM event generation register,               Address offset: 0x14 */
  __IO uint32_t CCMR1;           /*!< TIM  capture/compare mode register 1,        Address offset: 0x18 */
  __IO uint32_t CCMR2;           /*!< TIM  capture/compare mode register 2,        Address offset: 0x1C */
  __IO uint32_t CCER;            /*!< TIM capture/compare enable register,         Address offset: 0x20 */
  __IO uint32_t CNT;             /*!< TIM counter register,                        Address offset: 0x24 */
  __IO uint32_t PSC;             /*!< TIM prescaler register,                      Address offset: 0x28 */
  __IO uint32_t ARR;             /*!< TIM auto-reload register,                    Address offset: 0x2C */
  __IO uint32_t RCR;             /*!< TIM  repetition counter register,            Address offset: 0x30 */
  __IO uint32_t CCR1;            /*!< TIM capture/compare register 1,              Address offset: 0x34 */
  __IO uint32_t CCR2;            /*!< TIM capture/compare register 2,              Address offset: 0x38 */
  __IO uint32_t CCR3;            /*!< TIM capture/compare register 3,              Address offset: 0x3C */
  __IO uint32_t CCR4;            /*!< TIM capture/compare register 4,              Address offset: 0x40 */
  __IO uint32_t BDTR;            /*!< TIM break and dead-time register,            Address offset: 0x44 */
  __IO uint32_t DCR;             /*!< TIM DMA control register,                    Address offset: 0x48 */
  __IO uint32_t DMAR;            /*!< TIM DMA address for full transfer register,  Address offset: 0x4C */
  __IO uint32_t  RESERVED0;      //Reserved 0x50
  __IO uint32_t CCR5;            /*!< TIM capture/compare register 5,              Address offset: 0x54 */
  __IO uint32_t CCMR3;           /*!< TIM  capture/compare mode register 3,        Address offset: 0x58 */
  __IO uint32_t  RESERVED1;	     //Reserved 0x5C
  __IO uint32_t CCR6;            /*!< TIM capture/compare register 6,              Address offset: 0x60 */
  __IO uint32_t CCR1N;           /*!< TIM capture/compare 1N register 1,           Address offset: 0x64 */
  __IO uint32_t CCR2N;           /*!< TIM capture/compare 2N register 2,           Address offset: 0x68 */
  __IO uint32_t CCR3N;           /*!< TIM capture/compare 3N register 3,           Address offset: 0x6C */	
}TIM_TypeDef;


/** 
  * @brief Universal Synchronous Asynchronous Receiver Transmitter
  */
 
typedef struct
{
  __IO uint32_t SR;         /*!< USART Status register,                   Address offset: 0x00 */
  __IO uint32_t DR;         /*!< USART Data register,                     Address offset: 0x04 */
  __IO uint32_t BRR;        /*!< USART Baud rate register,                Address offset: 0x08 */
  __IO uint32_t CR1;        /*!< USART Control register 1,                Address offset: 0x0C */
  __IO uint32_t CR2;        /*!< USART Control register 2,                Address offset: 0x10 */
  __IO uint32_t CR3;        /*!< USART Control register 3,                Address offset: 0x14 */
  __IO uint32_t GTPR;       /*!< USART Guard time and prescaler register, Address offset: 0x18 */
} USART_TypeDef;

/** 
  * @brief Universal Serial Bus Full Speed Device
  */
  
typedef struct
{
  __IO uint16_t EP0R;                 /*!< USB Endpoint 0 register,                   Address offset: 0x00 */ 
  __IO uint16_t RESERVED0;            /*!< Reserved */     
  __IO uint16_t EP1R;                 /*!< USB Endpoint 1 register,                   Address offset: 0x04 */
  __IO uint16_t RESERVED1;            /*!< Reserved */       
  __IO uint16_t EP2R;                 /*!< USB Endpoint 2 register,                   Address offset: 0x08 */
  __IO uint16_t RESERVED2;            /*!< Reserved */       
  __IO uint16_t EP3R;                 /*!< USB Endpoint 3 register,                   Address offset: 0x0C */ 
  __IO uint16_t RESERVED3;            /*!< Reserved */       
  __IO uint16_t EP4R;                 /*!< USB Endpoint 4 register,                   Address offset: 0x10 */
  __IO uint16_t RESERVED4;            /*!< Reserved */       
  __IO uint16_t EP5R;                 /*!< USB Endpoint 5 register,                   Address offset: 0x14 */
  __IO uint16_t RESERVED5;            /*!< Reserved */       
  __IO uint16_t EP6R;                 /*!< USB Endpoint 6 register,                   Address offset: 0x18 */
  __IO uint16_t RESERVED6;            /*!< Reserved */       
  __IO uint16_t EP7R;                 /*!< USB Endpoint 7 register,                   Address offset: 0x1C */
  __IO uint16_t RESERVED7[17];        /*!< Reserved */     
  __IO uint16_t CNTR;                 /*!< Control register,                          Address offset: 0x40 */
  __IO uint16_t RESERVED8;            /*!< Reserved */       
  __IO uint16_t ISTR;                 /*!< Interrupt status register,                 Address offset: 0x44 */
  __IO uint16_t RESERVED9;            /*!< Reserved */       
  __IO uint16_t FNR;                  /*!< Frame number register,                     Address offset: 0x48 */
  __IO uint16_t RESERVEDA;            /*!< Reserved */       
  __IO uint16_t DADDR;                /*!< Device address register,                   Address offset: 0x4C */
  __IO uint16_t RESERVEDB;            /*!< Reserved */       
  __IO uint16_t BTABLE;               /*!< Buffer Table address register,             Address offset: 0x50 */
  __IO uint16_t RESERVEDC;            /*!< Reserved */       
} USB_TypeDef;


/** 
  * @brief Window WATCHDOG
  */

typedef struct
{
  __IO uint32_t CR;   /*!< WWDG Control register,       Address offset: 0x00 */
  __IO uint32_t CFR;  /*!< WWDG Configuration register, Address offset: 0x04 */
  __IO uint32_t SR;   /*!< WWDG Status register,        Address offset: 0x08 */
} WWDG_TypeDef;

/**
  * @}
  */
  
/** @addtogroup Peripheral_memory_map
  * @{
  */


#define FLASH_BASE            0x08000000U /*!< FLASH base address in the alias region */
#define FLASH_BANK1_END       0x0801FFFFU /*!< FLASH END address of bank1 */
#define SRAM_BASE             0x20000000U /*!< SRAM base address in the alias region */
#define PERIPH_BASE           0x40000000U /*!< Peripheral base address in the alias region */

#define SRAM_BB_BASE          0x22000000U /*!< SRAM base address in the bit-band region */
#define PERIPH_BB_BASE        0x42000000U /*!< Peripheral base address in the bit-band region */


/*!< Peripheral memory map */
#define APB1PERIPH_BASE       PERIPH_BASE
#define APB2PERIPH_BASE       (PERIPH_BASE + 0x00010000U)
#define AHBPERIPH_BASE        (PERIPH_BASE + 0x00020000U)

#define TIM2_BASE             (APB1PERIPH_BASE + 0x00000000U)
#define TIM3_BASE             (APB1PERIPH_BASE + 0x00000400U)
#define TIM4_BASE             (APB1PERIPH_BASE + 0x00000800U)
#define RTC_BASE              (APB1PERIPH_BASE + 0x00002800U)
#define WWDG_BASE             (APB1PERIPH_BASE + 0x00002C00U)
#define IWDG_BASE             (APB1PERIPH_BASE + 0x00003000U)
#define SPI2_BASE             (APB1PERIPH_BASE + 0x00003800U)
#define USART2_BASE           (APB1PERIPH_BASE + 0x00004400U)
#define USART3_BASE           (APB1PERIPH_BASE + 0x00004800U)
#define I2C1_BASE             (APB1PERIPH_BASE + 0x00005400U)
#define I2C2_BASE             (APB1PERIPH_BASE + 0x00005800U)
#define CAN1_BASE             (APB1PERIPH_BASE + 0x00006400U)
#define BKP_BASE              (APB1PERIPH_BASE + 0x00006C00U)
#define PWR_BASE              (APB1PERIPH_BASE + 0x00007000U)
#define DAC1_BASE             (APB1PERIPH_BASE + 0x00007400U)
#define DAC2_BASE             (APB1PERIPH_BASE + 0x00007500U)
#define DAC3_BASE             (APB1PERIPH_BASE + 0x00007600U)

#define SYSCFG_BASE           (APB2PERIPH_BASE + 0x00000000U)
#define EXTI_BASE             (APB2PERIPH_BASE + 0x00000400U)
#define GPIOA_BASE            (APB2PERIPH_BASE + 0x00000800U)
#define GPIOB_BASE            (APB2PERIPH_BASE + 0x00000C00U)
#define GPIOC_BASE            (APB2PERIPH_BASE + 0x00001000U)
#define GPIOD_BASE            (APB2PERIPH_BASE + 0x00001400U)
#define GPIOE_BASE            (APB2PERIPH_BASE + 0x00001800U)
#define ADC1_BASE             (APB2PERIPH_BASE + 0x00002400U)
#define ADC2_BASE             (APB2PERIPH_BASE + 0x00002800U)
#define TIM1_BASE             (APB2PERIPH_BASE + 0x00002C00U)
#define SPI1_BASE             (APB2PERIPH_BASE + 0x00003000U)
#define TIM8_BASE             (APB2PERIPH_BASE + 0x00003400U)
#define USART1_BASE           (APB2PERIPH_BASE + 0x00003800U)

#define ADC3_BASE             (APB2PERIPH_BASE + 0x00003C00U)
#define PGA_BASE              (APB2PERIPH_BASE + 0x00007800U)
#define CMP1_BASE             (APB2PERIPH_BASE + 0x00007C00U)
#define CMP2_BASE             (APB2PERIPH_BASE + 0x00007C10U)
#define CMP3_BASE             (APB2PERIPH_BASE + 0x00007C20U)
#define CRV_BASE              (APB2PERIPH_BASE + 0x00007D00U)
#define TIM20_BASE            (APB2PERIPH_BASE + 0x00005000U)

//#define SDIO_BASE             (PERIPH_BASE + 0x00018000U)

#define DMA1_BASE             (AHBPERIPH_BASE + 0x00000000U)
#define DMA1_Channel1_BASE    (AHBPERIPH_BASE + 0x00000008U)
#define DMA1_Channel2_BASE    (AHBPERIPH_BASE + 0x0000001CU)
#define DMA1_Channel3_BASE    (AHBPERIPH_BASE + 0x00000030U)
#define DMA1_Channel4_BASE    (AHBPERIPH_BASE + 0x00000044U)
#define DMA1_Channel5_BASE    (AHBPERIPH_BASE + 0x00000058U)
#define DMA1_Channel6_BASE    (AHBPERIPH_BASE + 0x0000006CU)
#define DMA1_Channel7_BASE    (AHBPERIPH_BASE + 0x00000080U)

#define DMA3_BASE            (AHBPERIPH_BASE + 0x00000800U)
#define DMA3_Channel1_BASE   (AHBPERIPH_BASE + 0x00000808U)
#define DMA3_Channel2_BASE   (AHBPERIPH_BASE + 0x0000081CU)
#define DMA3_Channel3_BASE   (AHBPERIPH_BASE + 0x00000830U)
#define DMA3_Channel4_BASE   (AHBPERIPH_BASE + 0x00000844U)
#define DMA3_Channel5_BASE   (AHBPERIPH_BASE + 0x00000858U)
#define DMA3_Channel6_BASE   (AHBPERIPH_BASE + 0x0000086CU)
#define DMA3_Channel7_BASE   (AHBPERIPH_BASE + 0x00000880U)
#define DMA3_Channel8_BASE   (AHBPERIPH_BASE + 0x00000894U)

#define AMPU_BASE             (AHBPERIPH_BASE + 0x00000C00U)

#define RCC_BASE              (AHBPERIPH_BASE + 0x00001000U)

#define UFMAC_BASE            (AHBPERIPH_BASE + 0x00001400U)

#define CCMSRAM_BASE          (AHBPERIPH_BASE + 0x00001C00U)

#define CRC_BASE              (AHBPERIPH_BASE + 0x00003000U)

#define PID1_BASE             (AHBPERIPH_BASE + 0x00004000U)
#define PID2_BASE             (AHBPERIPH_BASE + 0x00004100U)
#define PID3_BASE             (AHBPERIPH_BASE + 0x00004200U)
#define PID4_BASE             (AHBPERIPH_BASE + 0x00004300U)
#define PID5_BASE             (AHBPERIPH_BASE + 0x00004400U)
#define PID6_BASE             (AHBPERIPH_BASE + 0x00004500U)

#define ME1_BASE              (AHBPERIPH_BASE + 0x00005000U)
#define ME2_BASE              (AHBPERIPH_BASE + 0x00005400U)

#define FLASH_R_BASE          (AHBPERIPH_BASE + 0x00002000U) /*!< Flash registers base address */
#define FLASHSIZE_BASE        0x1FFFF7E0U    /*!< FLASH Size register base address */
#define UID_BASE              0x1FFFF7E8U    /*!< Unique device ID register base address */
#define OB_BASE               0x1FFFF800U    /*!< Flash Option Bytes base address */



#define DBGMCU_BASE          0xE0042000U /*!< Debug MCU registers base address */

/* USB device FS */
//#define USB_BASE              (APB1PERIPH_BASE + 0x00005C00U) /*!< USB_IP Peripheral Registers base address */
//#define USB_PMAADDR           (APB1PERIPH_BASE + 0x00006000U) /*!< USB_IP Packet Memory Area base address */


/**
  * @}
  */
  
/** @addtogroup Peripheral_declaration
  * @{
  */  

#define TIM2                ((TIM_TypeDef *)TIM2_BASE)
#define TIM3                ((TIM_TypeDef *)TIM3_BASE)
#define TIM4                ((TIM_TypeDef *)TIM4_BASE)
#define RTC                 ((RTC_TypeDef *)RTC_BASE)
#define WWDG                ((WWDG_TypeDef *)WWDG_BASE)
#define IWDG                ((IWDG_TypeDef *)IWDG_BASE)
#define SPI2                ((SPI_TypeDef *)SPI2_BASE)
#define USART2              ((USART_TypeDef *)USART2_BASE)
#define USART3              ((USART_TypeDef *)USART3_BASE)
#define I2C1                ((I2C_TypeDef *)I2C1_BASE)
#define I2C2                ((I2C_TypeDef *)I2C2_BASE)
#define USB                 ((USB_TypeDef *)USB_BASE)
#define CAN1                ((CAN_TypeDef *)CAN1_BASE)
#define BKP                 ((BKP_TypeDef *)BKP_BASE)
#define PWR                 ((PWR_TypeDef *)PWR_BASE)
#define DAC1                ((DAC_TypeDef *)DAC1_BASE)
#define DAC2                ((DAC_TypeDef *)DAC2_BASE)
#define DAC3                ((DAC_TypeDef *)DAC3_BASE)

#define SYSCFG              ((SYSCFG_TypeDef *)SYSCFG_BASE)
#define EXTI                ((EXTI_TypeDef *)EXTI_BASE)
#define GPIOA               ((GPIO_TypeDef *)GPIOA_BASE)
#define GPIOB               ((GPIO_TypeDef *)GPIOB_BASE)
#define GPIOC               ((GPIO_TypeDef *)GPIOC_BASE)
#define GPIOD               ((GPIO_TypeDef *)GPIOD_BASE)
#define GPIOE               ((GPIO_TypeDef *)GPIOE_BASE)
#define ADC1                ((ADC_TypeDef *)ADC1_BASE)
#define ADC2                ((ADC_TypeDef *)ADC2_BASE)
#define ADC12_COMMON        ((ADC_Common_TypeDef *)ADC1_BASE)
#define TIM1                ((TIM_TypeDef *)TIM1_BASE)
#define SPI1                ((SPI_TypeDef *)SPI1_BASE)

#define TIM8                ((TIM_TypeDef *)TIM8_BASE)

#define USART1              ((USART_TypeDef *)USART1_BASE)

#define ADC3                ((ADC_TypeDef *)ADC3_BASE)

#define CMP1                ((CMP_TypeDef *)CMP1_BASE)
#define CMP2                ((CMP_TypeDef *)CMP2_BASE)
#define CMP3                ((CMP_TypeDef *)CMP3_BASE)

#define CRV                ((CRV_TypeDef *)CRV_BASE)

#define PGA                 ((PGA_TypeDef *)PGA_BASE)
#define TIM20              ((TIM_TypeDef *)TIM20_BASE)

//#define SDIO                ((SDIO_TypeDef *)SDIO_BASE)
#define DMA1                ((DMA_TypeDef *)DMA1_BASE)
#define DMA1_Channel1       ((DMA_Channel_TypeDef *)DMA1_Channel1_BASE)
#define DMA1_Channel2       ((DMA_Channel_TypeDef *)DMA1_Channel2_BASE)
#define DMA1_Channel3       ((DMA_Channel_TypeDef *)DMA1_Channel3_BASE)
#define DMA1_Channel4       ((DMA_Channel_TypeDef *)DMA1_Channel4_BASE)
#define DMA1_Channel5       ((DMA_Channel_TypeDef *)DMA1_Channel5_BASE)
#define DMA1_Channel6       ((DMA_Channel_TypeDef *)DMA1_Channel6_BASE)
#define DMA1_Channel7       ((DMA_Channel_TypeDef *)DMA1_Channel7_BASE)

#define AMPU                ((AMPU_TypeDef *)AMPU_BASE)
#define AMPU_Region0        ((AMPU_Region_TypeDef *)AMPU_Region0_BASE)
#define AMPU_Region1        ((AMPU_Region_TypeDef *)AMPU_Region1_BASE)
#define AMPU_Region2        ((AMPU_Region_TypeDef *)AMPU_Region2_BASE)
#define AMPU_Region3        ((AMPU_Region_TypeDef *)AMPU_Region3_BASE)

#define RCC                 ((RCC_TypeDef *)RCC_BASE)

#define UFMAC               ((UFMAC_TypeDef *)UFMAC_BASE)
#define CCMSRAM             ((CCMSRAM_TypeDef *)CCMSRAM_BASE)

#define CRC                 ((CRC_TypeDef *)CRC_BASE)
#define FLASH               ((FLASH_TypeDef *)FLASH_R_BASE)
#define OB                  ((OB_TypeDef *)OB_BASE)
#define DBGMCU              ((DBGMCU_TypeDef *)DBGMCU_BASE)

#define DMA3               ((DMA_TypeDef *)DMA3_BASE)
#define DMA3_Channel1      ((DMA_Channel_TypeDef *)DMA3_Channel1_BASE)
#define DMA3_Channel2      ((DMA_Channel_TypeDef *)DMA3_Channel2_BASE)
#define DMA3_Channel3      ((DMA_Channel_TypeDef *)DMA3_Channel3_BASE)
#define DMA3_Channel4      ((DMA_Channel_TypeDef *)DMA3_Channel4_BASE)
#define DMA3_Channel5      ((DMA_Channel_TypeDef *)DMA3_Channel5_BASE)
#define DMA3_Channel6      ((DMA_Channel_TypeDef *)DMA3_Channel6_BASE)
#define DMA3_Channel7      ((DMA_Channel_TypeDef *)DMA3_Channel7_BASE)
#define DMA3_Channel8      ((DMA_Channel_TypeDef *)DMA3_Channel8_BASE)

#define PID1               ((PID_TypeDef *)PID1_BASE)
#define PID2               ((PID_TypeDef *)PID2_BASE)
#define PID3               ((PID_TypeDef *)PID3_BASE)
#define PID4               ((PID_TypeDef *)PID4_BASE)
#define PID5               ((PID_TypeDef *)PID5_BASE)
#define PID6               ((PID_TypeDef *)PID6_BASE)


#define ME1                ((ME_TypeDef *)ME1_BASE)
#define ME2                ((ME_TypeDef *)ME2_BASE)



/**
  * @}
*/ 

/** @addtogroup Exported_macro
  * @{
  */

/****************************** ADC Instances *********************************/
#define IS_ADC_ALL_INSTANCE(INSTANCE) (((INSTANCE) == ADC1) || \
                                       ((INSTANCE) == ADC2))

#define IS_ADC_COMMON_INSTANCE(INSTANCE) ((INSTANCE) == ADC12_COMMON)

#define IS_ADC_MULTIMODE_MASTER_INSTANCE(INSTANCE) ((INSTANCE) == ADC1)

#define IS_ADC_DMA_CAPABILITY_INSTANCE(INSTANCE) ((INSTANCE) == ADC1)

/****************************** CAN Instances *********************************/    
#define IS_CAN_ALL_INSTANCE(INSTANCE) ((INSTANCE) == CAN1)

/****************************** CRC Instances *********************************/
#define IS_CRC_ALL_INSTANCE(INSTANCE) ((INSTANCE) == CRC)

/****************************** DAC Instances *********************************/

/****************************** DMA Instances *********************************/
#define IS_DMA_ALL_INSTANCE(INSTANCE) (((INSTANCE) == DMA1_Channel1) || \
                                       ((INSTANCE) == DMA1_Channel2) || \
                                       ((INSTANCE) == DMA1_Channel3) || \
                                       ((INSTANCE) == DMA1_Channel4) || \
                                       ((INSTANCE) == DMA1_Channel5) || \
                                       ((INSTANCE) == DMA1_Channel6) || \
                                       ((INSTANCE) == DMA1_Channel7))
  
/******************************* GPIO Instances *******************************/
#define IS_GPIO_ALL_INSTANCE(INSTANCE) (((INSTANCE) == GPIOA) || \
                                        ((INSTANCE) == GPIOB) || \
                                        ((INSTANCE) == GPIOC) || \
                                        ((INSTANCE) == GPIOD) || \
                                        ((INSTANCE) == GPIOE))

/**************************** GPIO Alternate Function Instances ***************/
#define IS_GPIO_AF_INSTANCE(INSTANCE) IS_GPIO_ALL_INSTANCE(INSTANCE)

/**************************** GPIO Lock Instances *****************************/
#define IS_GPIO_LOCK_INSTANCE(INSTANCE) IS_GPIO_ALL_INSTANCE(INSTANCE)

/******************************** I2C Instances *******************************/
#define IS_I2C_ALL_INSTANCE(INSTANCE) (((INSTANCE) == I2C1) || \
                                       ((INSTANCE) == I2C2))

/******************************* SMBUS Instances ******************************/
#define IS_SMBUS_ALL_INSTANCE         IS_I2C_ALL_INSTANCE

/****************************** IWDG Instances ********************************/
#define IS_IWDG_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == IWDG)

/******************************** SPI Instances *******************************/
#define IS_SPI_ALL_INSTANCE(INSTANCE) (((INSTANCE) == SPI1) || \
                                       ((INSTANCE) == SPI2))

/****************************** START TIM Instances ***************************/
/****************************** TIM Instances *********************************/
#define IS_TIM_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_ADVANCED_INSTANCE(INSTANCE) ((INSTANCE) == TIM1)

#define IS_TIM_CC1_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CC2_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CC3_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CC4_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CLOCKSOURCE_ETRMODE1_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CLOCKSOURCE_ETRMODE2_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CLOCKSOURCE_TIX_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_CLOCKSOURCE_ITRX_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_OCXREF_CLEAR_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_ENCODER_INTERFACE_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_XOR_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_MASTER_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_SLAVE_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_SYNCHRO_INSTANCE(INSTANCE)  IS_TIM_MASTER_INSTANCE(INSTANCE)

#define IS_TIM_DMABURST_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_BREAK_INSTANCE(INSTANCE)\
  ((INSTANCE) == TIM1)

#define IS_TIM_CCX_INSTANCE(INSTANCE, CHANNEL) \
   ((((INSTANCE) == TIM1) &&                  \
     (((CHANNEL) == TIM_CHANNEL_1) ||          \
      ((CHANNEL) == TIM_CHANNEL_2) ||          \
      ((CHANNEL) == TIM_CHANNEL_3) ||          \
      ((CHANNEL) == TIM_CHANNEL_4)))           \
    ||                                         \
    (((INSTANCE) == TIM2) &&                   \
     (((CHANNEL) == TIM_CHANNEL_1) ||          \
      ((CHANNEL) == TIM_CHANNEL_2) ||          \
      ((CHANNEL) == TIM_CHANNEL_3) ||          \
      ((CHANNEL) == TIM_CHANNEL_4)))           \
    ||                                         \
    (((INSTANCE) == TIM3) &&                   \
     (((CHANNEL) == TIM_CHANNEL_1) ||          \
      ((CHANNEL) == TIM_CHANNEL_2) ||          \
      ((CHANNEL) == TIM_CHANNEL_3) ||          \
      ((CHANNEL) == TIM_CHANNEL_4)))           \
    ||                                         \
    (((INSTANCE) == TIM4) &&                   \
     (((CHANNEL) == TIM_CHANNEL_1) ||          \
      ((CHANNEL) == TIM_CHANNEL_2) ||          \
      ((CHANNEL) == TIM_CHANNEL_3) ||          \
      ((CHANNEL) == TIM_CHANNEL_4))))

#define IS_TIM_CCXN_INSTANCE(INSTANCE, CHANNEL) \
    (((INSTANCE) == TIM1) &&                    \
     (((CHANNEL) == TIM_CHANNEL_1) ||           \
      ((CHANNEL) == TIM_CHANNEL_2) ||           \
      ((CHANNEL) == TIM_CHANNEL_3)))

#define IS_TIM_COUNTER_MODE_SELECT_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_REPETITION_COUNTER_INSTANCE(INSTANCE)\
  ((INSTANCE) == TIM1)

#define IS_TIM_CLOCK_DIVISION_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))

#define IS_TIM_DMA_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))
    
#define IS_TIM_DMA_CC_INSTANCE(INSTANCE)\
  (((INSTANCE) == TIM1)    || \
   ((INSTANCE) == TIM2)    || \
   ((INSTANCE) == TIM3)    || \
   ((INSTANCE) == TIM4))
    
#define IS_TIM_COMMUTATION_EVENT_INSTANCE(INSTANCE)\
  ((INSTANCE) == TIM1)

#define IS_TIM_ETR_INSTANCE(INSTANCE)  (((INSTANCE) == TIM1)    || \
                                        ((INSTANCE) == TIM2)    || \
                                        ((INSTANCE) == TIM3)    || \
                                        ((INSTANCE) == TIM4))

#define IS_TIM_HALL_SENSOR_INTERFACE_INSTANCE(INSTANCE) (((INSTANCE) == TIM1)    || \
                                                         ((INSTANCE) == TIM2)    || \
                                                         ((INSTANCE) == TIM3)    || \
                                                         ((INSTANCE) == TIM4))

#define IS_TIM_32B_COUNTER_INSTANCE(INSTANCE)           0U

/****************************** END TIM Instances *****************************/


/******************** USART Instances : Synchronous mode **********************/                                           
#define IS_USART_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                     ((INSTANCE) == USART2) || \
                                     ((INSTANCE) == USART3))

/******************** UART Instances : Asynchronous mode **********************/
#define IS_UART_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                    ((INSTANCE) == USART2) || \
                                    ((INSTANCE) == USART3))

/******************** UART Instances : Half-Duplex mode **********************/
#define IS_UART_HALFDUPLEX_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                               ((INSTANCE) == USART2) || \
                                               ((INSTANCE) == USART3))

/******************** UART Instances : LIN mode **********************/
#define IS_UART_LIN_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                        ((INSTANCE) == USART2) || \
                                        ((INSTANCE) == USART3))

/****************** UART Instances : Hardware Flow control ********************/                                    
#define IS_UART_HWFLOW_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                           ((INSTANCE) == USART2) || \
                                           ((INSTANCE) == USART3))

/********************* UART Instances : Smard card mode ***********************/
#define IS_SMARTCARD_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                         ((INSTANCE) == USART2) || \
                                         ((INSTANCE) == USART3))

/*********************** UART Instances : IRDA mode ***************************/
#define IS_IRDA_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                    ((INSTANCE) == USART2) || \
                                    ((INSTANCE) == USART3))

/***************** UART Instances : Multi-Processor mode **********************/
#define IS_UART_MULTIPROCESSOR_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                                   ((INSTANCE) == USART2) || \
                                                   ((INSTANCE) == USART3))

/***************** UART Instances : DMA mode available **********************/
#define IS_UART_DMA_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                        ((INSTANCE) == USART2) || \
                                        ((INSTANCE) == USART3))

/****************************** RTC Instances *********************************/
#define IS_RTC_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == RTC)

/**************************** WWDG Instances *****************************/
#define IS_WWDG_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == WWDG)

/****************************** USB Instances ********************************/
#define IS_USB_ALL_INSTANCE(INSTANCE)   ((INSTANCE) == USB)



#define RCC_HSI_MIN         4000000U
#define RCC_HSI_MAX        16000000U

#define RCC_MAX_FREQUENCY  72000000U

/**
  * @}
  */ 
/******************************************************************************/
/*  For a painless codes migration between the rx32s50   device product       */
/*  lines, the aliases defined below are put in place to overcome the         */
/*  differences in the interrupt handlers and IRQn definitions.               */
/*  No need to update developed interrupt code when moving across             */ 
/*  product lines within the same rx32s50 Family                              */
/******************************************************************************/

/* Aliases for __IRQn */
#define ADC1_IRQn               ADC1_2_IRQn
#define TIM9_IRQn               TIM1_BRK_IRQn
#define TIM1_BRK_TIM9_IRQn      TIM1_BRK_IRQn
#define TIM1_BRK_TIM15_IRQn     TIM1_BRK_IRQn
#define TIM1_TRG_COM_TIM17_IRQn TIM1_TRG_COM_IRQn
#define TIM1_TRG_COM_TIM11_IRQn TIM1_TRG_COM_IRQn
#define TIM11_IRQn              TIM1_TRG_COM_IRQn
#define TIM1_UP_TIM10_IRQn      TIM1_UP_IRQn
#define TIM1_UP_TIM16_IRQn      TIM1_UP_IRQn
#define TIM10_IRQn              TIM1_UP_IRQn
#define OTG_FS_WKUP_IRQn        USBWakeUp_IRQn
#define CEC_IRQn                USBWakeUp_IRQn
#define USB_HP_IRQn             USB_HP_CAN1_TX_IRQn
#define CAN1_TX_IRQn            USB_HP_CAN1_TX_IRQn
#define CAN1_RX0_IRQn           USB_LP_CAN1_RX0_IRQn
#define USB_LP_IRQn             USB_LP_CAN1_RX0_IRQn


/* Aliases for __IRQHandler */
#define ADC1_IRQHandler               ADC1_2_IRQHandler
#define TIM9_IRQHandler               TIM1_BRK_IRQHandler
#define TIM1_BRK_TIM9_IRQHandler      TIM1_BRK_IRQHandler
#define TIM1_BRK_TIM15_IRQHandler     TIM1_BRK_IRQHandler
#define TIM1_TRG_COM_TIM17_IRQHandler TIM1_TRG_COM_IRQHandler
#define TIM1_TRG_COM_TIM11_IRQHandler TIM1_TRG_COM_IRQHandler
#define TIM11_IRQHandler              TIM1_TRG_COM_IRQHandler
#define TIM1_UP_TIM10_IRQHandler      TIM1_UP_IRQHandler
#define TIM1_UP_TIM16_IRQHandler      TIM1_UP_IRQHandler
#define TIM10_IRQHandler              TIM1_UP_IRQHandler
#define OTG_FS_WKUP_IRQHandler        USBWakeUp_IRQHandler
#define CEC_IRQHandler                USBWakeUp_IRQHandler
#define USB_HP_IRQHandler             USB_HP_CAN1_TX_IRQHandler
#define CAN1_TX_IRQHandler            USB_HP_CAN1_TX_IRQHandler
#define CAN1_RX0_IRQHandler           USB_LP_CAN1_RX0_IRQHandler
#define USB_LP_IRQHandler             USB_LP_CAN1_RX0_IRQHandler


/**
  * @}
  */

/**
  * @}
  */


#ifdef __cplusplus
  }
#endif /* __cplusplus */
  
#endif
