/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_cmp.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/
#ifndef __rx32s11xx_CMP_H__
#define __rx32s11xx_CMP_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "rx32s11xx.h"

/*
**********************************************************************************************************
*                         Peripheral Registers_Bits_Definition of CMP Module
**********************************************************************************************************
*/
/**********************  Bit definition for CR1 register of CMP_TypeDef ***********************/
#define CMP_CR1                   ((uint32_t)0x0000f7ff)				
#define CMP_CR1_EN                ((uint32_t)0x00000001)
	
#define CMP_CR1_MODE              ((uint32_t)0x00000006)			
#define CMP_CR1_MODE_Ultralow     ((uint32_t)0x00000000) 
#define CMP_CR1_MODE_Low          ((uint32_t)0x00000002)		
#define CMP_CR1_MODE_Medium       ((uint32_t)0x00000004)	
#define CMP_CR1_MODE_High         ((uint32_t)0x00000006)	

#define CMP_CR1_POL               ((uint32_t)0x00000008)			  
#define CMP_CR1_POL_Positive   		((uint32_t)0x00000000) 
#define CMP_CR1_POL_Negative    	((uint32_t)0x00000008) 

#define CMP_CR1_Ready             ((uint32_t)0x00000020)
#define CMP_CR1_OUT               ((uint32_t)0x00000040)

#define CMP_CR1_LOCK              ((uint32_t)0x00000080) 

#define CMP_CR1_OFLT              ((uint32_t)0x00000700)		
#define CMP_CR1_OFLT_1            ((uint32_t)0x00000000)	
#define CMP_CR1_OFLT_4            ((uint32_t)0x00000100)	
#define CMP_CR1_OFLT_16           ((uint32_t)0x00000200)	
#define CMP_CR1_OFLT_32           ((uint32_t)0x00000300)	
#define CMP_CR1_OFLT_64           ((uint32_t)0x00000400)	
#define CMP_CR1_OFLT_128          ((uint32_t)0x00000500) 
#define CMP_CR1_OFLT_256          ((uint32_t)0x00000600) 
#define CMP_CR1_OFLT_512          ((uint32_t)0x00000700) 

#define CMP_CR1_RIE               ((uint32_t)0x00001000) 
#define CMP_CR1_FIE               ((uint32_t)0x00002000) 
                                  
#define CMP_CR1_RIF               ((uint32_t)0x00004000) 
#define CMP_CR1_FIF               ((uint32_t)0x00008000)

#define CMP_CR1_RHYST             ((uint32_t)0x00030000)	  
#define CMP_CR1_RHYST_0mV         ((uint32_t)0x00000000) 
#define CMP_CR1_RHYST_5mV         ((uint32_t)0x00010000) 
#define CMP_CR1_RHYST_10mV        ((uint32_t)0x00020000) 
#define CMP_CR1_RHYST_20mV        ((uint32_t)0x00030000) 

#define CMP_CR1_FHYST             ((uint32_t)0x000C0000)	  
#define CMP_CR1_FHYST_0mV         ((uint32_t)0x00000000)  
#define CMP_CR1_FHYST_5mV         ((uint32_t)0x00040000) 
#define CMP_CR1_FHYST_10mV        ((uint32_t)0x00080000) 
#define CMP_CR1_FHYST_20mV        ((uint32_t)0x000C0000)

#define CMP_CR1_OutPad            ((uint32_t)0x00100000)
#define CMP_CR1_OutPad_Sample     ((uint32_t)0x00000000)
#define CMP_CR1_OutPad_Direct     ((uint32_t)0x00100000)

/**********************  Bit definition for CR2 register of CMP_TypeDef ***********************/
#define CMP_CR2               ((uint32_t)0xC0007fff)				 
#define CMP_CR2_CRV_SEL       ((uint32_t)0x000F)		 
#define CMP_CR2_CRV_SEL_1_20  ((uint32_t)0x0000) /*!<   1/20 AVCC   */
#define CMP_CR2_CRV_SEL_2_20  ((uint32_t)0x0001) /*!<   2/20 AVCC   */
#define CMP_CR2_CRV_SEL_3_20  ((uint32_t)0x0002) /*!<   3/20 AVCC   */
#define CMP_CR2_CRV_SEL_4_20  ((uint32_t)0x0003) /*!<   4/20 AVCC   */
#define CMP_CR2_CRV_SEL_5_20  ((uint32_t)0x0004) /*!<   5/20 AVCC   */
#define CMP_CR2_CRV_SEL_6_20  ((uint32_t)0x0005) /*!<   6/20 AVCC   */
#define CMP_CR2_CRV_SEL_7_20  ((uint32_t)0x0006) /*!<   7/20 AVCC   */
#define CMP_CR2_CRV_SEL_8_20  ((uint32_t)0x0007) /*!<   8/20 AVCC   */
#define CMP_CR2_CRV_SEL_9_20  ((uint32_t)0x0008) /*!<   9/20 AVCC   */
#define CMP_CR2_CRV_SEL_10_20 ((uint32_t)0x0009) /*!<   10/20 AVCC   */
#define CMP_CR2_CRV_SEL_11_20 ((uint32_t)0x000A) /*!<   11/20 AVCC   */
#define CMP_CR2_CRV_SEL_12_20 ((uint32_t)0x000B) /*!<   12/20 AVCC   */
#define CMP_CR2_CRV_SEL_13_20 ((uint32_t)0x000C) /*!<   13/20 AVCC   */
#define CMP_CR2_CRV_SEL_14_20 ((uint32_t)0x000D) /*!<   14/20 AVCC   */
#define CMP_CR2_CRV_SEL_15_20 ((uint32_t)0x000E) /*!<   15/20 AVCC   */
#define CMP_CR2_CRV_SEL_16_20 ((uint32_t)0x000F) /*!<   16/20 AVCC   */

#define CMP_CR2_CRV_EN        ((uint32_t)0x0010)		
#define CMP_CR2_CRV_SRC       ((uint32_t)0x0020)		
#define CMP_CR2_CRV_SRC_VBGP  ((uint32_t)0x0000) /*!< VBGP 1.2V   */
#define CMP_CR2_CRV_SRC_AVCC  ((uint32_t)0x0020) /*!< AVCC   */

#define CMP_CR2_INP_SEL       			 ((uint32_t)0x00001F00)
#define CMP_CR2_INP_SEL_PC3       	 ((uint32_t)0x00000300)
#define CMP_CR2_INP_SEL_PC0       	 ((uint32_t)0x00000B00)
#define CMP_CR2_INP_SEL_PB7   			 ((uint32_t)0x00001300)
#define CMP_CR2_INP_SEL_PGA0_P       ((uint32_t)0x00000000)
#define CMP_CR2_INP_SEL_PGA1_P       ((uint32_t)0x00000100)
#define CMP_CR2_INP_SEL_PGA2_P       ((uint32_t)0x00000200)
#define CMP_CR2_INP_SEL_PB5       	 ((uint32_t)0x00000300)

#define CMP_CR2_INM_SEL       		((uint32_t)0x000060C0)
#define CMP_CR2_INM_SEL_CRV       ((uint32_t)0x00000040)
#define CMP_CR2_INM_SEL_PC4       ((uint32_t)0x00000000)
#define CMP_CR2_INM_SEL_PC1       ((uint32_t)0x00002000)
#define CMP_CR2_INM_SEL_PB8       ((uint32_t)0x00004000)


#define CMP_CR2_BLANKING             ((uint32_t)0xC0000000)
#define CMP_CR2_BLANKING_NoBlanking  ((uint32_t)0x00000000)
#define CMP_CR2_BLANKING_TIM8_OC4    ((uint32_t)0x40000000)
#define CMP_CR2_BLANKING_TIM8_OC5    ((uint32_t)0x80000000)

/**********************  Bit definition for CAL register of CMP_TypeDef ***********************/

#define CMP_CALP_ADJ   ((uint32_t)0x000F)	/*!< CAL_ADJ  */
#define CMP_CALN_ADJ   ((uint32_t)0x0F00) /*!< CAL_ADJ  */

#define CMP_CALP_EN    ((uint32_t)0x10) /*!< CALP_EN */
#define CMP_CALN_EN    ((uint32_t)0x20) /*!< CALN_EN */

/**********************  Bit definition for PLC register of CMP_TypeDef ***********************/
#define CMP_PLC_INP_SEL       			 ((uint32_t)0x00000003)
#define CMP_PLC_INP_SEL_DISABLE      ((uint32_t)0x00000000)
#define CMP_PLC_INP_SEL_GPIO       	 ((uint32_t)0x00000001)
#define CMP_PLC_INP_SEL_PGA       	 ((uint32_t)0x00000002)

#define CMP_PLC_INM_SEL       			 ((uint32_t)0x00000004)
#define CMP_PLC_INM_SEL_DISABLE      ((uint32_t)0x00000000)
#define CMP_PLC_INM_SEL_GPIO       	 ((uint32_t)0x00000004)

#define CMP_PLC_CH1_EN       			 	 ((uint32_t)0x00000010)
#define CMP_PLC_CH2_EN       			 	 ((uint32_t)0x00000020)
#define CMP_PLC_CH3_EN       			 	 ((uint32_t)0x00000040)

#define CMP_PLC_Period       					((uint32_t)0x00003000)
#define CMP_PLC_Period_32_CLK       	((uint32_t)0x00000000)
#define CMP_PLC_Period_64_CLK       	((uint32_t)0x00001000)
#define CMP_PLC_Period_128_CLK        ((uint32_t)0x00002000)
#define CMP_PLC_Period_256_CLK        ((uint32_t)0x00003000)

#define CMP_PLC_MaskT       					((uint32_t)0x00030000)
#define CMP_PLC_MaskT_0_CLK       		((uint32_t)0x00000000)
#define CMP_PLC_MaskT_8_CLK       		((uint32_t)0x00010000)
#define CMP_PLC_MaskT_16_CLK        	((uint32_t)0x00020000)
#define CMP_PLC_MaskT_32_CLK        	((uint32_t)0x00030000)

#define CMP_PLC_Match_IT       				((uint32_t)0x00100000)

#define CMP_PLC_OFLT_Mode       			((uint32_t)0x00200000)
#define CMP_PLC_OFLT_After_MaskT      ((uint32_t)0x00200000)
#define CMP_PLC_OFLT_Parallel_MaskT   ((uint32_t)0x00200000)

#define CMP_PLC_Match_Data1       		((uint32_t)0x01000000)
#define CMP_PLC_Match_Data2       		((uint32_t)0x02000000)
#define CMP_PLC_Match_Data3       		((uint32_t)0x04000000)

/**********************  Bit definition for PLS register of CMP_TypeDef ***********************/
#define CMP_PLS_OUTP       		  ((uint32_t)0x00000007)

#define CMP_PLS_Ready       		((uint32_t)0x00000080)

#define CMP_PLS_Channel       	((uint32_t)0x00000300)
#define CMP_PLS_NULL       		  ((uint32_t)0x00000000)
#define CMP_PLS_Ready       		((uint32_t)0x00000080)
#define CMP_PLS_CH1       			((uint32_t)0x00000100)
#define CMP_PLS_CH2       			((uint32_t)0x00000200)
#define CMP_PLS_CH3       			((uint32_t)0x00000300)
#define CMP_PLS_CH1_Match       ((uint32_t)0x00001000)
#define CMP_PLS_CH2_Match       ((uint32_t)0x00002000)
#define CMP_PLS_CH3_Match       ((uint32_t)0x00004000)

/*
*********************************************************************************************************
*                                           Macro definition
*********************************************************************************************************
*/

typedef struct
{
	uint32_t Mode;    /** @arg CMP_CR1_MODE_Ultralow
												@arg CMP_CR1_MODE_Low
												@arg CMP_CR1_MODE_Medium
												@arg CMP_CR1_MODE_High ********/
	uint32_t Pol;	    /** @arg CMP_CR1_POL_Positive
												@arg CMP_CR1_POL_Negative ********/
	uint32_t OFLT;    /** @arg CMP_CR1_OFLT_1
												@arg CMP_CR1_OFLT_4
												@arg CMP_CR1_OFLT_16
												@arg CMP_CR1_OFLT_32
												@arg CMP_CR1_OFLT_64
												@arg CMP_CR1_OFLT_128
												@arg CMP_CR1_OFLT_256
												@arg CMP_CR1_OFLT_512 ********/
	uint32_t Rhyst;   /** @arg CMP_CR1_RHYST_0mV
												@arg CMP_CR1_RHYST_5mV
												@arg CMP_CR1_RHYST_10mV
												@arg CMP_CR1_RHYST_20mV ********/
	uint32_t Fhyst;   /** @arg CMP_CR1_FHYST_0mV
												@arg CMP_CR1_FHYST_5mV
												@arg CMP_CR1_FHYST_10mV
												@arg CMP_CR1_FHYST_20mV ********/
	uint32_t CRV; /** @arg CMP_CR2_CRV_SEL_1_20
                    @arg CMP_CR2_CRV_SEL_2_20
                    @arg CMP_CR2_CRV_SEL_3_20
                    @arg CMP_CR2_CRV_SEL_4_20
                    @arg CMP_CR2_CRV_SEL_5_20
                    @arg CMP_CR2_CRV_SEL_6_20
                    @arg CMP_CR2_CRV_SEL_7_20
                    @arg CMP_CR2_CRV_SEL_8_20
                    @arg CMP_CR2_CRV_SEL_9_20
                    @arg CMP_CR2_CRV_SEL_10_20
                    @arg CMP_CR2_CRV_SEL_11_20
                    @arg CMP_CR2_CRV_SEL_12_20
                    @arg CMP_CR2_CRV_SEL_13_20
                    @arg CMP_CR2_CRV_SEL_14_20
                    @arg CMP_CR2_CRV_SEL_15_20
                    @arg CMP_CR2_CRV_SEL_16_20 ********/
	FunctionalState CRV_En;	  /** @arg ENABLE
																@arg DISABLE ********/
	uint32_t CRV_Src; /** @arg CMP_CR2_CRV_SRC_AVCC
												@arg CMP_CR2_CRV_SRC_VBGP ********/
	uint32_t INP; /** @arg CMP_CR2_INP_SEL_PC3
                    @arg CMP_CR2_INP_SEL_PC0
                    @arg CMP_CR2_INP_SEL_PB7
                    @arg CMP_CR2_INP_SEL_PGA0_P
                    @arg CMP_CR2_INP_SEL_PGA1_P
                    @arg CMP_CR2_INP_SEL_PGA2_P
                    @arg CMP_CR2_INP_SEL_PB3
                    @arg CMP_CR2_INP_SEL_PB5 ********/
	uint32_t INM; /** @arg CMP_CR2_INM_SEL_CRV
                    @arg CMP_CR2_INM_SEL_PC4
                    @arg CMP_CR2_INM_SEL_PC1
                    @arg CMP_CR2_INM_SEL_PB8
                    @arg CMP_CR2_INM_SEL_PB4 ********/
}CMP_InitTypeDef;

typedef struct
{
  uint32_t Polling_INP; /** @arg CMP_PLC_INP_SEL_DISABLE
                            @arg CMP_PLC_INP_SEL_GPIO
                            @arg CMP_PLC_INP_SEL_PGA ********/
  uint32_t Polling_INM; /** @arg CMP_PLC_INM_SEL_DISABLE
                            @arg CMP_PLC_INM_SEL_GPIO ********/
  uint32_t Polling_Period;    /** @arg CMP_PLC_Period_32_CLK
                                  @arg CMP_PLC_Period_64_CLK
                                  @arg CMP_PLC_Period_128_CLK
                                  @arg CMP_PLC_Period_256_CLK ********/
  uint32_t Polling_MaskT;     /** @arg CMP_PLC_MaskT_0_CLK
                                  @arg CMP_PLC_MaskT_8_CLK
                                  @arg CMP_PLC_MaskT_16_CLK
                                  @arg CMP_PLC_MaskT_32_CLK ********/
  uint32_t Polling_OFLT_Mode;     /** @arg CMP_PLC_OFLT_After_MaskT
                                      @arg CMP_PLC_OFLT_Parallel_MaskT ********/                                        
}CMP_Polling_InitTypeDef;

/*
*********************************************************************************************************
*                                             Global variable
*********************************************************************************************************
*/

/** @defgroup CMP_OUTPUT_LEVEL: Comparator output
  * @{
  */
#define CMP_OUTPUT_LEVEL_HIGH               ((uint32_t)0x00000001)
#define CMP_OUTPUT_LEVEL_LOW                ((uint32_t)0x00000000)

/** @defgroup CMP_OUTPUT_LEVEL_Ready: Comparator output with ready signal
  * @{
  */
#define CMP_OUTPUT_LEVEL_HIGH_Ready      ((uint32_t)0x00000060)
#define CMP_OUTPUT_LEVEL_LOW_Ready       ((uint32_t)0x00000020)

	/*
*********************************************************************************************************
*                                           Global Function declaration
*********************************************************************************************************
*/
void CMP_DeInit(CMP_TypeDef* CMPx);
void CMP_StructInit(CMP_InitTypeDef *CMP_InitStruct);
void CMP_Init(CMP_TypeDef *CMPx, CMP_InitTypeDef *CMP_InitStruct);
void CMP_Enable(CMP_TypeDef *CMPx);
void CMP_Disable(CMP_TypeDef* CMPx);
void CMP_Set_Mode(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_POL(CMP_TypeDef *CMPx, uint32_t val);
uint32_t CMP_Read_OUT(CMP_TypeDef *CMPx);
uint32_t CMP_Read_OUT_Ready(CMP_TypeDef *CMPx);
void CMP_Enable_Lock(CMP_TypeDef *CMPx);
void CMP_Disable_Lock(CMP_TypeDef *CMPx);
void CMP_Set_OFLT(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Enable_IT(CMP_TypeDef *CMPx, uint32_t CMP_IT);
void CMP_Disable_IT(CMP_TypeDef *CMPx, uint32_t CMP_IT);
FlagStatus CMP_Get_Flag(CMP_TypeDef *CMPx, uint32_t Flag);
void CMP_Clear_Flag(CMP_TypeDef *CMPx, uint32_t Flag);
void CMP_Set_RHYST(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_FHYST(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_OutPad(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Enable_CRV(CMP_TypeDef *CMPx);
void CMP_Disable_CRV(CMP_TypeDef *CMPx);
void CMP_Set_CRV_SRC(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_CRV(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_INM(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_INP(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_BLANKING(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Enable_CALP(CMP_TypeDef *CMPx);
void CMP_Disable_CALP(CMP_TypeDef *CMPx);
void CMP_Enable_CALN(CMP_TypeDef *CMPx);
void CMP_Disable_CALN(CMP_TypeDef *CMPx);
void CMP_Set_CALP(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Set_CALN(CMP_TypeDef *CMPx, uint32_t val);

/***************************************CMP Polling Functions********************************************/
void CMP_Polling_Init(CMP_TypeDef *CMPx, CMP_Polling_InitTypeDef *CMP_InitStruct);
void CMP_Polling_Set_INP(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Set_INM(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Enable_CH1(CMP_TypeDef *CMPx);
void CMP_Polling_Disable_CH1(CMP_TypeDef *CMPx);
void CMP_Polling_Enable_CH2(CMP_TypeDef *CMPx);
void CMP_Polling_Disable_CH2(CMP_TypeDef *CMPx);
void CMP_Polling_Enable_CH3(CMP_TypeDef *CMPx);
void CMP_Polling_Disable_CH3(CMP_TypeDef *CMPx);
void CMP_Polling_Set_Period(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Set_MaskT(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Enable_IT(CMP_TypeDef *CMPx, uint32_t CMP_Polling_IT);
void CMP_Polling_Disable_IT(CMP_TypeDef *CMPx, uint32_t CMP_Polling_IT);
void CMP_Polling_Set_OFLT_Mode(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Set_Match_Data1(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Set_Match_Data2(CMP_TypeDef *CMPx, uint32_t val);
void CMP_Polling_Set_Match_Data3(CMP_TypeDef *CMPx, uint32_t val);
uint32_t CMP_Polling_Get_Current_Channle(CMP_TypeDef *CMPx);
FlagStatus CMP_Polling_Get_Flag(CMP_TypeDef *CMPx, uint32_t Flag);
void CMP_Polling_Clear_Flag(CMP_TypeDef *CMPx, uint32_t Flag);
uint32_t CMP_Polling_Read_OUTP(CMP_TypeDef *CMPx);
uint32_t CMP_Polling_Read_OUTP1(CMP_TypeDef *CMPx);
uint32_t CMP_Polling_Read_OUTP2(CMP_TypeDef *CMPx);
uint32_t CMP_Polling_Read_OUTP3(CMP_TypeDef *CMPx);
	
#ifdef __cplusplus
}
#endif

#endif
