/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_cmu.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_CMU_H__
#define __rx32s11xx_CMU_H__

#include "rx32s11xx.h"
/*
*********************************************************************************************************
*                                          
*********************************************************************************************************
*/

typedef struct
{	
	uint32_t SYSCLKSource;          /*!< The clock source (SYSCLKS) used as system clock.
                                       This parameter can be a value of @ref RCC_System_Clock_Source */
           
  uint32_t SYSDIV;  
	
} CMU_ClkInitTypeDef;


/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of CMU Module
**********************************************************************************************************
*/
/**************************  Bit definition for WPREG register of CMU_TypeDef ************************/
#define  CMU_WPREG                               ((uint32_t)0xffff)           
#define  CMU_WPREG_UnProtected                   ((uint32_t)0xA55A)           
#define  CMU_WPREG_Protected                     ((uint32_t)0x0000)              

/************************  Bit definition for SYSCLKCFG register of CMU_TypeDef **********************/
#define  CMU_SYSCLKCFG_WCLKEN                    ((uint32_t)0x0080)

#define  CMU_SYSCLKCFG_CLKSEL                    ((uint32_t)0x0083)           
#define  CMU_SYSCLKCFG_CLKSEL_LRC                ((uint32_t)0x0080)           
#define  CMU_SYSCLKCFG_CLKSEL_HSE                ((uint32_t)0x0081)          
#define  CMU_SYSCLKCFG_CLKSEL_HRC                ((uint32_t)0x0082)           

/*************************  Bit definition for JTAGSTA register of CMU_TypeDef ***********************/

#define  CMU_JTAGSTA_FLAG_SW                     ((uint32_t)0x0002)          

/*************************  Bit definition for LRCADJ register of CMU_TypeDef ************************/
#define  CMU_LRCADJ                              ((uint32_t)0x00ff)           

/*************************  Bit definition for HRCADJ register of CMU_TypeDef ************************/
#define  CMU_HRCADJ_TC                           ((uint32_t)0x3f00)           
#define  CMU_HRCADJ                              ((uint32_t)0x007f)                  

/*************************  Bit definition for CLKSTA register of CMU_TypeDef ************************/
#define  CMU_CLKSTA                              ((uint32_t)0x0044)           

#define  CMU_CLKSTA_HSEDET                       ((uint32_t)0x0004)         
#define  CMU_CLKSTA_HSELOCK                      ((uint32_t)0x0040)   
                    
/***********************  Bit definition for SYSCLKDIV register of CMU_TypeDef ***********************/
#define  CMU_SYSCLKDIV                           ((uint32_t)0x0007)           
#define  CMU_SYSCLKDIV_1                         ((uint32_t)0x0000)              
#define  CMU_SYSCLKDIV_2                         ((uint32_t)0x0001)            
#define  CMU_SYSCLKDIV_4                         ((uint32_t)0x0002)            
#define  CMU_SYSCLKDIV_8                         ((uint32_t)0x0003)            
#define  CMU_SYSCLKDIV_16                        ((uint32_t)0x0004)            
#define  CMU_SYSCLKDIV_32                        ((uint32_t)0x0005)            
#define  CMU_SYSCLKDIV_64                        ((uint32_t)0x0006)           
#define  CMU_SYSCLKDIV_128                       ((uint32_t)0x0007)        

/***********************  Bit definition for CLKOUTSEL register of CMU_TypeDef ***********************/
#define  CMU_CLKOUTSEL                           ((uint32_t)0x0003)           
#define  CMU_CLKOUTSEL_LRC                       ((uint32_t)0x0000)           /*!<LRC --> CLKOUT        */
#define  CMU_CLKOUTSEL_HSE                       ((uint32_t)0x0001)           /*!<OSC --> CLKOUT        */
#define  CMU_CLKOUTSEL_HRC                       ((uint32_t)0x0002)           /*!<HRC --> CLKOUT        */
#define  CMU_CLKOUTSEL_SYS                       ((uint32_t)0x0003)           /*!<SYS --> CLKOUT        */

/***********************  Bit definition for CLKOUTDIV register of CMU_TypeDef ***********************/
#define  CMU_CLKOUTDIV                          ((uint32_t)0x000f)           /*!<CLKOUT     */
#define  CMU_CLKOUTDIV_2                        ((uint32_t)0x0000)           /*!<CLKOUT /2     */
#define  CMU_CLKOUTDIV_4                        ((uint32_t)0x0001)           /*!<CLKOUT /4    */
#define  CMU_CLKOUTDIV_6                        ((uint32_t)0x0002)           /*!<CLKOUT /6   */
#define  CMU_CLKOUTDIV_8                        ((uint32_t)0x0003)           /*!<CLKOUT /8   */
#define  CMU_CLKOUTDIV_10                       ((uint32_t)0x0004)           /*!<CLKOUT /10   */
#define  CMU_CLKOUTDIV_12                       ((uint32_t)0x0005)           /*!<CLKOUT /12   */
#define  CMU_CLKOUTDIV_14                       ((uint32_t)0x0006)           /*!<CLKOUT /14   */
#define  CMU_CLKOUTDIV_16                       ((uint32_t)0x0007)           /*!<CLKOUT /16   */
#define  CMU_CLKOUTDIV_18                       ((uint32_t)0x0008)           /*!<CLKOUT /18   */
#define  CMU_CLKOUTDIV_20                       ((uint32_t)0x0009)           /*!<CLKOUT /20   */
#define  CMU_CLKOUTDIV_22                       ((uint32_t)0x000A)           /*!<CLKOUT /22   */
#define  CMU_CLKOUTDIV_24                       ((uint32_t)0x000B)           /*!<CLKOUT /24   */
#define  CMU_CLKOUTDIV_26                       ((uint32_t)0x000C)           /*!<CLKOUT /26   */
#define  CMU_CLKOUTDIV_28                       ((uint32_t)0x000D)           /*!<CLKOUT /28   */
#define  CMU_CLKOUTDIV_30                       ((uint32_t)0x000E)           /*!<CLKOUT /20   */
#define  CMU_CLKOUTDIV_32                       ((uint32_t)0x000F)           /*!<CLKOUT /32   */
 

/************************  Bit definition for CLKCTRL0 register of CMU_TypeDef ***********************/
#define  CMU_CLKCTRL0                           ((uint32_t)0xE03CA8)                   
#define  CMU_CLKCTRL0_I2CEN                      BIT3
#define  CMU_CLKCTRL0_HRCEN                      BIT5
#define  CMU_CLKCTRL0_HSEDETEN                   BIT7
#define  CMU_CLKCTRL0_HSELOCKEN                  BIT10
#define  CMU_CLKCTRL0_HSEEN                      BIT11
#define  CMU_CLKCTRL0_CLKOUTEN                   BIT12
#define  CMU_CLKCTRL0_1P5LBOREN                  BIT13
#define  CMU_CLKCTRL0_CMP4EN                     BIT21
#define  CMU_CLKCTRL0_CMP5EN                     BIT22
#define  CMU_CLKCTRL0_PGAEN                      BIT23

/************************  Bit definition for CLKCTRL1 register of CMU_TypeDef ***********************/
#define  CMU_CLKCTRL1                            ((uint32_t)0x1CC02F)          
#define  CMU_CLKCTRL1_TMR0EN                     BIT0           
#define  CMU_CLKCTRL1_TMR1EN                     BIT1           
#define  CMU_CLKCTRL1_TMR2EN                     BIT2           
#define  CMU_CLKCTRL1_TMR3EN                     BIT3            
#define  CMU_CLKCTRL1_UART1EN                    BIT5
#define  CMU_CLKCTRL1_SOFTWDTEN                  BIT14
#define  CMU_CLKCTRL1_DMAEN                      BIT15
#define  CMU_CLKCTRL1_TIM8EN                     BIT18          
#define  CMU_CLKCTRL1_MEEN                       BIT19           
#define  CMU_CLKCTRL1_SADCEN                    	 BIT20          
                                                                           
/**************************  Bit definition for MULTFUNCFG register of CMU_TypeDef ***********************/
#define  CMU_MULTFUNCFG_3T                            ((uint32_t)0x0001)          
#define  CMU_MULTFUNCFG_3T_ENABLE                     ((uint32_t)0x0001)  
#define  CMU_MULTFUNCFG_3T_DISABLE                    ((uint32_t)0x0000)  

/************************  Bit definition for FLASHCON register of CMU_TypeDef **********************/
#define  CMU_FLASHCON                            ((uint32_t)0x0007)        
#define  CMU_FLASHCON_FOP                        ((uint32_t)0x0003)         
#define  CMU_FLASHCON_FOP_READ                   ((uint32_t)0x0000)         
#define  CMU_FLASHCON_FOP_WRITE                  ((uint32_t)0x0001)         
#define  CMU_FLASHCON_FOP_PAGEERASE              ((uint32_t)0x0002)        
#define  CMU_FLASHCON_FOP_CHIPERASE              ((uint32_t)0x0003)        

#define  CMU_FLASHCON_BUSY                       ((uint32_t)0x0004)          

/************************  Bit definition for FLASHLOCK register of CMU_TypeDef **********************/
#define  CMU_FLASHLOCK                           ((uint32_t)0xffff)         
#define  CMU_FLASHLOCK_UnLocked                  ((uint32_t)0x7A68)         
#define  CMU_FLASHLOCK_Locked                    ((uint32_t)0x0000)   

/**************************  Bit definition for FLTCTR register of CMU_TypeDef ***********************/
#define  CMU_HSECTR                              ((uint32_t)0x0003) 
#define  CMU_LRCCTR                              ((uint32_t)0x000C) 
#define  CMU_HRCCTR                              ((uint32_t)0x0030) 

/*
*********************************************************************************************************
*                                             Global variable
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             Global Function declaration
*********************************************************************************************************
*/

void CMU_DeInit(CMU_TypeDef* CMUx);
void CMU_StructInit(CMU_ClkInitTypeDef *CMU_InitStruct);
void CMU_ClockConfig(CMU_TypeDef *CMUx,CMU_ClkInitTypeDef  *CMU_ClkInitStruct);

void CMU_Enable_CLKCTRL0(uint32_t val);
void CMU_Disable_CLKCTRL0(uint32_t val);
void CMU_Enable_CLKCTRL1(uint32_t val);
void CMU_Disable_CLKCTRL1(uint32_t val);
void CMU_Set_SYSCLK(uint32_t val);
void CMU_Set_SYSCLKDIV(uint32_t val);
uint32_t CMU_Get_CLKSTA(uint32_t val);
void CMU_Set_CLKOUTSEL(uint32_t val);
void CMU_Set_CLKOUTDIV(uint32_t val);
void CMU_Set_MULTFUNCFG(uint8_t val);
void CMU_Set_FLTCTR(uint32_t FLTCTR_X,uint8_t val);

#endif /* __rx32s11xx_CMU_H__ */
