/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_dma.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_DMA_H__
#define __rx32s11xx_DMA_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s11xx.h"
	
typedef struct
{
  uint32_t Channel; /** @arg DMA_CHANNEL_0
												@arg DMA_CHANNEL_1 ********/
  uint32_t CHNCTL_Mode; /** @arg DMA_CHNCTL_MODE_WordTransfer
														@arg DMA_CHNCTL_MODE_BlockTransfer ********/ 
  uint32_t CHNCTL_Cycle; /** @arg DMA_CHNCTL_CYCLE_DISABLE
														 @arg DMA_CHNCTL_CYCLE_ENABLE ********/ 
  uint32_t CHNCTL_Destininc; /** @arg DMA_CHNCTL_DESTININC_NOINC
																 @arg DMA_CHNCTL_DESTININC_INC
																 @arg DMA_CHNCTL_DESTININC_BLOCKINC	********/   
  uint32_t CHNCTL_Sourceinc; /** @arg DMA_CHNCTL_SOURCEINC_NOINC
																 @arg DMA_CHNCTL_SOURCEINC_INC
																 @arg DMA_CHNCTL_SOURCEINC_BLOCKINC	********/                   
  uint32_t CHNCTL_Psize; /** @arg DMA_CHNCTL_PSIZE_Byte
														 @arg DMA_CHNCTL_PSIZE_HalfWord
														 @arg DMA_CHNCTL_PSIZE_Word	********/   
  uint32_t CHNCTL_Channel; /** @arg DMA_CHNCTL_CHANNEL_Software
															 @arg DMA_CHNCTL_CHANNEL_UART0_TX
															 @arg DMA_CHNCTL_CHANNEL_UART0_RX
															 @arg DMA_CHNCTL_CHANNEL_UART1_TX
															 @arg DMA_CHNCTL_CHANNEL_UART1_RX
															 @arg DMA_CHNCTL_CHANNEL_I2C_TX
															 @arg DMA_CHNCTL_CHANNEL_I2C_RX
															 @arg DMA_CHNCTL_CHANNEL_SADC
															 @arg DMA_CHNCTL_CHANNEL_TIM8_UP
															 @arg DMA_CHNCTL_CHANNEL_TIM8_TRIG
															 @arg DMA_CHNCTL_CHANNEL_TIMER0
															 @arg DMA_CHNCTL_CHANNEL_TIMER1
															 @arg DMA_CHNCTL_CHANNEL_TIMER2
															 @arg DMA_CHNCTL_CHANNEL_TIMER3
															 @arg DMA_CHNCTL_CHANNEL_TIM8_COM
															 @arg DMA_CHNCTL_CHANNEL_TIM8_CH1
															 @arg DMA_CHNCTL_CHANNEL_TIM8_CH2
															 @arg DMA_CHNCTL_CHANNEL_TIM8_CH3
															 @arg DMA_CHNCTL_CHANNEL_TIM8_CH4
															 @arg DMA_CHNCTL_CHANNEL_TIM8_CH5	********/  
  uint32_t SrcAddress;
  uint32_t TarAddress;
  uint32_t DataLength;                               
} DMA_InitTypeDef;

/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of DMA Module
**********************************************************************************************************
*/
/**************************  Bit definition for DMAIE register of DMA_TypeDef ************************/
#define  DMA_DMAIE                               ((uint32_t)0x0777)          

#define  DMA_DMAIE_TCIE0                         ((uint32_t)0x0001)        
#define  DMA_DMAIE_TCIE1                         ((uint32_t)0x0002)         
#define  DMA_DMAIE_TCIE2                         ((uint32_t)0x0004)           

#define  DMA_DMAIE_BCIE0                         ((uint32_t)0x0010)           
#define  DMA_DMAIE_BCIE1                         ((uint32_t)0x0020)          
#define  DMA_DMAIE_BCIE2                         ((uint32_t)0x0040)         

#define  DMA_DMAIE_TEIE0                         ((uint32_t)0x0100)           
#define  DMA_DMAIE_TEIE1                         ((uint32_t)0x0200)          
#define  DMA_DMAIE_TEIE2                         ((uint32_t)0x0400)          

/**************************  Bit definition for DMAIF register of DMA_TypeDef ************************/
#define  DMA_DMAIF                               ((uint32_t)0x0777)         

#define  DMA_DMAIF_TCIF0                         ((uint32_t)0x0001)           
#define  DMA_DMAIF_TCIF1                         ((uint32_t)0x0002)           
#define  DMA_DMAIF_TCIF2                         ((uint32_t)0x0004)          

#define  DMA_DMAIF_BCIF0                         ((uint32_t)0x0010)          
#define  DMA_DMAIF_BCIF1                         ((uint32_t)0x0020)           
#define  DMA_DMAIF_BCIF2                         ((uint32_t)0x0040)          

#define  DMA_DMAIF_TEIF0                         ((uint32_t)0x0100)           
#define  DMA_DMAIF_TEIF1                         ((uint32_t)0x0200)           
#define  DMA_DMAIF_TEIF2                         ((uint32_t)0x0400)           

/**************************  Bit definition for CHNSTA register of DMA_TypeDef ***********************/
#define  DMA_CHNSTA                              ((uint32_t)0x0007)           
#define  DMA_CHNSTA_BUSY0                        ((uint32_t)0x0001)          
#define  DMA_CHNSTA_BUSY1                        ((uint32_t)0x0002)           
#define  DMA_CHNSTA_BUSY2                        ((uint32_t)0x0004)         

/*********************  Bit definition for CHNCTL register of DMA_Channel_TypeDef ********************/
#define  DMA_CHNCTL                              ((uint32_t)0x3ffff)         

#define  DMA_CHNCTL_CHNEN                        ((uint32_t)0x0001)          

#define  DMA_CHNCTL_PSIZE                        ((uint32_t)0x0006)          
#define  DMA_CHNCTL_PSIZE_8BIT                   ((uint32_t)0x0000)          
#define  DMA_CHNCTL_PSIZE_16BIT                  ((uint32_t)0x0002)           
#define  DMA_CHNCTL_PSIZE_32BIT                  ((uint32_t)0x0004)           

#define  DMA_CHNCTL_PSIZE_Byte                   ((uint32_t)0x0000)           
#define  DMA_CHNCTL_PSIZE_HalfWord               ((uint32_t)0x0002)         
#define  DMA_CHNCTL_PSIZE_Word                   ((uint32_t)0x0004)        

#define  DMA_CHNCTL_MODE                         ((uint32_t)0x0008)          
#define  DMA_CHNCTL_MODE_WordTransfer            ((uint32_t)0x0000)           
#define  DMA_CHNCTL_MODE_BlockTransfer           ((uint32_t)0x0008)           

#define  DMA_CHNCTL_CYCLE                        ((uint32_t)0x0010)          
#define  DMA_CHNCTL_CYCLE_DISABLE                ((uint32_t)0x0000)           
#define  DMA_CHNCTL_CYCLE_ENABLE                 ((uint32_t)0x0010)          

#define  DMA_CHNCTL_SOURCEINC                    ((uint32_t)0x0060)          
#define  DMA_CHNCTL_SOURCEINC_NOINC              ((uint32_t)0x0000)          
#define  DMA_CHNCTL_SOURCEINC_INC                ((uint32_t)0x0020)          
#define  DMA_CHNCTL_SOURCEINC_BLOCKINC           ((uint32_t)0x0040)          

#define  DMA_CHNCTL_DESTININC                    ((uint32_t)0x0180)        
#define  DMA_CHNCTL_DESTININC_NOINC              ((uint32_t)0x0000)         
#define  DMA_CHNCTL_DESTININC_INC                ((uint32_t)0x0080)         
#define  DMA_CHNCTL_DESTININC_BLOCKINC           ((uint32_t)0x0100)       

#define  DMA_CHNCTL_CHANNEL                      ((uint32_t)0x7e00)          
#define  DMA_CHNCTL_CHANNEL_Software             ((uint32_t)0x00000000 << 9)
#define  DMA_CHNCTL_CHANNEL_UART0_TX             ((uint32_t)0x00000001 << 9)
#define  DMA_CHNCTL_CHANNEL_UART0_RX             ((uint32_t)0x00000002 << 9)
#define  DMA_CHNCTL_CHANNEL_UART1_TX             ((uint32_t)0x00000003 << 9)
#define  DMA_CHNCTL_CHANNEL_UART1_RX             ((uint32_t)0x00000004 << 9)
#define  DMA_CHNCTL_CHANNEL_I2C_TX               ((uint32_t)0x00000013 << 9)
#define  DMA_CHNCTL_CHANNEL_I2C_RX               ((uint32_t)0x00000014 << 9)
#define  DMA_CHNCTL_CHANNEL_SADC                  ((uint32_t)0x00000015 << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_UP              ((uint32_t)0x00000016 << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_TRIG            ((uint32_t)0x00000017 << 9)
#define  DMA_CHNCTL_CHANNEL_TIMER0               ((uint32_t)0x00000018 << 9)
#define  DMA_CHNCTL_CHANNEL_TIMER1               ((uint32_t)0x00000019 << 9)
#define  DMA_CHNCTL_CHANNEL_TIMER2               ((uint32_t)0x0000001A << 9)
#define  DMA_CHNCTL_CHANNEL_TIMER3               ((uint32_t)0x0000001B << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_COM             ((uint32_t)0x0000001C << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_CH1             ((uint32_t)0x0000001D << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_CH2             ((uint32_t)0x0000001E << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_CH3             ((uint32_t)0x0000001F << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_CH4             ((uint32_t)0x00000020 << 9)
#define  DMA_CHNCTL_CHANNEL_TIM8_CH5             ((uint32_t)0x00000021 << 9)

#define  DMA_CHNCTL_SOFTREQUEST_TRIG             ((uint32_t)0x8000)

#define  DMA_CHNCTL_CHANNEL_PRIORITY             ((uint32_t)0x30000)
#define  DMA_CHNCTL_CHANNEL_PRIORITY_LOW         ((uint32_t)0x00000)
#define  DMA_CHNCTL_CHANNEL_PRIORITY_MEDIUM      ((uint32_t)0x10000)
#define  DMA_CHNCTL_CHANNEL_PRIORITY_HIGH        ((uint32_t)0x20000)
#define  DMA_CHNCTL_CHANNEL_PRIORITY_HIGHEST     ((uint32_t)0x30000)

/*********************  Bit definition for CHNSRC register of DMA_Channel_TypeDef ********************/
#define  DMA_CHNSRC                              ((uint32_t)0xffffffff)      

/*********************  Bit definition for CHNTAR register of DMA_Channel_TypeDef ********************/
#define  DMA_CHNTAR                              ((uint32_t)0xffffffff)       

/*********************  Bit definition for CHNCNT register of DMA_Channel_TypeDef ********************/
#define  DMA_CHNCNT                              ((uint32_t)0xffff)          

/********************  Bit definition for CHNTCCNT register of DMA_Channel_TypeDef *******************/
#define  DMA_CHNTCCNT                            ((uint32_t)0xffff)           

/*******************  Bit definition for CHNBULKNUM register of DMA_Channel_TypeDef ******************/
#define  DMA_CHNBULKNUM                          ((uint32_t)0xffffff)          
#define  DMA_CHNBULKNUM_NUM                      ((uint32_t)0x00ffff)
#define  DMA_CHNBULKNUM_CYCLE                    ((uint32_t)0xff0000)

/******************* Array used to get the DMA channel register offset versus channel index DMA_CHANNEL_x ******************/
static const uint8_t CHANNEL_OFFSET_TAB[] =
{
  (uint8_t)(DMA_Channel0_BASE - DMA_BASE),
  (uint8_t)(DMA_Channel1_BASE - DMA_BASE),
};

void DMA_DeInit(DMA_TypeDef* DMAx);
void DMA_StructInit(DMA_InitTypeDef *DMA_InitStruct);
void DMA_Init(DMA_TypeDef *DMAx, DMA_InitTypeDef* DMA_InitStruct);
void DMA_Set_Transfer(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t Configuration);
void DMA_Set_Addresses(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t SrcAddress,uint32_t TarAddress);
void DMA_Set_DataLength(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t DataLength);
uint32_t DMA_Get_TCDataLength(DMA_TypeDef* DMAx, uint32_t Channel );
void DMA_Set_BulkCycle(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t CycleLength);
void DMA_Set_BulkNumber(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t Cyclenumber);
void DMA_Enable_Channel(DMA_TypeDef* DMAx, uint32_t Channel);
void DMA_Disable_Channel(DMA_TypeDef* DMAx, uint32_t Channel);
void DMA_Set_ChannelPriority( DMA_TypeDef* DMAx, uint32_t Channel, uint32_t val );
void DMA_SoftRequestTrig( DMA_TypeDef* DMAx, uint32_t Channel );
void DMA_Enable_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT);
void DMA_Disable_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT);
uint32_t DMA_Get_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT);
uint32_t DMA_Get_Flag(DMA_TypeDef* DMAx, uint32_t Flag);
void DMA_Clear_Flag(DMA_TypeDef* DMAx, uint32_t Flag);
uint32_t DMA_Get_Busy(DMA_TypeDef* DMAx, uint32_t IFlag);

#ifdef __cplusplus
}
#endif

#endif /* __rx32s11xx_DMA_H__ */


