/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_exti.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_EXTI_H__
#define __rx32s11xx_EXTI_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s11xx.h"
  
/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of EXTI Module
**********************************************************************************************************
*/
/*********************  Bit definition for EXTIE & EXTIE2 register of EXTI_TypeDef ********************/
#define EXTI_ENABLE_FIE              ((uint16_t)0x0001)
#define EXTI_ENABLE_RIE              ((uint16_t)0x0001)

/**************************  Bit definition for EXTIF register of EXTI_TypeDef ************************/
#define EXTI_READ_FIF                ((uint16_t)0x0001)
#define EXTI_READ_RIF                ((uint16_t)0x0001)

/**************************  Bit definition for FILTEN register of EXTI_TypeDef ***********************/
#define EXTI_PIN_FLT                 ((uint16_t)0x0001)

/*************************  Bit definition for RXFILTEN register of EXTI_TypeDef **********************/
#define RX_PIN_FLT                   ((uint16_t)0x0001)

/*************************  Bit definition for FILTSEL register of EXTI_TypeDef ***********************/
#define EXTI_Filter_80_ns            ((uint32_t)0x0000)
#define EXTI_Filter_50_ns            ((uint32_t)0x0003)
#define EXTI_RAED_FILTSEL            ((uint32_t)0x0003)
/*
**********************************************************************************************************
*                                 EXTI Number Definition of EXTI Module
**********************************************************************************************************
*/
#define  UART1_RX   (1)

#define  EXTI_0     (0)
#define  EXTI_1     (1)
#define  EXTI_2     (2)
#define  EXTI_3     (3)
#define  EXTI_4     (4)
#define  EXTI_5     (5)
#define  EXTI_6     (6)
#define  EXTI_7     (7)
#define  EXTI_8     (8)

/*
**********************************************************************************************************
*                         EXTI_InitTypeDef Bit Definition of EXTI Module
**********************************************************************************************************
*/
/***************************  Bit definition for Mode of EXTI_InitTypeDef *****************************/
#define Rising_Edge_Trigger           ((uint16_t)0)
#define Falling_Edge_Trigger		      ((uint16_t)1)

/***************************  Bit definition for FltValue of EXTI_InitTypeDef *************************/
#define EXTI_No_Filter                ((uint16_t)1)

/*
**********************************************************************************************************
*                                 Initial structure of EXTI Module
**********************************************************************************************************
*/
typedef struct
{
  uint16_t ExtiNumber;   /* EXTI_x */
	uint16_t TrigMode;     /* Rising_Edge_Trigger or Falling_Edge_Trigger */
	uint16_t FltValue;     /* EXTI_No_Filter or EXTI_Filter_50_ns or EXTI_Filter_80_ns */
}EXTI_InitTypeDef;

/*
**********************************************************************************************************
*                                Function Declaration of EXTI Module
**********************************************************************************************************
*/
void EXTI_Init(EXTI_InitTypeDef *EXTI_InitStruct);
void EXTI_DeInit(void);
void EXTI_StructInit(EXTI_InitTypeDef *EXTI_InitStruct);

void EXTI_Enable_FallingTrig(uint32_t exti_x);
void EXTI_Disable_FallingTrig(uint32_t exti_x);
uint32_t EXTI_Get_FallingTrig(uint32_t exti_x);

uint32_t EXTI_Get_FallingFlag(uint32_t exti_x);
void EXTI_Clear_FallingFlag(uint32_t exti_x);

void EXTI_Enable_RisingTrig(uint32_t exti_x);
void EXTI_Disable_RisingTrig(uint32_t exti_x);
uint32_t EXTI_Get_RisingTrig(uint32_t exti_x);

uint32_t EXTI_Get_RisingFlag(uint32_t exti_x);
void EXTI_Clear_RisingFlag(uint32_t exti_x);

void EXTI_Enable_FLT(uint32_t exti_x);
void EXTI_Disable_FLT(uint32_t exti_x);
uint32_t EXTI_Get_INTFLT(uint32_t exti_x);

void EXTI_Set_FilterValue(uint32_t exti_x ,uint32_t val);
uint32_t EXTI_Get_FilterValue(uint32_t exti_x);

void RX_Enable_Filter(uint32_t UARTx_RX);
void RX_Disable_Filter(uint32_t UARTx_RX);
uint32_t RX_Get_FilterStatus(uint32_t UARTx_RX);

#ifdef __cplusplus
}
#endif

#endif
