/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_gpio.h
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_GPIO_H__
#define __rx32s11xx_GPIO_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "rx32s11xx.h"

/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of GPIO Module
**********************************************************************************************************
*/
/**************************  Bit definition for IOCFG register of GPIO_TypeDef ************************/
#define GPIO_MODE                  ((uint16_t)0x0000) 
#define AF_MODE                    ((uint16_t)0x0001) 
#define GPIO_READ_IOCFG            ((uint16_t)0x0001)

/*********************  Bit definition for AFCFG1 & AFCFG2 register of GPIO_TypeDef *******************/
#define AF_FUNCTION1               ((uint16_t)0x0000)
#define AF_FUNCTION2               ((uint16_t)0x0001)
#define AF_FUNCTION3               ((uint16_t)0x0002)
#define AF_FUNCTION4               ((uint16_t)0x0003)
#define AF_READ_FUNCTION           ((uint16_t)0x0003)

/**************************  Bit definition for PTDIR register of GPIO_TypeDef ************************/
#define GPIO_INPUT                 ((uint16_t)0x0000)
#define GPIO_OUTPUT                ((uint16_t)0x0001)
#define GPIO_READ_PTDIR            ((uint16_t)0x0001)

/**************************  Bit definition for PUPDN register of GPIO_TypeDef ************************/
#define GPIO_PULL_UP               ((uint32_t)0x00010000)
#define GPIO_PULL_DOWN             ((uint32_t)0x00000001)
#define GPIO_FLOATING              ((uint32_t)0x00010001)
#define GPIO_READ_PUPDN            ((uint32_t)0x00010001)

/**************************  Bit definition for PTDAT register of GPIO_TypeDef ************************/
#define GPIO_READ_PTDAT            ((uint16_t)0x0001)

/**************************  Bit definition for PTSET register of GPIO_TypeDef ************************/
#define GPIO_SETOUTPUT             ((uint16_t)0x0001)

/**************************  Bit definition for PTCLR register of GPIO_TypeDef ************************/
#define GPIO_CLROUTPUT             ((uint16_t)0x0001)

/**************************  Bit definition for PTTOG register of GPIO_TypeDef ************************/
#define GPIO_TOGOUTPUT             ((uint16_t)0x0001)

/**************************  Bit definition for PTOD register of GPIO_TypeDef *************************/
#define GPIO_OPENDRAIN             ((uint16_t)0x0000)
#define GPIO_PUSHPULL              ((uint16_t)0x0001)
#define GPIO_READ_PTOD             ((uint16_t)0x0001)

/**************************  Bit definition for HIIPM register of GPIO_TypeDef ************************/
#define GPIO_HIIPM_DISABLE         ((uint16_t)0x0000)
#define GPIO_HIIPM_ENABLE          ((uint16_t)0x0001)
#define GPIO_READ_HIIPM            ((uint16_t)0x0001)

/***************************  Bit definition for IDR register of GPIO_TypeDef *************************/
#define GPIO_READ_IDR              ((uint16_t)0x0001)
/*
**********************************************************************************************************
*                                 Pin Definition of GPIO Module
**********************************************************************************************************
*/
#define GPIO_PIN_0    (0)   /*!< Pin 0 selected    */
#define GPIO_PIN_1    (1)   /*!< Pin 1 selected    */
#define GPIO_PIN_2    (2)   /*!< Pin 2 selected    */
#define GPIO_PIN_3    (3)   /*!< Pin 3 selected    */
#define GPIO_PIN_4    (4)   /*!< Pin 4 selected    */
#define GPIO_PIN_5    (5)   /*!< Pin 5 selected    */
#define GPIO_PIN_6    (6)   /*!< Pin 6 selected    */
#define GPIO_PIN_7    (7)   /*!< Pin 7 selected    */
#define GPIO_PIN_8    (8)   /*!< Pin 8 selected    */
#define GPIO_PIN_9    (9)   /*!< Pin 9 selected    */
#define GPIO_PIN_10   (10)  /*!< Pin 10 selected   */
#define GPIO_PIN_11   (11)  /*!< Pin 11 selected   */
#define GPIO_PIN_12   (12)  /*!< Pin 12 selected   */
#define GPIO_PIN_13   (13)  /*!< Pin 13 selected   */
#define GPIO_PIN_14   (14)  /*!< Pin 14 selected   */
#define GPIO_PIN_15   (15)  /*!< Pin 15 selected   */

/*
**********************************************************************************************************
*                         GPIO_InitTypeDef Bit Definition of GPIO Module
**********************************************************************************************************
*/
/***************************  Bit definition for Mode of GPIO_InitTypeDef *****************************/
#define GPIO_MODE_INPUT         ((uint16_t)0)
#define GPIO_MODE_OUTPUT_PP			((uint16_t)1)
#define GPIO_MODE_OUTPUT_OD			((uint16_t)2)
#define GPIO_MODE_AF_PP         ((uint16_t)3)
#define GPIO_MODE_AF_OD         ((uint16_t)4)
#define GPIO_MODE_HiZ           ((uint16_t)5)

/*
**********************************************************************************************************
*                              Initial structure of GPIO Module
**********************************************************************************************************
*/
typedef struct
{
	uint16_t Pin;   /*GPIO_PIN_x */
	uint16_t Mode;  /*GPIO_MODE_INPUT or GPIO_MODE_OUTPUT_PP or GPIO_MODE_OUTPUT_OD or 
					 				  GPIO_MODE_AF_PP or GPIO_MODE_AF_OD or GPIO_MODE_HiZ*/
	uint32_t Pull;  /*GPIO_PULL_UP or GPIO_PULL_DOWN or  GPIO_FLOATING*/
	uint16_t AF;    /*AF_FUNCTION1 or  AF_FUNCTION2 or AF_FUNCTION3*/
}GPIO_InitTypeDef;

/*
**********************************************************************************************************
*                             Function Declaration of GPIO Module
**********************************************************************************************************
*/
void GPIO_Set_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Mode);
uint16_t GPIO_Get_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_Set_PinAF(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Sel_AF);
uint16_t GPIO_Get_PinAF(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_Set_DIR(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t DIR);
uint16_t GPIO_Get_DIR(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_Set_INPUTUP(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t UpDn);
uint32_t GPIO_Get_INPUTUP(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_Set_OutputMode(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t output_mode);
uint16_t GPIO_Get_OutputMode(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_Set_HIIPM(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t HIIPM);
uint16_t GPIO_Get_HIIPM(GPIO_TypeDef *GPIOx, uint32_t Pin);

uint16_t GPIO_Get_IDR(GPIO_TypeDef *GPIOx, uint32_t Pin);
uint16_t GPIO_Get_PTDAT(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Set_Output(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Clear_Output(GPIO_TypeDef *GPIOx, uint32_t Pin);
void GPIO_Toggle_Output(GPIO_TypeDef *GPIOx, uint32_t Pin);

void GPIO_DeInit(GPIO_TypeDef *GPIOx);
void GPIO_StructInit(GPIO_InitTypeDef *GPIO_InitStruct);
void GPIO_Init(GPIO_TypeDef *GPIOx, GPIO_InitTypeDef *GPIO_InitStruct);

#ifdef __cplusplus
}
#endif

#endif
