/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_iic_lib.h
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_IIC_H__
#define __rx32s11xx_IIC_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "rx32s11xx.h"
	
/*
**********************************************************************************************************
*                         Peripheral Registers_Bits_Definition of I2C Module
**********************************************************************************************************
*/
/**********************  Bit definition for I2CCON register of I2C_TypeDef ***********************/
	
#define I2C_I2CCON_EN                ((uint32_t)0x0040)
#define I2C_I2CCON_Start             ((uint32_t)0x0020)
#define I2C_I2CCON_Stop              ((uint32_t)0x0010)
#define I2C_I2CCON_Ack               ((uint32_t)0x0004)
	
/**********************  Bit definition for I2CADR register of I2C_TypeDef ***********************/
	
#define I2C_I2CADR_GeneralCall                ((uint32_t)0x0001)

	
/** @defgroup I2C_Start_Cmd
  * @{
  */
#define I2C_Start_Set 	0x0020U
#define I2C_Start_Reset 0x0000U

/** @defgroup I2C_Stop_Cmd
  * @{
  */
#define I2C_Stop_Set 		0x0010U
#define I2C_Stop_Reset 	0x0000U

/** @defgroup I2C_SI_Cmd
  * @{
  */
#define I2C_SI_Set 		0x0008U
#define I2C_SI_Reset 	0x0000U

/** @defgroup I2C_Acknowledge_Cmd
  * @{
  */
#define I2C_Acknowledge_Set 0x0004U
#define I2C_Acknowledge_Reset 0x0000U

typedef struct
{
		uint32_t Fsys; /** @arg This parameter must be set to the real system clock ********/

		uint32_t ClockSpeed; /** @arg 0 ~ 400000 ********/

		uint16_t OwnAddress; /** @arg 0x00 ~ 0x7F ********/

		uint16_t GeneralCallMode; /** @arg ENABLE
																	@arg DISABLE********/

		FunctionalState Ack; /** @arg ENABLE
														 @arg DISABLE********/
} I2C_InitTypeDef;

typedef struct
{
		uint8_t DevAddress; /** @arg 0x00 ~ 0x7F ********/

		uint32_t MemAddress; /** @arg 0x0000 ~ 0xFFFF ********/

		uint8_t MemAddressSize; /** @arg 8 or 16 ********/

		uint8_t *pData; /** @arg Pointer to I2C Data buffer ********/

		uint16_t Size; /** @arg I2C transfer size ********/

} I2C_HandleTypeDef;


void I2C_DeInit(I2C_TypeDef* I2Cx);
void I2C_StructInit(I2C_InitTypeDef *I2C_InitStruct);
void I2C_Init(I2C_TypeDef *I2Cx, I2C_InitTypeDef *I2C_InitStruct);
void I2C_Enable(I2C_TypeDef *I2Cx);
void I2C_Generate_Start(I2C_TypeDef *I2Cx);
void I2C_Generate_Stop(I2C_TypeDef *I2Cx);
void I2C_Send_Data(I2C_TypeDef *I2Cx, uint8_t Data);
uint8_t I2C_Receive_Data(I2C_TypeDef *I2Cx);
uint32_t I2C_Get_Status(I2C_TypeDef *I2Cx);
void I2C_Set_SSIA(I2C_TypeDef *I2Cx, uint16_t I2C_StaCmd, uint16_t I2C_Stop_Cmd, uint16_t I2C_SI_Cmd, uint16_t I2C_Acknowledge_Cmd);
void I2C_Set_SSI(I2C_TypeDef *I2Cx, uint16_t I2C_StaCmd, uint16_t I2C_Stop_Cmd, uint16_t I2C_SI_Cmd);
void I2C_Enable_Acknowledge(I2C_TypeDef *I2Cx);
void I2C_Disable_Acknowledge(I2C_TypeDef *I2Cx);
void I2C_Enable_GeneralCall(I2C_TypeDef *I2Cx);
void I2C_Disable_GeneralCall(I2C_TypeDef *I2Cx);

/* IO operation functions  ****************************************************/
/******* Blocking mode: Polling */
void I2C_Master_Transmit(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);
void I2C_Master_Receive(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);
void I2C_Slave_Transmit(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);
void I2C_Slave_Receive(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);
void I2C_Mem_Master_Write(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);
void I2C_Mem_Master_Read(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle);

#ifdef __cplusplus
}
#endif

#endif /* __rx32s11xx_IIC_H__ */
