/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_pmu.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_PMU_H__
#define __rx32s11xx_PMU_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s11xx.h"

/*
* @brief  PMU_PDLDO
*/
#define PMU_PDLDO_ENABLE                  ((uint32_t)0x0000)
#define PMU_PDLDO_DISABLE                 ((uint32_t)0x0001)
#define PMU_PDLDO_SHIFT                   (5)
#define PMU_READ_PDLDO                    ((uint32_t)0x0001)

/*
* @brief  PMU_LDO_INSLEEP
*/
#define PMU_LDO_ENABLE                    ((uint32_t)0x0001)
#define PMU_LDO_DISABLE                   ((uint32_t)0x0000)
#define PMU_LDO_SHIFT                     (4)
#define PMU_READ_LDO                      ((uint32_t)0x0001)

/*
* @brief  PMU_VCC_LVL
*/
#define PMU_VCCLVL_2_2_v                  ((uint32_t)0x0000)
#define PMU_VCCLVL_2_8_v                  ((uint32_t)0x0001)
#define PMU_VCCLVL_3_6_v                  ((uint32_t)0x0002)
#define PMU_VCCLVL_4_2_v                  ((uint32_t)0x0003)
#define PMU_VCCLVL_SHIFT                  (2)
#define PMU_READ_VCCLVL                   ((uint32_t)0x0003)

/*
* @brief  PMU_TIME_SHARING_DETECTION
*/
#define PMU_TIME_300_us                   ((uint32_t)0x0000)
#define PMU_TIME_360_us                   ((uint32_t)0x0001)
#define PMU_TIME_480_us                   ((uint32_t)0x0002)
#define PMU_TIME_1120_us                  ((uint32_t)0x0003)
#define PMU_TIME_SHIFT                    (3)
#define PMU_READ_TIME                     ((uint32_t)0x0003)

#define PMU_PERIOD_16_5_ms                ((uint32_t)0x0000)
#define PMU_PERIOD_33_ms                  ((uint32_t)0x0001)
#define PMU_PERIOD_67_ms                  ((uint32_t)0x0002)
#define PMU_PERIOD_134_ms                 ((uint32_t)0x0003)
#define PMU_PERIOD_268_ms                 ((uint32_t)0x0004)
#define PMU_PERIOD_536_ms                 ((uint32_t)0x0005)
#define PMU_PERIOD_1072_ms                ((uint32_t)0x0006)
#define PMU_PERIOD_2144_ms                ((uint32_t)0x0007)
#define PMU_PERIOD_SHIFT                  (0)
#define PMU_READ_PERIOD                   ((uint32_t)0x0007)

#define PMU_ENABLE_VCC_DET_IT             ((uint32_t)0x0001)
#define PMU_VCC_DET_IT_SHIFT              (0)
#define PMU_READ_VCC_DET_IT               ((uint32_t)0x0001)

#define PMU_READ_VCC_DET_IF               ((uint32_t)0x0001)
#define PMU_VCC_DET_IF_SHIFT              (0)

/*
* @brief  PMU_STATE
*/
#define PMU_READ_VCC_STATE               ((uint32_t)0x0001)
#define PMU_VCC_STATE_SHIFT              (0)
#define PMU_VCC_GREATER_VCCLVL           ((uint32_t)0x0001)
#define PMU_VCC_LESS_VCCLVL              ((uint32_t)0x0000)

/*
* @brief  PMU_WAKE_UP_FLAG
*/
#define PMU_WAKEIF_PMUWKIF               ((uint32_t)(0x0001<<0))
#define PMU_WAKEIF_INT0WKIF              ((uint32_t)(0x0001<<2))
#define PMU_WAKEIF_INT1WKIF              ((uint32_t)(0x0001<<3))
#define PMU_WAKEIF_INT2WKIF              ((uint32_t)(0x0001<<4))
#define PMU_WAKEIF_INT3WKIF              ((uint32_t)(0x0001<<5))
#define PMU_WAKEIF_INT4WKIF              ((uint32_t)(0x0001<<6))
#define PMU_WAKEIF_INT5WKIF              ((uint32_t)(0x0001<<7))
#define PMU_WAKEIF_INT6WKIF              ((uint32_t)(0x0001<<8))
#define PMU_WAKEIF_RX0WKIF               ((uint32_t)(0x0001<<9))
#define PMU_WAKEIF_RX1WKIF               ((uint32_t)(0x0001<<10))
#define PMU_WAKEIF_RTCWKIF               ((uint32_t)(0x0001<<20))
#define PMU_WAKEIF_INT7WKIF              ((uint32_t)(0x0001<<28))
#define PMU_WAKEIF_INT8WKIF              ((uint32_t)(0x0001<<29))
#define PMU_WAKEIF_INT9WKIF              ((uint32_t)(0x0001<<30))

void PMU_DeInit(PMU_TypeDef* PMUx);
void PMU_Set_LDOSLEEP(uint16_t ldo);
uint32_t PMU_Get_LDOSLEEP(void);

void PMU_Set_VCCLVL(uint16_t vcclvl);
uint32_t PMU_Get_VCCLVL(void);

void PMU_Set_TimeSharingDetection(uint16_t time,uint16_t period);

uint32_t PMU_Get_Time(void);
uint32_t PMU_Get_Period(void);

void PMU_Enable_VCCIT(void);
void PMU_Disable_VCCIT(void);
uint32_t PMU_Get_VCCIT(void);

uint32_t PMU_Get_FlagVCC(void);
void PMU_Clear_FlagVCC(void);

uint32_t PMU_Get_VCCState(void);

void PMU_Set_PDLDO(uint32_t status);

uint32_t PMU_Get_WakeupFlag(uint32_t wakeflag);

void Enter_Sleep1(void);
void Enter_Sleep0(void);
void Enter_Hold(void);

#ifdef __cplusplus
}
#endif

#endif 
