/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_timer.h
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_TIMER_H__
#define __rx32s11xx_TIMER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s11xx.h"

/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of Timer Module
**********************************************************************************************************
*/
/************************  Bit definition for TMRCON register of TMR_TypeDef  ************************/
#define  TMR_TMRCON                              ((uint32_t)0x80ff)

#define  TMR_TMRCON_CNTEN                        ((uint32_t)0x0001)

#define  TMR_TMRCON_MODE                         ((uint32_t)0x0006)
#define  TMR_TMRCON_MODE_EVENT                   ((uint32_t)0x0000)
#define  TMR_TMRCON_MODE_PWM                     ((uint32_t)0x0002)
#define  TMR_TMRCON_MODE_CAPTURE                 ((uint32_t)0x0004)
#define  TMR_TMRCON_MODE_PERIOD                  ((uint32_t)0x0006)
	
#define  TMR_TMRCON_CCMODE                       ((uint32_t)0x0008)
#define  TMR_TMRCON_CCMODE_RISE                  ((uint32_t)0x0000)
#define  TMR_TMRCON_CCMODE_FALL                  ((uint32_t)0x0008)

#define  TMR_TMRCON_PWMC                         ((uint32_t)0x0030)
#define  TMR_TMRCON_PWMC_UP                      ((uint32_t)0x0000)
#define  TMR_TMRCON_PWMC_DOWM                    ((uint32_t)0x0010)
#define  TMR_TMRCON_PWMC_CENTER                  ((uint32_t)0x0020)

#define  TMR_TMRCON_POLARITY                     ((uint32_t)0x0040)           
#define  TMR_TMRCON_POLARITY_POSITIVE            ((uint32_t)0x0000)           
#define  TMR_TMRCON_POLARITY_NEGATIVE            ((uint32_t)0x0040)           

#define  TMR_TMRCON_RESERVED                     ((uint32_t)0x0080)

/************************  Bit definition for TMRDIV register of TMR_TypeDef  ************************/
#define  TMR_TMRDIV                            ((uint32_t)0xffff)          

/************************  Bit definition for TMRPRD register of TMR_TypeDef  ************************/
#define  TMR_TMRPRD                            ((uint32_t)0xffff)          

/************************  Bit definition for TMRCAP register of TMR_TypeDef  ************************/
#define  TMR_TMRCAP                            ((uint32_t)0xffff)          

/************************  Bit definition for TMRCNT register of TMR_TypeDef  ************************/
#define  TMR_TMRCNT                            ((uint32_t)0xffff)          

/************************  Bit definition for TMRCMP register of TMR_TypeDef  ************************/
#define  TMR_TMRCMP                            ((uint32_t)0xffff)          

/************************  Bit definition for TMRIE register of TMR_TypeDef   ************************/
#define  TMR_TMRIE                               ((uint32_t)0x000f)          
#define  TMR_TMRIE_PRDIE                         ((uint32_t)0x0001) 
#define  TMR_TMRIE_CAPIE                         ((uint32_t)0x0002)
#define  TMR_TMRIE_CMPIE                         ((uint32_t)0x0004) 
#define  TMR_TMRIE_ACIE                          ((uint32_t)0x0008)  
/************************  Bit definition for TMRIF register of TMR_TypeDef   ************************/
#define  TMR_TMRIF                               ((uint32_t)0x000f)           
#define  TMR_TMRIF_PRDIF                         ((uint32_t)0x0001)           
#define  TMR_TMRIF_CAPIF                         ((uint32_t)0x0002)          
#define  TMR_TMRIF_CMPIF                         ((uint32_t)0x0004)          
#define  TMR_TMRIF_ACIF                          ((uint32_t)0x0008)          

/************************  RT_TMR_InitTypeDef   ************************/
typedef struct
{
	uint32_t Mode;        // TMR_TMRCON_MODE_PWM   / TMR_TMRCON_MODE_PERIOD
	uint32_t ICPolarity;  //TMR_TMRCON_CCMODE_RISE / TMR_TMRCON_CCMODE_FALL
	uint32_t Polarity;    //TMR_TMRCON_POLARITY_POSITIVE / TMR_TMRCON_POLARITY_NEGATIVE
	uint32_t CounterDIR;  //TMR_TMRCON_PWMC_UP    / TMR_TMRCON_PWMC_DOWM  / TMR_TMRCON_PWMC_CENTER
	uint32_t TMRPRD;
	uint32_t TMRCMP;
	uint32_t TMRDIV;
} TMR_InitTypeDef;   

/************************   Function   ************************/
void TMR_DeInit(TMR_TypeDef* TMRx);
void TMRx_StructInit(TMR_InitTypeDef* TMR_InitStruct);
void TMR_Init(TMR_TypeDef* TMRx,TMR_InitTypeDef* TMR_InitStruct);
void TMR_ICInit(TMR_TypeDef* TMRx,TMR_InitTypeDef* TMR_ICInitStruct);

void TMR_Enable_CNT(TMR_TypeDef* TMRx );
void TMR_Disable_CNT(TMR_TypeDef* TMRx );

void TMR_Set_MODE(TMR_TypeDef* TMRx ,uint32_t val);
void TMR_Set_CCMODE(TMR_TypeDef* TMRx ,uint32_t val);
void TMR_Set_PWMMODE(TMR_TypeDef* TMRx ,uint32_t val);
void TMR_Set_PWMPolarity(TMR_TypeDef* TMRx ,uint32_t val);

void TMR_Set_Prescaler(TMR_TypeDef* TMRx ,uint16_t val);
void TMR_Set_Period(TMR_TypeDef* TMRx ,uint16_t val);
uint16_t TMR_Get_Capture(TMR_TypeDef* TMRx);
void TMR_Set_Compare(TMR_TypeDef* TMRx ,uint16_t val);
uint16_t TMR_Get_Compare(TMR_TypeDef* TMRx);
void TMR_Set_Count(TMR_TypeDef* TMRx ,uint16_t val);
uint16_t TMR_Get_Count(TMR_TypeDef* TMRx);

void TMR_Enable_IT(TMR_TypeDef* TMRx,uint32_t ITFLAG);
void TMR_Disable_IT(TMR_TypeDef* TMRx,uint32_t ITFLAG);


uint32_t TMR_Get_Flag(TMR_TypeDef* TMRx,uint16_t TMR_FLAG);
void TMR_Clear_Flag(TMR_TypeDef* TMRx, uint32_t TMR_FLAG);

#ifdef __cplusplus
}
#endif

#endif 
