/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_uart.h
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#ifndef __rx32s11xx_UART_H__
#define __rx32s11xx_UART_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "rx32s11xx.h"

/*
**********************************************************************************************************
*                          Peripheral Registers_Bits_Definition of UART Module
**********************************************************************************************************
*/
/************************  Bit definition for UARTCON register of UART_TypeDef ***********************/
#define UART_UARTCON                   ((uint32_t)0x81ff)
#define UART_UARTCON_TXEN              ((uint32_t)0x0001)
#define UART_UARTCON_RXEN              ((uint32_t)0x0002)

#define UART_UARTCON_TXIE              ((uint32_t)0x0004)

#define UART_UARTCON_RXIE              ((uint32_t)0x0008)

#define UART_UARTCON_PRDIE             ((uint32_t)0x8000)

#define UART_UARTCON_PARITYEN          ((uint32_t)0x0010)
#define UART_UARTCON_PARITY_Enable     ((uint32_t)0x0010)
#define UART_UARTCON_PARITY_Disable    ((uint32_t)0x0000)

#define UART_UARTCON_PARITYSEL         ((uint32_t)0x0060)
#define UART_UARTCON_PARITYSEL_1       ((uint32_t)0x0060)
#define UART_UARTCON_PARITYSEL_0       ((uint32_t)0x0000)
#define UART_UARTCON_PARITYSEL_ODD     ((uint32_t)0x0020)
#define UART_UARTCON_PARITYSEL_EVEN    ((uint32_t)0x0040)

#define UART_UARTCON_LENSEL            ((uint32_t)0x0080)
#define UART_UARTCON_LENSEL_7Bit       ((uint32_t)0x0080)
#define UART_UARTCON_LENSEL_8Bit       ((uint32_t)0x0000)

#define UART_UARTCON_STOPSEL           ((uint32_t)0x0100)
#define UART_UARTCON_STOPSEL_2Bit      ((uint32_t)0x0100)
#define UART_UARTCON_STOPSEL_1Bit      ((uint32_t)0x0000)

/*************************  Bit definition for SREL register of UART_TypeDef *************************/
#define UART_SREL                      ((uint32_t)0xffff)

/*************************  Bit definition for SBUF register of UART_TypeDef *************************/
#define UART_SBUF                      ((uint32_t)0x00ff)

/************************  Bit definition for UARTSTA register of UART_TypeDef ***********************/
#define UART_UARTSTA                   ((uint32_t)0x000f) /*!<uart Status         */

#define UART_UARTSTA_TXIF              ((uint32_t)0x0001)
#define UART_UARTSTA_RXIF              ((uint32_t)0x0002)
#define UART_UARTSTA_PARITY            ((uint32_t)0x0004)
#define UART_UARTSTA_PRDIF             ((uint32_t)0x0008)

typedef struct
{
    uint32_t Baudrate;
    uint32_t StopSel;
    uint32_t LenSel;
	  uint32_t ParityEN; 
    uint32_t Paritysel;      
} UART_InitTypeDef;

/*
*********************************************************************************************************
*                                           Macro definition
*********************************************************************************************************
*/
void UART_DeInit(UART_TypeDef* UARTx);
void UART_StructInit(UART_InitTypeDef *UART_InitStruct);
void UART_Init(UART_TypeDef *UARTx, UART_InitTypeDef *UART_InitStruct);

void UART_Enable_IT(UART_TypeDef *UARTx, uint32_t ITFLAG);
void UART_Disable_IT(UART_TypeDef *UARTx, uint32_t ITFLAG);

void UART_Enable(UART_TypeDef *UARTx,uint32_t val);
void UART_Disable(UART_TypeDef *UARTx,uint32_t val);

void UART_Set_BaudRate(UART_TypeDef *UARTx, uint32_t fsys, uint32_t baudrate);

uint32_t UART_Get_Flag(UART_TypeDef *UARTx, uint16_t UART_FLAG);
void UART_Clear_Flag(UART_TypeDef *UARTx, uint32_t UART_FLAG);

void UART_Send_Data(UART_TypeDef *UARTx, uint8_t data);
uint8_t UART_Receive_Data(UART_TypeDef *UARTx);

void UART_Send_Buf(UART_TypeDef *UARTx, uint8_t *wbuf, uint8_t len);

#ifdef __cplusplus
}
#endif

#endif /* __rx32s11xx_UART_H__ */
