/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_adc.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s11xx
*********************************************************************************************************
*/



#include "rx32s11xx_adc.h"

/**
  * @brief  Deinitializes the SADCx peripheral registers to their default reset values.
  * @param  SADCx: where x can be 0, 1 to select the 
  *         SADC peripheral.
  * @retval None
  */
void SADC_DeInit(SADC_TypeDef* SADCx)
{
  SADCx->CALDAT0 = 0;
  SADCx->CALDAT1 = 0;
  SADCx->CALDAT2 = 0;
  SADCx->CALDAT3 = 0;
  SADCx->SADC0DAT = 0;
  SADCx->SADC1DAT = 0;
  SADCx->SADC2DAT = 0;
  SADCx->SADC3DAT = 0;
  SADCx->SADC4DAT = 0;
  SADCx->SADC5DAT = 0;
  SADCx->SADC6DAT = 0;
  SADCx->SADC7DAT = 0;
  SADCx->SADCCLKCFG = 0;
  SADCx->SADCCON = 0;
  SADCx->SADCDAT = 0;
  SADCx->SADCIE = 0;
  SADCx->SADCIF = 0;
  SADCx->SADCSAMP2 = 0;
  SADCx->SADCSAMP = 0;
  SADCx->SADCSEQCFG = 0;
  SADCx->SADCSTR = 0;
  SADCx->SADCTRGLEN = 0;
}
/**
  * @brief  Fills each SADC_InitStruct member with its default value.
  * @param  SADC_InitStruct: pointer to a SADC_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void SADC_StructInit(SADC_InitTypeDef *SADC_InitStruct)
{
  SADC_InitStruct->TriggerSource = SADC_ADTRIG_SADCSTR;
  SADC_InitStruct->ContMode = SADC_CONV_SINGLE;
  SADC_InitStruct->Discen = DISABLE;
  SADC_InitStruct->Prescaler = SADC_SADCCLK_DIV3;
  SADC_InitStruct->ScanMode = SADC_SADCSCAN_DISABLE;
  SADC_InitStruct->State = ENABLE;
}

/**
  * @function  SADC_Init
  * @brief  Initial the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  State:
  *         @arg ENABLE
  *         @arg DISABLE
  * @param  TriggerSource:
  *         @ref SADC_ADTRIG_SADCSTR
  *         @ref SADC_ADTRIG_RTC2
  *         @ref SADC_ADTRIG_INT3
  *         @ref SADC_ADTRIG_INT7
  *         @ref SADC_ADTRIG_TMR0
  *         @ref SADC_ADTRIG_TMR1
  *         @ref SADC_ADTRIG_TIM8_CH4
  *         @ref SADC_ADTRIG_TIM8_CH5
  *         @ref SADC_ADTRIG_TIM8_CH1
  *         @ref SADC_ADTRIG_TIM8_CH2
  *         @ref SADC_ADTRIG_TIM8_CH3
  *         @ref SADC_ADTRIG_TIM8_CH4_CH5
  * @param  ContMode:
  *         @arg SADC_CONV_CONTINUOUS
  *         @arg SADC_CONV_SINGLE
  * @param  ScanMode:
  *         @arg SADC_SADCSCAN_DISABLE
  *         @arg SADC_SADCSCAN_ENABLE
  * @param  Prescaler:
  *         @ref SADC_SADCCLK_DIV1
  *         @ref SADC_SADCCLK_DIV2
  *         @ref SADC_SADCCLK_DIV3
  *         @ref SADC_SADCCLK_DIV4
  *         @ref SADC_SADCCLK_DIV5
  *         @ref SADC_SADCCLK_DIV6
  *         @ref SADC_SADCCLK_DIV7
  *         @ref SADC_SADCCLK_DIV8
  * @param  Discen:
  *         @arg ENABLE
  *         @arg DISABLE
  * @retval None
  */
void SADC_Init(SADC_TypeDef* SADCx,SADC_InitTypeDef* SADC_InitStruct)
{
	SADC_Set_TriggerSource(SADCx, SADC_InitStruct->TriggerSource);       /* Set ADC trigger source */
	SADC_Set_ContMode(SADCx, SADC_InitStruct->ContMode);                 /* Set ADC cont mode */
	SADC_Set_ScanMode(SADCx, SADC_InitStruct->ScanMode);                 /* Set ADC scan mode */
  SADC_Set_ClkPrescaler(SADCx, SADC_InitStruct->Prescaler);            /* Set ADC clk Prescaler */
	if(SADC_InitStruct->Discen == ENABLE){
    SADC_Enable_DISCEN(SADC);                                          /* Enable DISCEN  */
	}
	if(SADC_InitStruct->Discen == DISABLE){
    SADC_Disable_DISCEN(SADC);                                         /* Disable DISCEN  */
	}	
  if(SADC_InitStruct->State == ENABLE){
    SADC_Enable(SADC);                                            /* Enable ADC  */
	}
	if(SADC_InitStruct->State == DISABLE){
    SADC_Disable(SADC);                                           /* Disable ADC  */
	}	
}

/**
  * @function  SADC_RankInit
  * @brief  Initial Rank of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  Length:
  *         @ref SADC_SEQLENG_1
  *         @ref SADC_SEQLENG_2
  *         @ref SADC_SEQLENG_3
  *         @ref SADC_SEQLENG_4
  *         @ref SADC_SEQLENG_5
  *         @ref SADC_SEQLENG_6
  *         @ref SADC_SEQLENG_7
  *         @ref SADC_SEQLENG_8
  * @param  Rank[x].Channel:
  *         @ref SADC_CHANNEL0       
  *         @ref SADC_CHANNEL1        
  *         @ref SADC_CHANNEL2        
  *         @ref SADC_CHANNEL3      
  *         @ref SADC_CHANNEL4       
  *         @ref SADC_CHANNEL5       
  *         @ref SADC_CHANNEL6       
  *         @ref SADC_CHANNEL7        
  *         @ref SADC_CHANNEL8       
  *         @ref SADC_CHANNEL9       
  *         @ref SADC_CHANNEL10  
  *         @ref SADC_CHANNEL11 or SADC_CHANNELVBG         
  *         @ref SADC_CHANNEL12 or SADC_CHANNELPGA0    
  *         @ref SADC_CHANNEL13 or SADC_CHANNELPGA1    
  *         @ref SADC_CHANNEL14 or SADC_CHANNELPGA2  
  * @param  Rank[x].SamplingTime:
  *         @ref SADC_SAMPLINGTIME_2CYCLES       
  *         @ref SADC_SAMPLINGTIME_4CYCLES        
  *         @ref SADC_SAMPLINGTIME_8CYCLES        
  *         @ref SADC_SAMPLINGTIME_16CYCLES      
  *         @ref SADC_SAMPLINGTIME_32YCLES       
  *         @ref SADC_SAMPLINGTIME_64CYCLES       
  *         @ref SADC_SAMPLINGTIME_128CYCLES       
  *         @ref SADC_SAMPLINGTIME_256CYCLES  
  * @retval None
  */
void SADC_RankInit(SADC_TypeDef* SADCx,SADC_RankInitTypeDef* SADC_RankInitStruct)
{
	int i,rank;
	rank    = SADC_RankInitStruct->Length >> 8 ;
  SADC_Set_SequencerLength(SADCx, SADC_RankInitStruct->Length);                                                           /* Set ADC scan Length */

	for(i=0; i<=rank; i++){
    SADC_Set_SequencerRanks(SADC,(i*4), SADC_RankInitStruct->Rank[i].Channel);                                            /* Set sequence rank channlel */
		
    SADC_Set_ChannelSamplingTime(SADC,SADC_RankInitStruct->Rank[i].Channel, SADC_RankInitStruct->Rank[i].SamplingTime);   /* Set channel sampling time */
	}
}

/**
  * @function  SADC_Enable
  * @brief  Enable SADC of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADCON);
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADCEN);
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADC_TRIG_EN);
}

/**
  * @function  SADC_Disable
  * @brief  Disable ADCON of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADCON);
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADCEN);
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADC_TRIG_EN);
}

/**
  * @function  SADC_Enable_SADCON
  * @brief  Enable SADCON of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_SADCON(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADCON);
}

/**
  * @function  SADC_Disable_SADCON
  * @brief  Disable ADCON of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_SADCON(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADCON);
}

/**
  * @function  SADC_Get_SADCON
  * @brief  Get SADCON state of the selected ADC 
  * @param  SADCx:
            @arg SADC Instance
  * @retval 0:SADCON is disabled, 1:SADCON is enabled
  */
uint32_t SADC_Get_SADCON(SADC_TypeDef* SADCx)
{
  return (READ_BIT(SADCx->SADCCON, SADC_SADCCON_SADCON) == (SADC_SADCCON_SADCON));
}

/**
  * @function  SADC_Enable_TRIG
  * @brief  Enable TRIG_EN of the selected ADC 
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_TRIG(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADC_TRIG_EN);
}

/**
  * @function  SADC_Disable_TRIG
  * @brief  Disable ADC TRIG_EN of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_TRIG(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADC_TRIG_EN);
}

/**
  * @function  SADC_Set_ScanMode
  * @brief  Set sequencers scan mode of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  ScanMode:
  *         @arg SADC_SADCSCAN_DISABLE
  *         @arg SADC_SADCSCAN_ENABLE
  * @retval None
  */
void SADC_Set_ScanMode(SADC_TypeDef* SADCx, uint32_t ScanMode)
{
  MODIFY_REG(SADCx->SADCCON, SADC_SADCCON_SADCSCAN, ScanMode);
}

/**
  * @function  SADC_Set_ContMode
  * @brief  Set continuous conversion mode of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  ContMode:
  *         @arg SADC_CONV_CONTINUOUS
  *         @arg SADC_CONV_SINGLE
  * @retval None
  */
void SADC_Set_ContMode(SADC_TypeDef* SADCx, uint32_t ContMode)
{
  MODIFY_REG(SADCx->SADCCON, SADC_SADCCON_SADCCONT, ContMode);
}

/**
  * @function  SADC_Enable_SADCEN
  * @brief  Enable SADCEN of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_SADCEN(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_SADCEN);
}

/**
  * @function  SADC_Disable_SADCEN
  * @brief  Disable SADCEN of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_SADCEN(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_SADCEN);
}

/**
  * @function  SADC_Get_SADCEN
  * @brief  Get SADCEN state of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval 0:SADCEN is disalbe, 1:SADCEN is enable
  */
uint32_t SADC_Get_SADCEN(SADC_TypeDef* SADCx)
{
  return (READ_BIT(SADCx->SADCCON, SADC_SADCCON_SADCEN) == (SADC_SADCCON_SADCEN));
}

/**
  * @function  SADC_Enable_DISCEN
  * @brief  Enable DISCEN of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_DISCEN(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCCON, SADC_SADCCON_DISCEN);
}

/**
  * @function  SADC_Disable_DISCEN
  * @brief  Disable DISCEN of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_DISCEN(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCCON, SADC_SADCCON_DISCEN);
}

/**
  * @function  SADC_Get_DISCEN
  * @brief  Get DISCEN state of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval 0:DISCEN is disalbe, 1:DISCEN is enable
  */
uint32_t SADC_Get_DISCEN(SADC_TypeDef* SADCx)
{
  return (READ_BIT(SADCx->SADCCON, SADC_SADCCON_DISCEN) == (SADC_SADCCON_DISCEN));
}

/**
  * @function  SADC_Start
  * @brief  Start STRS conversion of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Start(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCSTR, SADC_SADCSTR);
}

/**
  * @function  SADC_Set_ClkPrescaler
  * @brief  Set SADC clk Prescaler of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  Prescaler:
  *         @ref SADC_SADCCLK_DIV1
  *         @ref SADC_SADCCLK_DIV2
  *         @ref SADC_SADCCLK_DIV3
  *         @ref SADC_SADCCLK_DIV4
  *         @ref SADC_SADCCLK_DIV5
  *         @ref SADC_SADCCLK_DIV6
  *         @ref SADC_SADCCLK_DIV7
  *         @ref SADC_SADCCLK_DIV8
  * @retval None
  */
void SADC_Set_ClkPrescaler(SADC_TypeDef* SADCx, uint32_t Prescaler)
{
  MODIFY_REG(SADCx->SADCCLKCFG, SADC_SADCCLKCFG_SADCCLKDIV, Prescaler);
}

/**
  * @function  SADC_Enable_IT
  * @brief  Enable IE of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  IE
            @arg SADC_SADCIE_SIGIE
            @arg SADC_SADCIE_SEQIE
  * @retval None
  */
void SADC_Enable_IT(SADC_TypeDef* SADCx,uint32_t IE)
{
  SET_BIT(SADCx->SADCIE, IE);
}

/**
  * @function  SADC_Disable_IT
  * @brief  Enable IE of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  IE
            @arg SADC_SADCIE_SIGIE
            @arg SADC_SADCIE_SEQIE
  * @retval None
  */
void SADC_Disable_IT(SADC_TypeDef* SADCx,uint32_t IE)
{
  CLEAR_BIT(SADCx->SADCIE, IE);
}

/**
  * @function  SADC_Get_IT
  * @brief  Get IT state of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  IE
            @arg SADC_SADCIE_SIGIE
            @arg SADC_SADCIE_SEQIE
  * @retval 0:IT is disabled, 1:IT is enabled
  */
uint32_t SADC_Get_IT(SADC_TypeDef* SADCx,uint32_t IE)
{
  return (READ_BIT(SADCx->SADCIE, IE) == (IE));
}

/**
  * @function  SADC_Get_Flag
  * @brief  Get IF state of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  IF
            @arg SADC_SADCIF_SIGIF
            @arg SADC_SADCIF_SEQIF
  * @retval 0:IF is inactive, 1:IF is active
  */
uint32_t SADC_Get_Flag(SADC_TypeDef* SADCx,uint32_t IF)
{
  return (READ_BIT(SADCx->SADCIF, IF) == (IF));
}

/**
  * @function  SADC_Clear_Flag
  * @brief  Clear SIG IF of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  IF
            @arg SADC_SADCIF_SIGIF
            @arg SADC_SADCIF_SEQIF
  * @retval None
  */
void SADC_Clear_Flag(SADC_TypeDef* SADCx,uint32_t IF)
{
  CLEAR_BIT(SADCx->SADCIF, IF);
}

/**
  * @function  SADC_Set_ChannelSamplingTime
  * @brief  Set selected channel sampling time of the selected ADC
  *         channel 0-7  set ADCx->SADCSAMP
  *         channel 8-15 set ADCx->SADCSAMP2
  * @param  SADCx:
            @arg SADC Instance
  * @param  Channel:
  *         @ref SADC_CHANNEL0       
  *         @ref SADC_CHANNEL1        
  *         @ref SADC_CHANNEL2        
  *         @ref SADC_CHANNEL3      
  *         @ref SADC_CHANNEL4       
  *         @ref SADC_CHANNEL5       
  *         @ref SADC_CHANNEL6       
  *         @ref SADC_CHANNEL7        
  *         @ref SADC_CHANNEL8       
  *         @ref SADC_CHANNEL9       
  *         @ref SADC_CHANNEL10    
  *         @ref SADC_CHANNEL11 or SADC_CHANNELVBG         
  *         @ref SADC_CHANNEL12 or SADC_CHANNELPGA0    
  *         @ref SADC_CHANNEL13 or SADC_CHANNELPGA1    
  *         @ref SADC_CHANNEL14 or SADC_CHANNELPGA2     
  * @param  SamplingTime:
  *         @ref SADC_SAMPLINGTIME_2CYCLES       
  *         @ref SADC_SAMPLINGTIME_4CYCLES        
  *         @ref SADC_SAMPLINGTIME_8CYCLES        
  *         @ref SADC_SAMPLINGTIME_16CYCLES      
  *         @ref SADC_SAMPLINGTIME_32YCLES       
  *         @ref SADC_SAMPLINGTIME_64CYCLES       
  *         @ref SADC_SAMPLINGTIME_128CYCLES       
  *         @ref SADC_SAMPLINGTIME_256CYCLES            
  * @retval None
  */
void SADC_Set_ChannelSamplingTime(SADC_TypeDef* SADCx, uint32_t Channel, uint32_t SamplingTime)
{
	if(Channel < 8){
  MODIFY_REG(SADCx->SADCSAMP, 
             SADC_SAMP_NUMBER_MASK << (Channel*4),
             (SamplingTime & SADC_SAMP_NUMBER_MASK) << (Channel*4));
	}
	else{
  MODIFY_REG(SADCx->SADCSAMP2, 
             SADC_SAMP_NUMBER_MASK << ((Channel-8)*4),
             (SamplingTime & SADC_SAMP_NUMBER_MASK) << ((Channel-8)*4));
	}
}

/**
  * @function  SADC_Enable_TRIG_CH4
  * @brief  Enable the TRIG selected CH4 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_TRIG_CH4(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH4_EN);
}

/**
  * @function  SADC_Disable_TRIG_CH4
  * @brief  Disable the TRIG selected CH4
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_TRIG_CH4(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH4_EN);
}

/**
  * @function  SADC_Enable_TRIG_CH5
  * @brief  Enable the TRIG selected CH5 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Enable_TRIG_CH5(SADC_TypeDef* SADCx)
{
  SET_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH5_EN);
}

/**
  * @function  SADC_Disable_TRIG_CH5
  * @brief  Disable the TRIG selected CH5 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval None
  */
void SADC_Disable_TRIG_CH5(SADC_TypeDef* SADCx)
{
  CLEAR_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH5_EN);
}

/**
  * @function  SADC_Set_TriggerSource
  * @brief  Set SADC Trigger Source of the selected ADC
  *         select TIM8_CH4 should Enable ADC_SADCTRGLEN_CH4_EN
  *         select TIM8_CH5 should Enable ADC_SADCTRGLEN_CH5_EN
  * @param  SADCx:
            @arg SADC Instance
  * @param  TriggerSource:
  *         @ref SADC_ADTRIG_SADCSTR
  *         @ref SADC_ADTRIG_RTC2
  *         @ref SADC_ADTRIG_INT3
  *         @ref SADC_ADTRIG_INT7
  *         @ref SADC_ADTRIG_TMR0
  *         @ref SADC_ADTRIG_TMR1
  *         @ref SADC_ADTRIG_TIM8_CH4
  *         @ref SADC_ADTRIG_TIM8_CH5
  *         @ref SADC_ADTRIG_TIM8_CH1
  *         @ref SADC_ADTRIG_TIM8_CH2
  *         @ref SADC_ADTRIG_TIM8_CH3
  *         @ref SADC_ADTRIG_TIM8_CH4_CH5
  * @retval None
  */
void SADC_Set_TriggerSource(SADC_TypeDef* SADCx, uint32_t TriggerSource)
{
  MODIFY_REG(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_ADTRIG, (TriggerSource & SADC_SADCTRGLEN_ADTRIG));
	if(TriggerSource == SADC_ADTRIG_TIM8_CH4){
    SET_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH4_EN);	
	}
	if(TriggerSource == SADC_ADTRIG_TIM8_CH5){
    SET_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH5_EN);	
	}
	if(TriggerSource == SADC_ADTRIG_TIM8_CH4_CH5){
    SET_BIT(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_CH4_EN | SADC_SADCTRGLEN_CH5_EN);	
	}
}

/**
  * @function  SADC_Set_SequencerLength
  * @brief  Set SADC Sequencer Length of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  Length:
  *         @ref SADC_SEQLENG_1
  *         @ref SADC_SEQLENG_2
  *         @ref SADC_SEQLENG_3
  *         @ref SADC_SEQLENG_4
  *         @ref SADC_SEQLENG_5
  *         @ref SADC_SEQLENG_6
  *         @ref SADC_SEQLENG_7
  *         @ref SADC_SEQLENG_8
  * @retval None
  */
void SADC_Set_SequencerLength(SADC_TypeDef* SADCx, uint32_t Length)
{
  MODIFY_REG(SADCx->SADCTRGLEN, SADC_SADCTRGLEN_SEQLENG, Length);
}

/**
  * @function  SADC_Set_SequencerRanks
  * @brief  Set ADC group regular sequence channel of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  Rank:
  *         @ref SADC_SEQCFG_SEQ0_SEL
  *         @ref SADC_SEQCFG_SEQ1_SEL
  *         @ref SADC_SEQCFG_SEQ2_SEL
  *         @ref SADC_SEQCFG_SEQ3_SEL 
  *         @ref SADC_SEQCFG_SEQ4_SEL
  *         @ref SADC_SEQCFG_SEQ5_SEL
  *         @ref SADC_SEQCFG_SEQ6_SEL
  *         @ref SADC_SEQCFG_SEQ7_SEL
  * @param  Channel:
  *         @ref SADC_CHANNEL0       
  *         @ref SADC_CHANNEL1        
  *         @ref SADC_CHANNEL2        
  *         @ref SADC_CHANNEL3      
  *         @ref SADC_CHANNEL4       
  *         @ref SADC_CHANNEL5       
  *         @ref SADC_CHANNEL6       
  *         @ref SADC_CHANNEL7        
  *         @ref SADC_CHANNEL8       
  *         @ref SADC_CHANNEL9       
  *         @ref SADC_CHANNEL10    
  *         @ref SADC_CHANNEL11 or SADC_CHANNELVBG         
  *         @ref SADC_CHANNEL12 or SADC_CHANNELPGA0    
  *         @ref SADC_CHANNEL13 or SADC_CHANNELPGA1    
  *         @ref SADC_CHANNEL14 or SADC_CHANNELPGA2      
  * @retval None
  */
void SADC_Set_SequencerRanks(SADC_TypeDef* SADCx, uint32_t Rank, uint32_t Channel)
{
  MODIFY_REG(SADCx->SADCSEQCFG, 
             SADC_SEQ_NUMBER_MASK << Rank,
             (Channel & SADC_SEQ_NUMBER_MASK) << Rank);
}

/**
  * @function  SADC_Read_SADCDAT
  * @brief  Get offest of SADCDAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADCDAT
  */
uint32_t SADC_Read_SADCDAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADCDAT));
}

/**
  * @function  SADC_Read_SADC0DAT
  * @brief  Get offest of SADC0DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC0DAT
  */
uint32_t SADC_Read_SADC0DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC0DAT));
}

/**
  * @function  SADC_Read_SADC1DAT
  * @brief  Get offest of SADC1DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC1DAT
  */
uint32_t SADC_Read_SADC1DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC1DAT));
}

/**
  * @function  SADC_Read_SADC2DAT
  * @brief  Get offest of SADC2DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC2DAT
  */
uint32_t SADC_Read_SADC2DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC2DAT));
}

/**
  * @function  SADC_Read_SADC3DAT
  * @brief  Get offest of SADC3DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC3DAT
  */
uint32_t SADC_Read_SADC3DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC3DAT));
}

/**
  * @function  SADC_Read_SADC4DAT
  * @brief  Get offest of SADC4DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC4DAT
  */
uint32_t SADC_Read_SADC4DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC4DAT));
}

/**
  * @function  SADC_Read_SADC5DAT
  * @brief  Get offest of SADC5DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC5DAT
  */
uint32_t SADC_Read_SADC5DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC5DAT));
}

/**
  * @function  SADC_Read_SADC6DAT
  * @brief  Get offest of SADC6DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC6DAT
  */
uint32_t SADC_Read_SADC6DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC6DAT));
}

/**
  * @function  SADC_Read_SADC7DAT
  * @brief  Get offest of SADC7DAT of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of SADC7DAT
  */
uint32_t SADC_Read_SADC7DAT(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->SADC7DAT));
}

/**
  * @function  SADC_Set_CALDAT0
  * @brief  Set SADC CALDAT0 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void SADC_Set_CALDAT0(SADC_TypeDef* SADCx, uint32_t offest)
{
  MODIFY_REG(SADCx->CALDAT0, SADC_CALDAT0, offest);
}

/**
  * @function  SADC_Read_CALDAT0
  * @brief  Get CALDAT0 offest of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of CALDAT0.
  */
uint32_t SADC_Read_CALDAT0(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->CALDAT0));
}

/**
  * @function  SADC_Set_CALDAT1
  * @brief  Set SADC CALDAT1 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void SADC_Set_CALDAT1(SADC_TypeDef* SADCx, uint32_t offest)
{
  MODIFY_REG(SADCx->CALDAT1, SADC_CALDAT1, offest);
}

/**
  * @function  SADC_Read_CALDAT1
  * @brief  Get CALDAT1 offest of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of CALDAT1.
  */
uint32_t SADC_Read_CALDAT1(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->CALDAT1));
}

/**
  * @function  SADC_Set_CALDAT2
  * @brief  Set SADC CALDAT2 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void SADC_Set_CALDAT2(SADC_TypeDef* SADCx, uint32_t offest)
{
  MODIFY_REG(SADCx->CALDAT2, SADC_CALDAT2, offest);
}

/**
  * @function  SADC_Read_CALDAT2
  * @brief  Get CALDAT2 offest of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of CALDAT2.
  */
uint32_t SADC_Read_CALDAT2(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->CALDAT2));
}

/**
  * @function  SADC_Set_CALDAT3
  * @brief  Set SADC CALDAT3 of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @param  offest:
  *         @ref offest
  * @retval None
  */
void SADC_Set_CALDAT3(SADC_TypeDef* SADCx, uint32_t offest)
{
  MODIFY_REG(SADCx->CALDAT3, SADC_CALDAT3, offest);
}

/**
  * @function  SADC_Read_CALDAT3
  * @brief  Get CALDAT3 offest of the selected ADC
  * @param  SADCx:
            @arg SADC Instance
  * @retval offest of CALDAT3.
  */
uint32_t SADC_Read_CALDAT3(SADC_TypeDef* SADCx)
{
  return (READ_REG(SADCx->CALDAT3));
}
