/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_cmp.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_cmp.h"

/**
  * @brief  Deinitializes the CMPx peripheral registers to their default reset values.
  * @param  CMPx: where x can be 4, 5 to select the 
  *         CMP peripheral.
  * @retval None
  */
void CMP_DeInit(CMP_TypeDef* CMPx)
{
  CMPx->CAL = 0;
  CMPx->CR1 = 0;
  CMPx->CR2 = 0;
  CMPx->PLC = 0;
  CMPx->PLS = 0;
}
/**
  * @brief  Fills each CMP_InitStruct member with its default value.
  * @param  CMP_InitStruct: pointer to a CMP_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void CMP_StructInit(CMP_InitTypeDef *CMP_InitStruct)
{
  CMP_InitStruct->CRV = CMP_CR2_CRV_SEL_16_20;
  CMP_InitStruct->CRV_En = DISABLE;
  CMP_InitStruct->CRV_Src = CMP_CR2_CRV_SRC_AVCC;
  CMP_InitStruct->Fhyst = CMP_CR1_FHYST_0mV;
  CMP_InitStruct->INM = CMP_CR2_INM_SEL_PC4;;
  CMP_InitStruct->INP = CMP_CR2_INP_SEL_PC3;
  CMP_InitStruct->Mode =CMP_CR1_MODE_High;
  CMP_InitStruct->OFLT = CMP_CR1_OFLT_1;
  CMP_InitStruct->Pol = CMP_CR1_POL_Positive;
  CMP_InitStruct->Rhyst= CMP_CR1_RHYST_0mV;
}

/**
***********************************************************************************
*@Function CMP_Init
*@brief    Initial of CMPx
*@param1   CMPx
*@param2  CMP_InitStruct
* 					@param Mode;
*									@arg CMP_CR1_MODE_Ultralow
*									@arg CMP_CR1_MODE_Low
*									@arg CMP_CR1_MODE_Medium
*									@arg CMP_CR1_MODE_High
* 					@param Pol;
*									@arg CMP_CR1_POL_Positive
*									@arg CMP_CR1_POL_Negative
* 					@param OFLT;
*									@arg CMP_CR1_OFLT_1
*									@arg CMP_CR1_OFLT_4
*									@arg CMP_CR1_OFLT_16
*									@arg CMP_CR1_OFLT_32
*									@arg CMP_CR1_OFLT_64
*									@arg CMP_CR1_OFLT_128
*									@arg CMP_CR1_OFLT_256
*									@arg CMP_CR1_OFLT_512
* 					@param Rhyst;
*									@arg CMP_CR1_RHYST_0mV
*									@arg CMP_CR1_RHYST_5mV
*									@arg CMP_CR1_RHYST_10mV
*									@arg CMP_CR1_RHYST_20mV
* 					@param Fhyst;
*									@arg CMP_CR1_FHYST_0mV
*									@arg CMP_CR1_FHYST_5mV
*									@arg CMP_CR1_FHYST_10mV
*									@arg CMP_CR1_FHYST_20mV
* 					@param CRV_Sel;
*									@arg CMP_CR2_CRV_SEL_1_20
*									@arg CMP_CR2_CRV_SEL_2_20
*									@arg CMP_CR2_CRV_SEL_3_20
*									@arg CMP_CR2_CRV_SEL_4_20
*									@arg CMP_CR2_CRV_SEL_5_20
*									@arg CMP_CR2_CRV_SEL_6_20
*									@arg CMP_CR2_CRV_SEL_7_20
*									@arg CMP_CR2_CRV_SEL_8_20
*									@arg CMP_CR2_CRV_SEL_9_20
*									@arg CMP_CR2_CRV_SEL_10_20
*									@arg CMP_CR2_CRV_SEL_11_20
*									@arg CMP_CR2_CRV_SEL_12_20
*									@arg CMP_CR2_CRV_SEL_13_20
*									@arg CMP_CR2_CRV_SEL_14_20
*									@arg CMP_CR2_CRV_SEL_15_20
*									@arg CMP_CR2_CRV_SEL_16_20
* 					@param CRV_En;
*									@arg ENABLE
*									@arg DISABLE
* 					@param CRV_Src;
*									@arg CMP_CR2_CRV_SRC_avcc
*									@arg CMP_CR2_CRV_SRC_vbgp
* 					@param INP_Sel;
*									@arg CMP_CR2_INP_SEL_PC3
*									@arg CMP_CR2_INP_SEL_PC0
*									@arg CMP_CR2_INP_SEL_PB7
*									@arg CMP_CR2_INP_SEL_PGA0_P
*									@arg CMP_CR2_INP_SEL_PGA1_P
*									@arg CMP_CR2_INP_SEL_PGA2_P
*									@arg CMP_CR2_INP_SEL_PB5
* 					@param INM_Sel;
*									@arg CMP_CR2_INM_SEL_CRV
*									@arg CMP_CR2_INM_SEL_PC4
*									@arg CMP_CR2_INM_SEL_PC1
*									@arg CMP_CR2_INM_SEL_PB8
*@retval None
***********************************************************************************
  */
void CMP_Init(CMP_TypeDef *CMPx, CMP_InitTypeDef *CMP_InitStruct)
{
  MODIFY_REG(CMPx->CR1, (CMP_CR1_MODE | CMP_CR1_POL | CMP_CR1_OFLT | CMP_CR1_RHYST | CMP_CR1_FHYST),\
							(CMP_InitStruct->Mode | CMP_InitStruct->Pol | CMP_InitStruct->OFLT | CMP_InitStruct->Rhyst | CMP_InitStruct->Fhyst));
	
  MODIFY_REG(CMPx->CR2, CMP_CR2_CRV_SEL | CMP_CR2_CRV_EN | CMP_CR2_CRV_SRC | CMP_CR2_INM_SEL | CMP_CR2_INP_SEL,\
							(CMP_InitStruct->CRV | (CMP_InitStruct->CRV_En << 4) | CMP_InitStruct->CRV_Src | CMP_InitStruct->INM | CMP_InitStruct->INP));
}

/**
***********************************************************************************
*@Function CMP_Polling_Init
*@brief    Initial of CMPx Polling
*@param1   CMPx
*@param2  CMP_Polling_InitStruct
* 					@param Polling_INP;
*									@arg CMP_PLC_INP_SEL_DISABLE
*									@arg CMP_PLC_INP_SEL_GPIO
*									@arg CMP_PLC_INP_SEL_PGA
* 					@param Polling_INM;
*									@arg CMP_PLC_INM_SEL_DISABLE
*									@arg CMP_PLC_INM_SEL_GPIO
* 					@param Polling_Period;
*									@arg CMP_PLC_Period_32_CLK
*									@arg CMP_PLC_Period_64_CLK
*									@arg CMP_PLC_Period_128_CLK
*									@arg CMP_PLC_Period_256_CLK
* 					@param Polling_MaskT;
*									@arg CMP_PLC_MaskT_0_CLK
*									@arg CMP_PLC_MaskT_8_CLK
*									@arg CMP_PLC_MaskT_16_CLK
*									@arg CMP_PLC_MaskT_32_CLK
* 					@param Polling_OFLT_Mode;
*									@arg CMP_PLC_OFLT_After_MaskT
*									@arg CMP_PLC_OFLT_Parallel_MaskT
*@retval None
***********************************************************************************
  */
void CMP_Polling_Init(CMP_TypeDef *CMPx, CMP_Polling_InitTypeDef *CMP_Polling_InitStruct)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_INP_SEL | CMP_PLC_INM_SEL | CMP_PLC_Period | CMP_PLC_MaskT | CMP_PLC_OFLT_Mode,\
							(CMP_Polling_InitStruct->Polling_INP | CMP_Polling_InitStruct->Polling_INM | CMP_Polling_InitStruct->Polling_Period | CMP_Polling_InitStruct->Polling_MaskT | CMP_Polling_InitStruct->Polling_OFLT_Mode));
}

/**
***********************************************************************************
*@Function CMP_Enable
*@brief    Enable CMP.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Enable(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->CR1, CMP_CR1_EN);
}

/**
***********************************************************************************
*@Function CMP_Disable
*@brief    Disable CMP.
*@param1   CMPx: 
*          @arg  CMP4 
*          @arg  CMP5
*@retval None
***********************************************************************************
  */
void CMP_Disable(CMP_TypeDef* CMPx)
{
  CLEAR_BIT(CMPx->CR1, CMP_CR1_EN);	
}

/**
***********************************************************************************
*@Function CMP_Set_MODE
*@brief    Set CMP MODE.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   val:
*         @arg CMP_CR1_MODE_Ultralow
*         @arg CMP_CR1_MODE_Low
*         @arg CMP_CR1_MODE_Medium
*         @arg CMP_CR1_MODE_High
*@retval None
***********************************************************************************
  */
void CMP_Set_Mode(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_MODE, val);
}

/**
***********************************************************************************
*@Function CMP_Set_POL
*@brief   Set CMP POL.
*@param1  CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   val:
*          @arg CMP_CR1_POL_Positive
*          @arg CMP_CR1_POL_Negative
*@retval None
***********************************************************************************
  */
void CMP_Set_POL(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_POL, val);
}

/**
***********************************************************************************
*@Function CMP_Read_OUT
*@brief   READ CMP OUTPUT.
*@param1  CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval   @arg  CMP_OUTPUT_LEVEL_HIGH
*          @arg  CMP_OUTPUT_LEVEL_LOW
***********************************************************************************
  */
uint32_t CMP_Read_OUT(CMP_TypeDef *CMPx)
{
  return (READ_BIT(CMPx->CR1, CMP_CR1_OUT) == (CMP_CR1_OUT));
}

/**
***********************************************************************************
*@Function CMP_Read_OUT_Ready
*@brief   READ CMP OUTPUT with Ready Flag.
*@param1  CMPx:
*          @arg  CMP4
*          @arg  CMP5       
*@retval  @arg  CMP_OUTPUT_LEVEL_HIGH_Ready
*         @arg  CMP_OUTPUT_LEVEL_LOW_Ready
***********************************************************************************
  */
uint32_t CMP_Read_OUT_Ready(CMP_TypeDef *CMPx)
{
  return (READ_REG(CMPx->CR1) & 0x00000060);
}

/**
***********************************************************************************
*@Function CMP_Enable_Lock
*@brief   Enable CR2 Lock.
*@param1  CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
  */
void CMP_Enable_Lock(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->CR1, CMP_CR1_LOCK);
}

/**
***********************************************************************************
*@Function CMP_Disable_Lock
*@brief    SET CR2 can read and write.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
  */
void CMP_Disable_Lock(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->CR1, CMP_CR1_LOCK);
}

/**
***********************************************************************************
*@Function CMP_Set_OFLT
*@brief   SET CR1 OFLT.
*@param1  CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg CMP_CR1_OFLT_1
*         @arg CMP_CR1_OFLT_4
*         @arg CMP_CR1_OFLT_16
*         @arg CMP_CR1_OFLT_32
*         @arg CMP_CR1_OFLT_64
*         @arg CMP_CR1_OFLT_128
*         @arg CMP_CR1_OFLT_256
*         @arg CMP_CR1_OFLT_512
*@retval None
***********************************************************************************
  */
void CMP_Set_OFLT(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_OFLT, val);
}

/**
*********************************************************************************************************
*@Function CMP_Enable_IT
*@brief    Enable CMP IT
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_IT:
*          @arg  CMP_CR1_RIE
*          @arg  CMP_CR1_FIE
*@retval None
*********************************************************************************************************
*/
void CMP_Enable_IT(CMP_TypeDef *CMPx, uint32_t CMP_IT)
{
  SET_BIT(CMPx->CR1, CMP_IT);
}

/**
*********************************************************************************************************
*@Function CMP_Disable_IT
*@brief    Disable CMP IT
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_IT:
*          @arg  CMP_CR1_RIE
*          @arg  CMP_CR1_FIE
*@retval None
*********************************************************************************************************
*/
void CMP_Disable_IT(CMP_TypeDef *CMPx, uint32_t CMP_IT)
{
  CLEAR_BIT(CMPx->CR1, CMP_IT);
}

/**
***********************************************************************************
*@Function CMP_Get_Flag
*@brief    Get CMP Flag.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param1   Flag:
*          @arg  CMP_CR1_RIF
*          @arg  CMP_CR1_FIF
*@retval FlagStatus (SET or RESET).
***********************************************************************************
  */
FlagStatus CMP_Get_Flag(CMP_TypeDef *CMPx, uint32_t Flag)
{
  return (FlagStatus)(READ_BIT(CMPx->CR1, Flag) == (Flag));
}

/**
***********************************************************************************
*@Function CMP_Clear_Flag
*@brief    Clear CMP Flag.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_IT:
*          @arg  CMP_CR1_RIF
*          @arg  CMP_CR1_FIF
*@retval   None.
***********************************************************************************
  */
void CMP_Clear_Flag(CMP_TypeDef *CMPx, uint32_t Flag)
{
  CLEAR_BIT(CMPx->CR1, Flag);
}

/**
***********************************************************************************
*@Function CMP_Set_RHYST
*@brief    SET CR1 RHYST.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg CMP_CR1_RHYST_0mV
*         @arg CMP_CR1_RHYST_5mV
*         @arg CMP_CR1_RHYST_10mV
*         @arg CMP_CR1_RHYST_20mV
*@retval None
***********************************************************************************
  */
void CMP_Set_RHYST(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_RHYST, val);
}

/**
***********************************************************************************
*@Function CMP_Set_FHYST
*@brief    SET CR1 FHYST.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg CMP_CR1_FHYST_0mV
*         @arg CMP_CR1_FHYST_5mV
*         @arg CMP_CR1_FHYST_10mV
*         @arg CMP_CR1_FHYST_20mV
*@retval None
***********************************************************************************
  */
void CMP_Set_FHYST(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_FHYST, val);
}

/**
***********************************************************************************
*@Function CMP_Set_OutPad
*@brief    SET CMP output pad signal mode.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg CMP_CR1_OutPad_Sample
*         @arg CMP_CR1_OutPad_Direct
*@retval None
***********************************************************************************
  */
void CMP_Set_OutPad(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR1, CMP_CR1_OutPad, val);
}

/**
*********************************************************************************************************
*@Function CMP_Enable_CRV
*@brief    Enable CRV_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Enable_CRV(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->CR2, CMP_CR2_CRV_EN);
}

/**
*********************************************************************************************************
*@Function CMP_Disable_CRV
*@brief    Disable CRV_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Disable_CRV(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->CR2, CMP_CR2_CRV_EN);
}

/**
***********************************************************************************
*@Function CMP_Set_CRV_SRC
*@brief    SET CR2_CRV_SRC.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   val:
*          @arg  CMP_CR2_CRV_SRC_VBGP
*          @arg  CMP_CR2_CRV_SRC_AVCC
*@retval None
***********************************************************************************
  */
void CMP_Set_CRV_SRC(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR2, CMP_CR2_CRV_SRC, val);
}

/**
***********************************************************************************
*@Function CMP_Set_CRV
*@brief    SET CMP_CR2_CRV_SEL.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg   CMP_CR2_CRV_SEL_1_20
*         @arg   CMP_CR2_CRV_SEL_2_20
*         @arg   CMP_CR2_CRV_SEL_3_20
*         @arg   CMP_CR2_CRV_SEL_4_20
*         @arg   CMP_CR2_CRV_SEL_5_20
*         @arg   CMP_CR2_CRV_SEL_6_20
*         @arg   CMP_CR2_CRV_SEL_7_20
*         @arg   CMP_CR2_CRV_SEL_8_20
*         @arg   CMP_CR2_CRV_SEL_9_20
*         @arg   CMP_CR2_CRV_SEL_10_20
*         @arg   CMP_CR2_CRV_SEL_11_20
*         @arg   CMP_CR2_CRV_SEL_12_20
*         @arg   CMP_CR2_CRV_SEL_13_20
*         @arg   CMP_CR2_CRV_SEL_14_20
*         @arg   CMP_CR2_CRV_SEL_15_20
*         @arg   CMP_CR2_CRV_SEL_16_20
*@retval None
***********************************************************************************
  */
void CMP_Set_CRV(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR2, CMP_CR2_CRV_SEL, val);
}

/**
***********************************************************************************
*@Function CMP_Set_INM
*@brief    SET CMP_CR2_INM_SEL.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*					 @arg CMP_CR2_INM_SEL_CRV
*					 @arg CMP_CR2_INM_SEL_PC4
*				 	 @arg CMP_CR2_INM_SEL_PC1
*					 @arg CMP_CR2_INM_SEL_PB8
*@retval None
***********************************************************************************
  */
void CMP_Set_INM(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR2, CMP_CR2_INM_SEL, val);
}

/**
***********************************************************************************
*@Function CMP_Set_INP
*@brief    SET CMP_CR2_INP_SEL.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*					 @arg CMP_CR2_INP_SEL_PC3
*					 @arg CMP_CR2_INP_SEL_PC0
*					 @arg CMP_CR2_INP_SEL_PB7
*					 @arg CMP_CR2_INP_SEL_PGA0_P
*					 @arg CMP_CR2_INP_SEL_PGA1_P
*					 @arg CMP_CR2_INP_SEL_PGA2_P
*					 @arg CMP_CR2_INP_SEL_PB5
*@retval None
***********************************************************************************
  */
void CMP_Set_INP(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR2, CMP_CR2_INP_SEL, val);
}

/**
***********************************************************************************
*@Function CMP_Set_BLANKING
*@brief    SET CMP_CR2_BLANKING.
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg   CMP_CR2_BLANKING_NoBlanking
*         @arg   CMP_CR2_BLANKING_TIM8_OC4  
*         @arg   CMP_CR2_BLANKING_TIM8_OC5  
*@retval None
***********************************************************************************
  */
void CMP_Set_BLANKING(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CR2, CMP_CR2_BLANKING, val);
}


/**
*********************************************************************************************************
*@Function CMP_Enable_CALP
*@brief    Enable CALP_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Enable_CALP(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->CAL, CMP_CALP_EN);
}

/**
*********************************************************************************************************
*@Function CMP_Disable_CALP
*@brief    Enable CALP_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Disable_CALP(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->CAL, CMP_CALP_EN);
}

/**
*********************************************************************************************************
*@Function CMP_Enable_CALN
*@brief    Enable CALN_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Enable_CALN(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->CAL, CMP_CALN_EN);
}

/**
*********************************************************************************************************
*@Function CMP_Disable_CALN
*@brief    Enable CALN_EN
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
*********************************************************************************************************
*/
void CMP_Disable_CALN(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->CAL, CMP_CALN_EN);
}

/**
***********************************************************************************
*@Function CMP_Set_CALP
*@brief    SET CMP_CALP_ADJ.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val: 0~15
***********************************************************************************
*/
void CMP_Set_CALP(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CAL, CMP_CALP_ADJ, val);
}

/**
***********************************************************************************
*@Function CMP_Set_CALN
*@brief    SET CMP_CALN_ADJ.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val: 0~15
***********************************************************************************
  */
void CMP_Set_CALN(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->CAL, CMP_CALN_ADJ, (val << 8));
}

/***************************************CMP Polling Functions********************************************/

/**
***********************************************************************************
*@Function CMP_Polling_Set_INP
*@brief    Set CMP Polling PLUS
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg   CMP_PLC_INP_SEL_DISABLE
*         @arg   CMP_PLC_INP_SEL_GPIO  
*         @arg   CMP_PLC_INP_SEL_PGA  
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_INP(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_INP_SEL, val);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_INM
*@brief    Set CMP Polling MINUS
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg   CMP_PLC_INM_SEL_DISABLE
*         @arg   CMP_PLC_INM_SEL_GPIO   
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_INM(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_INM_SEL, val);
}

/**
***********************************************************************************
*@Function CMP_Polling_Enable_CH1
*@brief    Enable CMP Polling Channel 1.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Enable_CH1(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->PLC, CMP_PLC_CH1_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Disable_CH1
*@brief    Disable CMP Polling Channel 1.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Disable_CH1(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->PLC, CMP_PLC_CH1_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Enable_CH2
*@brief    Enable CMP Polling Channel 2.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Enable_CH2(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->PLC, CMP_PLC_CH2_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Disable_CH2
*@brief    Disable CMP Polling Channel 2.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Disable_CH2(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->PLC, CMP_PLC_CH2_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Enable_CH3
*@brief    Enable CMP Polling Channel 3.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Enable_CH3(CMP_TypeDef *CMPx)
{
  SET_BIT(CMPx->PLC, CMP_PLC_CH3_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Disable_CH3
*@brief    Disable CMP Polling Channel 3.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval None
***********************************************************************************
*/
void CMP_Polling_Disable_CH3(CMP_TypeDef *CMPx)
{
  CLEAR_BIT(CMPx->PLC, CMP_PLC_CH3_EN);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_Period
*@brief    Set CMP Polling Period
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg   CMP_PLC_Period_32_CLK
*         @arg   CMP_PLC_Period_64_CLK
*         @arg   CMP_PLC_Period_128_CLK
*         @arg   CMP_PLC_Period_256_CLK  
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_Period(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_Period, val);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_MaskT
*@brief    Set CMP Polling MaskT
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg   CMP_PLC_MaskT_0_CLK
*         @arg   CMP_PLC_MaskT_8_CLK
*         @arg   CMP_PLC_MaskT_16_CLK
*         @arg   CMP_PLC_MaskT_32_CLK  
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_MaskT(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_MaskT, val);
}

/**
*********************************************************************************************************
*@Function CMP_Polling_Enable_IT
*@brief    Enable CMP Polling Match IT
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_Polling_IT:
*          @arg  CMP_PLC_Match_IT
*@retval None
*********************************************************************************************************
*/
void CMP_Polling_Enable_IT(CMP_TypeDef *CMPx, uint32_t CMP_Polling_IT)
{
  SET_BIT(CMPx->PLC, CMP_Polling_IT);
}

/**
*********************************************************************************************************
*@Function CMP_Polling_Disable_IT
*@brief    Enable CMP Polling Match IT
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_Polling_IT:
*          @arg  CMP_PLC_Match_IT
*@retval None
*********************************************************************************************************
*/
void CMP_Polling_Disable_IT(CMP_TypeDef *CMPx, uint32_t CMP_Polling_IT)
{
  SET_BIT(CMPx->PLC, CMP_Polling_IT);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_OFLT_Mode
*@brief    SET the mode of CMP filter and MaskT under polling mode.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2  val:
*         @arg CMP_PLC_OFLT_After_MaskT
*         @arg CMP_PLC_OFLT_Parallel_MaskT
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_OFLT_Mode(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_OFLT_Mode, val);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_Match_Data1
*@brief    Set CMP Polling CH1 Match Data
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_Match_Data1(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_Match_Data1, val << 24);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_Match_Data2
*@brief    Set CMP Polling CH1 Match Data
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_Match_Data2(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_Match_Data2, val << 25);
}

/**
***********************************************************************************
*@Function CMP_Polling_Set_Match_Data3
*@brief    Set CMP Polling CH1 Match Data
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@param2  val:
*         @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
*@retval None
***********************************************************************************
  */
void CMP_Polling_Set_Match_Data3(CMP_TypeDef *CMPx, uint32_t val)
{
  MODIFY_REG(CMPx->PLC, CMP_PLC_Match_Data3, val << 26);
}

/**
***********************************************************************************
*@Function CMP_Polling_Get_Current_Channle
*@brief    Get CMP Polling Flag.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@retval   @arg  CMP_PLS_CH1
*          @arg  CMP_PLS_CH2
*          @arg  CMP_PLS_CH3
*@retval None
***********************************************************************************
  */
uint32_t CMP_Polling_Get_Current_Channle(CMP_TypeDef *CMPx)
{
  return (CMPx->PLS & CMP_PLS_Channel);
}

/**
***********************************************************************************
*@Function CMP_Polling_Get_Flag
*@brief    Get CMP Polling Flag.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   Flag: This parameter can be any combination of the following values:
*          @arg  CMP_PLS_NULL
*          @arg  CMP_PLS_RDY
*          @arg  CMP_PLS_CH1
*          @arg  CMP_PLS_CH2
*          @arg  CMP_PLS_CH3
*          @arg  CMP_PLS_CH1_Match
*          @arg  CMP_PLS_CH2_Match
*          @arg  CMP_PLS_CH3_Match
*@retval FlagStatus (SET or RESET).
***********************************************************************************
  */
FlagStatus CMP_Polling_Get_Flag(CMP_TypeDef *CMPx, uint32_t Flag)
{
  return (FlagStatus)((READ_REG(CMPx->PLS) & Flag) == (Flag));
}

/**
***********************************************************************************
*@Function CMP_Polling_Clear_Flag
*@brief    Clear CMP Polling Flag.
*@param1   CMPx:
*          @arg  CMP4
*          @arg  CMP5
*@param2   CMP_IT:
*          @arg  CMP_PLS_CH1_Match
*          @arg  CMP_PLS_CH2_Match
*          @arg  CMP_PLS_CH3_Match
*@retval   None.
***********************************************************************************
  */
void CMP_Polling_Clear_Flag(CMP_TypeDef *CMPx, uint32_t Flag)
{
  CLEAR_BIT(CMPx->PLS, Flag);
}

/**
***********************************************************************************
*@Function CMP_Polling_Read_OUTP
*@brief    GET PLS DAT OUTP
*@param1  CMPx:
*         @arg  CMP4
*@retval  bit(x) is the output result corresponding to the input CH(x+1)_INP,x is 0 to 2.
*         bitx is 1 meaning the highoutput,bitx is 0 meaning the lowoutput.
***********************************************************************************
*/
uint32_t CMP_Polling_Read_OUTP(CMP_TypeDef *CMPx)
{
  return (READ_REG(CMPx->PLS) & 0x00000007);
}

/**
***********************************************************************************
*@Function CMP_Polling_Read_OUTP1
*@brief    GET PLS DAT OUTP1
*@param1  CMPx:
*         @arg  CMP4
*@retval  @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
***********************************************************************************
*/
uint32_t CMP_Polling_Read_OUTP1(CMP_TypeDef *CMPx)
{
  return ((READ_REG(CMPx->PLS) & 0x00000001) >> 0);
}

/**
***********************************************************************************
*@Function CMP_Polling_Read_OUTP2
*@brief    GET PLS DAT OUTP2
*@param1  CMPx:
*         @arg  CMP4
*@retval  @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
***********************************************************************************
*/
uint32_t CMP_Polling_Read_OUTP2(CMP_TypeDef *CMPx)
{
  return ((READ_REG(CMPx->PLS) & 0x00000002) >> 1);
}

/**
***********************************************************************************
*@Function CMP_Polling_Read_OUTP3
*@brief    GET PLS DAT OUTP3
*@param1  CMPx:
*         @arg  CMP4
*         @arg  CMP5
*@retval  @arg  CMP_OUTPUT_LEVEL_HIGH
*         @arg  CMP_OUTPUT_LEVEL_LOW
***********************************************************************************
*/
uint32_t CMP_Polling_Read_OUTP3(CMP_TypeDef *CMPx)
{
  return ((READ_REG(CMPx->PLS) & 0x00000004) >> 2);
}
