/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_cmu.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx.h"
#include "rx32s11xx_cmu.h"
#include "rx32s11xx_gpio.h"

/*
*********************************************************************************************************
*                                             
*********************************************************************************************************
*/
#define Wait2mS (WaitPllLock)

/*
*********************************************************************************************************
*                                            Local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            Local function
*********************************************************************************************************
*/

/**
  * @brief  Deinitializes the UARTx peripheral registers to their default reset values.
  * @param  UARTx: where x can be 0, 1 to select the 
  *         UART peripheral.
  * @retval None
  */
void CMU_DeInit(CMU_TypeDef* CMUx)
{
  CMUx->CHIPID = 0;
  CMUx->CLKCTRL0 = 0;
  CMUx->CLKCTRL1 = 0;
  CMUx->CLKOUTDIV = 0;
  CMUx->CLKOUTSEL = 0;
  CMUx->CLKSTA = 0;
  CMUx->CPUCFG = 0;
  CMUx->FLASHCON = 0;
  CMUx->FLASHLOCK = 0;
  CMUx->FLTCTR = 0;
  CMUx->HRCADJ = 0;
  CMUx->INFOLOCK = 0;
  CMUx->JTAGSTA = 0;
  CMUx->LFCLKCFG = 0;
  CMUx->LRCADJ = 0;
  CMUx->MULTFUNCFG = 0;
  CMUx->PLLLOCK_PRE = 0;
  CMUx->PORRSTSET = 0;
  CMUx->SYSCLKCFG = 0;
  CMUx->SYSCLKDIV = 0;
  CMUx->WPREG = 0;
}
/**
  * @brief  Fills each CMU_InitStruct member with its default value.
  * @param  CMU_InitStruct: pointer to a CMU_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void CMU_StructInit(CMU_ClkInitTypeDef *CMU_InitStruct)
{
  CMU_InitStruct->SYSCLKSource = CMU_SYSCLKCFG_CLKSEL_HRC;
  CMU_InitStruct->SYSDIV = CMU_SYSCLKDIV_4;
}

/**
***********************************************************************************
*@Function CMU_ClockConfig
*@brief    Config SYSCLK
*@param  CMUx :
*         @arg CMU
*@param  CMU_ClkInitStruct: pointer to a CMU_ClkInitTypeDef structure that contains
*         the configuration information for the specified CMU peripheral.
  * @param  SYSCLKSource:
	*         @arg CMU_SYSCLKCFG_CLKSEL_LRC
	*         @arg CMU_SYSCLKCFG_CLKSEL_HSE
  *         @arg CMU_SYSCLKCFG_CLKSEL_HRC
  * @param  SYSDIV:
  *         @arg CMU_SYSCLKDIV_1
	*         @arg CMU_SYSCLKDIV_2
	*         @arg CMU_SYSCLKDIV_4
	*         @arg CMU_SYSCLKDIV_8
  *         @arg CMU_SYSCLKDIV_16
	*         @arg CMU_SYSCLKDIV_32
	*         @arg CMU_SYSCLKDIV_64
	*         @arg CMU_SYSCLKDIV_128
* @retval None
***********************************************************************************
  */
void CMU_ClockConfig(CMU_TypeDef *CMUx,CMU_ClkInitTypeDef  *CMU_ClkInitStruct)
{
	CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
	
	switch(CMU_ClkInitStruct->SYSCLKSource)
	{
		case CMU_SYSCLKCFG_CLKSEL_HRC:		 
      CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HRC);
			break;
		case CMU_SYSCLKCFG_CLKSEL_LRC:
			CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_LRC);
			break;
		case CMU_SYSCLKCFG_CLKSEL_HSE:
      // IO PD7_FUNC1 OSC_IN PD8_FUNC1 OSC_OUT
			GPIO_Set_HIIPM(GPIOD, GPIO_PIN_8, GPIO_HIIPM_DISABLE);
			GPIO_Set_OutputMode(GPIOD, GPIO_PIN_8, GPIO_PUSHPULL);
			GPIO_Set_PinAF(GPIOD, GPIO_PIN_8,AF_FUNCTION1);
			GPIO_Set_PinMode(GPIOD,GPIO_PIN_8, AF_MODE);   
    
			GPIO_Set_HIIPM(GPIOD, GPIO_PIN_7, GPIO_HIIPM_DISABLE);
			GPIO_Set_OutputMode(GPIOD, GPIO_PIN_7, GPIO_PUSHPULL);
			GPIO_Set_PinAF(GPIOD, GPIO_PIN_7,AF_FUNCTION1);
			GPIO_Set_PinMode(GPIOD,GPIO_PIN_7, AF_MODE);      
  
			CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_HSEEN);
      while((CMU->CLKSTA & CMU_CLKSTA_HSELOCK) == 0);//Wait HSE_LOCK 
    	CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HSEDETEN); 		
	    CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_HSE);			
			break;
    default:;
	}
  CMU_Set_SYSCLKDIV(CMU_ClkInitStruct->SYSDIV);
}
/**
***********************************************************************************
*@Function CMU_Enable_CLKCTRL0
*@brief    Enable  CLKCTRL0.
*@param    val This parameter can be one of the following values:
*          @arg  CMU_CLKCTRL0_I2CEN                      BIT3
*          @arg  CMU_CLKCTRL0_HRCEN                      BIT5
*          @arg  CMU_CLKCTRL0_HSEDETEN                   BIT7
*          @arg  CMU_CLKCTRL0_HSELOCKEN                  BIT10
*          @arg  CMU_CLKCTRL0_HSEEN                      BIT11
*          @arg  CMU_CLKCTRL0_CLKOUTEN                   BIT12
*          @arg  CMU_CLKCTRL0_1P5LBOREN                  BIT13
*          @arg  CMU_CLKCTRL0_CMP4EN                     BIT21
*          @arg  CMU_CLKCTRL0_CMP5EN                     BIT22
*          @arg  CMU_CLKCTRL0_PGAEN                      BIT23
*@retval None
***********************************************************************************
  */
void CMU_Enable_CLKCTRL0(uint32_t val)
{
    CMU->WPREG = CMU_WPREG_UnProtected;

    SET_BIT(CMU->CLKCTRL0, val);

    CMU->WPREG = 0;
}

/**
***********************************************************************************
*@Function CMU_Disable_CLKCTRL0
*@brief    Disable  CLKCTRL0.
*@param:   val This parameter can be one of the following values:
*          @arg  CMU_CLKCTRL0_I2CEN                      BIT3
*          @arg  CMU_CLKCTRL0_HRCEN                      BIT5
*          @arg  CMU_CLKCTRL0_HSEDETEN                   BIT7
*          @arg  CMU_CLKCTRL0_HSELOCKEN                  BIT10
*          @arg  CMU_CLKCTRL0_HSEEN                      BIT11
*          @arg  CMU_CLKCTRL0_CLKOUTEN                   BIT12
*          @arg  CMU_CLKCTRL0_1P5LBOREN                  BIT13
*          @arg  CMU_CLKCTRL0_CMP4EN                     BIT21
*          @arg  CMU_CLKCTRL0_CMP5EN                     BIT22
*          @arg  CMU_CLKCTRL0_PGAEN                      BIT23
*@retval None
***********************************************************************************
  */
void CMU_Disable_CLKCTRL0(uint32_t val)
{
    CMU->WPREG = CMU_WPREG_UnProtected;

    CLEAR_BIT(CMU->CLKCTRL0, val);

    CMU->WPREG = 0;
}

/**
***********************************************************************************
*@Function CMU_Enable_CLKCTRL1
*@brief    Enable  CLKCTRL1.
*@param   val This parameter can be one of the following values:
*         @arg  CMU_CLKCTRL1_TMR0EN                     BIT0
*         @arg  CMU_CLKCTRL1_TMR1EN                     BIT1
*         @arg  CMU_CLKCTRL1_TMR2EN                     BIT2
*         @arg  CMU_CLKCTRL1_TMR3EN                     BIT3
*         @arg  CMU_CLKCTRL1_UART1EN                    BIT5
*         @arg  CMU_CLKCTRL1_SOFTWDTEN                  BIT14
*         @arg  CMU_CLKCTRL1_DMAEN                      BIT15
*         @arg  CMU_CLKCTRL1_TIM8EN                     BIT18
*         @arg  CMU_CLKCTRL1_MEEN                       BIT19
*         @arg  CMU_CLKCTRL1_SADCEN                    	BIT20
*@retval None
***********************************************************************************
  */
void CMU_Enable_CLKCTRL1(uint32_t val)
{
  CMU->WPREG = CMU_WPREG_UnProtected;
  SET_BIT(CMU->CLKCTRL1, val);
  CMU->WPREG = CMU_WPREG_Protected;
}

/**
***********************************************************************************
*@Function CMU_Disable_CLKCTRL1
*@brief   Disable  CLKCTRL1.
*@param:  val This parameter can be one of the following values:
*         @arg  CMU_CLKCTRL1_TMR0EN                     BIT0
*         @arg  CMU_CLKCTRL1_TMR1EN                     BIT1
*         @arg  CMU_CLKCTRL1_TMR2EN                     BIT2
*         @arg  CMU_CLKCTRL1_TMR3EN                     BIT3
*         @arg  CMU_CLKCTRL1_UART1EN                    BIT5
*         @arg  CMU_CLKCTRL1_SOFTWDTEN                  BIT14
*         @arg  CMU_CLKCTRL1_DMAEN                      BIT15
*         @arg  CMU_CLKCTRL1_TIM8EN                     BIT18
*         @arg  CMU_CLKCTRL1_MEEN                       BIT19
*         @arg  CMU_CLKCTRL1_SADCEN                    	BIT20
*@retval None
***********************************************************************************
  */
void CMU_Disable_CLKCTRL1(uint32_t val)
{
  CMU->WPREG = CMU_WPREG_UnProtected;
  CLEAR_BIT(CMU->CLKCTRL1, val);
  CMU->WPREG = CMU_WPREG_Protected;
}
/**
***********************************************************************************
*@Function CMU_Set_SYSCLK
*@brief    Set  system clock .
*@param    val This parameter can be one of the following values:
*          @arg  CMU_SYSCLKCFG_CLKSEL_LRC
*          @arg  CMU_SYSCLKCFG_CLKSEL_HSE
*          @arg  CMU_SYSCLKCFG_CLKSEL_HRC
*@retval None
***********************************************************************************
  */
void CMU_Set_SYSCLK(uint32_t val)
{
  CMU->WPREG = CMU_WPREG_UnProtected;
  MODIFY_REG(CMU->SYSCLKCFG, CMU_SYSCLKCFG_CLKSEL, val);
  CMU->WPREG = CMU_WPREG_Protected;
}

/**
***********************************************************************************
*@Function CMU_Set_SYSCLKDIV
*@brief    Set adc clock prescaler.
*@param    val This parameter can be one of the following values:
  *         @arg CMU_SYSCLKDIV_1
	*         @arg CMU_SYSCLKDIV_2
	*         @arg CMU_SYSCLKDIV_4
	*         @arg CMU_SYSCLKDIV_8
  *         @arg CMU_SYSCLKDIV_16
	*         @arg CMU_SYSCLKDIV_32
	*         @arg CMU_SYSCLKDIV_64
	*         @arg CMU_SYSCLKDIV_128
*@retval None
***********************************************************************************
*/
void CMU_Set_SYSCLKDIV(uint32_t val)
{
 CMU->WPREG = CMU_WPREG_UnProtected;
 MODIFY_REG(CMU->SYSCLKDIV, CMU_SYSCLKDIV, val);
 CMU->WPREG = CMU_WPREG_Protected;
}

/**
***********************************************************************************
*@Function CMU_Get_CLKSTA
*@brief    Indicate the status of CLKSTA.
*@param    val This parameter can be one of the following values:
*          @arg  CMU_CLKSTA_HSEDET
*          @arg  CMU_CLKSTA_HSELOCK
*@retval State of bit (1 or 0).
***********************************************************************************
  */
uint32_t CMU_Get_CLKSTA(uint32_t val)
{
  return (READ_BIT(CMU->CLKSTA, val) == (val));
}

/**
***********************************************************************************
*@Function CMU_Set_CLKOUTSEL
*@brief    Set  CLKOUTSEL.
*@param    val This parameter can be one of the following values:
*          @arg  CMU_CLKOUTSEL_LRC
*          @arg  CMU_CLKOUTSEL_HSE
*          @arg  CMU_CLKOUTSEL_HRC
*          @arg  CMU_CLKOUTSEL_SYS
*@retval None
***********************************************************************************
  */
void CMU_Set_CLKOUTSEL(uint32_t val)
{
 CMU->WPREG = CMU_WPREG_UnProtected;
 MODIFY_REG(CMU->CLKOUTSEL, CMU_CLKOUTSEL, val);
 CMU->WPREG = CMU_WPREG_Protected;
}
/**
***********************************************************************************
*@Function CMU_Set_CLKOUTDIV
*@brief    Set  CLKOUTDIV.
*@param    val This parameter can be one of the following values:
*          @arg  CMU_CLKOUTDIV_2 
*          @arg  CMU_CLKOUTDIV_4 
*          @arg  CMU_CLKOUTDIV_6 
*          @arg  CMU_CLKOUTDIV_8 
*          @arg  CMU_CLKOUTDIV_10
*          @arg  CMU_CLKOUTDIV_12
*          @arg  CMU_CLKOUTDIV_14
*          @arg  CMU_CLKOUTDIV_16
*          @arg  CMU_CLKOUTDIV_18
*          @arg  CMU_CLKOUTDIV_20
*          @arg  CMU_CLKOUTDIV_22
*          @arg  CMU_CLKOUTDIV_24
*          @arg  CMU_CLKOUTDIV_26
*          @arg  CMU_CLKOUTDIV_28
*          @arg  CMU_CLKOUTDIV_30
*          @arg  CMU_CLKOUTDIV_32
*@retval None
***********************************************************************************
  */
void CMU_Set_CLKOUTDIV(uint32_t val)
{
 CMU->WPREG = CMU_WPREG_UnProtected;
 MODIFY_REG(CMU->CLKOUTDIV, CMU_CLKOUTDIV, val);
 CMU->WPREG = CMU_WPREG_Protected;
}

/**
***********************************************************************************
*@Function CMU_Set_MULTFUNCFG
*@brief    set PREFETCH.
*@param    val This parameter can be one of the following values:
*          @arg CMU_MULTFUNCFG_3T_DISABLE          0x00
*          @arg CMU_MULTFUNCFG_3T_ENABLE           0x01
*@retval None
***********************************************************************************
*/
void CMU_Set_MULTFUNCFG(uint8_t val)
{
 uint8_t bit = 0; //Bit of PREFETCH
 CMU->WPREG = CMU_WPREG_UnProtected;
 if( val==1 )
 CMU->MULTFUNCFG |= (0x01<<bit);
 else if( val==0 )
 CMU->MULTFUNCFG &= (~(0x01<<bit));
 CMU->WPREG = CMU_WPREG_Protected;
}

/**
***********************************************************************************
*@Function CMU_Set_FLTCTR
*@brief    Set  FLTCTR .
*@param1    FLTCTR_X:
*          @arg  CMU_HSECTR
*          @arg  CMU_LRCCTR
*          @arg  CMU_HRCCTR
*@param2    val:
*          @arg  0x00
*          @arg  0x01
*          @arg  0x02
*          @arg  0x03
*@retval None
***********************************************************************************
  */
void CMU_Set_FLTCTR(uint32_t FLTCTR_X, uint8_t val)
{
 CMU->WPREG = CMU_WPREG_UnProtected;
 if (FLTCTR_X == CMU_HSECTR)
     MODIFY_REG(CMU->FLTCTR, FLTCTR_X, val);
 else if (FLTCTR_X == CMU_LRCCTR)
     MODIFY_REG(CMU->FLTCTR, FLTCTR_X, val << 2);
 else if (FLTCTR_X == CMU_HRCCTR)
     MODIFY_REG(CMU->FLTCTR, FLTCTR_X, val << 4);
 CMU->WPREG = CMU_WPREG_Protected;
}
