/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_dma.c
* By           : RX_SD_Team
* Version      : 20210420
* brief  : Only rx32s11xx
*********************************************************************************************************
*/

#define   __rx32s11xx_DMA_C

#include "rx32s11xx_dma.h"
#include "main.h"


/**
  * @brief  Deinitializes the DMAx peripheral registers to their default reset values.
  * @param  DMAx: where x can be 0, 1 to select the 
  *         DMA peripheral.
  * @retval None
  */
void DMA_DeInit(DMA_TypeDef* DMAx)
{
  DMAx->CHNSTA = 0;
  DMAx->DMAIE = 0;
  DMAx->DMAIF = 0;
}
/**
  * @brief  Fills each USART_InitStruct member with its default value.
  * @param  DMA_InitStruct: pointer to a DMA_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void DMA_StructInit(DMA_InitTypeDef *DMA_InitStruct)
{
  DMA_InitStruct->Channel = DMA_CHANNEL_0;
  DMA_InitStruct->CHNCTL_Channel = DMA_CHNCTL_CHANNEL_Software;
  DMA_InitStruct->CHNCTL_Cycle= DMA_CHNCTL_CYCLE_DISABLE;
  DMA_InitStruct->CHNCTL_Destininc = DMA_CHNCTL_SOURCEINC_INC;
  DMA_InitStruct->CHNCTL_Mode = DMA_CHNCTL_MODE_WordTransfer;
  DMA_InitStruct->CHNCTL_Psize = DMA_CHNCTL_PSIZE_Word;
  DMA_InitStruct->CHNCTL_Sourceinc = DMA_CHNCTL_DESTININC_INC;
  DMA_InitStruct->DataLength = 1;
  DMA_InitStruct->SrcAddress = 0x20000800;
  DMA_InitStruct->TarAddress = 0x20000A00;
}

/**
  * @brief  Initialize the DMA registers according to the specified parameters in DMA_InitStruct.
  * @note   To convert DMAx_Channely Instance to DMAx Instance and Channely, use helper macros :
  * @param  DMAx DMAx Instance
  * @param  Configuration:
	*         @arg Channel
  *              @ref DMA_CHANNEL_0
  *              @ref DMA_CHANNEL_1
  *              @ref DMA_CHANNEL_2
  *         @arg CHNCTL_Mode
  *              @ref DMA_CHNCTL_MODE_WordTransfer 
  *              @ref DMA_CHNCTL_MODE_BlockTransfer
  *         @arg CHNCTL_Cycle
  *              @ref DMA_CHNCTL_CYCLE_DISABLE 
  *              @ref DMA_CHNCTL_CYCLE_ENABLE
  *         @arg CHNCTL_Destininc
  *              @ref DMA_CHNCTL_DESTININC_NOINC 
  *              @ref DMA_CHNCTL_DESTININC_INC
  *              @ref DMA_CHNCTL_DESTININC_BLOCKINC
  *         @arg CHNCTL_Sourceinc
  *              @ref DMA_CHNCTL_SOURCEINC_NOINC 
  *              @ref DMA_CHNCTL_SOURCEINC_INC
  *              @ref DMA_CHNCTL_SOURCEINC_BLOCKINC
  *         @arg CHNCTL_Psize
  *              @ref DMA_CHNCTL_PSIZE_Byte 
  *              @ref DMA_CHNCTL_PSIZE_HalfWord
  *              @ref DMA_CHNCTL_PSIZE_Word 
  *         @arg CHNCTL_Channel
  *              @ref DMA_CHNCTL_CHANNEL_Software
  *              @ref DMA_CHNCTL_CHANNEL_UART0_TX
  *              @ref DMA_CHNCTL_CHANNEL_UART0_RX 
  *              @ref DMA_CHNCTL_CHANNEL_UART1_TX 
  *              @ref DMA_CHNCTL_CHANNEL_UART1_RX   
  *              @ref DMA_CHNCTL_CHANNEL_I2C_TX       
  *              @ref DMA_CHNCTL_CHANNEL_I2C_RX          
  *              @ref DMA_CHNCTL_CHANNEL_SADC                
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_UP            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_TRIG           
  *              @ref DMA_CHNCTL_CHANNEL_TIMER0              
  *              @ref DMA_CHNCTL_CHANNEL_TIMER1               
  *              @ref DMA_CHNCTL_CHANNEL_TIMER2              
  *              @ref DMA_CHNCTL_CHANNEL_TIMER3              
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_COM            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH1           
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH2            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH3           
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH4            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH5 
  * @param  SrcAddress:
  *         @arg  SrcAddress 
  * @param  TarAddress:
  *         @arg  TarAddress 
  * @param  DataLength:
  *         @arg  DataLength 
  * @param  DMA_InitStruct pointer to a @ref LL_DMA_InitTypeDef structure
  * @retval None
	*/
void DMA_Init(DMA_TypeDef *DMAx, DMA_InitTypeDef* DMA_InitStruct)
{
  /* Configure the DMA transfer */
  /*  - DMA transfer in circular mode to match with ADC configuration:        */
  /*    DMA unlimited requests.                                               */
  /*  - DMA transfer from ADC without address increment.                      */
  /*  - DMA transfer to memory with address increment.                        */
  /*  - DMA transfer from ADC by half-word to match with ADC configuration:   */
  /*    ADC resolution 12 bits.                                               */
  /*  - DMA transfer to memory by half-word to match with ADC conversion data */
  /*    buffer variable type: half-word.                                      */
  DMA_Set_Transfer( DMAx,DMA_InitStruct->Channel,
                        DMA_InitStruct->CHNCTL_Mode| 
                        DMA_InitStruct->CHNCTL_Cycle| 
                        DMA_InitStruct->CHNCTL_Destininc| 
                        DMA_InitStruct->CHNCTL_Sourceinc| 
                        DMA_InitStruct->CHNCTL_Psize | 
                        DMA_InitStruct->CHNCTL_Channel );
                  
  /* Set DMA transfer addresses of source and destination */
  DMA_Set_Addresses(DMAx,DMA_InitStruct->Channel,
                        DMA_InitStruct->SrcAddress, 
                        DMA_InitStruct->TarAddress);
  
  /* Set DMA transfer size */
  DMA_Set_DataLength(DMAx,DMA_InitStruct->Channel,
                        DMA_InitStruct->DataLength);									 
}

/**
  * @function  DMA_Set_Transfer
  * @brief  Configure all parameters link to transfer of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg DMA_CHANNEL_0 
  *         @arg DMA_CHANNEL_1
  *         @arg DMA_CHANNEL_2
  * @param  Configuration:
  *         @arg DMAx_CHNCTL_MODE
  *              @ref DMA_CHNCTL_MODE_WordTransfer 
  *              @ref DMA_CHNCTL_MODE_BlockTransfer
  *         @arg DMAx_CHNCTL_CYCLE
  *              @ref DMA_CHNCTL_CYCLE_DISABLE 
  *              @ref DMA_CHNCTL_CYCLE_ENABLE
  *         @arg DMAx_CHNCTL_DESTININC
  *              @ref DMA_CHNCTL_DESTININC_NOINC 
  *              @ref DMA_CHNCTL_DESTININC_INC
  *              @ref DMA_CHNCTL_DESTININC_BLOCKINC
  *         @arg DMAx_CHNCTL_SOURCEINC
  *              @ref DMA_CHNCTL_SOURCEINC_NOINC 
  *              @ref DMA_CHNCTL_SOURCEINC_INC
  *              @ref DMA_CHNCTL_SOURCEINC_BLOCKINC
  *         @arg DMAx_CHNCTL_PSIZE
  *              @ref DMA_CHNCTL_PSIZE_Byte 
  *              @ref DMA_CHNCTL_PSIZE_HalfWord
  *              @ref DMA_CHNCTL_PSIZE_Word 
  *         @arg DMAx_CHNCTL_CHANNEL
  *              @ref DMA_CHNCTL_CHANNEL_Software
  *              @ref DMA_CHNCTL_CHANNEL_UART0_TX
  *              @ref DMA_CHNCTL_CHANNEL_UART0_RX 
  *              @ref DMA_CHNCTL_CHANNEL_UART1_TX 
  *              @ref DMA_CHNCTL_CHANNEL_UART1_RX    
  *              @ref DMA_CHNCTL_CHANNEL_I2C_TX       
  *              @ref DMA_CHNCTL_CHANNEL_I2C_RX          
  *              @ref DMA_CHNCTL_CHANNEL_SADC                
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_UP            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_TRIG           
  *              @ref DMA_CHNCTL_CHANNEL_TIMER0              
  *              @ref DMA_CHNCTL_CHANNEL_TIMER1               
  *              @ref DMA_CHNCTL_CHANNEL_TIMER2              
  *              @ref DMA_CHNCTL_CHANNEL_TIMER3              
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_COM            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH1           
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH2            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH3           
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH4            
  *              @ref DMA_CHNCTL_CHANNEL_TIM8_CH5  
  * @retval None
  */
void DMA_Set_Transfer(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t Configuration)
{
  MODIFY_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCTL,
             DMA_CHNCTL_MODE | DMA_CHNCTL_CYCLE | DMA_CHNCTL_DESTININC | DMA_CHNCTL_SOURCEINC | DMA_CHNCTL_PSIZE | DMA_CHNCTL_CHANNEL,
             Configuration);
}

/**
  * @function  DMA_Set_Addresses
  * @brief  Configure the Source and Destination addresses of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @param  SrcAddress:
  *         @arg  SrcAddress 
  * @param  TarAddress:
  *         @arg  TarAddress 
  * @retval None
  */
void DMA_Set_Addresses(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t SrcAddress,uint32_t TarAddress)
{
    WRITE_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNSRC, SrcAddress);
    WRITE_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNTAR, TarAddress);
}

/**
  * @function  DMA_Set_DataLength
  * @brief  Set Number of data to transfer of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @param  DataLength:
  *         @arg  DataLength 
  * @retval None
  */
void DMA_Set_DataLength(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t DataLength)
{
  MODIFY_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCNT,
             DMA_CHNCNT, DataLength);
}

/**
  * @function  DMA_Get_TCDataLength
  * @brief  Get Number of data that they transmission is completed in the selected DMA 
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @retval the number of TCData in the selected DMA
  */
uint32_t DMA_Get_TCDataLength(DMA_TypeDef* DMAx, uint32_t Channel )
{
 return( READ_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNTCCNT) );
}

/**
  * @function  DMA_Set_BulkCycle
  * @brief  Set Number of bulk cycle.
  * @note   This action has no effect if
  *         channel is enabled.
  * @rmtoll CHNXBULKNUM          DMA_SetBulkCycle
  * @param  DMAx DMAx Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref DMA_CHANNEL_0
  *         @arg @ref DMA_CHANNEL_1
  *         @arg @ref DMA_CHANNEL_2
  * @param  CycleLength Between Min_Data = 0 and Max_Data = 0x000000FF
  * @retval None
  */
void DMA_Set_BulkCycle(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t CycleLength)
{
	uint8_t bit = 16;//bit is CHN0BULKNUM_CYCLE's BitL
  MODIFY_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNBULKNUM,
             DMA_CHNBULKNUM_CYCLE, (CycleLength<<bit));
}

/**
  * @function  DMA_Set_BulkNumber
  * @brief  Set Number of data to transfer.
  * @note   This action has no effect if
  *         channel is enabled.
  * @rmtoll CHNXBULKNUM          DMA_SetBulkNumber
  * @param  DMAx DMAx Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref DMA_CHANNEL_0
  *         @arg @ref DMA_CHANNEL_1
  *         @arg @ref DMA_CHANNEL_2
  * @param  Cycle Between Min_Data = 0 and Max_Data = 0x0000ffff
  * @retval None
  */
void DMA_Set_BulkNumber(DMA_TypeDef* DMAx, uint32_t Channel, uint32_t Cyclenumber)
{
	uint8_t bit = 0;//bit is CHN0BULKNUM_Num's BitL
  MODIFY_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNBULKNUM,
             DMA_CHNBULKNUM_NUM, (Cyclenumber<<bit));
}

/**
  * @function  DMA_Enable_Channel
  * @brief  Enable channel of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @retval None
  */
void DMA_Enable_Channel(DMA_TypeDef* DMAx, uint32_t Channel)
{
	SET_BIT(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCTL, DMA_CHNCTL_CHNEN);    //Enable Channel
}

/**
  * @function  DMA_Disable_Channel
  * @brief  Disable channel of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @retval None
  */
void DMA_Disable_Channel(DMA_TypeDef* DMAx, uint32_t Channel)
{
	CLEAR_BIT(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCTL, DMA_CHNCTL_CHNEN);  //Disable Channel
}

/**
  * @function  DMA_Set_ChannelPriority
  * @brief  Config channel priority of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @param  val:
  *         @arg  DMA_CHNCTL_CHANNEL_PRIORITY_LOW         ((uint32_t)0x00000)
  *         @arg  DMA_CHNCTL_CHANNEL_PRIORITY_MEDIUM      ((uint32_t)0x10000)
  *         @arg  DMA_CHNCTL_CHANNEL_PRIORITY_HIGH        ((uint32_t)0x20000)
  *         @arg  DMA_CHNCTL_CHANNEL_PRIORITY_HIGHEST     ((uint32_t)0x30000)
  * @retval None
  */
void DMA_Set_ChannelPriority( DMA_TypeDef* DMAx, uint32_t Channel, uint32_t val )
{
	MODIFY_REG(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCTL,
             DMA_CHNCTL_CHANNEL_PRIORITY, val);
}

/**
  * @function  DMA_SoftRequestTrig
  * @brief  Config soft request trig of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  Channel:
  *         @arg  DMA_CHANNEL_0 
  *         @arg  DMA_CHANNEL_1
  *         @arg  DMA_CHANNEL_2
  * @retval None
  */
void DMA_SoftRequestTrig( DMA_TypeDef* DMAx, uint32_t Channel )
{
	SET_BIT(((DMA_Channel_TypeDef *)((uint32_t)((uint32_t)DMAx + CHANNEL_OFFSET_TAB[Channel])))->CHNCTL,
					DMA_CHNCTL_SOFTREQUEST_TRIG);
}

/**
  * @function  DMA_Enable_IT
  * @brief  Enable interrupt of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  DMA_IT:
  *         @arg  DMA_DMAIE_TCIE0 
  *         @arg  DMA_DMAIE_TCIE1 
  *         @arg  DMA_DMAIE_TCIE2 
  *         @arg  DMA_DMAIE_BCIE0 
  *         @arg  DMA_DMAIE_BCIE1 
  *         @arg  DMA_DMAIE_BCIE2 
  *         @arg  DMA_DMAIE_TEIE0 
  *         @arg  DMA_DMAIE_TEIE1 
  *         @arg  DMA_DMAIE_TEIE2 
  * @retval None
  */
void DMA_Enable_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT)
{
	SET_BIT(DMAx->DMAIE, DMA_IT);              /*!< Enable DMA Interrupt*/
}

/**
  * @function  DMA_Disable_IT
  * @brief  Enable interrupt of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  DMA_IT:
  *         @arg  DMA_DMAIE_TCIE0 
  *         @arg  DMA_DMAIE_TCIE1 
  *         @arg  DMA_DMAIE_TCIE2 
  *         @arg  DMA_DMAIE_BCIE0 
  *         @arg  DMA_DMAIE_BCIE1 
  *         @arg  DMA_DMAIE_BCIE2 
  *         @arg  DMA_DMAIE_TEIE0 
  *         @arg  DMA_DMAIE_TEIE1 
  *         @arg  DMA_DMAIE_TEIE2 
  * @retval None
  */
void DMA_Disable_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT)
{
	CLEAR_BIT(DMAx->DMAIE, DMA_IT);              /*!< Disable DMA Interrupt*/
}

/**
  * @function  DMA_Get_IT
  * @brief  read complete interrupt status of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  DMA_IT:
  *         @arg  DMA_DMAIE_TCIE0 
  *         @arg  DMA_DMAIE_TCIE1 
  *         @arg  DMA_DMAIE_TCIE2 
  *         @arg  DMA_DMAIE_BCIE0 
  *         @arg  DMA_DMAIE_BCIE1 
  *         @arg  DMA_DMAIE_BCIE2 
  *         @arg  DMA_DMAIE_TEIE0 
  *         @arg  DMA_DMAIE_TEIE1 
  *         @arg  DMA_DMAIE_TEIE2 
  * @retval State of bit (1 or 0).
  */
uint32_t DMA_Get_IT(DMA_TypeDef* DMAx, uint32_t DMA_IT)
{
  return (READ_BIT(DMAx->DMAIF, DMA_IT) == (DMA_IT));
}

/**
  * @function  DMA_Get_Flag
  * @brief  read complete interrupt flag of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  ITFlag:
  *         @arg  DMA_DMAIF_TCIF0 
  *         @arg  DMA_DMAIF_TCIF1 
  *         @arg  DMA_DMAIF_TCIF2 
  *         @arg  DMA_DMAIF_BCIF0 
  *         @arg  DMA_DMAIF_BCIF1 
  *         @arg  DMA_DMAIF_BCIF2 
  *         @arg  DMA_DMAIF_TEIF0 
  *         @arg  DMA_DMAIF_TEIF1 
  *         @arg  DMA_DMAIF_TEIF2 
  * @retval State of bit (1 or 0).
  */
uint32_t DMA_Get_Flag(DMA_TypeDef* DMAx, uint32_t Flag)
{
  return (READ_BIT(DMAx->DMAIF, Flag) == (Flag));
}

/**
  * @function  DMA_Clear_Flag
  * @brief  clear complete interrupt flag of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  ITFlag:
  *         @arg  DMA_DMAIF_TCIF0 
  *         @arg  DMA_DMAIF_TCIF1 
  *         @arg  DMA_DMAIF_TCIF2 
  *         @arg  DMA_DMAIF_BCIF0 
  *         @arg  DMA_DMAIF_BCIF1 
  *         @arg  DMA_DMAIF_BCIF2 
  *         @arg  DMA_DMAIF_TEIF0 
  *         @arg  DMA_DMAIF_TEIF1 
  *         @arg  DMA_DMAIF_TEIF2 
  * @retval None
  */
void DMA_Clear_Flag(DMA_TypeDef* DMAx, uint32_t Flag)
{
  CLEAR_BIT(DMAx->DMAIF, Flag);                 /*!< Clear DMA Interrupt Flag       */
}

/**
  * @function  DMA_Get_Busy
  * @brief  read busy flag of the selected DMA
  * @param  DMAx :
            @arg DMA Instance
  * @param  ITFlag:
  *         @arg  DMA_CHNSTA_BUSY0        ((uint32_t)0x0001) 
  *         @arg  DMA_CHNSTA_BUSY1        ((uint32_t)0x0002) 
  *         @arg  DMA_CHNSTA_BUSY2        ((uint32_t)0x0004) 
  * @retval State of bit (1 or 0).
  */
uint32_t DMA_Get_Busy(DMA_TypeDef* DMAx, uint32_t Busy)
{
  return( READ_BIT(DMAx->CHNSTA, Busy) == (Busy) );
}
