/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                               rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_exti.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_exti.h"

const uint32_t OFFSET_TAB_REG[] =
{
  0x00U, /* EXTI_0 */
  0x00U, /* EXTI_1 */
  0x00U, /* EXTI_2 */
  0x00U, /* EXTI_3 */
  0x00U, /* EXTI_4 */
  0x00U, /* EXTI_5 */
  0x00U, /* EXTI_6 */
  0x10U, /* EXTI_7 */
  0x10U, /* EXTI_8 */
};

const uint32_t SHIFT_TAB_EXTI[] =
{
  0, /* EXTI_0	*/
  0, /* EXTI_1 */
  0, /* EXTI_2	*/
  0, /* EXTI_3 */
  0, /* EXTI_4	*/
  0, /* EXTI_5 */
  0, /* EXTI_6	*/
  7, /* EXTI_7 */
  7, /* EXTI_8	*/
};

/**
  * @brief  Initializes the EXTI peripheral according to the specified
  *         parameters in the EXTI_InitStruct.
  * @param  EXTI_InitStruct: pointer to a GPIO_InitTypeDef structure that
  *         contains the configuration information for the specified EXTI peripheral.
  * @retval None
  */
void EXTI_Init(EXTI_InitTypeDef *EXTI_InitStruct)
{
  //Set EXTI Filter Value
  if( EXTI_InitStruct->FltValue != EXTI_No_Filter )
  {
    EXTI_Enable_FLT( EXTI_InitStruct->ExtiNumber );
    EXTI_Set_FilterValue( EXTI_InitStruct->ExtiNumber, EXTI_InitStruct->FltValue );
  }
  else 
  {
    EXTI_Disable_FLT( EXTI_InitStruct->ExtiNumber ); 
  }
  //Set Trigger Mode
  if( EXTI_InitStruct->TrigMode == Falling_Edge_Trigger )
  {
    EXTI_Enable_FallingTrig( EXTI_InitStruct->ExtiNumber );
  }
  else if( EXTI_InitStruct->TrigMode == Rising_Edge_Trigger )
  {
    EXTI_Enable_RisingTrig( EXTI_InitStruct->ExtiNumber );
  }
  //Clear Falling & Rising Flag
	EXTI_Clear_FallingFlag( EXTI_InitStruct->ExtiNumber );
	EXTI_Clear_RisingFlag( EXTI_InitStruct->ExtiNumber );
}

/**
  * @brief  Initializes the EXTI peripheral in the EXTI_InitStruct.
  * @retval None
  */
void EXTI_DeInit(void)
{  
  EXTI->EXTIE    = 0x0000;
  EXTI->EXTIE2   = 0x0000;
  EXTI->EXTIF    = 0x0000;
  EXTI->EXTIF2   = 0x0000;
  EXTI->PINFLTEN = 0x01ff;
  EXTI->RXFLTEN  = 0x0000;
  EXTI->FLTSEL   = 0x3ffff;
}

/**
  * @brief  Fills each EXTI_InitStruct member with its default value.
  * @param  GPIO_InitStruct : pointer to a EXTI_InitTypeDef structure which will be initialized.
  * @retval None
  */
void EXTI_StructInit(EXTI_InitTypeDef *EXTI_InitStruct)
{
  EXTI_InitStruct->ExtiNumber = EXTI_0;
  EXTI_InitStruct->FltValue   = EXTI_No_Filter;
  EXTI_InitStruct->TrigMode   = Falling_Edge_Trigger;
}

/**
  * @brief  Enable ExtiLine Falling Edge Trigger for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Enable_FallingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  SET_BIT(*pReg, (EXTI_ENABLE_FIE << (exti_x - SHIFT_TAB_EXTI[exti_x])));
}

/**
  * @brief  Disable ExtiLine Falling Edge Trigger for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Disable_FallingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  CLEAR_BIT(*pReg, (EXTI_ENABLE_FIE << (exti_x - SHIFT_TAB_EXTI[exti_x])));
}

/**
  * @brief  Check if falling edge trigger is enabled for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_FallingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  return (READ_BIT(*pReg, EXTI_ENABLE_FIE << (exti_x - SHIFT_TAB_EXTI[exti_x])) == (EXTI_ENABLE_FIE << (exti_x - SHIFT_TAB_EXTI[exti_x])));
}

/**
  * @brief  Enable ExtiLine Rising Edge Trigger for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Enable_RisingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  SET_BIT(*pReg, (EXTI_ENABLE_RIE << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)));
}

/**
  * @brief  Disable ExtiLine Rising Edge Trigger for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Disable_RisingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  CLEAR_BIT(*pReg, (EXTI_ENABLE_RIE << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)));
}

/**
  * @brief  Check if rising edge trigger is enabled for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_RisingTrig(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIE) + OFFSET_TAB_REG[exti_x]);
  return (READ_BIT(*pReg, EXTI_ENABLE_RIE << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)) == (EXTI_ENABLE_FIE << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)));
}

/**
  * @brief  Check if the falling Flag is set or not for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_FallingFlag(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIF) + OFFSET_TAB_REG[exti_x]);
  return (READ_BIT(*pReg, EXTI_READ_FIF << (exti_x - SHIFT_TAB_EXTI[exti_x])) == (EXTI_READ_FIF << (exti_x - SHIFT_TAB_EXTI[exti_x])));
}

/**
  * @brief  Clear ExtLine Falling Flags for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Clear_FallingFlag(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIF) + OFFSET_TAB_REG[exti_x]);
  CLEAR_BIT(*pReg, EXTI_READ_FIF << (exti_x - SHIFT_TAB_EXTI[exti_x]));
}

/**
  * @brief  Check if the rising Flag is set or not for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval State of bit (1 or 0).
  */
uint32_t EXTI_Get_RisingFlag(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIF) + OFFSET_TAB_REG[exti_x]);
  return (READ_BIT(*pReg, EXTI_READ_RIF << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)) == (EXTI_READ_RIF << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8)));
}

/**
  * @brief  Clear ExtLine Rising Flags for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Clear_RisingFlag(uint32_t exti_x)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&EXTI->EXTIF) + OFFSET_TAB_REG[exti_x]);
  CLEAR_BIT(*pReg, EXTI_READ_RIF << (exti_x - SHIFT_TAB_EXTI[exti_x] + 8));
}

/**
  * @brief  Enable PIN filter for Lines
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref EXTI_0
  *         @arg @ref EXTI_1
  *         @arg @ref EXTI_2
  *         @arg @ref EXTI_3
  *         @arg @ref EXTI_4
  *         @arg @ref EXTI_5
  *         @arg @ref EXTI_6
  *         @arg @ref EXTI_7
  *         @arg @ref EXTI_8
  * @retval None
  */
void EXTI_Enable_FLT(uint32_t exti_x)
{
  CLEAR_BIT(EXTI->PINFLTEN, (EXTI_PIN_FLT<<exti_x));
}

/**
   * @brief  Disable PIN filter for Lines
   * @param  ExtiLine This parameter can be a combination of the following values:
   *				 @arg @ref EXTI_0
   *				 @arg @ref EXTI_1
   *				 @arg @ref EXTI_2
   *				 @arg @ref EXTI_3
   *				 @arg @ref EXTI_4
   *				 @arg @ref EXTI_5
   *				 @arg @ref EXTI_6
   *				 @arg @ref EXTI_7
   *				 @arg @ref EXTI_8
   * @retval None
   */
void EXTI_Disable_FLT(uint32_t exti_x)
{
  SET_BIT(EXTI->PINFLTEN, (EXTI_PIN_FLT<<exti_x));
}

/**
   * @brief  Check if the INTFLT is set or not for Lines
   * @param  ExtiLine This parameter can be a combination of the following values:
   *		     @arg @ref EXTI_0
   *		     @arg @ref EXTI_1
   *		     @arg @ref EXTI_2
   *		     @arg @ref EXTI_3
   *		     @arg @ref EXTI_4
   *		     @arg @ref EXTI_5
   *		     @arg @ref EXTI_6
   *		     @arg @ref EXTI_7
   *		     @arg @ref EXTI_8
   * @retval State of bit (1 or 0).
   */
uint32_t EXTI_Get_INTFLT(uint32_t exti_x)
{
  return(READ_BIT(EXTI->PINFLTEN, (EXTI_PIN_FLT<<exti_x)) == (EXTI_PIN_FLT<<exti_x));
}

/**
  * @brief   SET filtsel.
  * @param1: select INTx
  *          @arg @ref EXTI_0
  *          @arg @ref EXTI_1
  *          @arg @ref EXTI_2
  *          @arg @ref EXTI_3
  *          @arg @ref EXTI_4
  *          @arg @ref EXTI_5
  *          @arg @ref EXTI_6
  *          @arg @ref EXTI_7
  *          @arg @ref EXTI_8
  * @param2: val This parameter can be one of the following values:
  *          @arg @ref EXTI_Filter_80_ns
  *          @arg @ref EXTI_Filter_50_ns
  * @retval None
  */
void EXTI_Set_FilterValue(uint32_t exti_x, uint32_t val)
{
  MODIFY_REG( EXTI->FLTSEL, (EXTI_RAED_FILTSEL<<(exti_x*2)), (val<<(exti_x*2)) );
}

/**
  * @brief   GET filtsel.
  * @param1: select INTx
  *          @arg @ref EXTI_0
  *          @arg @ref EXTI_1
  *          @arg @ref EXTI_2
  *          @arg @ref EXTI_3
  *          @arg @ref EXTI_4
  *          @arg @ref EXTI_5
  *          @arg @ref EXTI_6
  *          @arg @ref EXTI_7
  *          @arg @ref EXTI_8
  * @retval: val This parameter can be one of the following values:
  *          @arg @ref EXTI_Filter_80_ns
  *          @arg @ref EXTI_Filter_50_ns
  */
uint32_t EXTI_Get_FilterValue(uint32_t exti_x)
{
 return( READ_BIT(EXTI->FLTSEL,(EXTI_RAED_FILTSEL<<(exti_x*2))) >> (exti_x*2) );
}
/**
  * @brief  Enable PIN filter for Lines in RX 0 to 1
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref UART1_RX
  * @retval None
  */
void RX_Enable_Filter(uint32_t UARTx_RX)
{
  SET_BIT(EXTI->RXFLTEN, (RX_PIN_FLT << UARTx_RX));
}

/**
  * @brief  Disable PIN filter for Lines in RX 0 to 1
  * @param  ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref UART1_RX
  * @retval None
  */
void RX_Disable_Filter(uint32_t UARTx_RX)
{
  CLEAR_BIT(EXTI->RXFLTEN, (RX_PIN_FLT << UARTx_RX));
}

/**
  * @brief	Check if the RXFLT is set or not for Lines in RX 0 to 1
  * @param	ExtiLine This parameter can be a combination of the following values:
  *         @arg @ref UART1_RX
  * @retval State of bit (1 or 0).
  */
uint32_t RX_Get_FilterStatus(uint32_t UARTx_RX)
{
  return (READ_BIT(EXTI->RXFLTEN, (RX_PIN_FLT << UARTx_RX)) == (RX_PIN_FLT << UARTx_RX) );
}

