/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_iic_lib.h
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_gpio.h"

/*
*********************************************************************************************************
                                           Local variable
*********************************************************************************************************
*/
const uint32_t OFFSET_TAB_AFCFGx[] =
{
  0x00U, /* GPIO_PIN_0  */
  0x00U, /* GPIO_PIN_1  */
  0x00U, /* GPIO_PIN_2  */
  0x00U, /* GPIO_PIN_3  */
  0x00U, /* GPIO_PIN_4  */
  0x00U, /* GPIO_PIN_5  */
  0x00U, /* GPIO_PIN_6  */
  0x00U, /* GPIO_PIN_7  */
  0x2CU, /* GPIO_PIN_8  */
  0x2CU, /* GPIO_PIN_9  */
  0x2CU, /* GPIO_PIN_10 */
  0x2CU, /* GPIO_PIN_11 */
  0x2CU, /* GPIO_PIN_12 */
  0x2CU, /* GPIO_PIN_13 */
  0x2CU, /* GPIO_PIN_14 */
  0x2CU  /* GPIO_PIN_15 */
};

const uint16_t SHIFT_TAB_GPIO[] =
{
  0, /* GPIO_PIN_0  */
  0, /* GPIO_PIN_1  */
  0, /* GPIO_PIN_2  */
  0, /* GPIO_PIN_3  */
  0, /* GPIO_PIN_4  */
  0, /* GPIO_PIN_5  */
  0, /* GPIO_PIN_6  */
  0, /* GPIO_PIN_7  */
  8, /* GPIO_PIN_8  */
  8, /* GPIO_PIN_9  */
  8, /* GPIO_PIN_10 */
  8, /* GPIO_PIN_11 */
  8, /* GPIO_PIN_12 */
  8, /* GPIO_PIN_13 */
  8, /* GPIO_PIN_14 */
  8  /* GPIO_PIN_15 */
};

/**
  * @brief  Initializes the GPIOx peripheral according to the specified
  *         parameters in the GPIO_InitStruct.
  * @param  GPIOx: where x can be (A..G) to select the GPIO peripheral.
  * @param  GPIO_InitStruct: pointer to a GPIO_InitTypeDef structure that
  *         contains the configuration information for the specified GPIO peripheral.
  * @retval None
  */
void GPIO_Init(GPIO_TypeDef *GPIOx, GPIO_InitTypeDef *GPIO_InitStruct)
{
	if( GPIO_InitStruct->Pull != (uint32_t)0x00 )
	{
		GPIO_Set_INPUTUP(GPIOx, GPIO_InitStruct->Pin, GPIO_InitStruct->Pull);//set pull
	}
	switch( GPIO_InitStruct->Mode )
	{
		case GPIO_MODE_INPUT:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_DISABLE);//default:HiZ is closed when GPIO_MODE
			GPIO_Set_DIR(GPIOx, GPIO_InitStruct->Pin, GPIO_INPUT);          //set GPIO_INPUT
			GPIO_Set_PinMode(GPIOx, GPIO_InitStruct->Pin, GPIO_MODE);       //set GPIO_MODE
			break;
		case GPIO_MODE_OUTPUT_PP:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_DISABLE);//default:HiZ is closed when GPIO_MODE
			GPIO_Set_OutputMode(GPIOx, GPIO_InitStruct->Pin, GPIO_PUSHPULL);//set GPIO_PUSHPULL
			GPIO_Set_DIR(GPIOx, GPIO_InitStruct->Pin, GPIO_OUTPUT);         //set GPIO_OUTPUT
			GPIO_Set_PinMode(GPIOx, GPIO_InitStruct->Pin, GPIO_MODE);       //set GPIO_MODE
			break;
		case GPIO_MODE_OUTPUT_OD:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_DISABLE); //default:HiZ is closed when GPIO_MODE
			GPIO_Set_OutputMode(GPIOx, GPIO_InitStruct->Pin, GPIO_OPENDRAIN);//set GPIO_OPENDRAIN
			GPIO_Set_DIR(GPIOx, GPIO_InitStruct->Pin, GPIO_OUTPUT);          //set GPIO_OUTPUT
			GPIO_Set_PinMode(GPIOx, GPIO_InitStruct->Pin, GPIO_MODE);        //set GPIO_MODE			
			break;
		case GPIO_MODE_AF_PP:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_DISABLE);//default:HiZ is closed when AF_MODE
			GPIO_Set_OutputMode(GPIOx, GPIO_InitStruct->Pin, GPIO_PUSHPULL);//set GPIO_PUSHPULL
			GPIO_Set_PinAF(GPIOx, GPIO_InitStruct->Pin,GPIO_InitStruct->AF);//set af
			GPIO_Set_PinMode(GPIOx, GPIO_InitStruct->Pin, AF_MODE);         //set AF_MODE			
			break;
		case GPIO_MODE_AF_OD:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_DISABLE); //default:HiZ is closed when AF_MODE
			GPIO_Set_OutputMode(GPIOx, GPIO_InitStruct->Pin, GPIO_OPENDRAIN);//set GPIO_OPENDRAIN
			GPIO_Set_PinAF(GPIOx, GPIO_InitStruct->Pin,GPIO_InitStruct->AF); //set af
			GPIO_Set_PinMode(GPIOx, GPIO_InitStruct->Pin, AF_MODE);          //set AF_MODE
			break;
		case GPIO_MODE_HiZ:
			GPIO_Set_HIIPM(GPIOx, GPIO_InitStruct->Pin, GPIO_HIIPM_ENABLE);//set GPIO_HIIPM_ENABLE
			break;
		default:;
	}
}

/**
  * @brief  Initializes the GPIOx peripheral in the GPIO_InitStruct.
  * @param  GPIOx: where x can be (A..G) to select the GPIO peripheral.
  * @param  GPIO_InitStruct: pointer to a GPIO_InitTypeDef structure that
  *         contains the configuration information for the specified GPIO peripheral.
  * @retval None
  */
void GPIO_DeInit(GPIO_TypeDef *GPIOx)
{
	if(GPIOx == GPIOA)
	{GPIOA->IOCFG = 0x0003;GPIOA->PTOD = 0x00F3;}
	else if(GPIOx == GPIOB)
	{GPIOB->IOCFG = 0x0000;GPIOB->PTOD = 0x01A0;}
	else if(GPIOx == GPIOC)
	{GPIOC->IOCFG = 0x0000;GPIOC->PTOD = 0x065B;}
	else if(GPIOx == GPIOD)
	{GPIOD->IOCFG = 0x0000;GPIOD->PTOD = 0x018F;}
  
	GPIOx->AFCFG1 = 0x0000;
	GPIOx->PTDIR  = 0x0000;
	GPIOx->PUPDN  = 0xffffffff;
	GPIOx->PTDAT  = 0x0000;
  GPIOx->HIIPM  = 0x0000;
	GPIOx->AFCFG2 = 0x0000;
}

/**
  * @brief  Fills each GPIO_InitStruct member with its default value.
  * @param  GPIO_InitStruct : pointer to a GPIO_InitTypeDef structure which will be initialized.
  * @retval None
  */
void GPIO_StructInit(GPIO_InitTypeDef *GPIO_InitStruct)
{
  GPIO_InitStruct->Pin  = GPIO_PIN_0;
  GPIO_InitStruct->Mode = GPIO_MODE;
  GPIO_InitStruct->AF   = AF_FUNCTION1;
  GPIO_InitStruct->Pull = GPIO_FLOATING;
}

/**
  * @brief  Configure gpio mode for a dedicated pin on dedicated port.
  * @note   I/O mode can be GPIO,PIN.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll IOCFG        PT         GPIO_Set_PinMode
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  Mode This parameter can be one of the following values:
  *         @arg @ref GPIO_MODE
  *         @arg @ref AF_MODE

  * @retval None
  */
void GPIO_Set_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Mode)
{
  MODIFY_REG(GPIOx->IOCFG,(GPIO_READ_IOCFG<<Pin),(Mode<<Pin));
}

/**
  * @brief  Return gpio mode for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll IOCFG        PT         GPIO_Get_PinMode
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref GPIO_MODE
  *         @arg @ref AF_MODE
  */
uint16_t GPIO_Get_PinMode(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  return(READ_BIT(GPIOx->IOCFG,(GPIO_READ_IOCFG<<Pin))>>Pin);
}

/**
  * @brief  Configure gpio AF for a dedicated pin on dedicated port.
  * @note   I/O AF can be AF_FUNCTION1, AF_FUNCTION2 or AF_FUNCTION3.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll AFCFG1      SELx       GPIO_Set_PinAF
  * @rmtoll AFCFG2      SELx       GPIO_Set_PinAF
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  Sel_AF This parameter can be one of the following values:
  *         @arg @ref AF_FUNCTION1
  *         @arg @ref AF_FUNCTION2
  *         @arg @ref AF_FUNCTION3
  *         @arg @ref AF_FUNCTION4
  * @retval None
  */
void GPIO_Set_PinAF(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t Sel_AF)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&GPIOx->AFCFG1) + OFFSET_TAB_AFCFGx[Pin]);
  MODIFY_REG(*pReg, (AF_READ_FUNCTION << ((Pin - SHIFT_TAB_GPIO[Pin]) * 2U)), Sel_AF << ((Pin - SHIFT_TAB_GPIO[Pin]) * 2U));
}

/**
  * @brief  Return gpio AF for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll AFCFG1      SELx       GPIO_Get_PinAF
  * @rmtoll AFCFG2      SELx       GPIO_Get_PinAF
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref AF_FUNCTION1
  *         @arg @ref AF_FUNCTION2
  *         @arg @ref AF_FUNCTION3
  *         @arg @ref AF_FUNCTION4
  */
uint16_t GPIO_Get_PinAF(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  register uint32_t *pReg = (uint32_t *)((uint32_t)(&GPIOx->AFCFG1) + OFFSET_TAB_AFCFGx[Pin]);
  return (READ_BIT(*pReg, (AF_READ_FUNCTION << ((Pin - SHIFT_TAB_GPIO[Pin]) * 2U))) >> ((Pin - SHIFT_TAB_GPIO[Pin]) * 2U));
}

/**
  * @brief  Configure gpio DIR  for a dedicated pin on dedicated port.
  * @note   I/O mode can be GPIO_INPUT,GPIO_OUTPUT.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTDIR        PT         GPIO_Set_DIR
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  DIR This parameter can be one of the following values:
  *         @arg @ref GPIO_INPUT
  *         @arg @ref GPIO_OUTPUT
  * @retval None
  */
void GPIO_Set_DIR(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t DIR)
{
  MODIFY_REG(GPIOx->PTDIR, (GPIO_READ_PTDIR << Pin), (DIR << Pin));
}

/**
  * @brief  Return gpio DIR for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTDIR        PT         GPIO_Get_DIR
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref GPIO_INPUT
  *         @arg @ref GPIO_OUTPUT
  */
uint16_t GPIO_Get_DIR(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  return (READ_BIT(GPIOx->PTDIR, (GPIO_READ_PTDIR << Pin)) >> Pin);
}

/**
  * @brief  Configure gpio input Type for a dedicated pin on dedicated port.
  * @note   I/O mode can be GPIO_INPUT,GPIO_OUTPUT.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PUPDN        PT         GPIO_Set_INPUTUP
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  UP This parameter can be one of the following values:
  *         @arg @ref GPIO_PULL_UP
  *         @arg @ref GPIO_PULL_DOWN
  *         @arg @ref GPIO_FLOATING
  * @retval None
  */
void GPIO_Set_INPUTUP(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t UpDn)
{
  MODIFY_REG(GPIOx->PUPDN, (GPIO_READ_PUPDN << Pin), (UpDn << Pin));
}

/**
  * @brief  Return gpio input Type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PUPDN        PT         GPIO_Get_INPUTUP
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref GPIO_PULL_UP
  *         @arg @ref GPIO_PULL_DOWN
  *         @arg @ref GPIO_FLOATING
  */
uint32_t GPIO_Get_INPUTUP(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  return (READ_BIT(GPIOx->PUPDN, (GPIO_READ_PUPDN << Pin)) >> Pin);
}

/**
  * @brief  Return gpio output Type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTDAT        PT         GPIO_Get_PTDAT
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref 0
  *         @arg @ref 1
  */
uint16_t GPIO_Get_PTDAT(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  return (READ_BIT(GPIOx->PTDAT, (GPIO_READ_PTDAT << Pin)) >> Pin);
}

/**
  * @brief  Set gpio output type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTSET        PT         GPIO_Set_Output
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval None
  */
void GPIO_Set_Output(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  SET_BIT(GPIOx->PTSET, (GPIO_SETOUTPUT << Pin));
}

/**
  * @brief  Set gpio output type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTCLR        PT         GPIO_CLR_Output
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval None
  */
void GPIO_Clear_Output(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  SET_BIT(GPIOx->PTCLR, (GPIO_CLROUTPUT << Pin));
}

/**
  * @brief  Set gpio output type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTTOG        PT         GPIO_TOG_Output
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval None
  */
void GPIO_Toggle_Output(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  SET_BIT(GPIOx->PTTOG, (GPIO_TOGOUTPUT << Pin));
}

/**
  * @brief  Configure gpio output mode  for a dedicated pin on dedicated port.
  * @note   I/O mode can be GPIO_OUTPUT_OPENDRAIN,GPIO_OUTPUT_PUSHPULL.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTOD       PT         GPIO_Set_OUTPUTMODE
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  output_mode This parameter can be one of the following values:
  *         @arg @ref GPIO_OPENDRAIN
  *         @arg @ref GPIO_PUSHPULL
  * @retval None
  */
void GPIO_Set_OutputMode(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t output_mode)
{
  MODIFY_REG(GPIOx->PTOD, (GPIO_READ_PTOD << Pin), (output_mode << Pin));
}

/**
  * @brief  Return gpio output Type for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll PTOD       PT         GPIO_Get_OUTPUTMODE
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref GPIO_PENDRAIN
  *         @arg @ref GPIO_PUSHPULL
  */
uint16_t GPIO_Get_OutputMode(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
  return (READ_BIT(GPIOx->PTOD, (GPIO_READ_PTOD << Pin)) >> Pin);
}

/**
  * @brief  Configure gpio HIIPM mode  for a dedicated pin on dedicated port.
  * @note   I/O mode can be GPIO_HIIPM_DISABLE,GPIO_HIIPM_ENABLE.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll HIIPM       Hiipm        GPIO_Set_HIIPM
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @param  HIIPM This parameter can be one of the following values:
  *         @arg @ref GPIO_HIIPM_DISABLE
  *         @arg @ref GPIO_HIIPM_ENABLE
  * @retval None
  */
void GPIO_Set_HIIPM(GPIO_TypeDef *GPIOx, uint32_t Pin, uint32_t HIIPM)
{
 MODIFY_REG(GPIOx->HIIPM, (GPIO_READ_HIIPM << Pin), (HIIPM << Pin));
}

/**
  * @brief  Return gpio HIIPM mode for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll HIIPM       Hiipm        GPIO_Get_HIIPM
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref GPIO_HIIPM_DISABLE
  *         @arg @ref GPIO_HIIPM_ENABLE
  */
uint16_t GPIO_Get_HIIPM(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
 return (READ_BIT(GPIOx->HIIPM, (GPIO_READ_HIIPM << Pin)) >> Pin);
}

/**
  * @brief  Return gpio level status for a dedicated pin on dedicated port.
  * @note   Warning: only one pin can be passed as parameter.
  * @rmtoll IDR       idr        GPIO_Get_IDR
  * @param  GPIOx GPIO Port
  * @param  Pin This parameter can be one of the following values:
  *         @arg @ref GPIO_PIN_0
  *         @arg @ref GPIO_PIN_1
  *         @arg @ref GPIO_PIN_2
  *         @arg @ref GPIO_PIN_3
  *         @arg @ref GPIO_PIN_4
  *         @arg @ref GPIO_PIN_5
  *         @arg @ref GPIO_PIN_6
  *         @arg @ref GPIO_PIN_7
  *         @arg @ref GPIO_PIN_8
  *         @arg @ref GPIO_PIN_9
  *         @arg @ref GPIO_PIN_10
  *         @arg @ref GPIO_PIN_11
  *         @arg @ref GPIO_PIN_12
  *         @arg @ref GPIO_PIN_13
  *         @arg @ref GPIO_PIN_14
  *         @arg @ref GPIO_PIN_15
  * @retval Returned value can be one of the following values:
  *         @arg @ref 0
  *         @arg @ref 1
  */
uint16_t GPIO_Get_IDR(GPIO_TypeDef *GPIOx, uint32_t Pin)
{
 return (READ_BIT(GPIOx->IDR, (GPIO_READ_IDR << Pin)) >> Pin);
}
