/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_me.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define  __rx32s11xx_ME_C

#include "rx32s11xx_me.h"


/**
  * @brief  Deinitializes the MEx peripheral registers to their default reset values.
  * @param  MEx: where x can be 0, 1 to select the 
  *         ME peripheral.
  * @retval None
  */
void ME_DeInit(ME_TypeDef* MEx)
{ 
    MEx->CR = 0;                    
	  MEx->Ia = 0;                    
	  MEx->Ib = 0;                    
	  MEx->Ic = 0;                   
	  MEx->Ialpha = 0;                
    MEx->Ibeta = 0;                 
	  MEx->Iq = 0;                   
	  MEx->Id = 0;                    
	  MEx->Vq = 0;                   
	  MEx->Vd = 0;                   
    MEx->Valpha = 0;                
	  MEx->Vbeta = 0;                 
	  MEx->Angle = 0;                 
	  MEx->Cos = 0;                   
	  MEx->Sin = 0;                  
	  MEx->IPDSin = 0;               
   	MEx->Kslid = 0;                 
	  MEx->Kslf = 0;                 
	  MEx->KF = 0;                    
	  MEx->KG = 0;                    
	  MEx->E0 = 0;                   
	  MEx->Ealpha = 0;               
	  MEx->Ebeta = 0;                
	  MEx->Zalpha = 0;             
    MEx->Zbeta = 0;                
	  MEx->IalphaError = 0;          
    MEx->IbetaError = 0;          
	  MEx->EstIalpha = 0;            
	  MEx->EstIbeta = 0;             
	  MEx->SMOTheta = 0;             
	  MEx->SMOIalpha = 0;            
	  MEx->SMOIbeta = 0;             
   	MEx->BEMFA = 0;               
	  MEx->BEMFB = 0;                
	  MEx->IPDTheta = 0;                             
	  MEx->SVZone = 0;                
	  MEx->FOVM = 0;                
	  MEx->TPWM = 0;                 
	  MEx->LSMIN = 0;               
    MEx->PDCH1 = 0;                
	  MEx->PDCH2 = 0;                
	  MEx->PDCH3 = 0;               
		MEx->IER = 0;                 
	  MEx->IFR = 0;                  	
	  MEx->MCYC = 0;                 
}


/**
  * @function  ME_SQRT
  * @brief  SQRT function
  * @param  m: input number
  * @retval output number
  */
uint16_t ME_SQRT(uint32_t m)
{
	CLEAR_BIT(SQRT->CR,SQRT_CR_RDY);
	SQRT->DI = m;
	while((SQRT->CR & SQRT_CR_RDY)==0);
	return SQRT->DO;
}

/**
  * @function  ME_Config_IT
  * @brief  ME interrupt config of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IT:
            @arg ME_IER_DIVFE
            @arg ME_IER_PID1E
            @arg ME_IER_PID2E
            @arg ME_IER_PID3E
            @arg ME_IER_CDTRE
            @arg ME_IER_IPDE
            @arg ME_IER_SMOE
            @arg ME_IER_SVE
						@arg ME_IER_F4E
  * @retval None
  */
void ME_Config_IT(ME_TypeDef *MEx,uint16_t ME_IT, FunctionalState NewState)
{
    /*  asseparam  */
    if (NewState == DISABLE)
    {
	    CLEAR_BIT(MEx->IER,ME_IT); //Disable
    }
    else
    {
	    SET_BIT(MEx->IER,ME_IT);  //Enable
    }
}

/**
  * @function  ME_Get_ITStatus
  * @brief  read complete interrupt status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IT:
            @arg ME_IER_DIVFE
            @arg ME_IER_PID1E
            @arg ME_IER_PID2E
            @arg ME_IER_PID3E
            @arg ME_IER_CDTRE
            @arg ME_IER_IPDE
            @arg ME_IER_SMOE
            @arg ME_IER_SVE
						@arg ME_IER_F4E
  * @retval State of bit (1 or 0).
  */
uint32_t ME_Get_ITStatus(ME_TypeDef *MEx,uint16_t ITEn)
{
  return (READ_BIT(MEx->IER, ITEn) == (ITEn));
}

/**
  * @function  ME_Get_IFStatus
  * @brief  read complete interrupt Flag status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IF:
            @arg ME_IFR_DIVFF
            @arg ME_IFR_PID1F
            @arg ME_IFR_PID2F
            @arg ME_IFR_PID3F
            @arg ME_IFR_CDTRF
            @arg ME_IFR_IPDF
            @arg ME_IFR_SMOF
            @arg ME_IFR_SVF
						@arg ME_IFR_F4F
  * @retval State of bit (1 or 0).
  */
uint32_t ME_Get_IFStatus(ME_TypeDef *MEx,uint32_t IFlag)
{
  return (READ_BIT(MEx->IFR, IFlag) == (IFlag));
}

/**
  * @function  ME_Clear_IF
  * @brief  read complete interrupt Flag status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IF:
            @arg ME_IFR_DIVFF
            @arg ME_IFR_PID1F
            @arg ME_IFR_PID2F
            @arg ME_IFR_PID3F
            @arg ME_IFR_CDTRF
            @arg ME_IFR_IPDF
            @arg ME_IFR_SMOF
            @arg ME_IFR_SVF
						@arg ME_IFR_F4F
  * @retval None
  */
void ME_Clear_IF(ME_TypeDef *MEx,uint32_t IFlag)
{
  CLEAR_BIT(MEx->IFR, IFlag);
}

/**
  * @function  ME_SET_STR
  * @brief  SET STR of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_STR:
            @arg ME_CR_CDTR_STR
            @arg ME_CR_IPD_STR
            @arg ME_CR_SMO_STR
            @arg ME_CR_SV_STR
            @arg ME_CR_F4_STR
  * @retval None
  */
void ME_SET_STR(ME_TypeDef *MEx,uint16_t ME_STR)
{
    SET_BIT(MEx->CR, ME_STR);
}

/**
  * @function  ME_CLEAR_RDY
  * @brief  clear RDY Flag status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IF:
            @arg ME_CR_CDTR_RDY
            @arg ME_CR_IPD_RDY
            @arg ME_CR_SMO_RDY
            @arg ME_CR_SV_RDY
  * @retval None
  */
void ME_CLEAR_RDY(ME_TypeDef *MEx,uint32_t RDY)
{
  CLEAR_BIT(MEx->CR, RDY);
}

/**
  * @function  ME_Get_RDYStatus
  * @brief  read complete RDY Flag status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IF:
            @arg ME_CR_CDTR_RDY
            @arg ME_CR_IPD_RDY
            @arg ME_CR_SMO_RDY
            @arg ME_CR_SV_RDY
  * @retval State of bit (1 or 0).
  */
uint32_t ME_Get_RDYStatus(ME_TypeDef *MEx,uint32_t RDY)
{
  return (READ_BIT(MEx->CR, RDY) == (RDY));
}

/**
  * @function  ME_Get_BUSYStatus
  * @brief  read complete BUSY Flag status of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_IF:
            @arg ME_CR_CDTR_BUSY
            @arg ME_CR_IPD_BUSY
            @arg ME_CR_SMO_BUSY
            @arg ME_CR_SV_BUSY
  * @retval State of bit (1 or 0).
  */
uint32_t ME_Get_BUSYStatus(ME_TypeDef *MEx,uint32_t BUSY)
{
  return (READ_BIT(MEx->IFR, BUSY) == (BUSY));
}

/**
  * @function  ME_SET_FIVE
  * @brief  set sv_five of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  ME_FIVE:
            @arg  ME_CR_SV_FIVE_NoRightShift                  ((uint32_t)0x20000)
            @arg  ME_CR_SV_FIVE_RightShiftOneBit              ((uint32_t)0x00000)
  * @retval None
  */
void ME_SET_FIVE( ME_TypeDef *MEx,uint16_t ME_FIVE )
{
	MODIFY_REG(MEx->CR, ME_READ_CR_SV_FIVE, ME_FIVE);
}

/**
  * @function  ME_SELECT_CDTRCH
  * @brief  Select CDTR Channel of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  Channel:
            @arg ME_CR_CDTR_CH0
            @arg ME_CR_CDTR_CH1
            @arg ME_CR_CDTR_CH2
            @arg ME_CR_CDTR_CH3
            @arg ME_CR_CDTR_CH4
            @arg ME_CR_CDTR_CH5
            @arg ME_CR_CDTR_CH6
            @arg ME_CR_CDTR_CH7
  * @retval None
  */
void ME_SELECT_CDTRCH(ME_TypeDef *MEx,uint32_t Channel)
{
  MODIFY_REG(MEx->CR, ME_CR_CDTR_CH, Channel);
}

/**
  * @function  ME_CLR_SMO
  * @brief  CLAER SMO of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @retval None
  */
void ME_CLR_SMO(ME_TypeDef *MEx)
{
  SET_BIT(MEx->CR, ME_CR_SMO_CLR);
}

/**
  * @function  ME_IsRDY_SQRT
  * @brief  Get RDY state of the selected SQRT 
  * @param  SQRTx:
            @arg SQRT Instance
  * @retval 0:SQRT_RDY is inactive, 1:SQRT_RDY is active
  */
uint32_t ME_IsRDY_SQRT(SQRT_TypeDef *SQRTx)
{
  return (READ_BIT(SQRTx->CR, SQRT_CR_RDY) == (SQRT_CR_RDY));
}

/**
  * @function  ME_CLEAR_RDY_SQRT
  * @brief  CLAER DIV_RDY of the selected SQRT
  * @param  SQRTx:
            @arg SQRT Instance
  * @retval None
  */
void ME_CLEAR_RDY_SQRT(SQRT_TypeDef *SQRTx)
{
  CLEAR_BIT(SQRTx->CR, SQRT_CR_RDY);
}

/**
  * @function  ME_STR_PID
  * @brief  STR PID of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval None
  */
void ME_STR_PID(PID_TypeDef *PIDx)
{
  SET_BIT(PIDx->CR, PID_CR_STR);
}

/**
  * @function  ME_CLR_PID
  * @brief  CLAER PID of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval None
  */
void ME_CLR_PID(PID_TypeDef *PIDx)
{
  SET_BIT(PIDx->CR, PID_CR_CLR);
}

/**
  * @function  ME_IsRDY_PID
  * @brief  Get RDY state of the selected PID 
  * @param  PIDx:
            @arg PID Instance
  * @retval 0:PID_RDY is inactive, 1:PID_RDY is active
  */
uint32_t ME_IsRDY_PID(PID_TypeDef *PIDx)
{
  return (READ_BIT(PIDx->CR, PID_CR_RDY) == (PID_CR_RDY));
}

/**
  * @function  ME_CLEAR_RDY_PID
  * @brief  CLAER PID_RDY of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval None
  */
void ME_CLEAR_RDY_PID(PID_TypeDef *PIDx)
{
  CLEAR_BIT(PIDx->CR, PID_CR_RDY);
}

/**
  * @function  ME_Write_Ia
  * @brief  Set Ia of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ia(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ia, ME_Ia, value);
}

/**
  * @function  ME_Read_Ia
  * @brief  Get Ia value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ia.
  */
uint32_t ME_Read_Ia(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ia));
}

/**
  * @function  ME_Write_Ib
  * @brief  Set Ib of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ib(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ib, ME_Ib, value);
}

/**
  * @function  ME_Read_Ib
  * @brief  Get Ib value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ib.
  */
uint32_t ME_Read_Ib(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ib));
}

/**
  * @function  ME_Write_Ic
  * @brief  Set Ic of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ic(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ic, ME_Ic, value);
}

/**
  * @function  ME_Read_Ic
  * @brief  Get Ic value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ic.
  */
uint32_t ME_Read_Ic(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ic));
}

/**
  * @function  ME_Write_Ialpha
  * @brief  Set Ialpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ialpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ialpha, ME_Ialpha, value);
}

/**
  * @function  ME_Read_Ialpha
  * @brief  Get Ialpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ialpha.
  */
uint32_t ME_Read_Ialpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ialpha));
}

/**
  * @function  ME_Write_Ibeta
  * @brief  Set Ibeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ibeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ibeta, ME_Ibeta, value);
}

/**
  * @function  ME_Read_Ibeta
  * @brief  Get Ibeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ibeta.
  */
uint32_t ME_Read_Ibeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ibeta));
}

/**
  * @function  ME_Write_Iq
  * @brief  Set Iq of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Iq(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Iq, ME_Iq, value);
}

/**
  * @function  ME_Read_Iq
  * @brief  Get Iq value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Iq.
  */
uint32_t ME_Read_Iq(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Iq));
}

/**
  * @function  ME_Write_Id
  * @brief  Set Id of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Id(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Id, ME_Id, value);
}

/**
  * @function  ME_Read_Id
  * @brief  Get Id value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Id.
  */
uint32_t ME_Read_Id(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Id));
}

/**
  * @function  ME_Write_Vq
  * @brief  Set Vq of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Vq(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Vq, ME_Vq, value);
}

/**
  * @function  ME_Read_Vq
  * @brief  Get Vq value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Vq.
  */
uint32_t ME_Read_Vq(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Vq));
}

/**
  * @function  ME_Write_Vd
  * @brief  Set Vd of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Vd(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Vd, ME_Vd, value);
}

/**
  * @function  ME_Read_Vd
  * @brief  Get Vd value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Vd.
  */
uint32_t ME_Read_Vd(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Vd));
}

/**
  * @function  ME_Write_Valpha
  * @brief  Set Valpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Valpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Valpha, ME_Valpha, value);
}

/**
  * @function  ME_Read_Valpha
  * @brief  Get Valpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Valpha.
  */
uint32_t ME_Read_Valpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Valpha));
}

/**
  * @function  ME_Write_Vbeta
  * @brief  Set Vbeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Vbeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Vbeta, ME_Vbeta, value);
}

/**
  * @function  ME_Read_Vbeta
  * @brief  Get Vbeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Vbeta.
  */
uint32_t ME_Read_Vbeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Vbeta));
}

/**
  * @function  ME_Write_Angle
  * @brief  Set Angle of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Angle(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Angle, ME_Angle, value);
  __asm {
		nop;
   }
}

/**
  * @function  ME_Read_Angle
  * @brief  Get Angle value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Angle.
  */
uint32_t ME_Read_Angle(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Angle));
}

/**
  * @function  ME_Read_Cos
  * @brief  Get Cos value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Cos.
  */
uint32_t ME_Read_Cos(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Cos));
}

/**
  * @function  ME_Read_Sin
  * @brief  Get Sin value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Sin.
  */
uint32_t ME_Read_Sin(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Sin));
}

/**
  * @function  ME_Read_IPDSin
  * @brief  Get IPDSin value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of IPDSin.
  */
uint32_t ME_Read_IPDSin(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->IPDSin));
}

/**
  * @function  ME_Write_Kslid
  * @brief  Set Kslid of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Kslid(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Kslid, ME_Kslid, value);
}

/**
  * @function  ME_Read_Kslid
  * @brief  Get Kslid value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Kslid.
  */
uint32_t ME_Read_Kslid(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Kslid));
}

/**
  * @function  ME_Write_Kslf
  * @brief  Set Kslf of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Kslf(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Kslf, ME_Kslf, value);
}

/**
  * @function  ME_Read_Kslf
  * @brief  Get Kslf value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Kslf.
  */
uint32_t ME_Read_Kslf(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Kslf));
}

/**
  * @function  ME_Write_KF
  * @brief  Set KF of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_KF(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->KF, ME_KF, value);
}

/**
  * @function  ME_Read_KF
  * @brief  Get KF value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of KF.
  */
uint32_t ME_Read_KF(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->KF));
}

/**
  * @function  ME_Write_KG
  * @brief  Set KG of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_KG(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->KG, ME_KG, value);
}

/**
  * @function  ME_Read_KG
  * @brief  Get KG value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of KG.
  */
uint32_t ME_Read_KG(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->KG));
}

/**
  * @function  ME_Write_E0
  * @brief  Set E0 of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_E0(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->E0, ME_E0, value);
}

/**
  * @function  ME_Read_E0
  * @brief  Get E0 value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of E0.
  */
uint32_t ME_Read_E0(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->E0));
}

/**
  * @function  ME_Write_Ealpha
  * @brief  Set Ealpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ealpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ealpha, ME_Ealpha, value);
}

/**
  * @function  ME_Read_Ealpha
  * @brief  Get Ealpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ealpha.
  */
uint32_t ME_Read_Ealpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ealpha));
}

/**
  * @function  ME_Write_Ebeta
  * @brief  Set Ebeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Ebeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Ebeta, ME_Ebeta, value);
}

/**
  * @function  ME_Read_Ebeta
  * @brief  Get Ebeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Ebeta.
  */
uint32_t ME_Read_Ebeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Ebeta));
}

/**
  * @function  ME_Write_Zalpha
  * @brief  Set Zalpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Zalpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Zalpha, ME_Zalpha, value);
}

/**
  * @function  ME_Read_Zalpha
  * @brief  Get Zalpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Zalpha.
  */
uint32_t ME_Read_Zalpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Zalpha));
}

/**
  * @function  ME_Write_Zbeta
  * @brief  Set Zbeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_Zbeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->Zbeta, ME_Zbeta, value);
}

/**
  * @function  ME_Read_Zbeta
  * @brief  Get Zbeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of Zbeta.
  */
uint32_t ME_Read_Zbeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->Zbeta));
}

/**
  * @function  ME_Write_IalphaError
  * @brief  Set IalphaError of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_IalphaError(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->IalphaError, ME_IalphaError, value);
}

/**
  * @function  ME_Read_IalphaError
  * @brief  Get IalphaError value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of IalphaError.
  */
uint32_t ME_Read_IalphaError(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->IalphaError));
}

/**
  * @function  ME_Write_IbetaError
  * @brief  Set IbetaError of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_IbetaError(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->IbetaError, ME_IbetaError, value);
}

/**
  * @function  ME_Read_IbetaError
  * @brief  Get IbetaError value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of IbetaError.
  */
uint32_t ME_Read_IbetaError(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->IbetaError));
}

/**
  * @function  ME_Write_EstIalpha
  * @brief  Set EstIalpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_EstIalpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->EstIalpha, ME_EstIalpha, value);
}

/**
  * @function  ME_Read_EstIalpha
  * @brief  Get EstIalpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of EstIalpha.
  */
uint32_t ME_Read_EstIalpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->EstIalpha));
}

/**
  * @function  ME_Write_EstIbeta
  * @brief  Set EstIbeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_EstIbeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->EstIbeta, ME_EstIbeta, value);
}

/**
  * @function  ME_Read_EstIbeta
  * @brief  Get EstIbeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of EstIbeta.
  */
uint32_t ME_Read_EstIbeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->EstIbeta));
}

/**
  * @function  ME_Write_SMOTheta
  * @brief  Set SMOTheta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_SMOTheta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->SMOTheta, ME_SMOTheta, value);
}

/**
  * @function  ME_Read_SMOTheta
  * @brief  Get SMOTheta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of SMOTheta.
  */
uint32_t ME_Read_SMOTheta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->SMOTheta));
}

/**
  * @function  ME_Write_SMOIalpha
  * @brief  Set SMOIalpha of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_SMOIalpha(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->SMOIalpha, ME_SMOIalpha, value);
}

/**
  * @function  ME_Read_SMOIalpha
  * @brief  Get SMOIalpha value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of SMOIalpha.
  */
uint32_t ME_Read_SMOIalpha(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->SMOIalpha));
}

/**
  * @function  ME_Write_SMOIbeta
  * @brief  Set SMOIbeta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_SMOIbeta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->SMOIbeta, ME_SMOIbeta, value);
}

/**
  * @function  ME_Read_SMOIbeta
  * @brief  Get SMOIbeta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of SMOIbeta.
  */
uint32_t ME_Read_SMOIbeta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->SMOIbeta));
}

/**
  * @function  ME_Write_BEMFA
  * @brief  Set BEMFA of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_BEMFA(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->BEMFA, ME_BEMFA, value);
}

/**
  * @function  ME_Read_BEMFA
  * @brief  Get BEMFA value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of BEMFA.
  */
uint32_t ME_Read_BEMFA(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->BEMFA));
}

/**
  * @function  ME_Write_BEMFB
  * @brief  Set BEMFB of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_BEMFB(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->BEMFB, ME_BEMFB, value);
}

/**
  * @function  ME_Read_BEMFB
  * @brief  Get BEMFB value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of BEMFB.
  */
uint32_t ME_Read_BEMFB(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->BEMFB));
}

/**
  * @function  ME_Write_IPDTheta
  * @brief  Set IPDTheta of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_IPDTheta(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->IPDTheta, ME_IPDTheta, value);
}

/**
  * @function  ME_Read_IPDTheta
  * @brief  Get IPDTheta value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of IPDTheta.
  */
uint32_t ME_Read_IPDTheta(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->IPDTheta));
}

/**
  * @function  ME_Write_SVZone
  * @brief  Set SVZone of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_SVZone(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->SVZone, ME_SVZone, value);
}

/**
  * @function  ME_Read_SVZone
  * @brief  Get SVZone value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of SVZone.
  */
uint32_t ME_Read_SVZone(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->SVZone));
}

/**
  * @function  ME_Write_FOVM
  * @brief  Set FOVM of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_FOVM(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->FOVM, ME_FOVM, value);
}

/**
  * @function  ME_Read_FOVM
  * @brief  Get FOVM value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of FOVM.
  */
uint32_t ME_Read_FOVM(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->FOVM));
}

/**
  * @function  ME_Write_TPWM
  * @brief  Set TPWM of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_TPWM(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->TPWM, ME_TPWM, value);
}

/**
  * @function  ME_Read_TPWM
  * @brief  Get TPWM value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of TPWM.
  */
uint32_t ME_Read_TPWM(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->TPWM));
}

/**
  * @function  ME_Write_LSMIN
  * @brief  Set LSMIN of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_LSMIN(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->LSMIN, ME_LSMIN, value);
}

/**
  * @function  ME_Read_LSMIN
  * @brief  Get LSMIN value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of LSMIN.
  */
uint32_t ME_Read_LSMIN(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->LSMIN));
}

/**
  * @function  ME_Write_PDCH1
  * @brief  Set PDCH1 of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PDCH1(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->PDCH1, ME_PDCH1, value);
}

/**
  * @function  ME_Read_PDCH1
  * @brief  Get PDCH1 value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of PDCH1.
  */
uint32_t ME_Read_PDCH1(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->PDCH1));
}

/**
  * @function  ME_Write_PDCH2
  * @brief  Set PDCH2 of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PDCH2(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->PDCH2, ME_PDCH2, value);
}

/**
  * @function  ME_Read_PDCH2
  * @brief  Get PDCH2 value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of PDCH2.
  */
uint32_t ME_Read_PDCH2(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->PDCH2));
}

/**
  * @function  ME_Write_PDCH3
  * @brief  Set PDCH3 of the selected ME
  * @param  MEx:
            @arg ME Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PDCH3(ME_TypeDef *MEx, uint32_t value)
{
  MODIFY_REG(MEx->PDCH3, ME_PDCH3, value);
}

/**
  * @function  ME_Read_PDCH3
  * @brief  Get PDCH3 value of the selected ADC
  * @param  MEx:
            @arg ME Instance
  * @retval value of PDCH3.
  */
uint32_t ME_Read_PDCH3(ME_TypeDef *MEx)
{
  return (READ_REG(MEx->PDCH3));
}

/**
  * @function  ME_Write_PID_REF
  * @brief  Set REF of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_REF(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->REF, PID_REF, value);
}

/**
  * @function  ME_Read_PID_REF
  * @brief  Get REF value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of REF.
  */
uint32_t ME_Read_PID_REF(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->REF));
}

/**
  * @function  ME_Write_PID_FB
  * @brief  Set FB of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_FB(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->FB, PID_FB, value);
}

/**
  * @function  ME_Read_PID_FB
  * @brief  Get FB value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of FB.
  */
uint32_t ME_Read_PID_FB(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->FB));
}

/**
  * @function  ME_Read_PID_OUT
  * @brief  Get OUT value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of OUT.
  */
uint32_t ME_Read_PID_OUT(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->OUT));
}

/**
  * @function  ME_Read_PID_ERR
  * @brief  Get ERR value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of ERR.
  */
uint32_t ME_Read_PID_ERR(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->ERR));
}

/**
  * @function  ME_Read_PID_INTG
  * @brief  Get INTG value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of INTG.
  */
uint32_t ME_Read_PID_INTG(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->INTG));
}

/**
  * @function  ME_Write_PID_KPG
  * @brief  Set KPG of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KPG(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KPG, PID_KPG, value);
}

/**
  * @function  ME_Read_PID_KPG
  * @brief  Get KPG value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of KPG.
  */
uint32_t ME_Read_PID_KPG(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->KPG));
}

/**
  * @function  ME_Write_PID_KIG
  * @brief  Set KIG of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KIG(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KIG, PID_KIG, value);
}

/**
  * @function  ME_Read_PID_KIG
  * @brief  Get KIG value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of KIG.
  */
uint32_t ME_Read_PID_KIG(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->KIG));
}

/**
  * @function  ME_Write_PID_KDG
  * @brief  Set KDG of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KDG(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KDG, PID_KDG, value);
}

/**
  * @function  ME_Read_PID_KDG
  * @brief  Get KDG value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of KDG.
  */
uint32_t ME_Read_PID_KDG(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->KDG));
}

/**
  * @function  ME_Write_PID_KIMULP
  * @brief  Set KIMULP of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KIMULP(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KIMULP, PID_KIMULP, value);
}

/**
  * @function  ME_Read_PID_KIMULP
  * @brief  Get KIMULP value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of KIMULP.
  */
uint32_t ME_Read_PID_KIMULP(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->KIMULP));
}

/**
  * @function  ME_Write_PID_KDIV
  * @brief  Set KDIV of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KDIV(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KDIV, PID_KDIV, value);
}

/**
  * @function  ME_Read_PID_KDIV
  * @brief  Get KDIV value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of KDIV.
  */
uint32_t ME_Read_PID_KDIV(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->KDIV));
}

/**
  * @function  ME_Write_PID_KDIV_KPD
  * @brief  Set KDIV_KPD of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KDIV_KPD(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KDIV, PID_KDIV_KPD, value);
}

/**
  * @function  ME_Write_PID_KDIV_KID
  * @brief  Set KDIV_KID of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KDIV_KID(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KDIV, PID_KDIV_KID, value << 4);
}

/**
  * @function  ME_Write_PID_KDIV_KDD
  * @brief  Set KDIV_KDD of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_KDIV_KDD(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->KDIV, PID_KDIV_KDD, value << 8);
}

/**
  * @function  ME_Write_PID_INTGLIM
  * @brief  Set INTGLIM of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_INTGLIM(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->INTGLIM, PID_INTGLIM, value);
}

/**
  * @function  ME_Read_PID_INTGLIM
  * @brief  Get INTGLIM value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of INTGLIM.
  */
uint32_t ME_Read_PID_INTGLIM(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->INTGLIM));
}

/**
  * @function  ME_Write_PID_OUTLIM
  * @brief  Set OUTLIM of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_PID_OUTLIM(PID_TypeDef *PIDx, uint32_t value)
{
  MODIFY_REG(PIDx->OUTLIM, PID_OUTLIM, value);
}

/**
  * @function  ME_Read_PID_OUTLIM
  * @brief  Get OUTLIM value of the selected PID
  * @param  PIDx:
            @arg PID Instance
  * @retval value of OUTLIM.
  */
uint32_t ME_Read_PID_OUTLIM(PID_TypeDef *PIDx)
{
  return (READ_REG(PIDx->OUTLIM));
}

/**
  * @function  ME_Write_SQRT_DI
  * @brief  Set DI of the selected SQRT
  * @param  SQRTx:
            @arg SQRT Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_SQRT_DI(SQRT_TypeDef *SQRTx, uint32_t value)
{
  MODIFY_REG(SQRTx->DI, SQRT_DI, value);
}

/**
  * @function  ME_Read_SQRT_DI
  * @brief  Get DI value of the selected SQRT
  * @param  SQRTx:
            @arg SQRT Instance
  * @retval value of DI.
  */
uint32_t ME_Read_SQRT_DI(SQRT_TypeDef *SQRTx)
{
  return (READ_REG(SQRTx->DI));
}

/**
  * @function  ME_Read_SQRT_DO
  * @brief  Get DO value of the selected SQRT
  * @param  SQRTx:
            @arg SQRT Instance
  * @retval value of DO.
  */
uint32_t ME_Read_SQRT_DO(SQRT_TypeDef *SQRTx)
{
  return (READ_REG(SQRTx->DO));
}


/**
  * @function  ME_Write_DIV_DD
  * @brief  Set DD of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_DIV_DD(DIV_TypeDef *DIVx, uint32_t value)
{
  MODIFY_REG(DIVx->DD, DIV_DD, value);
}

/**
  * @function  ME_Read_DIV_DD
  * @brief  Get DD value of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval value of DD.
  */
uint32_t ME_Read_DIV_DD(DIV_TypeDef *DIVx)
{
  return (READ_REG(DIVx->DD));
}


/**
  * @function  ME_Write_DIV_DI
  * @brief  Set DI of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @param  value:
  *         @ref value
  * @retval None
  */
void ME_Write_DIV_DI(DIV_TypeDef *DIVx, uint32_t value)
{
  MODIFY_REG(DIVx->DI, DIV_DI, value);
}

/**
  * @function  ME_Read_DIV_DI
  * @brief  Get DI value of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval value of DI.
  */
uint32_t ME_Read_DIV_DI(DIV_TypeDef *DIVx)
{
  return (READ_REG(DIVx->DI));
}


/**
  * @function  ME_CLEAR_RDY_DIV
  * @brief  CLAER DIV_RDY of the selected DIV
  * @param  MEx:
            @arg ME Instance
  * @retval None
  */
void ME_CLEAR_RDY_DIV(DIV_TypeDef *DIVx)
{
  CLEAR_BIT(DIVx->CR, DIV_CR_RDY);
}

/**
  * @function  ME_SET_DIVUSN
  * @brief  select unsigned of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval None
  */
void ME_SET_DIVUSN(DIV_TypeDef *DIVx)
{
  CLEAR_BIT(DIVx->CR, DIV_CR_SN);
}

/**
  * @function  DIV_STR_DIV
  * @brief  STR DIV of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval None
  */
void ME_STR_DIV(DIV_TypeDef *DIVx)
{
  SET_BIT(DIVx->CR, DIV_CR_STR);
}

/**
  * @function  ME_Read_DIV_DQ
  * @brief  Get DQ value of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval value of DQ.
  */
uint32_t ME_Read_DIV_DQ(DIV_TypeDef *DIVx)
{
  return (READ_REG(DIVx->DQ));
}

/**
  * @function  ME_Read_DIV_DR
  * @brief  Get DR value of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval value of DR.
  */
uint32_t ME_Read_DIV_DR(DIV_TypeDef *DIVx)
{
  return (READ_REG(DIVx->DR));
}

/**
  * @function  ME_UDIV_DQ
  * @brief  ME unsigned DIV
  * @param  m: Dividend
  * @param  n: Divisor         
  * @retval Quotient
  */
uint32_t ME_UDIV_DQ(uint32_t m,uint32_t n)
{
	if(n ==0)
		return 0;
	
	DIV->DD = m;
	DIV->DI = n; 
  DIV->CR = DIV_CR_STR;
  
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
	
	return DIV->DQ;
}

/**
  * @function  ME_UDIV_DR
  * @brief  ME unsigned DIV
  * @param  m: Dividend
  * @param  n: Divisor         
  * @retval Remainder
  */
uint32_t ME_UDIV_DR(uint32_t m,uint32_t n)
{
	if(n ==0)
		return 0;
	
	DIV->DD = m;
	DIV->DI = n;
	CLEAR_BIT(DIV->CR,DIV_CR_RDY);
	SET_BIT(DIV->CR,DIV_CR_STR);
	while((DIV->CR & DIV_CR_RDY)==0);
  
  __NOP();
	
	return DIV->DR;
}

/**
  * @function  ME_SET_DIVSN
  * @brief  select signed of the selected DIV
  * @param  DIVx:
            @arg DIV Instance
  * @retval None
  */
void ME_SET_DIVSN(DIV_TypeDef *DIVx)
{
  SET_BIT(DIVx->CR, DIV_CR_SN);
}

/**
  * @function  ME_IsRDY_DIV
  * @brief  Get RDY state of the selected DIV 
  * @param  MEx:
            @arg ME Instance
  * @retval 0:DIV_RDY is inactive, 1:DIV_RDY is active
  */
uint32_t ME_IsRDY_DIV(DIV_TypeDef *DIVx)
{
  return (READ_BIT(DIVx->CR, DIV_CR_RDY) == (DIV_CR_RDY));
}

/**
  * @function  ME_SDIV_DQ
  * @brief  ME signed DIV
  * @param  m: Dividend
  * @param  n: Divisor         
  * @retval Quotient
  */
uint32_t ME_SDIV_DQ(int32_t m,int32_t n)
{
	if(n ==0)
		return 0;
	
	DIV->DD = m;
	DIV->DI = n;
	DIV->CR = (DIV_CR_STR | DIV_CR_SN);
  
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
  __NOP();
	
	return DIV->DQ;
}

/**
  * @function  ME_SDIV_DR
  * @brief  ME signed DIV
  * @param  m: Dividend
  * @param  n: Divisor         
  * @retval Remainder
  */
uint32_t ME_SDIV_DR(int32_t m,int32_t n)
{
	if(n ==0)
		return 0;
	
	DIV->DD = m;
	DIV->DI = n;
	CLEAR_BIT(DIV->CR,DIV_CR_RDY);
	SET_BIT(DIV->CR,(DIV_CR_STR|DIV_CR_SN));
	while((DIV->CR & DIV_CR_RDY)==0);
  
  __NOP();
	
	return DIV->DR;
}



