/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_pga.c
* By           : RX_SD
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_pga.h"

/**
  * @brief  Deinitializes the PGAx peripheral registers to their default reset values.
  * @param  PGAx: where x can be 0, 1 to select the 
  *         PGA peripheral.
  * @retval None
  */
void PGA_DeInit(PGA_TypeDef* PGAx)
{
  PGAx->PGA_CR2 = 0;
  PGAx->PGA_CR = 0;
}
/**
  * @brief  Fills each PGA_InitStruct member with its default value.
  * @param  PGA_InitStruct: pointer to a PGA_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void PGA_StructInit(PGA_InitTypeDef *PGA_InitStruct)
{
  PGA_InitStruct->PGA0_EN = ENABLE;
  PGA_InitStruct->PGA1_EN = ENABLE;
  PGA_InitStruct->PGA2_EN  = ENABLE;
  PGA_InitStruct->PGA0_GAIN = PGA_CR_PGA0GAIN_4X;
  PGA_InitStruct->PGA1_GAIN = PGA_CR_PGA1GAIN_4X;
  PGA_InitStruct->PGA2_GAIN = PGA_CR_PGA2GAIN_4X;
}

/**
***********************************************************************************
*@Function PGA_Init
*@brief    Initial the selected PGAx
*@param  PGAx :
*         @arg PGA
*@param  PGA_InitStruct: pointer to a PGA_InitTypeDef structure that contains
*         the configuration information for the specified PGAx peripheral.
  * @param  PGA0_EN:
  *         @arg ENABLE
  *         @arg DISABLE
  * @param  PGA0_GAIN:
  *         @arg PGA_CR_PGA0GAIN_2X    
  *         @arg PGA_CR_PGA0GAIN_4X     
  *         @arg PGA_CR_PGA0GAIN_8X     
  * @param  PGA1_EN:
  *         @arg ENABLE
  *         @arg DISABLE
  * @param  PGA1_GAIN:
  *         @arg PGA_CR_PGA1GAIN_2X    
  *         @arg PGA_CR_PGA1GAIN_4X     
  *         @arg PGA_CR_PGA1GAIN_8X     
  * @param  PGA2_EN:
  *         @arg ENABLE
  *         @arg DISABLE
  * @param  PGA2_GAIN:
  *         @arg PGA_CR_PGA2GAIN_2X    
  *         @arg PGA_CR_PGA2GAIN_4X     
  *         @arg PGA_CR_PGA2GAIN_8X   
* @retval None
***********************************************************************************
  */
void PGA_Init(PGA_TypeDef *PGAx, PGA_InitTypeDef *PGA_InitStruct)
{	
	if (PGA_InitStruct->PGA0_EN)
	{
	  PGA_Set_GAIN(PGA_CR_PGA0GAIN,PGA_InitStruct->PGA0_GAIN);
	  PGA_Enable(PGA_CR_PGA0EN);    
  }
  else 
  {
    CLEAR_BIT(PGA->PGA_CR, PGA_CR_PGA0EN);
  }

  if (PGA_InitStruct->PGA1_EN)
	{
	  PGA_Set_GAIN(PGA_CR_PGA1GAIN,PGA_InitStruct->PGA1_GAIN);
	  PGA_Enable(PGA_CR_PGA1EN);    
  }
  else 
  {
    CLEAR_BIT(PGA->PGA_CR, PGA_CR_PGA1EN);
  }

  if (PGA_InitStruct->PGA2_EN)
	{
	  PGA_Set_GAIN(PGA_CR_PGA2GAIN,PGA_InitStruct->PGA2_GAIN);
	  PGA_Enable(PGA_CR_PGA2EN);    
  }
  else 
  {
    CLEAR_BIT(PGA->PGA_CR, PGA_CR_PGA2EN);
  }
}


/**
***********************************************************************************
*@Function PGA_Enable
*@brief    Enable PGA.
*@param1  val:
*         @arg   PGA_CR_PGA0EN
*         @arg   PGA_CR_PGA1EN
*         @arg   PGA_CR_PGA2EN
*@retval None
***********************************************************************************
  */
void PGA_Enable(uint32_t val)
{
    SET_BIT(PGA->PGA_CR, val);
}

/**
***********************************************************************************
*@Function PGA_Disable
*@brief    Disable PGA.
*@param1  val:
*         @arg   PGA_CR_PGA0EN
*         @arg   PGA_CR_PGA1EN
*         @arg   PGA_CR_PGA2EN
*@retval None
***********************************************************************************
  */
void PGA_Disable(uint32_t val)
{
    CLEAR_BIT(PGA->PGA_CR, val);
}

/**
***********************************************************************************
*@Function PGA_Set_GAIN
*@brief    Set PGA_GAIN.
*@param1  PGAx:
*         @arg   PGA_CR_PGA0GAIN
*         @arg   PGA_CR_PGA1GAIN
*         @arg   PGA_CR_PGA2GAIN
*@param2  val:
*         @arg   PGA_CR_PGA0GAIN_2X     ((uint32_t)0x0000)
*         @arg   PGA_CR_PGA0GAIN_4X     ((uint32_t)0x0002)
*         @arg   PGA_CR_PGA0GAIN_8X     ((uint32_t)0x0004)
*         
*         @arg   PGA_CR_PGA1GAIN_2X     ((uint32_t)0x0000)
*         @arg   PGA_CR_PGA1GAIN_4X     ((uint32_t)0x0020)
*         @arg   PGA_CR_PGA1GAIN_8X     ((uint32_t)0x0040)
*
*         @arg   PGA_CR_PGA2GAIN_2X     ((uint32_t)0x0000)
*         @arg   PGA_CR_PGA2GAIN_4X     ((uint32_t)0x0200)
*         @arg   PGA_CR_PGA2GAIN_8X     ((uint32_t)0x0400)
*@retval None
***********************************************************************************
*/
void PGA_Set_GAIN(uint32_t PGAx, uint32_t val)
{
    MODIFY_REG(PGA->PGA_CR, PGAx, val);
}

/**
***********************************************************************************
*@Function PGA_Set_TEST
*@brief    Enable PGA TEST.
*@param1  val:
*         @arg   PGA_CR2_TEST_Normal
*         @arg   PGA_CR2_TEST_PGA0
*         @arg   PGA_CR2_TEST_PGA1
*         @arg   PGA_CR2_TEST_PGA2
*@retval None
***********************************************************************************
  */
void PGA_Set_TEST(uint32_t val)
{
    MODIFY_REG(PGA->PGA_CR2,PGA_CR2_TEST, val);
}


