/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                             rx32s11xx
*                                         Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_rtc.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define __rx32s11xx_RTC_C

#include "rx32s11xx_rtc.h"

/*
*********************************************************************************************************
*                                        Local macro/Structure
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            Local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                       Local Function declaration
*********************************************************************************************************
*/

/**
  * @brief  Deinitializes the RTCx peripheral registers to their default reset values.
  * @param  RTCx: where x can be 0, 1 to select the 
  *         RTC peripheral.
  * @retval None
  */
void RTC_DeInit(RTC_TypeDef* RTCx)
{
  RTCx->RTCCON = 0;
  RTCx->RTCIE = 0;
  RTCx->RTCIF = 0;
  RTCx->RTCTMR2 = 0;
}

/**
* @Register:RTCCON
*/
/**
  * @function RTC_Enable_TMR2EN
  * @brief  Enables the specified RTC_TMR peripheral.
  * @param RTCx:
  *         @arg RTC
  * @retval None
  */
void RTC_Enable_TMR2EN(RTC_TypeDef *RTCx)
{
  SET_BIT(RTCx->RTCCON,RTC_RTCCON_RTC2EN);
}

/**
  * @function RTC_Disable_TMR2EN
  * @brief  Disables the specified RTC_TMR peripheral.
  * @param RTCx:
  *         @arg RTC
  * @retval None
  */
void RTC_Disable_TMR2EN(RTC_TypeDef *RTCx)
{
  CLEAR_BIT(RTCx->RTCCON,RTC_RTCCON_RTC2EN);
}

/**
* @Register:RTCIE
*/
/**
  * @function  RTC_Enable_IT
  * @brief  Enables the specified RTC Interrupt peripheral.
  * @param  RTCx:
  *         @arg RTC
  * @param  IE
            @arg RTC_RTCIE_RTC2IE
  * @retval None
  */
void RTC_Enable_IT(RTC_TypeDef *RTCx,uint32_t IE)
{
  SET_BIT(RTCx->RTCIE, IE);  
}

/**
  * @function  RTC_Disable_IT
  * @brief  Disables the specified RTC Interrupt peripheral.
  * @param  RTCx:
  *         @arg RTC
  * @param  IE
            @arg RTC_RTCIE_RTC2IE
  * @retval None
  */
void RTC_Disable_IT(RTC_TypeDef *RTCx,uint32_t IE)
{
  CLEAR_BIT(RTCx->RTCIE, IE);
}

/**
  * @function  RTC_Get_IT
  * @brief  Get IT Status of the selected RTC
  * @param  RTCx:
  *         @arg RTC
  * @param 
            @arg RTC_RTCIE_RTC2IE
  * @retval 0:IT is disabled, 1:IT is enabled
  */
uint32_t RTC_Get_IT(RTC_TypeDef *RTCx,uint32_t IE)
{
  return (READ_BIT(RTCx->RTCIE, IE) == (IE));
}

/**
* @Register:RTCIF
*/
/**
  * @function  RTC_Clear_FLAG
  * @brief  Clear RTC IT Flag
  * @param  RTCx:
  *         @arg RTC
  * @param  IF
            @arg RTC_RTCIF_RTC2IF
  * @retval None
  */
void RTC_Clear_FLAG(RTC_TypeDef *RTCx,uint32_t IF)
{
  CLEAR_BIT(RTCx->RTCIF, IF);
}

/**
  * @function  RTC_Get_FLAG
  * @brief  Check whether the specified RTC IT flag is set or not.
  * @param  RTCx:
  *         @arg RTC
  * @param  IF
            @arg RTC_RTCIF_RTC2IF
  * @retval State of bit (1 or 0).
  */
uint32_t RTC_Get_FLAG(RTC_TypeDef *RTCx,uint32_t IF)
{
  return( READ_BIT(RTCx->RTCIF, IF) == (IF) );
}

/**
* @Register:RTCTMR2
*/
/**
  * @function RTC_Set_RTCTMR2
  * @brief  Specifies the RTC TMR clock cycle.
  * @param  Count: The RTC TMR clock cycle.
  * @retval None
  */
void RTC_Set_TMR2(RTC_TypeDef *RTCx,uint16_t Count)
{
  MODIFY_REG(RTCx->RTCTMR2, RTC_RTCTMR2_CNT, Count);
}
