/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                               rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_tim8.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/
#include <stdio.h>
#include "rx32s11xx_tim8.h"


static void TIM_Set_TI1(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,uint16_t TIM_ICFilter);
static void TIM_Set_TI2(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,uint16_t TIM_ICFilter);
static void TIM_Set_TI3(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,uint16_t TIM_ICFilter);
static void TIM_Set_TI4(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,uint16_t TIM_ICFilter);

const uint8_t OFFSET_TAB_CCMRx[] =
{
  0x00U,            /* 0: TIM_CH1  */
  0x00U,            /* 1: TIM_CH1N */
  0x00U,            /* 2: TIM_CH2  */
  0x00U,            /* 3: TIM_CH2N */
  0x04U,            /* 4: TIM_CH3  */
  0x04U,            /* 5: TIM_CH3N */
  0x04U,            /* 6: TIM_CH4 */
	0x3CU,            /* 7: TIM_CH5  */
	0x3CU             /* 8: TIM_CH6  */  
};

 const uint8_t SHIFT_TAB_OCxx[] =
{
  0U,            /* 0: OC1M, OC1FE, OC1PE */
  0U,            /* 1: - NA */
  8U,            /* 2: OC2M, OC2FE, OC2PE */
  0U,            /* 3: - NA */
  0U,            /* 4: OC3M, OC3FE, OC3PE */
  0U,            /* 5: - NA */
  8U,            /* 6: OC4M, OC4FE, OC4PE */
	0U,            /* 7: OC5M, OC5FE, OC5PE */
	6U             /* 8: OC6M, OC6FE, OC6PE */
};

 const uint8_t SHIFT_TAB_ICxx[] =
{
  0U,            /* 0: CC1S, IC1PSC, IC1F */
  0U,            /* 1: - NA */
  8U,            /* 2: CC2S, IC2PSC, IC2F */
  0U,            /* 3: - NA */
  0U,            /* 4: CC3S, IC3PSC, IC3F */
  0U,            /* 5: - NA */
  8U,            /* 6: CC4S, IC4PSC, IC4F */
  0U,            /* 7: - NA */
  0U             /* 8: - NA */
};

 const uint8_t SHIFT_TAB_CCxP[] =
{
  0U,            /* 0: CC1P */
  2U,            /* 1: CC1NP */
  4U,            /* 2: CC2P */
  6U,            /* 3: CC2NP */
  8U,            /* 4: CC3P */
  10U,           /* 5: CC3NP */
  12U,           /* 6: CC4P */
	14U,           /* 7: CC5P */
	16U            /* 8: CC6P */
};
 const uint8_t SHIFT_TAB_OISx[] =
{
  0U,            /* 0: OIS1 */
  1U,            /* 1: OIS1N */
  2U,            /* 2: OIS2 */
  3U,            /* 3: OIS2N */
  4U,            /* 4: OIS3 */
  5U,            /* 5: OIS3N */
  6U,            /* 6: OIS4 */
	7U,            /* 7: OIS5 */
	8U             /* 8: OIS6 */
};

/**
  * @brief  Deinitializes the TIMx peripheral registers to their default reset values.
  * @param  TIMx: where x can be  8 to select the TIM peripheral.
  * @retval None

  */
void TIM_DeInit(TIM_TypeDef* TIMx)
{
  TIMx->CR1   = 0x0000;
	TIMx->CR2   = 0x0000;
	TIMx->SMCR  = 0x0000;
	TIMx->DIER  = 0x0000;
	TIMx->SR    = 0x0000;
	TIMx->EGR   = 0x0000;
	TIMx->CCMR1 = 0x0000;
	TIMx->CCMR2 = 0x0000;
	TIMx->CCER  = 0x0000;
	TIMx->CNT   = 0x0000;
	TIMx->PSC   = 0x0000;
	TIMx->ARR   = 0x0000;
	TIMx->RCR   = 0x0000;
	TIMx->CCR1  = 0x0000;
	TIMx->CCR2  = 0x0000;
	TIMx->CCR3  = 0x0000;
	TIMx->CCR4  = 0x0000;
	TIMx->BDTR  = 0x0000;
	TIMx->CCR5  = 0x0000;
	TIMx->CCMR3  = 0x0000;	
}

/**
  * @brief  Fills each TIM_TimeBaseInitStruct member with its default value.
  * @param  TIM_TimeBaseInitStruct : pointer to a ATIM_TimeBaseInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void TIM_TimeBaseStructInit(TIM_TimeBaseInitTypeDef* TIM_TimeBaseInitStruct)
{
  /* Set the default configuration */
  TIM_TimeBaseInitStruct->Period            = 0xFFFF;
  TIM_TimeBaseInitStruct->Prescaler         = 0x0000;
  TIM_TimeBaseInitStruct->ClockDivision     = 0x0000;
  TIM_TimeBaseInitStruct->CounterMode       = TIM_COUNTERMODE_UP;
  TIM_TimeBaseInitStruct->RepetitionCounter = 0x0000;
}
/**
  * @brief  Fills each TIM_OCInitStruct member with its default value.
  * @param  TIM_OCInitStruct: pointer to a ATIM_OCInitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void TIM_OCStructInit(TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  /* Set the default configuration */
  TIM_OCInitStruct->OCMode       = TIM_OCMODE_FROZEN;
  TIM_OCInitStruct->OutputState  = TIM_OutputState_Disable;
  TIM_OCInitStruct->OutputNState = TIM_OutputNState_Disable;
  TIM_OCInitStruct->Pulse        = 0x00000000;
  TIM_OCInitStruct->OCPolarity   = TIM_OCPolarity_High;
  TIM_OCInitStruct->OCNPolarity  = TIM_OCNPolarity_High;
  TIM_OCInitStruct->OCIdleState  = TIM_OCIdleState_Reset;
  TIM_OCInitStruct->OCNIdleState = TIM_OCNIdleState_Reset;
}

/**
  * @brief  Fills each TIM_ICInitStruct member with its default value.
  * @param  TIM_ICInitStruct: pointer to a ATIM_ICInitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void TIM_ICStructInit(TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  /* Set the default configuration */
  TIM_ICInitStruct->Channel     = TIM_CHANNEL_CH1;
  TIM_ICInitStruct->ICPolarity  = TIM_ICPolarity_Rising;
  TIM_ICInitStruct->ICSelection = TIM_ICSelection_DirectTI;
  TIM_ICInitStruct->ICPrescaler = TIM_ICPSC_DIV1;
  TIM_ICInitStruct->ICFilter    = 0x00;
}

/**
  * @brief  Fills each TIM_BDTRInitStruct member with its default value.
  * @param  TIM_BDTRInitStruct: pointer to a ATIM_BDTRInitTypeDef structure which
  *         will be initialized.
  * @retval None
  */
void TIM_BDTRStructInit(TIM_BDTRInitTypeDef* TIM_BDTRInitStruct)
{
  /* Set the default configuration */
  TIM_BDTRInitStruct->OSSRState       = TIM_OSSR_DISABLE;
  TIM_BDTRInitStruct->OSSIState       = TIM_OSSI_DISABLE;
  TIM_BDTRInitStruct->LOCKLevel       = TIM_LOCKLevel_OFF;
  TIM_BDTRInitStruct->DeadTime        = 0x00;
  TIM_BDTRInitStruct->BreakState      = TIM_Break_Enable;
  TIM_BDTRInitStruct->BreakPolarity   = TIM_BREAK_POLARITY_LOW;
  TIM_BDTRInitStruct->AutomaticOutput = TIM_AutomaticOutput_Disable;
}

/**
  * @function  TIM_TimeBaseInit
  * @brief  Initializes the TIMx Time Base Unit peripheral according to 
  *         the specified parameters in the TIM_TimeBaseInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  Prescaler between Min_Data=0 and Max_Data=65535
  * @param  CounterMode
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  * @param  Period between Min_Data=0 and Max_Data=65535
  * @param  ClockDivision
  *         @arg @ref TIM_CKD_DIV1
  *         @arg @ref TIM_CKD_DIV2
  *         @arg @ref TIM_CKD_DIV4
  * @param  Period between Min_Data=0 and Max_Data=255
  * @retval None
  */
void TIM_TimeBaseInit(TIM_TypeDef* TIMx, TIM_TimeBaseInitTypeDef* TIM_TimeBaseInitStruct)
{
  uint32_t tmpcr1 = 0;

  tmpcr1 = TIMx->CR1;

	/* Select the Counter Mode */
	tmpcr1 &= (uint32_t)(~((uint32_t)(TIM_CR1_DIR | TIM_CR1_CMS)));
	tmpcr1 |= (uint32_t)TIM_TimeBaseInitStruct->CounterMode;

	/* Set the clock division */
	tmpcr1 &= (uint32_t)(~((uint32_t)TIM_CR1_CKD));
	tmpcr1 |= (uint32_t)TIM_TimeBaseInitStruct->ClockDivision;

  TIMx->CR1 = tmpcr1;

  /* Set the Autoreload value */
  TIMx->ARR = TIM_TimeBaseInitStruct->Period ;
 
  /* Set the Prescaler value */
  TIMx->PSC = TIM_TimeBaseInitStruct->Prescaler;

  /* Set the Repetition Counter value */
  TIMx->RCR = TIM_TimeBaseInitStruct->RepetitionCounter;

  /* Generate an update event to reload the Prescaler and the Repetition counter
     values immediately */
  TIMx->EGR = 0x0001;           
}

/**
  * @function  TIM_OC1Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC1Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
 /* Disable the Channel 1: Reset the CC1E Bit */
  TIMx->CCER &= (uint32_t)(~(uint32_t)TIM_CCER_CC1E);
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR1 register value */
  tmpccmrx = TIMx->CCMR1;
    
  /* Reset the Output Compare Mode Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_OC1M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_CC1S));

  /* Select the Output Compare Mode */
  tmpccmrx |= TIM_OCInitStruct->OCMode;
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1P));
  /* Set the Output Compare Polarity */
  tmpccer |= TIM_OCInitStruct->OCPolarity;
  
  /* Set the Output State */
  tmpccer |= TIM_OCInitStruct->OutputState;
      
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1NP));
	/* Set the Output N Polarity */
	tmpccer |= TIM_OCInitStruct->OCNPolarity;
	
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC1NE));    
	/* Set the Output N State */
	tmpccer |= TIM_OCInitStruct->OutputNState;
	
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS1));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS1N));
	
	/* Set the Output Idle state */
	tmpcr2 |= TIM_OCInitStruct->OCIdleState;
	/* Set the Output N Idle state */
	tmpcr2 |= TIM_OCInitStruct->OCNIdleState;

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR1 */
  TIMx->CCMR1 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR1 = TIM_OCInitStruct->Pulse; 
 
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC2Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC2Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
   /* Disable the Channel 2: Reset the CC2E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC2E));
  
  /* Get the TIMx CCER register value */  
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR1 register value */
  tmpccmrx = TIMx->CCMR1;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_OC2M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR1_CC2S));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 8);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 4);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 4);
    
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2NP));
	/* Set the Output N Polarity */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OCNPolarity << 4);
	
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC2NE));    
	/* Set the Output N State */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputNState << 4);
	
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS2));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS2N));
	
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 2);
	/* Set the Output N Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCNIdleState << 2);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR1 */
  TIMx->CCMR1 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR2 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC3Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC3Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;

  /* Disable the Channel 3: Reset the CC3E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC3E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR2;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_OC3M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_CC3S));  
  /* Select the Output Compare Mode */
  tmpccmrx |= TIM_OCInitStruct->OCMode;
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 8);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 8);
    
	/* Reset the Output N Polarity level */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3NP));
	/* Set the Output N Polarity */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OCNPolarity << 8);
	/* Reset the Output N State */
	tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC3NE));
	
	/* Set the Output N State */
	tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputNState << 8);
	/* Reset the Output Compare and Output Compare N IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS3));
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS3N));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 4);
	/* Set the Output N Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCNIdleState << 4);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */
  TIMx->CCMR2 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR3 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC4Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC4Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 4: Reset the CC4E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC4E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR2;
    
  /* Reset the Output Compare mode and Capture/Compare selection Bits */
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_OC4M));
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR2_CC4S));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 8);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC4P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 12);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 12);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS4));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 6);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */  
  TIMx->CCMR2 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR4 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC5Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC5Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 5: Reset the CC5E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC5E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR2 register value */
  tmpccmrx = TIMx->CCMR3;
    
  /* Reset the Output Compare mode and Capture*/
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR3_OC5M));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC5P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 14);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 14);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS5));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 7);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */  
  TIMx->CCMR3 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR5 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_OC6Init
  * @brief  Initializes the TIMx Channel1 according to the specified
  *         parameters in the TIM_OCInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  OCMode: 
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @param  OutputState:
  *         @arg @ref TIM_OutputState_Disable
  *         @arg @ref TIM_OutputState_Enable
  * @param  OutputNState:
  *         @arg @ref TIM_OutputNState_Disable
  *         @arg @ref TIM_OutputNState_Enable
  * @param  Pulse between Min_Data=0 and Max_Data=65535
  * @param  OCPolarity:
  *         @arg @ref TIM_OCPolarity_High
  *         @arg @ref TIM_OCPolarity_Low
  * @param  OCNPolarity:
  *         @arg @ref TIM_OCNPolarity_High
  *         @arg @ref TIM_OCNPolarity_Low	
  * @param  OCIdleState:
  *         @arg @ref TIM_OCIdleState_Set
  *         @arg @ref TIM_OCIdleState_Reset	
	* @param  OCNIdleState:
  *         @arg @ref TIM_OCNIdleState_Set
  *         @arg @ref TIM_OCNIdleState_Reset	
  * @retval None
  */
void TIM_OC6Init(TIM_TypeDef* TIMx, TIM_OCInitTypeDef* TIM_OCInitStruct)
{
  uint32_t tmpccmrx = 0, tmpccer = 0, tmpcr2 = 0;
  
  /* Disable the Channel 5: Reset the CC5E Bit */
  TIMx->CCER &= (uint32_t)(~((uint32_t)TIM_CCER_CC6E));
  
  /* Get the TIMx CCER register value */
  tmpccer = TIMx->CCER;
  /* Get the TIMx CR2 register value */
  tmpcr2 =  TIMx->CR2;
  
  /* Get the TIMx CCMR3 register value */
  tmpccmrx = TIMx->CCMR3;
    
  /* Reset the Output Compare mode and Capture*/
  tmpccmrx &= (uint32_t)(~((uint32_t)TIM_CCMR3_OC6M));
  
  /* Select the Output Compare Mode */
  tmpccmrx |= (uint32_t)(TIM_OCInitStruct->OCMode << 6);
  
  /* Reset the Output Polarity level */
  tmpccer &= (uint32_t)(~((uint32_t)TIM_CCER_CC6P));
  /* Set the Output Compare Polarity */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OCPolarity << 16);
  
  /* Set the Output State */
  tmpccer |= (uint32_t)(TIM_OCInitStruct->OutputState << 16);
    
	/* Reset the Output Compare IDLE State */
	tmpcr2 &= (uint32_t)(~((uint32_t)TIM_CR2_OIS6));
	/* Set the Output Idle state */
	tmpcr2 |= (uint32_t)(TIM_OCInitStruct->OCIdleState << 8);

  /* Write to TIMx CR2 */
  TIMx->CR2 = tmpcr2;
  
  /* Write to TIMx CCMR2 */  
  TIMx->CCMR3 = tmpccmrx;

  /* Set the Capture Compare Register value */
  TIMx->CCR6 = TIM_OCInitStruct->Pulse;
  
  /* Write to TIMx CCER */
  TIMx->CCER = tmpccer;
}

/**
  * @function  TIM_ICInit
  * @brief  Initializes the TIM peripheral according to the specified
  *         parameters in the TIM_ICInitStruct
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel 
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPolarity
  *         @arg @ref TIM_ICPolarity_Rising
  *         @arg @ref TIM_ICPolarity_Falling
  *         @arg @ref TIM_ICPolarity_BothEdge
  * @param  ICSelection
  *         @arg @ref TIM_ICSelection_DirectTI
  *         @arg @ref TIM_ICSelection_IndirectTI
  *         @arg @ref TIM_ICSelection_TRC
  * @param  ICPrescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8	
  * @retval None
  */
void TIM_ICInit(TIM_TypeDef* TIMx, TIM_ICInitTypeDef* TIM_ICInitStruct)
{
  if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH1)
  {
    /* TI1 Configuration */
    TIM_Set_TI1(TIMx, TIM_ICInitStruct->ICPolarity,
               TIM_ICInitStruct->ICSelection,
               TIM_ICInitStruct->ICFilter);
    /* Set the Input Capture Prescaler value */
    TIM_Set_IC_Prescaler(TIMx, TIM_CHANNEL_CH1,TIM_ICInitStruct->ICPrescaler);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH2)
  {
    /* TI2 Configuration */
    TIM_Set_TI2(TIMx, TIM_ICInitStruct->ICPolarity,
               TIM_ICInitStruct->ICSelection,
               TIM_ICInitStruct->ICFilter);
    /* Set the Input Capture Prescaler value */
    TIM_Set_IC_Prescaler(TIMx, TIM_CHANNEL_CH2,TIM_ICInitStruct->ICPrescaler);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH3)
  {
    /* TI3 Configuration */
    TIM_Set_TI3(TIMx,  TIM_ICInitStruct->ICPolarity,
               TIM_ICInitStruct->ICSelection,
               TIM_ICInitStruct->ICFilter);
    /* Set the Input Capture Prescaler value */
    TIM_Set_IC_Prescaler(TIMx, TIM_CHANNEL_CH3,TIM_ICInitStruct->ICPrescaler);
  }
  else if (TIM_ICInitStruct->Channel == TIM_CHANNEL_CH4)
  {
    /* TI4 Configuration */
    TIM_Set_TI4(TIMx, TIM_ICInitStruct->ICPolarity,
               TIM_ICInitStruct->ICSelection,
               TIM_ICInitStruct->ICFilter);
    /* Set the Input Capture Prescaler value */
    TIM_Set_IC_Prescaler(TIMx, TIM_CHANNEL_CH4,TIM_ICInitStruct->ICPrescaler);
  }
}

/**
  * @function  TIM_BDTRInit
  * @brief  Configures the: Break feature, dead time, Lock level, the OSSI,
  *         the OSSR State and the AOE(automatic output enable).
  * @param  TIMx:
            @arg TIM Instance
  * @param  OSSRState 
  *         @arg @ref TIM_OSSR_ENABLE
  *         @arg @ref TIM_OSSR_DISABLE
  * @param  OSSIState
  *         @arg @ref TIM_OSSI_ENABLE
  *         @arg @ref TIM_OSSI_DISABLE
  * @param  LOCKLevel
  *         @arg @ref TIM_LOCKLevel_OFF
  *         @arg @ref TIM_LOCKLevel_1
  *         @arg @ref TIM_LOCKLevel_2
  *         @arg @ref TIM_LOCKLevel_3
  * @param  DeadTime between 0x00 and 0xFF
  * @param  BreakState
  *         @arg @ref TIM_Break_Enable
  *         @arg @ref TIM_Break_Disable	
  * @param  BreakPolarity
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  *         @arg @ref TIM_BREAK_POLARITY_LOW	
  * @param  AutomaticOutput
  *         @arg @ref TIM_AutomaticOutput_Enable
  *         @arg @ref TIM_AutomaticOutput_Disable	
  * @param  CMP4_BreakState
  *         @arg @ref TIM_CMP4_Break_Enable
  *         @arg @ref TIM_CMP4_Break_Disable	
  * @param  CMP4_BreakPolarity
  *         @arg @ref TIM_CMP4OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP4OEP_POLARITY_HIGH	
  * @param  CMP5_BreakState
  *         @arg @ref TIM_CMP5_Break_Enable
  *         @arg @ref TIM_CMP5_Break_Disable	
  * @param  CMP5_BreakPolarity
  *         @arg @ref TIM_CMP5OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP5OEP_POLARITY_HIGH	
  * @param  DOE
  *         @arg @ref TIM_DOE_Enable
  *         @arg @ref TIM_DOE_Disable	
  * @param  HDFE
  *         @arg @ref TIM_HDFE_Enable
  *         @arg @ref TIM_HDFE_Disable	
  * @retval None
  */
void TIM_BDTRInit(TIM_TypeDef* TIMx, TIM_BDTRInitTypeDef *TIM_BDTRInitStruct)
{
  /* Set the Lock level, the Break enable Bit and the Ploarity, the OSSR State,
     the OSSI State, the dead time value and the Automatic Output Enable Bit */
  TIMx->BDTR = (uint32_t)TIM_BDTRInitStruct->OSSRState | TIM_BDTRInitStruct->OSSIState |
             TIM_BDTRInitStruct->LOCKLevel | TIM_BDTRInitStruct->DeadTime |
             TIM_BDTRInitStruct->BreakState | TIM_BDTRInitStruct->BreakPolarity |
             TIM_BDTRInitStruct->CMP4_BreakState | TIM_BDTRInitStruct->CMP4_BreakPolarity |
						 TIM_BDTRInitStruct->CMP5_BreakState | TIM_BDTRInitStruct->CMP5_BreakPolarity |
             TIM_BDTRInitStruct->AutomaticOutput | TIM_BDTRInitStruct->DOE | TIM_BDTRInitStruct->HDFE;
}


/**
  * @function  TIM_ENCODERInit
  * @brief  Initializes the encoder interface of the TIMx 
  * @param  TIMx:
            @arg TIM Instance
  * @param  EncoderMode 
  *         @arg @ref TIM_ENCODERMODE_X2_TI1
  *         @arg @ref TIM_ENCODERMODE_X2_TI2
  *         @arg @ref TIM_ENCODERMODE_X4_TI12
  * @param  IC1Polarity
  *         @arg @ref TIM_ICPolarity_Rising
  *         @arg @ref TIM_ICPolarity_Falling
  *         @arg @ref TIM_ICPolarity_BothEdge
  * @param  IC1Selection
  *         @arg @ref TIM_ICSelection_DirectTI
  *         @arg @ref TIM_ICSelection_IndirectTI
  *         @arg @ref TIM_ICSelection_TRC
  * @param  IC1Prescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  IC1Filter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8	
  * @param  IC2Polarity
  *         @arg @ref TIM_ICPolarity_Rising
  *         @arg @ref TIM_ICPolarity_Falling
  *         @arg @ref TIM_ICPolarity_BothEdge
  * @param  IC2Selection
  *         @arg @ref TIM_ICSelection_DirectTI
  *         @arg @ref TIM_ICSelection_IndirectTI
  *         @arg @ref TIM_ICSelection_TRC
  * @param  IC2Prescaler
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @param  IC2Filter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8		
  * @retval None
  */
	
void TIM_ENCODERInit(TIM_TypeDef *TIMx, TIM_ENCODERInitTypeDef *TIM_ENCODERInitStruct)
{
  /* Configure TI1 */
	TIM_Set_IC_ActiveInput(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Selection);
	TIM_Set_IC_Filter(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Filter);	
	TIM_Set_IC_Prescaler(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Prescaler);	
	
	
  /* Configure TI2 */
	TIM_Set_IC_ActiveInput(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC1Selection);
	TIM_Set_IC_Filter(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC1Filter);	
	TIM_Set_IC_Prescaler(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC1Prescaler);	

  /* Set TI1 and TI2 polarity and enable TI1 and TI2 */
	TIM_Set_IC_Polarity(TIMx,TIM_CHANNEL_CH1,TIM_ENCODERInitStruct->IC1Polarity);		
	TIM_Set_IC_Polarity(TIMx,TIM_CHANNEL_CH2,TIM_ENCODERInitStruct->IC1Polarity);		
  TIM_Enable_CC_Channel(TIMx,TIM_CHANNEL_CH1);
  TIM_Enable_CC_Channel(TIMx,TIM_CHANNEL_CH2);	
	
  /* Set encoder mode */
  TIM_Set_EncoderMode(TIMx, TIM_ENCODERInitStruct->EncoderMode);

}

/**
  * @function  TIM_Enable_CEN
  * @brief  Enable counter of the selected TIMx
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CEN(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_CEN);
}

/**
  * @function  TIM_Disable_CEN
  * @brief  Disable counter of the selected TIMx
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CEN(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_CEN);
}

/**
  * @function  TIM_Get_CENStatus
  * @brief  Indicates whether the timer counter is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:CEN counter is disalbe, 1:CEN counter is enable
  */
 uint32_t TIM_Get_CENStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_CEN) == (TIM_CR1_CEN));
}

/**
  * @function  TIM_Enable_UpdateEvent
  * @brief  Enable update event generation
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_UpdateEvent(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_UDIS);
}

/**
  * @function  TIM_Disable_UpdateEvent
  * @brief  Disable update event generation
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_UpdateEvent(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_UDIS);
}

/**
  * @function  TIM_Get_UpdateEventStatus
  * @brief  Indicates whether update event generation is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:UpdateEvent is disalbe, 1:UpdateEvent is enable
  */
 uint32_t TIM_Get_UpdateEventStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_UDIS) == TIM_CR1_UDIS);
}

/**
  * @function  TIM_Set_UpdateSource
  * @brief  Set update event sorce
  * @param  TIMx:
            @arg TIM Instance
  * @param  UpdateSource This parameter can be one of the following values:
  *         @arg @ref TIM_UPDATESOURCE_REGULAR
  *         @arg @ref TIM_UPDATESOURCE_COUNTER
  * @retval None
  */
 void TIM_Set_UpdateSource(TIM_TypeDef *TIMx,uint32_t UpdateSource)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_URS, UpdateSource);
}

/**
  * @function  TIM_Get_UpdateSource
  * @brief  Get actual event update source
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_UPDATESOURCE_REGULAR
  *         @arg @ref TIM_UPDATESOURCE_COUNTER
  */
 uint32_t TIM_Get_UpdateSource(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1,TIM_CR1_URS));
}

/**
  * @function  TIM_Set_OnePulseMode
  * @brief  Set one pulse mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  OnePulseMode This parameter can be one of the following values:
  *         @arg @ref TIM_ONEPULSEMODE_SINGLE
  *         @arg @ref TIM_ONEPULSEMODE_REPETITIVE
  * @retval None
  */
 void TIM_Set_OnePulseMode(TIM_TypeDef *TIMx,uint32_t OnePulseMode)
{
  MODIFY_REG(TIMx->CR1,TIM_CR1_OPM, OnePulseMode);
}

/**
  * @function  TIM_Get_OnePulseMode
  * @brief  Get one pulse mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ONEPULSEMODE_SINGLE
  *         @arg @ref TIM_ONEPULSEMODE_REPETITIVE
  */
 uint32_t TIM_Get_OnePulseMode(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_OPM));
}

/**
  * @function  TIM_Set_CounterMode
  * @brief  Set TIM8 counter counting mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  CounterMode This parameter can be one of the following values:
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  * @retval None
  */
 void TIM_Set_CounterMode(TIM_TypeDef *TIMx,uint32_t CounterMode)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_DIR|TIM_CR1_CMS, CounterMode);
}

/**
  * @function  TIM_Get_CounterMode
  * @brief  Get actual counter mode
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_COUNTERMODE_UP
  *         @arg @ref TIM_COUNTERMODE_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP
  *         @arg @ref TIM_COUNTERMODE_CENTER_DOWN
  *         @arg @ref TIM_COUNTERMODE_CENTER_UP_DOWN
  */
 uint32_t TIM_Get_CounterMode(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_DIR|TIM_CR1_CMS));
}

/**
  * @function  TIM_Enable_ARRPreload
  * @brief  Enable auto-reload (ARR) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_ARRPreload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR1, TIM_CR1_APRE);
}

/**
  * @function  TIM_Disable_ARRPreload
  * @brief  Disable auto-reload (ARR) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_ARRPreload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR1, TIM_CR1_APRE);
}

/**
  * @function  TIM_Get_ARRPreloadStatus
  * @brief  Indicates whether auto-reload (ARR) preload is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:auto-reload (ARR) preload is disalbe, 1:auto-reload (ARR) preload is enable
  */
 uint32_t TIM_Get_ARRPreloadStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR1, TIM_CR1_APRE) == TIM_CR1_APRE);
}

/**
  * @function  TIM_Set_ClockDivision
  * @brief  Set the division ratio between the timer clock  and the sampling clock used by the dead-time generators (when supported) and the digital filters.
  * @param  TIMx:
            @arg TIM Instance
  * @param  ClockDivision This parameter can be one of the following values:
  *         @arg @ref TIM_CLOCKDIVISION_DIV1
  *         @arg @ref TIM_CLOCKDIVISION_DIV2
  *         @arg @ref TIM_CLOCKDIVISION_DIV4
  * @retval None
  */
 void TIM_Set_ClockDivision(TIM_TypeDef *TIMx,uint32_t ClockDivision)
{
  MODIFY_REG(TIMx->CR1, TIM_CR1_CKD, ClockDivision);
}

/**
  * @function  TIM_Get_ClockDivision
  * @brief  Get the actual division ratio between the timer clock  and the sampling clock used by the dead-time generators (when supported) and the digital filters.
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_CLOCKDIVISION_DIV1
  *         @arg @ref TIM_CLOCKDIVISION_DIV2
  *         @arg @ref TIM_CLOCKDIVISION_DIV4
  */
 uint32_t TIM_Get_ClockDivision(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_CKD));
}

/**
  * @function  TIM_Set_CounterValue
  * @brief  Set the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @param  Counter Counter value (between Min_Data=0 and Max_Data=0xFFFF)
  * @retval None
  */
 void TIM_Set_CounterValue(TIM_TypeDef *TIMx,uint32_t Counter)
{
  WRITE_REG(TIMx->CNT, Counter);
}

/**
  * @function  TIM_Clear_CounterValue
  * @brief  Clear the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Clear_CounterValue(TIM_TypeDef *TIMx)
{
  CLEAR_REG(TIMx->CNT);
}

/**
  * @function  TIM_Get_CounterValue
  * @brief  Get the counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval Counter value (between Min_Data=0 and Max_Data=0xFFFF)
  */
 uint32_t TIM_Get_CounterValue(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CNT));
}

/**
  * @function  TIM_Get_Direction
  * @brief  Get the counter direction
  * @param  TIMx:
            @arg TIM Instance
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_COUNTERDIRECTION_UP
  *         @arg @ref TIM_COUNTERDIRECTION_DOWN
  */
 uint32_t TIM_Get_Direction(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_BIT(TIMx->CR1, TIM_CR1_DIR));
}

/**
  * @function  TIM_Set_Prescaler
  * @brief  Set TIMx the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @param  Prescaler between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_Prescaler(TIM_TypeDef *TIMx,uint32_t Prescaler)
{
  WRITE_REG(TIMx->PSC, Prescaler);
}

/**
  * @function  TIM_Get_Prescaler
  * @brief  Get the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @retval  Prescaler value between Min_Data=0 and Max_Data=65535
  */
 uint32_t TIM_Get_Prescaler(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->PSC));
}

/**
  * @function  TIM_Set_AutoReload
  * @brief  Set TIMx the auto-reload value
  * @param  TIMx:
            @arg TIM Instance
  * @param  AutoReload between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_AutoReload(TIM_TypeDef *TIMx,uint32_t AutoReload)
{
  WRITE_REG(TIMx->ARR, AutoReload);
}

/**
  * @function  TIM_Get_AutoReload
  * @brief  Get the prescaler value
  * @param  TIMx:
            @arg TIM Instance
  * @retval  Auto-reload value value between Min_Data=0 and Max_Data=65535
  */
 uint32_t TIM_Get_AutoReload(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->ARR));
}

/**
  * @function  TIM_Set_RepetitionCounter
  * @brief  Set the repetition counter value
  * @param  TIMx:
            @arg TIM Instance
  * @param  RepetitionCounter value between Min_Data=0 and Max_Data=255
  * @retval None
  */
 void TIM_Set_RepetitionCounter(TIM_TypeDef *TIMx,uint32_t RepetitionCounter)
{
  WRITE_REG(TIMx->RCR, RepetitionCounter);
}

/**
  * @function  TIM_Get_RepetitionCounter
  * @brief  Get the repetition counter value
  * @param  TIMx:
            @arg TIM Instance
  * @retval RepetitionCounter value between Min_Data=0 and Max_Data=255
  */
 uint32_t TIM_Get_RepetitionCounter(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->RCR));
}


/** @defgroup TIM_Capture_Compare Capture Compare configuration
  * @{
  */

/**
  * @function  TIM_Enable_CC_Preload
  * @brief  Enable  the capture/compare control bits (CCxE, CCxNE and OCxM) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CC_Preload(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR2, TIM_CR2_CCPC);
}

/**
  * @function  TIM_Disable_CC_Preload
  * @brief  Disable  the capture/compare control bits (CCxE, CCxNE and OCxM) preload
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CC_Preload(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR2,TIM_CR2_CCPC);
}

/**
  * @function  TIM_Set_CC_Update
  * @brief  Set the updated source of the capture/compare control bits (CCxE, CCxNE and OCxM)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CCUpdateSource This parameter can be one of the following values:
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_ONLY
  *         @arg @ref TIM_CCUPDATESOURCE_COMG_AND_TRGI
  * @retval None
  */
 void TIM_Set_CC_Update(TIM_TypeDef *TIMx,uint32_t CCUpdateSource)
{
  MODIFY_REG(TIMx->CR2,TIM_CR2_CCUS, CCUpdateSource);
}

/**
  * @function  TIM_Set_LockLevel
  * @brief  Set the lock level to freeze the
  *         configuration of several capture/compare parameters
  * @param  TIMx:
            @arg TIM Instance
  * @param  LockLevel This parameter can be one of the following values:
  *         @arg @ref TIM_LOCKLevel_OFF
  *         @arg @ref TIM_LOCKLevel_1
  *         @arg @ref TIM_LOCKLevel_2
  *         @arg @ref TIM_LOCKLevel_3
  * @retval None
  */
 void TIM_Set_LockLevel(TIM_TypeDef *TIMx,uint32_t LockLevel)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_LOCK, LockLevel);
}

/**
  * @function  TIM_Set_DeadTime
  * @brief  Set the dead-time delay (delay inserted between the rising edge of the OCxREF signal and the rising edge if the Ocx and OCxN signals)
  * @param  TIMx:
            @arg TIM Instance
  * @param  DeadTime between Min_Data=0 and Max_Data=255
  * @retval None
  */
 void TIM_Set_DeadTime(TIM_TypeDef *TIMx,uint32_t DeadTime)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_DTG, DeadTime);
}

/**
  * @function  TIM_Enable_CC_Channel
  * @brief  Enable capture/compare channels.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5  
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_CC_Channel(TIM_TypeDef *TIMx,uint32_t Channels)
{
  SET_BIT(TIMx->CCER, Channels);
}

/**
  * @function  TIM_Disable_CC_Channel
  * @brief  Disable capture/compare channels.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_CC_Channel(TIM_TypeDef *TIMx,uint32_t Channels)
{
  CLEAR_BIT(TIMx->CCER, Channels);
}

/**
  * @function  TIM_Get_CC_ChannelStatus
  * @brief  Indicate whether channel(s) is(are) enabled.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channels This parameter can be a combination of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0:channel is disalbe, 1:channel is enable
  */
 uint32_t TIM_Get_CC_ChannelStatus(TIM_TypeDef *TIMx,uint32_t Channels)
{
  return (READ_BIT(TIMx->CCER, Channels) == (Channels));
}

/** 
  @defgroup TIM_Output_Channel Output channel configuration
  */
/**
  * @function  TIM_Set_OC_Output
  * @brief  Configure an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  Configuration This parameter must be a combination of all the following values:
  *         @arg @ref TIM_OCPOLARITY_HIGH or @ref TIM_OCPOLARITY_LOW
  *         @arg @ref TIM_OCIDLESTATE_LOW or @ref TIM_OCIDLESTATE_HIGH
  * @retval None
  */
 void TIM_Set_OC_Output(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Configuration)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_CC1S<< SHIFT_TAB_OCxx[iChannel]));
  MODIFY_REG(TIMx->CCER, (TIM_CCER_CC1P << SHIFT_TAB_CCxP[iChannel]),
             (Configuration & TIM_CCER_CC1P) << SHIFT_TAB_CCxP[iChannel]);
  MODIFY_REG(TIMx->CR2, (TIM_CR2_OIS1 << SHIFT_TAB_OISx[iChannel]),
             (Configuration & TIM_CR2_OIS1) << SHIFT_TAB_OISx[iChannel]);
}


/**
  * @function  TIM_Set_OC_Mode
  * @brief  Define the behavior of the output reference signal OCxREF from which
  *         OCx and OCxN (when relevant) are derived.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  Mode This parameter can be one of the following values:
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  * @retval None
  */
 void TIM_Set_OC_Mode(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Mode)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_OC1M  | TIM_CCMR1_CC1S) << SHIFT_TAB_OCxx[iChannel]),  Mode << SHIFT_TAB_OCxx[iChannel]);
}

/**
  * @function  TIM_Get_OC_Mode
  * @brief  Get the output compare mode of an output channel.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_OCMODE_FROZEN
  *         @arg @ref TIM_OCMODE_ACTIVE
  *         @arg @ref TIM_OCMODE_INACTIVE
  *         @arg @ref TIM_OCMODE_TOGGLE
  *         @arg @ref TIM_OCMODE_FORCED_INACTIVE
  *         @arg @ref TIM_OCMODE_FORCED_ACTIVE
  *         @arg @ref TIM_OCMODE_PWM1
  *         @arg @ref TIM_OCMODE_PWM2
  */
 uint32_t TIM_Get_OC_Mode(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return (READ_BIT(*pReg, ((TIM_CCMR1_OC1M |TIM_CCMR1_CC1S) << SHIFT_TAB_OCxx[iChannel])) >> SHIFT_TAB_OCxx[iChannel]);
}

/**
  * @function  TIM_Set_OC_Polarity
  * @brief  Set the polarity of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  Polarity This parameter can be one of the following values:
  *         @arg @ref TIM_OCPOLARITY_HIGH
  *         @arg @ref TIM_OCPOLARITY_LOW
  * @retval None
  */
 void TIM_Set_OC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t Polarity)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CCER, (TIM_CCER_CC1P << SHIFT_TAB_CCxP[iChannel]),  Polarity << SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Get_OC_Polarity
  * @brief  Get the polarity of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_OCPOLARITY_HIGH
  *         @arg @ref TIM_OCPOLARITY_LOW
  */
 uint32_t TIM_Get_OC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CCER, (TIM_CCER_CC1P<< SHIFT_TAB_CCxP[iChannel])) >> SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Set_OC_IdleState
  * @brief  Set the IDLE state of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @param  IdleState This parameter can be one of the following values:
  *         @arg @ref TIM_OCIDLESTATE_LOW
  *         @arg @ref TIM_OCIDLESTATE_HIGH
  * @retval None
  */
 void TIM_Set_OC_IdleState(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t IdleState)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CR2, (TIM_CR2_OIS1<< SHIFT_TAB_OISx[iChannel]),  IdleState << SHIFT_TAB_OISx[iChannel]);
}

/**
  * @function  TIM_Get_OC_IdleState
  * @brief  Get the IDLE state of an output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH1N
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH2N
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH3N
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_OCIDLESTATE_LOW
  *         @arg @ref TIM_OCIDLESTATE_HIGH
  */
 uint32_t TIM_Get_OC_IdleState(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CR2, (TIM_CR2_OIS1<< SHIFT_TAB_OISx[iChannel])) >> SHIFT_TAB_OISx[iChannel]);
}

/**
  * @function  TIM_Enable_OC_Fast
  * @brief  Enable fast mode for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Fast(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1FE << SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Fast
  * @brief  Disable fast mode for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Fast(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1FE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_FastStatus
  * @brief  Indicates whether fast mode is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  fast disable, 1:Output Compare  fast enable
  */
 uint32_t TIM_Get_OC_FastStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1FE<< SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Enable_OC_Preload
  * @brief  Enable compare register (TIM_CCRx) preload for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Preload(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Preload
  * @brief  Disable compare register (TIM_CCRx) preload for the output channel.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Preload(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_PreloadStatus
  * @brief  Indicates whether compare register (TIM_CCRx) preload is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  preload  disable, 1:Output Compare  preload enable
  */
 uint32_t TIM_Get_OC_PreloadStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1PE<< SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Enable_OC_Clear
  * @brief  Enable clearing the output channel on an external event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Enable_OC_Clear(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  SET_BIT(*pReg, (TIM_CCMR1_OC1CE<< SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Disable_OC_Clear
  * @brief  Disable clearing the output channel on an external event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval None
  */
 void TIM_Disable_OC_Clear(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  CLEAR_BIT(*pReg, (TIM_CCMR1_OC1CE << SHIFT_TAB_OCxx[iChannel]));
}

/**
  * @function  TIM_Get_OC_ClearStatus
  * @brief  Indicates clearing the output channel on an external event is enabled for the output channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  *         @arg @ref TIM_CHANNEL_CH5
  *         @arg @ref TIM_CHANNEL_CH6
  * @retval 0: Output Compare  clear  disable, 1:Output Compare  clear enable
  */
 uint32_t TIM_Get_OC_ClearStatus(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  register uint32_t bitfield = TIM_CCMR1_OC1CE << SHIFT_TAB_OCxx[iChannel];
  return (READ_BIT(*pReg, bitfield) == bitfield);
}

/**
  * @function  TIM_Set_OC_CompareCH1
  * @brief  Set compare value for output channel 1 (TIM_CCR1)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH1(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR1, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH2
  * @brief  Set compare value for output channel 2 (TIM_CCR2)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH2(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR2, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH3
  * @brief  Set compare value for output channel 3 (TIM_CCR3)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH3(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR3, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH4
  * @brief  Set compare value for output channel 4 (TIM_CCR4)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH4(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR4, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH5
  * @brief  Set compare value for output channel 5 (TIM_CCR5)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH5(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR5, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH6
  * @brief  Set compare value for output channel 6 (TIM_CCR6)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH6(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR6, CompareValue);
}

/**
  * @function  TIM_Get_OC_CompareCH1
  * @brief  Get compare value (TIM_CCR1) set for  output channel 1
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH1(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1));
}

/**
  * @function  TIM_Get_OC_CompareCH2
  * @brief  Get compare value (TIM_CCR2) set for  output channel 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH2(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2));
}

/**
  * @function  TIM_Get_OC_CompareCH3
  * @brief  Get compare value (TIM_CCR3) set for  output channel 3
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH3(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3));
}

/**
  * @function  TIM_Get_OC_CompareCH4
  * @brief  Get compare value (TIM_CCR4) set for  output channel 4
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH4(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR4));
}

/**
  * @function  TIM_Get_OC_CompareCH5
  * @brief  Get compare value (TIM_CCR5) set for  output channel 5
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH5(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR5));
}

/**
  * @function  TIM_Get_OC_CompareCH6
  * @brief  Get compare value (TIM_CCR6) set for  output channel 6
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH6(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR6));
}

/**
  * @function  TIM_Set_IC_ActiveInput
  * @brief  Set the active input
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICActiveInput This parameter can be one of the following values:
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  * @retval None
  */
void TIM_Set_IC_ActiveInput(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICActiveInput)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_CC1S) << SHIFT_TAB_ICxx[iChannel]), (ICActiveInput) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_ActiveInput
  * @brief  Get the current active input
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ACTIVEINPUT_DIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_INDIRECTTI
  *         @arg @ref TIM_ACTIVEINPUT_TRC
  */
uint32_t TIM_Get_IC_ActiveInput(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_CC1S) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}


/**
  * @function  TIM_Set_IC_Prescaler
  * @brief  Set the prescaler of input channel
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPrescaler This parameter can be one of the following values:
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  * @retval None
  */
void TIM_Set_IC_Prescaler(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICPrescaler)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_IC1PSC) << SHIFT_TAB_ICxx[iChannel]), (ICPrescaler >> 16U) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_Prescaler
  * @brief  Get the current prescaler value acting on an  input channel.
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ICPSC_DIV1
  *         @arg @ref TIM_ICPSC_DIV2
  *         @arg @ref TIM_ICPSC_DIV4
  *         @arg @ref TIM_ICPSC_DIV8
  */
uint32_t TIM_Get_IC_Prescaler(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_IC1PSC) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}

/**
  * @function  TIM_Set_IC_Filter
  * @brief  Set the input filter duration
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICFilter This parameter can be one of the following values:
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
 void TIM_Set_IC_Filter(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICFilter)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  MODIFY_REG(*pReg, ((TIM_CCMR1_IC1F) << SHIFT_TAB_ICxx[iChannel]), (ICFilter << 4U) << SHIFT_TAB_ICxx[iChannel]);
}

/**
  * @function  TIM_Get_IC_Filter
  * @brief  Get the input filter duration
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  */
 uint32_t TIM_Get_IC_Filter(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  register uint32_t *pReg = (uint32_t *)((uint32_t)((uint32_t)(&TIMx->CCMR1) + OFFSET_TAB_CCMRx[iChannel]));
  return ((READ_BIT(*pReg, ((TIM_CCMR1_IC1F) << SHIFT_TAB_ICxx[iChannel])) >> SHIFT_TAB_ICxx[iChannel]) << 16U);
}

/**
  * @function  TIM_Set_IC_Polarity
  * @brief  Set the input channel polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @param  ICPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_ICPolarity_Rising
  *         @arg @ref TIM_ICPolarity_Falling
  *         @arg @ref TIM_ICPolarity_BothEdge
  * @retval None
  */
 void TIM_Set_IC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel, uint32_t ICPolarity)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  MODIFY_REG(TIMx->CCER, ((TIM_CCER_CC1NP|TIM_CCER_CC1P) << SHIFT_TAB_CCxP[iChannel]),
             ICPolarity << SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Get_IC_Polarity
  * @brief  Get the current input channel polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  Channel This parameter can be one of the following values:
  *         @arg @ref TIM_CHANNEL_CH1
  *         @arg @ref TIM_CHANNEL_CH2
  *         @arg @ref TIM_CHANNEL_CH3
  *         @arg @ref TIM_CHANNEL_CH4
  * @retval Returned value can be one of the following values:
  *         @arg @ref TIM_ICPolarity_Rising
  *         @arg @ref TIM_ICPolarity_Falling
  *         @arg @ref TIM_ICPolarity_BothEdge
  */
 uint32_t TIM_Get_IC_Polarity(TIM_TypeDef *TIMx,uint32_t Channel)
{
  register uint8_t iChannel = TIM_GET_CHANNEL_INDEX(Channel);
  return (READ_BIT(TIMx->CCER, ((TIM_CCER_CC1NP|TIM_CCER_CC1P)  << SHIFT_TAB_CCxP[iChannel])) >>
          SHIFT_TAB_CCxP[iChannel]);
}

/**
  * @function  TIM_Enable_IC_XORCombination
  * @brief  Connect the TIM_CH1, CH2 and CH3 pins  to the TI1 input (XOR combination)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_IC_XORCombination(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CR2, (TIM_CR2_TI1S));
}

/**
  * @function  TIM_Disable_IC_XORCombination
  * @brief  Disconnect the TIM_CH1, CH2 and CH3 pins  from the TI1 input
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_IC_XORCombination(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CR2, TIM_CR2_TI1S);
}

/**
  * @function  TIM_Get_IC_XORCombinationStatus
  * @brief  Indicates whether the TIM_CH1, CH2 and CH3 pins are connectected to the TI1 input
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0: Connect TIM_CH1 to TI1 input, 1:Connect the TIM_CH1/CH2/CH3 pins to the TI1 input
  */
 uint32_t TIM_Get_IC_XORCombinationStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->CR2, TIM_CR2_TI1S) == (TIM_CR2_TI1S));
}

/**
  * @function  TIM_Get_IC_CaptureCH1
  * @brief  Get captured value for input channel 1
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH1(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1));
}

/**
  * @function  TIM_Get_IC_CaptureCH2
  * @brief  Get captured value for input channel 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH2(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2));
}

/**
  * @function  TIM_Get_IC_CaptureCH3
  * @brief  Get captured value for input channel 3
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH3(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3));
}

/**
  * @function  TIM_Get_IC_CaptureCH4
  * @brief  Get captured value for input channel 4
  * @param  TIMx:
            @arg TIM Instance
  * @retval CapturedValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_IC_CaptureCH4(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR4));
}

/** 
  @defgroup TIM_Clock_Selection Counter clock selection
  */
/**
  * @function  TIM_Enable_ExternalClock
  * @brief  Enable external clock mode 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_ExternalClock(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->SMCR, TIM_SMCR_ECE);
}

/**
  * @function  TIM_Disable_ExternalClock
  * @brief  Disable external clock mode 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_ExternalClock(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->SMCR, TIM_SMCR_ECE);
}

/**
  * @function  TIM_Get_ExternalClockStatus
  * @brief  Indicate whether external clock mode 2 is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0:disable external clock,1:enable external clock
  */
 uint32_t TIM_Get_ExternalClockStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->SMCR, TIM_SMCR_ECE) == TIM_SMCR_ECE);
}

/**
  * @function  TIM_Set_ClockSource
  * @brief  Set the clock source of the counter clock
  * @param  TIMx:
            @arg TIM Instance
  * @param  ClockSource This parameter can be one of the following values:
  *         @arg @ref TIM_CLOCKSOURCE_INTERNAL
  *         @arg @ref TIM_CLOCKSOURCE_EXT_MODE1
  *         @arg @ref TIM_CLOCKSOURCE_EXT_MODE2
  * @retval None
  */
 void TIM_Set_ClockSource(TIM_TypeDef *TIMx,uint32_t ClockSource)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS| TIM_SMCR_ECE, ClockSource);
}

/**
  * @function  TIM_Set_EncoderMode
  * @brief  Set the encoder interface mode
  * @param  TIMx:
            @arg TIM Instance
  * @param  EncoderMode This parameter can be one of the following values:
  *         @arg @ref TIM_ENCODERMODE_X2_TI1
  *         @arg @ref TIM_ENCODERMODE_X2_TI2
  *         @arg @ref TIM_ENCODERMODE_X4_TI12
  * @retval None
  */
 void TIM_Set_EncoderMode(TIM_TypeDef *TIMx,uint32_t EncoderMode)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS, EncoderMode);
}


/** 
 @defgroup TIM_Timer_Synchronization Timer synchronisation configuration
  */
/**
  * @function  TIM_Set_SlaveMode
  * @brief  Set the synchronization mode of a slave timer
  * @param  TIMx:
            @arg TIM Instance
  * @param  SlaveMode This parameter can be one of the following values:
  *         @arg @ref TIM_SLAVEMODE_DISABLED
  *         @arg @ref TIM_SLAVEMODE_RESET
  *         @arg @ref TIM_SLAVEMODE_GATED
  *         @arg @ref TIM_SLAVEMODE_TRIGGER
  * @retval None
  */
 void TIM_Set_SlaveMode(TIM_TypeDef *TIMx,uint32_t SlaveMode)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_SMS, SlaveMode);
}

/**
  * @function  TIM_Set_TriggerInput
  * @brief  Set the selects the trigger input to be used to synchronize the counter
  * @param  TIMx:
            @arg TIM Instance
  * @param  TriggerInput This parameter can be one of the following values:
  *         @arg @ref TIM_TS_TI1F_ED
  *         @arg @ref TIM_TS_TI1FP1
  *         @arg @ref TIM_TS_TI2FP2
  *         @arg @ref TIM_TS_ETRF
  * @retval None
  */
 void TIM_Set_TriggerInput(TIM_TypeDef *TIMx,uint32_t TriggerInput)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_TS, TriggerInput);
}

/**
  * @function  TIM_Set_ETR
  * @brief  Configure the external trigger (ETR) input
  * @param  TIMx:
            @arg TIM Instance
  * @param  ETRPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_ETR_POLARITY_NONINVERTED
  *         @arg @ref TIM_ETR_POLARITY_INVERTED
  * @param  ETRPrescaler This parameter can be one of the following values:
  *         @arg @ref TIM_ETR_PRESCALER_DIV1
  *         @arg @ref TIM_ETR_PRESCALER_DIV2
  *         @arg @ref TIM_ETR_PRESCALER_DIV4
  *         @arg @ref TIM_ETR_PRESCALER_DIV8
  * @param  ETRFilter This parameter can be one of the following values:
  *         @arg @ref TIM_ETR_FILTER_FDIV1
  *         @arg @ref TIM_ETR_FILTER_FDIV1_N2
  *         @arg @ref TIM_ETR_FILTER_FDIV1_N4
  *         @arg @ref TIM_ETR_FILTER_FDIV1_N8
  *         @arg @ref TIM_ETR_FILTER_FDIV2_N6
  *         @arg @ref TIM_ETR_FILTER_FDIV2_N8
  *         @arg @ref TIM_ETR_FILTER_FDIV4_N6
  *         @arg @ref TIM_ETR_FILTER_FDIV4_N8
  *         @arg @ref TIM_ETR_FILTER_FDIV8_N6
  *         @arg @ref TIM_ETR_FILTER_FDIV8_N8
  *         @arg @ref TIM_ETR_FILTER_FDIV16_N5
  *         @arg @ref TIM_ETR_FILTER_FDIV16_N6
  *         @arg @ref TIM_ETR_FILTER_FDIV16_N8
  *         @arg @ref TIM_ETR_FILTER_FDIV32_N5
  *         @arg @ref TIM_ETR_FILTER_FDIV32_N6
  *         @arg @ref TIM_ETR_FILTER_FDIV32_N8
  * @retval None
  */
 void TIM_Set_ETR(TIM_TypeDef *TIMx,uint32_t ETRPolarity, uint32_t ETRPrescaler,uint32_t ETRFilter)
{
  MODIFY_REG(TIMx->SMCR, TIM_SMCR_ETP | TIM_SMCR_ETPS | TIM_SMCR_ETF, ETRPolarity | ETRPrescaler | ETRFilter);
}


/** 
  @defgroup TIM_Break_Function Break function configuration
  */
/**
  * @function  TIM_Enable_BRK
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_BRK(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_BKE);
}

/**
  * @function  TIM_Disable_BRK
  * @brief  Disable the break function.
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_BRK(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_BKE);
}

/**
  * @function  TIM_Set_BreakPolarity
  * @brief  Configure the break input
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_BREAK_POLARITY_LOW
  *         @arg @ref TIM_BREAK_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKP, BreakPolarity);
}

/**
  * @function  TIM_Enable_CMP4Break
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CMP4Break(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_CMP4OE);
}

/**
  * @function  TIM_Disable_CMP4Break
  * @brief  Disable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CMP4Break(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_CMP4OE);
}

/**
  * @function  TIM_Set_CMP4BreakPolarity
  * @brief  Configure CMP4 out polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_CMP4OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP4OEP_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_CMP4BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_CMP4OEP, BreakPolarity);
}

/**
  * @function  TIM_Enable_CMP5Break
  * @brief  Enable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CMP5Break(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_CMP5OE);
}

/**
  * @function  TIM_Disable_CMP5Break
  * @brief  Disable the break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CMP5Break(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_CMP5OE);
}

/**
  * @function  TIM_Set_CMP5BreakPolarity
  * @brief  Configure CMP5 out polarity
  * @param  TIMx:
            @arg TIM Instance
  * @param  BreakPolarity This parameter can be one of the following values:
  *         @arg @ref TIM_CMP5OEP_POLARITY_LOW
  *         @arg @ref TIM_CMP5OEP_POLARITY_HIGH
  * @retval None
  */
 void TIM_Set_CMP5BreakPolarity(TIM_TypeDef *TIMx,uint32_t BreakPolarity)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_CMP5OEP, BreakPolarity);
}

/**
  * @function  TIM_Enable_DOE
  * @brief  Enable DOE function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_DOE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_DOE);
}

/**
  * @function  TIM_Disable_DOE
  * @brief  Disable DOE function
  * @param  TIMx:
            @arg TIM Instance
  * @note The channel wait a dead-time to outputs idle state after MOE=0
  * @retval None
  */
 void TIM_Disable_DOE(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_DOE);
}

/**
  * @function  TIM_Enable_HardFaultBreak
  * @brief  Enable hardfault output of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Enable_HardFaultBreak( TIM_TypeDef *TIMx )
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_HardFault_OE);
}

/**
  * @function  TIM_Disable_HardFaultBreak
  * @brief  Disnable hardfault output of break function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
void TIM_Disable_HardFaultBreak( TIM_TypeDef *TIMx )
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_HardFault_OE);
}

/**
  * @function  TIM_Set_BRKFilter
  * @brief  Set the sampling frequency and the filter length of break input 
  * @param  TIMx:
            @arg TIM Instance
  * @param  BKFilter This parameter can be one of the following values:
  *         @arg TIM_BDTR_BKF_NoFilter_AsynBreak  ((uint32_t)0<<22)
  *         @arg TIM_BDTR_BKF_Fck_N2              ((uint32_t)1<<22)
  *         @arg TIM_BDTR_BKF_Fck_N4              ((uint32_t)2<<22)
  *         @arg TIM_BDTR_BKF_Fck_N8              ((uint32_t)3<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv2_N6         ((uint32_t)4<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv2_N8         ((uint32_t)5<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv4_N6         ((uint32_t)6<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv4_N8         ((uint32_t)7<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv8_N6         ((uint32_t)8<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv8_N8         ((uint32_t)9<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv16_N5        ((uint32_t)10<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv16_N6        ((uint32_t)11<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv16_N8        ((uint32_t)12<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv32_N5        ((uint32_t)13<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv32_N6        ((uint32_t)14<<22)
  *         @arg TIM_BDTR_BKF_FdtsDiv32_N8        ((uint32_t)15<<22)
  * @retval None
  */
void TIM_Set_BRKFilter(TIM_TypeDef *TIMx, uint32_t BKFilter)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_BKF, BKFilter);
}

/**
  * @function  TIM_Set_OffStates
  * @brief  Select the outputs off state (enabled v.s. disabled) in Idle and Run modes
  * @param  TIMx:
            @arg TIM Instance
  * @param  OffStateIdle This parameter can be one of the following values:
  *         @arg @ref TIM_OSSI_DISABLE
  *         @arg @ref TIM_OSSI_ENABLE
  * @param  OffStateRun This parameter can be one of the following values:
  *         @arg @ref TIM_OSSR_DISABLE
  *         @arg @ref TIM_OSSR_ENABLE
  * @retval None
  */
 void TIM_Set_OffStates(TIM_TypeDef *TIMx,uint32_t OffStateIdle, uint32_t OffStateRun)
{
  MODIFY_REG(TIMx->BDTR, TIM_BDTR_OSSI |TIM_BDTR_OSSR, OffStateIdle | OffStateRun);
}

/**
  * @function  TIM_Enable_AOE
  * @brief  Enable automatic output (MOE can be set by software or automatically when a break input is active)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_AOE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR,TIM_BDTR_AOE);
}

/**
  * @function  TIM_Disable_AOE
  * @brief  Disable automatic output (MOE can be set only by software)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_AOE(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_AOE);
}

/**
  * @function  TIM_Get_AOEStatus
  * @brief  Indicate whether automatic output is enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0: disable automatic output,1:enable automatic output
  */
 uint32_t TIM_Get_AOEStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->BDTR, TIM_BDTR_AOE) ==TIM_BDTR_AOE);
}

/**
  * @function  TIM_Enable_MOE
  * @brief  Enable the outputs (set the MOE bit in TIM_BDTR register)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_MOE(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->BDTR, TIM_BDTR_MOE);
}

/**
  * @function  TIM_Disable_MOE
  * @brief  Disable the outputs (reset the MOE bit in TIM_BDTR register)
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_MOE(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->BDTR, TIM_BDTR_MOE);
}

/**
  * @function  TIM_Get_MOEStatus
  * @brief  Indicates whether outputs are enabled
  * @param  TIMx:
            @arg TIM Instance
  * @retval 0: disable outputs,1:enable outputs
  */
 uint32_t TIM_Get_MOEStatus(TIM_TypeDef *TIMx)
{
  return (READ_BIT(TIMx->BDTR, TIM_BDTR_MOE) == TIM_BDTR_MOE);
}

/***
  * @function  TIM_Set_TI1
  * @brief  Configure the TI1 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_ICPolarity_Rising
  *     @arg TIM_ICPolarity_Falling
  *     @arg TIM_ICPolarity_BothEdge
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ICSelection_DirectTI: TIM Input 1 is selected to be connected to IC1
  *     @arg TIM_ICSelection_IndirectTI: TIM Input 1 is selected to be connected to IC2
  *     @arg TIM_ICSelection_TRC: TIM Input 1 is selected to be connected to TRC
  * @param  TIM_ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
static void TIM_Set_TI1(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,
                       uint16_t TIM_ICFilter)
{
  uint16_t tmpccmr1 = 0, tmpccer = 0;
  /* Disable the Channel 1: Reset the CC1E Bit */
  TIMx->CCER &= (uint16_t)~((uint16_t)TIM_CCER_CC1E);
  tmpccmr1 = TIMx->CCMR1;
  tmpccer = TIMx->CCER;
  /* Select the Input and set the filter */
  tmpccmr1 &= (uint16_t)(((uint16_t)~((uint16_t)TIM_CCMR1_CC1S)) & ((uint16_t)~((uint16_t)TIM_CCMR1_IC1F)));
  tmpccmr1 |= (uint16_t)(TIM_ICSelection | (uint16_t)(TIM_ICFilter << (uint16_t)4));

	/* Select the Polarity and set the CC1E Bit */
	tmpccer &= (uint16_t)~((uint16_t)(TIM_CCER_CC1P));
	tmpccer |= (uint16_t)(TIM_ICPolarity | (uint16_t)TIM_CCER_CC1E);

  /* Write to TIMx CCMR1 and CCER registers */
  TIMx->CCMR1 = tmpccmr1;
  TIMx->CCER = tmpccer;
}

/***
  * @function  TIM_Set_TI2
  * @brief  Configure the TI2 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_ICPolarity_Rising
  *     @arg TIM_ICPolarity_Falling
  *     @arg TIM_ICPolarity_BothEdge
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ICSelection_DirectTI: TIM Input 2 is selected to be connected to IC2
  *     @arg TIM_ICSelection_IndirectTI: TIM Input 2 is selected to be connected to IC1
  *     @arg TIM_ICSelection_TRC: TIM Input 2 is selected to be connected to TRC
  * @param  TIM_ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
static void TIM_Set_TI2(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,
                       uint16_t TIM_ICFilter)
{
  uint16_t tmpccmr1 = 0, tmpccer = 0, tmp = 0;
  /* Disable the Channel 2: Reset the CC2E Bit */
  TIMx->CCER &= (uint16_t)~((uint16_t)TIM_CCER_CC2E);
  tmpccmr1 = TIMx->CCMR1;
  tmpccer = TIMx->CCER;
  tmp = (uint16_t)(TIM_ICPolarity << 4);
  /* Select the Input and set the filter */
  tmpccmr1 &= (uint16_t)(((uint16_t)~((uint16_t)TIM_CCMR1_CC2S)) & ((uint16_t)~((uint16_t)TIM_CCMR1_IC2F)));
  tmpccmr1 |= (uint16_t)(TIM_ICFilter << 12);
  tmpccmr1 |= (uint16_t)(TIM_ICSelection << 8);

	/* Select the Polarity and set the CC2E Bit */
	tmpccer &= (uint16_t)~((uint16_t)(TIM_CCER_CC2P));
	tmpccer |=  (uint16_t)(tmp | (uint16_t)TIM_CCER_CC2E);
  
  /* Write to TIMx CCMR1 and CCER registers */
  TIMx->CCMR1 = tmpccmr1 ;
  TIMx->CCER = tmpccer;
}

/***
  * @function  TIM_Set_TI3
  * @brief  Configure the TI3 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity
  *   This parameter can be one of the following values:
  *     @arg TIM_ICPolarity_Rising
  *     @arg TIM_ICPolarity_Falling
  *     @arg TIM_ICPolarity_BothEdge
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ICSelection_DirectTI: TIM Input 3 is selected to be connected to IC3
  *     @arg TIM_ICSelection_IndirectTI: TIM Input 3 is selected to be connected to IC4
  *     @arg TIM_ICSelection_TRC: TIM Input 3 is selected to be connected to TRC
  * @param  TIM_ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
static void TIM_Set_TI3(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,
                       uint16_t TIM_ICFilter)
{
  uint16_t tmpccmr2 = 0, tmpccer = 0, tmp = 0;
  /* Disable the Channel 3: Reset the CC3E Bit */
  TIMx->CCER &= (uint16_t)~((uint16_t)TIM_CCER_CC3E);
  tmpccmr2 = TIMx->CCMR2;
  tmpccer = TIMx->CCER;
  tmp = (uint16_t)(TIM_ICPolarity << 8);
  /* Select the Input and set the filter */
  tmpccmr2 &= (uint16_t)(((uint16_t)~((uint16_t)TIM_CCMR2_CC3S)) & ((uint16_t)~((uint16_t)TIM_CCMR2_IC3F)));
  tmpccmr2 |= (uint16_t)(TIM_ICSelection | (uint16_t)(TIM_ICFilter << (uint16_t)4));

	/* Select the Polarity and set the CC3E Bit */
	tmpccer &= (uint16_t)~((uint16_t)(TIM_CCER_CC3P));
	tmpccer |= (uint16_t)(tmp | (uint16_t)TIM_CCER_CC3E);

  /* Write to TIMx CCMR2 and CCER registers */
  TIMx->CCMR2 = tmpccmr2;
  TIMx->CCER = tmpccer;
}

/***
  * @function  TIM_Set_TI4
  * @brief  Configure the TI4 as Input
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_ICPolarity : The Input Polarity.
  *   This parameter can be one of the following values:
  *     @arg TIM_ICPolarity_Rising
  *     @arg TIM_ICPolarity_Falling
  *     @arg TIM_ICPolarity_BothEdge
  * @param  TIM_ICSelection: specifies the input to be used
  *   This parameter can be one of the following values:
  *     @arg TIM_ICSelection_DirectTI: TIM Input 4 is selected to be connected to IC4
  *     @arg TIM_ICSelection_IndirectTI: TIM Input 4 is selected to be connected to IC3
  *     @arg TIM_ICSelection_TRC: TIM Input 4 is selected to be connected to TRC
  * @param  TIM_ICFilter
  *         @arg @ref TIM_IC_FILTER_FDIV1
  *         @arg @ref TIM_IC_FILTER_FDIV1_N2	
  *         @arg @ref TIM_IC_FILTER_FDIV1_N4
  *         @arg @ref TIM_IC_FILTER_FDIV1_N8
  *         @arg @ref TIM_IC_FILTER_FDIV2_N6
  *         @arg @ref TIM_IC_FILTER_FDIV2_N8
  *         @arg @ref TIM_IC_FILTER_FDIV4_N6
  *         @arg @ref TIM_IC_FILTER_FDIV4_N8
  *         @arg @ref TIM_IC_FILTER_FDIV8_N6
  *         @arg @ref TIM_IC_FILTER_FDIV8_N8
  *         @arg @ref TIM_IC_FILTER_FDIV16_N5
  *         @arg @ref TIM_IC_FILTER_FDIV16_N6
  *         @arg @ref TIM_IC_FILTER_FDIV16_N8
  *         @arg @ref TIM_IC_FILTER_FDIV32_N5
  *         @arg @ref TIM_IC_FILTER_FDIV32_N6	
  *         @arg @ref TIM_IC_FILTER_FDIV32_N8
  * @retval None
  */
static void TIM_Set_TI4(TIM_TypeDef* TIMx, uint16_t TIM_ICPolarity, uint16_t TIM_ICSelection,
                       uint16_t TIM_ICFilter)
{
  uint16_t tmpccmr2 = 0, tmpccer = 0, tmp = 0;

   /* Disable the Channel 4: Reset the CC4E Bit */
  TIMx->CCER &= (uint16_t)~((uint16_t)TIM_CCER_CC4E);
  tmpccmr2 = TIMx->CCMR2;
  tmpccer = TIMx->CCER;
  tmp = (uint16_t)(TIM_ICPolarity << 12);
  /* Select the Input and set the filter */
  tmpccmr2 &= (uint16_t)((uint16_t)(~(uint16_t)TIM_CCMR2_CC4S) & ((uint16_t)~((uint16_t)TIM_CCMR2_IC4F)));
  tmpccmr2 |= (uint16_t)(TIM_ICSelection << 8);
  tmpccmr2 |= (uint16_t)(TIM_ICFilter << 12);

	/* Select the Polarity and set the CC4E Bit */
	tmpccer &= (uint16_t)~((uint16_t)(TIM_CCER_CC4P));
	tmpccer |= (uint16_t)(tmp | (uint16_t)TIM_CCER_CC4E);
  
  /* Write to TIMx CCMR2 and CCER registers */
  TIMx->CCMR2 = tmpccmr2;
  TIMx->CCER = tmpccer;
}


/**
  * @function  TIM_Enable_IT
  * @brief  TIM enable IT
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_IT:
  *   This parameter can be one of the following values:
  *     @arg 
  *     TIM_DIER_UIE          ((uint32_t)0x00001) 
  *     TIM_DIER_CC1IE        ((uint32_t)0x00002) 
  *     TIM_DIER_CC2IE        ((uint32_t)0x00004) 
  *     TIM_DIER_CC3IE        ((uint32_t)0x00008) 
  *     TIM_DIER_CC4IE        ((uint32_t)0x00010) 
  *     TIM_DIER_COMIE        ((uint32_t)0x00020) 
  *     TIM_DIER_TIE          ((uint32_t)0x00040) 
  *     TIM_DIER_BIE          ((uint32_t)0x00080) 
  * @retval None
  */
void TIM_Enable_IT(TIM_TypeDef* TIMx, uint32_t TIM_IT)
{  
        /* Enable the Interrupt sources */
        TIMx->DIER |= TIM_IT;
}


/**
  * @function  TIM_Disable_IT
  * @brief  TIM disable IT
  * @param  TIMx:
            @arg TIM Instance
  * @param  TIM_IT:
  *   This parameter can be one of the following values:
  *     @arg 
  *     TIM_DIER_UIE          ((uint32_t)0x00001) 
  *     TIM_DIER_CC1IE        ((uint32_t)0x00002) 
  *     TIM_DIER_CC2IE        ((uint32_t)0x00004) 
  *     TIM_DIER_CC3IE        ((uint32_t)0x00008) 
  *     TIM_DIER_CC4IE        ((uint32_t)0x00010) 
  *     TIM_DIER_COMIE        ((uint32_t)0x00020) 
  *     TIM_DIER_TIE          ((uint32_t)0x00040) 
  *     TIM_DIER_BIE          ((uint32_t)0x00080) 
  * @retval None
  */
void TIM_Disable_IT(TIM_TypeDef* TIMx, uint32_t TIM_IT)
{  
        /* Disable the Interrupt sources */
        TIMx->DIER &= (uint32_t)~TIM_IT;

}

/**
  * @function  TIM_Get_Flag
  * @brief  TIM_ClearITPendingBit
  * @param  TIMx:
            @arg TIM Instance
  * @retval  Flag:
  *   This parameter can be one of the following values:
  *     @arg 
  *     TIM_SR_UIF          ((uint32_t)0x00001) 
  *     TIM_SR_CC1IF        ((uint32_t)0x00002) 
  *     TIM_SR_CC2IF        ((uint32_t)0x00004) 
  *     TIM_SR_CC3IF        ((uint32_t)0x00008) 
  *     TIM_SR_CC4IF        ((uint32_t)0x00010) 
  *     TIM_SR_COMIF        ((uint32_t)0x00020) 
  *     TIM_SR_TIF          ((uint32_t)0x00040) 
  *     TIM_SR_BIF          ((uint32_t)0x00080) 
  *     TIM_SR_CC1OF        ((uint32_t)0x02000)
  *     TIM_SR_CC2OF        ((uint32_t)0x04000)
  *     TIM_SR_CC3OF        ((uint32_t)0x08000)
  *     TIM_SR_CC4OF        ((uint32_t)0x10000)
  */
uint32_t TIM_Get_Flag(TIM_TypeDef* TIMx, uint32_t Flag)
{
  return (READ_BIT(TIMx->SR, Flag) == (Flag));
}

/**
  * @function  TIM_Clear_Flag
  * @brief  Clear IT flag
  * @param  TIMx:
            @arg TIM Instance
  * @param  IT_Flag:
  *   This parameter can be one of the following values:
  *     @arg 
  *     TIM_SR_UIF          ((uint32_t)0x00001) 
  *     TIM_SR_CC1IF        ((uint32_t)0x00002) 
  *     TIM_SR_CC2IF        ((uint32_t)0x00004) 
  *     TIM_SR_CC3IF        ((uint32_t)0x00008) 
  *     TIM_SR_CC4IF        ((uint32_t)0x00010) 
  *     TIM_SR_COMIF        ((uint32_t)0x00020) 
  *     TIM_SR_TIF          ((uint32_t)0x00040) 
  *     TIM_SR_BIF          ((uint32_t)0x00080) 
  *     TIM_SR_CC1OF        ((uint32_t)0x02000)
  *     TIM_SR_CC2OF        ((uint32_t)0x04000)
  *     TIM_SR_CC3OF        ((uint32_t)0x08000)
  *     TIM_SR_CC4OF        ((uint32_t)0x10000)
  * @retval None
  */
 void TIM_Clear_Flag(TIM_TypeDef *TIMx, uint32_t IT_Flag)
{
  CLEAR_BIT(TIMx->SR, IT_Flag);
}

/**
  * @function  TIM_Set_GenerateEvent
  * @brief  Generate event
  * @param  TIMx:
            @arg TIM Instance
  * @param  Event:
  *   This parameter can be one of the following values:
  *     @arg 
  *     TIM_EGR_UG          ((uint32_t)0x00001) 
  *     TIM_EGR_CC1G        ((uint32_t)0x00002) 
  *     TIM_EGR_CC2G        ((uint32_t)0x00004) 
  *     TIM_EGR_CC3G        ((uint32_t)0x00008) 
  *     TIM_EGR_CC4G        ((uint32_t)0x00010) 
  *     TIM_EGR_COMG        ((uint32_t)0x00020) 
  *     TIM_EGR_TG          ((uint32_t)0x00040) 
  *     TIM_EGR_BG          ((uint32_t)0x00080) 
  * @retval None
  */
 void TIM_Set_GenerateEvent(TIM_TypeDef *TIMx, uint32_t Event)
{
  SET_BIT(TIMx->EGR, Event);
}

/**
  * @function  TIM_Set_OC_CompareCH1N
  * @brief  Set compare value for output channel 1N (TIM_CCR1N)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH1N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR1N, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH2N
  * @brief  Set compare value for output channel 2N (TIM_CCR2N)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH2N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR2N, CompareValue);
}

/**
  * @function  TIM_Set_OC_CompareCH3N
  * @brief  Set compare value for output channel 3N (TIM_CCR3N)
  * @param  TIMx:
            @arg TIM Instance
  * @param  CompareValue between Min_Data=0 and Max_Data=65535
  * @retval None
  */
 void TIM_Set_OC_CompareCH3N(TIM_TypeDef *TIMx,uint32_t CompareValue)
{
  WRITE_REG(TIMx->CCR3N, CompareValue);
}

/**
  * @function  TIM_Get_OC_CompareCH1N
  * @brief  Get compare value (TIM_CCR1N) set for  output channel 1
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH1N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR1N));
}

/**
  * @function  TIM_Get_OC_CompareCH2N
  * @brief  Get compare value (TIM_CCR2N) set for  output channel 2
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH2N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR2N));
}

/**
  * @function  TIM_Get_OC_CompareCH3N
  * @brief  Get compare value (TIM_CCR3N) set for  output channel 3
  * @param  TIMx:
            @arg TIM Instance
  * @retval CompareValue (between Min_Data=0 and Max_Data=65535)
  */
 uint32_t TIM_Get_OC_CompareCH3N(TIM_TypeDef *TIMx)
{
  return (uint32_t)(READ_REG(TIMx->CCR3N));
}

/**
  * @function  TIM_Enable_CCR1N
  * @brief Enable the CCR1N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR1N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR1N, TIM_CCR1N_EN);
}

/**
  * @function  TIM_Disable_CCR1N
  * @brief Disable the CCR1N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR1N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR1N, TIM_CCR1N_EN);
}

/**
  * @function  TIM_Enable_CCR2N
  * @brief Enable the CCR2N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR2N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR2N, TIM_CCR2N_EN);
}

/**
  * @function  TIM_Disable_CCR2N
  * @brief Disable the CCR2N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR2N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR2N, TIM_CCR2N_EN);
}

/**
  * @function  TIM_Enable_CCR3N
  * @brief Enable the CCR3N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Enable_CCR3N(TIM_TypeDef *TIMx)
{
  SET_BIT(TIMx->CCR3N, TIM_CCR3N_EN);
}

/**
  * @function  TIM_Disable_CCR3N
  * @brief Disable the CCR3N function
  * @param  TIMx:
            @arg TIM Instance
  * @retval None
  */
 void TIM_Disable_CCR3N(TIM_TypeDef *TIMx)
{
  CLEAR_BIT(TIMx->CCR3N, TIM_CCR3N_EN);
}

