/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_timer.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_timer.h"
#include "rx32s11xx_uart.h"

/*
*********************************************************************************************************
*                                            Local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            Local function
*
*********************************************************************************************************
*/
/**
  * @brief  Deinitializes the TMRx peripheral registers to their default reset values.
  * @param  TMRx: where x can be 0 to 3 to select the TIM peripheral.
  * @retval None

  */
void TMR_DeInit(TMR_TypeDef* TMRx)
{  
	TMRx->TMRCON = 0x0000;  
	TMRx->TMRDIV = 0x0000;
	TMRx->TMRPRD = 0x0000;
	TMRx->TMRCAP = 0x0000;
	TMRx->TMRCNT = 0x0000;  
	TMRx->TMRCMP = 0x0000;
	TMRx->TMRIE = 0x0000;
	TMRx->TMRIF = 0x0000;
}

/**
  * @brief  Fills each TMRx_StructInit member with its default value.
  * @param  TMRx_StructInit : pointer to a TMRx_StructInit
  *         structure which will be initialized.
  * @retval None
  */
void TMRx_StructInit(TMR_InitTypeDef* TMR_InitStruct)
{
	TMR_InitStruct->Mode = TMR_TMRCON_MODE_EVENT;
  TMR_InitStruct->Polarity = TMR_TMRCON_POLARITY_POSITIVE;
  TMR_InitStruct->CounterDIR = TMR_TMRCON_PWMC_UP;
  TMR_InitStruct->TMRDIV = 0x00;
  TMR_InitStruct->TMRPRD = 0xFFFF;
  TMR_InitStruct->TMRCMP = 0x0000;
}
/**
***********************************************************************************
*@Function TMR_Init
*@brief    Initial the selected TMRx
*@param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
*@param  TMR_InitStruct: pointer to a TMR_InitTypeDef structure that contains
*         the configuration information for the specified TMRx peripheral.
* @param  Mode:
*         @arg TMR_TMRCON_MODE_PERIOD
*         @arg TMR_TMRCON_MODE_PWM
* @param  POLARITY:
*         @ref TMR_TMRCON_POLARITY_POSITIVE
*         @ref TMR_TMRCON_POLARITY_NEGATIVE
* @param  CounterDIR:
*         @arg TMR_TMRCON_PWMC_UP
*         @arg TMR_TMRCON_PWMC_DOWM
*         @arg TMR_TMRCON_PWMC_CENTER
* @param  TMRPRD:
*         @arg 0-65535
* @param  TMRCMP:
*         @arg 0-65535
* @param  TMRDIV:
*         @arg 0-65535
* @retval None
***********************************************************************************
  */
void TMR_Init(TMR_TypeDef* TMRx,TMR_InitTypeDef* TMR_InitStruct)
{	
	uint32_t tmp1 = 0;
	tmp1 = TMRx->TMRCON;
	
	/* Config the MODE */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_MODE));
	tmp1 |= (uint32_t)TMR_InitStruct->Mode;	
	/* Config the POLARITY:POSITIVE/NEGATIVE */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_POLARITY));
	tmp1 |= (uint32_t)TMR_InitStruct->Polarity;
	/* Config the PWMC */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_PWMC));
	tmp1 |= (uint32_t)TMR_InitStruct->CounterDIR;	

	TMRx->TMRCON = tmp1;	

	/* Config the DIV */
	TMRx->TMRDIV =TMR_InitStruct->TMRDIV;
	/* Config the Period */
	TMRx->TMRPRD = TMR_InitStruct->TMRPRD;		
  /* Config the CMP */	
	TMRx->TMRCMP = TMR_InitStruct->TMRCMP;		
  
}

/**
***********************************************************************************
*@Function TMR_ICInit
*@brief    TMRx INPUT Initial
*@param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
*@param  TMR_InitStruct: pointer to a TMR_InitTypeDef structure that contains
*         the configuration information for the specified TMRx peripheral.
* @param  Mode:
*         @arg TMR_TMRCON_MODE_CAPTURE
*         @arg TMR_TMRCON_MODE_EVENT
* @param  CCMODE
*         @arg TMR_TMRCON_CCMODE_RISE
*         @arg TMR_TMRCON_CCMODE_FALL
* @param  POLARITY:
*         @ref TMR_TMRCON_POLARITY_POSITIVE
*         @ref TMR_TMRCON_POLARITY_NEGATIVE
* @param  CounterDIR:
*         @arg TMR_TMRCON_PWMC_UP
*         @arg TMR_TMRCON_PWMC_DOWM
*         @arg TMR_TMRCON_PWMC_CENTER
* @param  TMRPRD:
*         @arg 0-65535
* @param  TMRCMP:
*         @arg 0-65535
* @param  TMRDIV:
*         @arg 0-65535
* @retval None
***********************************************************************************
  */
void TMR_ICInit(TMR_TypeDef* TMRx,TMR_InitTypeDef* TMR_ICInitStruct)
{	
	uint32_t tmp1 = 0;
	tmp1 = TMRx->TMRCON;
	
	/* Config the MODE */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_MODE));
	tmp1 |= (uint32_t)TMR_ICInitStruct->Mode;
	/* Config the CCMODE:RISE/FALL */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_CCMODE));
	tmp1 |= (uint32_t)TMR_ICInitStruct->ICPolarity;		
	/* Config the POLARITY:POSITIVE/NEGATIVE */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_POLARITY));
	tmp1 |= (uint32_t)TMR_ICInitStruct->Polarity;
	/* Config the PWMC */
  tmp1 &= (uint16_t)(~(uint16_t)(TMR_TMRCON_PWMC));
	tmp1 |= (uint32_t)TMR_ICInitStruct->CounterDIR;	

	TMRx->TMRCON = tmp1;	

	/* Config the DIV */
	TMRx->TMRDIV =TMR_ICInitStruct->TMRDIV;
	/* Config the Period */
	TMRx->TMRPRD = TMR_ICInitStruct->TMRPRD;		
  /* Config the CMP */	
	TMRx->TMRCMP = TMR_ICInitStruct->TMRCMP;			
  
}



/**
***********************************************************************************
*@Function TMR_Enable_CNT
* @brief  ENABLE CNT.
* @rmtoll TMRCON          CNTEN          
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @retval None
***********************************************************************************
*/
void TMR_Enable_CNT(TMR_TypeDef* TMRx )
{
	SET_BIT(TMRx->TMRCON,TMR_TMRCON_CNTEN);
}

/**
***********************************************************************************
*@Function TMR_Disable_CNT
* @brief  DISENABLE CNT.
* @rmtoll TMRCON          CNTEN          
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @retval None
***********************************************************************************
*/
void TMR_Disable_CNT(TMR_TypeDef* TMRx )
{
	CLEAR_BIT(TMRx->TMRCON,TMR_TMRCON_CNTEN);
}

/**
***********************************************************************************
* @Function TMR_Set_MODE
* @brief  select timer mode.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val:  This parameter can be one of the following values:
*         @arg TMR_TMRCON_MODE_EVENT          ((uint32_t)0x0000)
*         @arg TMR_TMRCON_MODE_PWM            ((uint32_t)0x0002)
*         @arg TMR_TMRCON_MODE_CAPTURE        ((uint32_t)0x0004)
*         @arg TMR_TMRCON_MODE_PERIOD         ((uint32_t)0x0006)
*         @arg TMR_TMRCON_MODE_ENCODER1       ((uint32_t)0x000A) 
*         @arg TMR_TMRCON_MODE_ENCODER2       ((uint32_t)0x000C)
*         @arg TMR_TMRCON_MODE_ENCODERX       ((uint32_t)0x000E)
* @retval None
***********************************************************************************
*/
void TMR_Set_MODE(TMR_TypeDef* TMRx ,uint32_t val)
{
	MODIFY_REG(TMRx->TMRCON, TMR_TMRCON_MODE, val);
}
/**
***********************************************************************************
* @Function TMR_Set_CCMODE
* @brief  select timer ccmode.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val:  This parameter can be one of the following values:
*         @arg TMR_TMRCON_CCMODE_RISE                  ((uint32_t)0x0000)
*         @arg TMR_TMRCON_CCMODE_FALL                  ((uint32_t)0x0008)
* @retval None
***********************************************************************************
*/
void TMR_Set_CCMODE(TMR_TypeDef* TMRx ,uint32_t val)
{
	MODIFY_REG(TMRx->TMRCON, TMR_TMRCON_CCMODE, val);
}

/**
***********************************************************************************
* @Function TMR_Set_PWMMODE
* @brief  select pwm mode.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: This parameter can be one of the following values:
*         @arg TMR_TMRCON_PWMC_UP             ((uint32_t)0x0000) 
*         @arg TMR_TMRCON_PWMC_DOWM           ((uint32_t)0x0010) 
*         @arg TMR_TMRCON_PWMC_CENTER         ((uint32_t)0x0020) 
* @retval None
***********************************************************************************
*/
void TMR_Set_PWMMODE(TMR_TypeDef* TMRx ,uint32_t val)
{
	MODIFY_REG(TMRx->TMRCON, TMR_TMRCON_PWMC, val);
}

/**
***********************************************************************************
* @Function TMR_Set_PWMPolarity
* @brief  select pwm polarity.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: This parameter can be one of the following values:
*         @arg TMR_TMRCON_POLARITY_POSITIVE               ((uint32_t)0x0000)
*         @arg TMR_TMRCON_POLARITY_NEGATIVE               ((uint32_t)0x0040)
* @retval None
***********************************************************************************
*/
void TMR_Set_PWMPolarity(TMR_TypeDef* TMRx ,uint32_t val)
{
	MODIFY_REG(TMRx->TMRCON, TMR_TMRCON_POLARITY, val);
}

/** @defgroup TMR_DIV&PRD&CAP&CNT&CMP_Management
  */
/**
***********************************************************************************
* @Function TMR_Set_Prescaler
* @brief  set timer prescaler.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: val is a number between 0 to 65535
* @retval None
***********************************************************************************
*/
void TMR_Set_Prescaler(TMR_TypeDef* TMRx ,uint16_t val)
{
	MODIFY_REG(TMRx->TMRDIV, TMR_TMRDIV, val);
}

/**
***********************************************************************************
* @Function TMR_Set_Prescaler
* @brief  set timer period.         
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: val is a number between 0 to 65535
* @retval None
***********************************************************************************
*/
void TMR_Set_Period(TMR_TypeDef* TMRx ,uint16_t val)
{
	MODIFY_REG(TMRx->TMRPRD, TMR_TMRPRD, val);
}

/**
***********************************************************************************
* @Function TMR_Get_capture
* @brief  get timer capture.
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3 
* @retval:  retval is a number between 0 to 65535
***********************************************************************************
*/
uint16_t TMR_Get_Capture(TMR_TypeDef* TMRx)
{
	return( (READ_REG(TMRx->TMRCAP))&TMR_TMRCAP );
}

/**
***********************************************************************************
* @Function TMR_Set_Count
* @brief  set timer count.
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: val is a number between 0 to 65535
* @retval None
***********************************************************************************
*/
void TMR_Set_Count(TMR_TypeDef* TMRx ,uint16_t val)
{
	MODIFY_REG(TMRx->TMRCNT, TMR_TMRCNT, val);
}

/**
***********************************************************************************
* @Function TMR_Get_count
* @brief  get timer count.
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @retval:  retval is a number between 0 to 65535
***********************************************************************************
*/
uint16_t TMR_Get_Count(TMR_TypeDef* TMRx)
{
	return( (READ_REG(TMRx->TMRCNT))&TMR_TMRCNT );
}

/**
***********************************************************************************
* @Function TMR_Set_compare
* @brief  set timer compare.
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @param  val: val is a number between 0 to 65535
* @retval None
***********************************************************************************
*/
void TMR_Set_Compare(TMR_TypeDef* TMRx ,uint16_t val)
{
	MODIFY_REG(TMRx->TMRCMP, TMR_TMRCMP, val);
}

/**
***********************************************************************************
* @Function TMR_Get_Compare
* @brief  get timer compare.
* @param  TMRx :
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
* @retval:  retval is a number between 0 to 65535
***********************************************************************************
*/
uint16_t TMR_Get_Compare(TMR_TypeDef* TMRx)
{
	return( (READ_REG(TMRx->TMRCMP))&TMR_TMRCMP );
}

/** @defgroup TMR_IE&IF_Management
  */
/**
***********************************************************************************
*@Function TMR_Enable_IT
*@brief    TMR Initial
*@param    TMRx:
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
*@param   ITFLAG:
*         @arg  TMR_TMRIE_PRDIE 
*         @arg  TMR_TMRIE_CAPIE
*         @arg  TMR_TMRIE_CMPIE
*         @arg  TMR_TMRIE_ACIE
* @retval None
***********************************************************************************
*/
void TMR_Enable_IT(TMR_TypeDef* TMRx,uint32_t ITFLAG)
{
	/* Config the IT enable */
	SET_BIT(TMRx->TMRIE, ITFLAG);
}


/**
***********************************************************************************
*@Function TMR_Disable_IT
*@brief    TMR Initial
*@param    TMRx:
*         @arg TMR0
*         @arg TMR1
*         @arg TMR2
*         @arg TMR3
*@param   ITFLAG:
*         @arg  TMR_TMRIE_PRDIE 
*         @arg  TMR_TMRIE_CAPIE
*         @arg  TMR_TMRIE_CMPIE
*         @arg  TMR_TMRIE_ACIE
* @retval None
***********************************************************************************
*/
void TMR_Disable_IT(TMR_TypeDef* TMRx,uint32_t ITFLAG)
{
	/* Config the IT enable */
	CLEAR_BIT(TMRx->TMRIE, ITFLAG);
}

/**
***********************************************************************************
  * @function  TMR_Get_Flag
  * @brief  read complete flag of the selected TMR
  * @param  TMRx :
  *         @arg TMR0
  *         @arg TMR1
  *         @arg TMR2
  *         @arg TMR3
  * @param  ITFlag:
  *         @arg  TMR_TMRIF_PRDIF 
  *         @arg  TMR_TMRIF_CAPIF 
  *         @arg  TMR_TMRIF_CMPIF 
  *         @arg  TMR_TMRIF_ACIF 
  * @retval State of bit (1 or 0).
***********************************************************************************
*/
uint32_t TMR_Get_Flag(TMR_TypeDef* TMRx,uint16_t TMR_FLAG)
{
	 return (READ_BIT(TMRx->TMRIF, TMR_FLAG) == (TMR_FLAG));		
}

/**
***********************************************************************************
  * @function  TMR_Clear_Flag
  * @brief  clear complete flag of the selected TMR
  * @param  TMRx :
  *         @arg TMR0
  *         @arg TMR1
  *         @arg TMR2
  *         @arg TMR3
  * @param  ITFlag:
  *         @arg  TMR_TMRIF_PRDIF 
  *         @arg  TMR_TMRIF_CAPIF 
  *         @arg  TMR_TMRIF_CMPIF 
  *         @arg  TMR_TMRIF_ACIF 
  * @retval None
***********************************************************************************
*/
void TMR_Clear_Flag(TMR_TypeDef* TMRx, uint32_t TMR_FLAG)
{
  CLEAR_BIT(TMRx->TMRIF, TMR_FLAG);
}

