/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_uart.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include <stdio.h>
#include "rx32s11xx_uart.h"
#include "rx32s11xx_cmu.h"
#include "system_rx32s11xx.h"

#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */

PUTCHAR_PROTOTYPE
{
    UART1->SBUF = ch;
    while (0 == (UART1->UARTSTA & 0x01));
    UART1->UARTSTA &= ~((uint32_t)0x01);
    return ch;
}
/*
*********************************************************************************************************
*                                             Local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             Local function
*********************************************************************************************************
*/

/**
  * @brief  Deinitializes the UARTx peripheral registers to their default reset values.
  * @param  UARTx: where x can be 0, 1 to select the 
  *         UART peripheral.
  * @retval None
  */
void UART_DeInit(UART_TypeDef* UARTx)
{
  UARTx->UARTCON = 0;  
	UARTx->SREL    = 0;
	UARTx->SBUF    = 0;
	UARTx->UARTSTA = 0;
}
/**
  * @brief  Fills each USART_InitStruct member with its default value.
  * @param  USART_InitStruct: pointer to a USART_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void UART_StructInit(UART_InitTypeDef *UART_InitStruct)
{
	UART_InitStruct->Baudrate  = 0;
  UART_InitStruct->LenSel    = UART_UARTCON_LENSEL_8Bit;
  UART_InitStruct->StopSel   = UART_UARTCON_STOPSEL_1Bit;
  UART_InitStruct->Paritysel = UART_UARTCON_PARITYSEL_0;
  UART_InitStruct->ParityEN  = UART_UARTCON_PARITY_Disable;
}

/**
***********************************************************************************
*@Function UART_Init
*@brief    UART Initial
*@param   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param  UART_InitStruct: pointer to a UART_InitTypeDef structure that contains
*        the configuration information for the specified UART peripheral.
*        @arg  Fsys:
*              @ref SystemCoreClock This parameter must be set to the real system clock
*        @arg  Baudrate
*              @ref 115200 / 2400 / 9600 ....
*        @arg  StopSel
*              @ref UART_UARTCON_STOPSEL_1Bit
*              @ref UART_UARTCON_STOPSEL_2Bit
*        @arg  LenSel
*              @ref UART_UARTCON_LENSEL_7Bit
*              @ref UART_UARTCON_LENSEL_8Bit
*        @arg  ParityEN
*              @ref UART_UARTCON_PARITY_Enable
*              @ref UART_UARTCON_PARITY_Disable
*        @arg  Paritysel
*              @ref UART_UARTCON_PARITYSEL_1
*              @ref UART_UARTCON_PARITYSEL_0 
*              @ref UART_UARTCON_PARITYSEL_ODD
*              @ref UART_UARTCON_PARITYSEL_EVEN
* @retval None
***********************************************************************************
  */
void UART_Init(UART_TypeDef *UARTx, UART_InitTypeDef *UART_InitStruct)
{
  uint32_t tmp1 = UARTx->UARTCON;
  /* Config the Baudrate */
  UART_Set_BaudRate(UARTx, SystemCoreClock, UART_InitStruct->Baudrate);
  /* Config the STOPSEL */
  tmp1 &= (uint16_t)(~(uint16_t)(UART_UARTCON_STOPSEL));
  tmp1 |= (uint32_t)UART_InitStruct->StopSel;
  /* Config the LENSEL */
  tmp1 &= (uint16_t)(~(uint16_t)(UART_UARTCON_LENSEL));
  tmp1 |= (uint32_t)UART_InitStruct->LenSel;
  /* Config the Paritysel */
  tmp1 &= (uint16_t)(~(uint16_t)(UART_UARTCON_PARITYSEL));
  tmp1 |= (uint32_t)UART_InitStruct->Paritysel;
  /* Config the PARITY*/
  tmp1 &= (uint16_t)(~(uint16_t)(UART_UARTCON_PARITYEN | UART_UARTCON_PARITYSEL));
  tmp1 |= (uint32_t)UART_InitStruct->Paritysel;
  tmp1 |= (uint32_t)UART_InitStruct->ParityEN;

  UARTx->UARTCON = tmp1;
}

/**
***********************************************************************************
*@Function UART_Enable_IT
*@brief    UART Initial
*@param1   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param2
*          @arg  UART_UARTCON_TXIE 
*          @arg  UART_UARTCON_RXIE 
*          @arg  UART_UARTCON_PRDIE
*@retval None
***********************************************************************************
*/
void UART_Enable_IT(UART_TypeDef *UARTx, uint32_t ITFLAG)
{
  SET_BIT(UARTx->UARTCON, ITFLAG);
}
/**
***********************************************************************************
*@Function UART_Disable_IT
*@brief     
*@param1   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param2
*          @arg  UART_UARTCON_TXIE 
*          @arg  UART_UARTCON_RXIE 
*          @arg  UART_UARTCON_PRDIE
* @retval None
***********************************************************************************
*/
void UART_Disable_IT(UART_TypeDef *UARTx, uint32_t ITFLAG)
{
  CLEAR_BIT(UARTx->UARTCON, ITFLAG);
}
/**
*********************************************************************************************************
*@Function UART_Enable
*@brief    Enable TXRX
*@param1   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param2
*          @arg  UART_UARTCON_TXEN 
*          @arg  UART_UARTCON_RXEN
*          @arg  UART_UARTCON_TXEN|UART_UARTCON_RXEN
* @retval None
*********************************************************************************************************
  */
void UART_Enable(UART_TypeDef *UARTx,uint32_t val)
{
  SET_BIT(UARTx->UARTCON, val);
}
/**
*********************************************************************************************************
*@Function UART_Disable
*@brief    Disable TXRX
*@param1   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param2
*          @arg  UART_UARTCON_TXEN 
*          @arg  UART_UARTCON_RXEN
*          @arg  UART_UARTCON_TXEN|UART_UARTCON_RXEN
* @retval None
*********************************************************************************************************
  */
void UART_Disable(UART_TypeDef *UARTx,uint32_t val)
{
  CLEAR_BIT(UARTx->UARTCON, val);
}

/**
*********************************************************************************************************
*@Function UART_Set_BaudRate
*@brief    Set BaudRate
*@param1   UARTx:
*          @arg  UART0
*          @arg  UART1
*@param2   fsys:
*          @arg  pclk, maximum is 42MHz(42 000 000)
*@param3   baudrate:
*          @arg  1200 to 115200
* @retval None
*********************************************************************************************************
*/
void UART_Set_BaudRate(UART_TypeDef *UARTx, uint32_t fsys, uint32_t baudrate)
{
  uint32_t tmpVal;
  tmpVal = (fsys / (2 * baudrate)) - 1;
  UARTx->SREL = tmpVal;
}

/**
*********************************************************************************************************
  * @function  UART_Get_Flag
  * @brief  read complete flag of the selected UART
  * @param  UARTx :
  *         @arg UART0
  *         @arg UART1
  * @param  UART_FLAG:
  *         @arg  UART_UARTSTA_TXIF
  *         @arg  UART_UARTSTA_RXIF
  *         @arg  UART_UARTSTA_PARITY
  *         @arg  UART_UARTSTA_PRDIF
  * @retval State of bit (1 or 0).
*********************************************************************************************************
*/
uint32_t UART_Get_Flag(UART_TypeDef *UARTx, uint16_t UART_FLAG)
{
  return (READ_BIT(UARTx->UARTSTA, UART_FLAG) == (UART_FLAG));
}

/**
*********************************************************************************************************
  * @function  UART_Clear_Flag
  * @brief  clear complete flag of the selected UART
  * @param  UARTx :
  *         @arg UART0
  *         @arg UART1
  * @param  UART_FLAG:
  *         @arg  UART_UARTSTA_TXIF
  *         @arg  UART_UARTSTA_RXIF
  *         @arg  UART_UARTSTA_PARITY
  *         @arg  UART_UARTSTA_PRDIF
  * @retval None
*********************************************************************************************************
*/
void UART_Clear_Flag(UART_TypeDef *UARTx, uint32_t UART_FLAG)
{
  WRITE_REG(UARTx->UARTSTA, ~(UART_FLAG));
}

/**
*********************************************************************************************************
  * @function  UART_Send_Data
  * @brief  send data by UART
  * @param  UARTx :
  *         @arg UART0
  *         @arg UART1
  * @param  data: the value is between 0 to 255
  * @retval None
*********************************************************************************************************  
*/
void UART_Send_Data(UART_TypeDef *UARTx, uint8_t data)
{
  UARTx->SBUF = data;
}

/**
*********************************************************************************************************
  * @function  UART_Receive_Data
  * @brief  receive data from UART
  * @param  UARTx :
  *         @arg UART0
  *         @arg UART1
  * @retval the value is between 0 to 255
*********************************************************************************************************  
*/
uint8_t UART_Receive_Data(UART_TypeDef *UARTx)
{
	return (UARTx->SBUF);
}

/**
*********************************************************************************************************
  * @function  UART_Send_Buf
  * @brief  send data by UART
  * @param  UARTx :
  *         @arg UART0
  *         @arg UART1
  * @param  *wbuf :
  * @param  data: the value is between 0 to 255
*********************************************************************************************************  
*/
void UART_Send_Buf(UART_TypeDef *UARTx, uint8_t *wbuf, uint8_t len)
{
	uint8_t i;
	for (i = 0; i < len; i++)
	{
		UART_Send_Data(UARTx, *(wbuf + i));
    while( UART_Get_Flag(UARTx, UART_UARTSTA_TXIF)==0 );
    UART_Clear_Flag( UARTx, UART_UARTSTA_TXIF );
	}
}


