/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                               rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_wdt.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  :Only rx32s11xx
*********************************************************************************************************
*/


#include "rx32s11xx_wdt.h"
#include "rx32s11xx.h"

/**
  * @brief  Deinitializes the WDTx peripheral registers to their default reset values.
  * @param  WDTx: where x can be 0, 1 to select the 
  *         WDT peripheral.
  * @retval None
  */
void WDT_DeInit(WDT_TypeDef* WDTx)
{
  WDTx->WDTCLR = 0;
}

/**
*********************************************************************************************************
*@function  WDG_Init
*@brief     Set dog .WDT overflow time = 64ms * (1+SET[7:0]).
*           The shortest time is 64ms,the longest time is 16384ms,default time is 4160ms.
*@param     OverFlowTime:(ms)
*						@arg 64, 2*64, 3*64, ..., 256*64
*@retval    None
*********************************************************************************************************
*/
void WDG_Init(WDT_TypeDef *IWDG,WDG_InitTypeDef  *WDG_InitStruct)
{
	WDT_Set_Overflow_Time(WDG_InitStruct->OverFlowTime);
}


/*
*********************************************************************************************************
*@function  WDT_FeedDog
*@brief     Feed dog .WDT overflow time = 64ms * (1+SET[7:0]).
*           The shortest time is 64ms,the longest time is 16384ms,defult time is 4160ms.
*@param     None
*@retval    None
*********************************************************************************************************
*/
void WDT_FeedDog(void)
{  
	WDT->WDTCLR = (WDT->WDTCLR & 0x00ff)|0xaa00;
}

/*
*********************************************************************************************************
*@function  WDT_ReadWDTCNT
*@brief     Read WDTCNT value.
*@param     None
*@retval    WDTCNT value
*********************************************************************************************************
*/
uint32_t WDT_Read_WDTCNT(void)
{
    return( READ_REG(WDT->WDTCNT) & (uint16_t)0x00ff );
}

/*
*********************************************************************************************************
*@function  WDT_Set_Overflow_Time_ms
*@brief     Set dog .WDT overflow time = 64ms * (1+SET[7:0]).
*           The shortest time is 64ms,the longest time is 16384ms,default time is 4160ms.
*@param     overflow_time:(ms)
*						@arg 64, 2*64, 3*64, ..., 256*64
*@retval    None
*********************************************************************************************************
*/
void WDT_Set_Overflow_Time(uint16_t overflow_time)
{
	uint8_t temp = 0;
	temp = ((overflow_time/64)-1) & 0xff;
	WDT->WDTCLR = (WDT->WDTCLR & 0xff00)|temp;
}


