;/*
;*********************************************************************************************
;*Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
;*All Rights Reserved,
;*Redistribution and use in source and binary forms, with or without modification, are permitted
;*provided that the following conditions are met:
;*1. Redistributions of source code must retain the above copyright notice, this list of
;*conditions and the following disclaimer.
;*2. Redistributions in binary form must reproduce the above copyright notice, this list of
;*conditions and the following disclaimer in the documentation and/or other materials provided 
;*with the distribution.
;*3. Neither the name of the copyright holder nor the names of its contributors may be used to 
;*endorse or promote products derived from this software without specific prior written
;*permission.
;*THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
;*OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
;*MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
;*COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
;*EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
;*GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
;*AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
;*NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
;*OF THE POSSIBILITY OF SUCH DAMAGE.
;**********************************************************************************************
;*/
;/*
;*********************************************************************************************************
;*                                             rx32s30xx
;*                                         Library Function
;*
;*                                       All Rights Reserved
;*
;*
;* Project      : rx32s30xx
;* File         : startup_rx32s30xx.s
;* By           : RX_SD_Team
;* Description  :
;*********************************************************************************************************
;*/

; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000400

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                DCD     PMU_IRQHandler            ; 16+ 0: PMU
                DCD     SADC_IRQHandler           ; 16+ 1: SADC
                DCD     EXTI0_IRQHandler          ; 16+ 2: EXTI0
                DCD     EXTI1_IRQHandler          ; 16+ 3: EXTI1
                DCD     EXTI2_IRQHandler          ; 16+ 4: EXTI2
                DCD     EXTI3_IRQHandler          ; 16+ 5: EXTI3
                DCD     EXTI4_IRQHandler          ; 16+ 6: EXTI4
                DCD     EXTI5_IRQHandler          ; 16+ 7: EXTI5
                DCD     EXTI6_IRQHandler          ; 16+ 8: EXTI6
                DCD     UART0_IRQHandler          ; 16+ 9: UART0
                DCD     UART1_IRQHandler          ; 16+10: UART1
                DCD     0                         ; 16+11: Reserved
                DCD     0                         ; 16+12: Reserved
                DCD     0                         ; 16+13: Reserved
                DCD     ME_IRQHandler             ; 16+14: ME
                DCD     TMR0_IRQHandler           ; 16+15: Timer0
                DCD     TMR1_IRQHandler           ; 16+16: Timer1
                DCD     TMR2_IRQHandler           ; 16+17: Timer2
                DCD     TMR3_IRQHandler            ; 16+18: Timer3
                DCD     0                         ; 16+19: Reserved
                DCD     RTC_IRQHandler            ; 16+20: RTC
                DCD     I2C_IRQHandler            ; 16+21: I2C
                DCD     SPI_IRQHandler           ; 16+22: SPI0
                DCD     0                         ; 16+23: Reserved
                DCD     0                         ; 16+24: Reserved
                DCD     CMP4_IRQHandler           ; 16+25: CMP4
                DCD     CMP5_IRQHandler           ; 16+26: CMP5
                DCD     TIM8_IRQHandler           ; 16+27: TIM8
                DCD     EXTI7_IRQHandler          ; 16+28: EXTI7
                DCD     EXTI8_IRQHandler          ; 16+29: EXTI8
                DCD     EXTI9_IRQHandler          ; 16+30: EXTI9
                DCD     DMA_IRQHandler            ; 16+31: DMA

                IF      :LNOT::DEF:NO_CRP
                AREA    |.ARM.__at_0x0FC0|, CODE, READONLY
; FLASHʹôCODE
; CRP_Key         DCD     0xFAFF00A2

; FLASHʹôCODE
CRP_Key         DCD     0xFAFFFFA2
                ENDIF


                AREA    |.text|, CODE, READONLY


; Reset Handler
_RDefault      EQU     0xFFFFFFFF
Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  SystemInit
                IMPORT  __main
                LDR     R1, =_RDefault
                MOV	R2, R1
                MOV	R3, R1
				MOV	R4, R1
				MOV	R5, R1
				MOV	R6, R1
				MOV	R7, R1
				MOV	R8, R1
				MOV	R9, R1
				MOV	R10, R1
				MOV	R12, R1
                LDR     R0, =SystemInit
                BLX     R0
                LDR     R0, =__main
                BX      R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)

; now, under COMMON NMI.c and NMI.h, a real NMI handler is created if NMI is enabled
; for particular peripheral.
NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  NMI_Handler               [WEAK]
                EXPORT  PMU_IRQHandler            [WEAK]
                EXPORT  SADC_IRQHandler           [WEAK]
                EXPORT  EXTI0_IRQHandler          [WEAK]
                EXPORT  EXTI1_IRQHandler          [WEAK]
                EXPORT  EXTI2_IRQHandler          [WEAK]
                EXPORT  EXTI3_IRQHandler          [WEAK]
                EXPORT  EXTI4_IRQHandler          [WEAK]
                EXPORT  EXTI5_IRQHandler          [WEAK]
                EXPORT  EXTI6_IRQHandler          [WEAK]
                EXPORT  UART0_IRQHandler          [WEAK]
                EXPORT  UART1_IRQHandler          [WEAK]
                EXPORT  CMP1_IRQHandler           [WEAK]
                EXPORT  CMP2_IRQHandler           [WEAK]
                EXPORT  CMP3_IRQHandler           [WEAK]
                EXPORT  ME_IRQHandler             [WEAK]
                EXPORT  TMR0_IRQHandler           [WEAK]
                EXPORT  TMR1_IRQHandler           [WEAK]
                EXPORT  TMR2_IRQHandler           [WEAK]
                EXPORT  TMR3_IRQHandler           [WEAK]
                EXPORT  RTC_IRQHandler            [WEAK]
                EXPORT  I2C_IRQHandler            [WEAK]
                EXPORT  SPI_IRQHandler            [WEAK]
                EXPORT  SelfTestFreq_IRQHandler   [WEAK]
				EXPORT  CMP4_IRQHandler           [WEAK]
                EXPORT  CMP5_IRQHandler	          [WEAK]
                EXPORT  TIM8_IRQHandler           [WEAK]
                EXPORT  EXTI7_IRQHandler          [WEAK]
                EXPORT  EXTI8_IRQHandler          [WEAK]
			    EXPORT  EXTI9_IRQHandler          [WEAK]
                EXPORT  DMA_IRQHandler            [WEAK]



PMU_IRQHandler
SADC_IRQHandler
EXTI0_IRQHandler
EXTI1_IRQHandler
EXTI2_IRQHandler
EXTI3_IRQHandler
EXTI4_IRQHandler
EXTI5_IRQHandler
EXTI6_IRQHandler
UART0_IRQHandler
UART1_IRQHandler
CMP1_IRQHandler
CMP2_IRQHandler
CMP3_IRQHandler
ME_IRQHandler
TMR0_IRQHandler
TMR1_IRQHandler
TMR2_IRQHandler
TMR3_IRQHandler
RTC_IRQHandler
I2C_IRQHandler
SPI_IRQHandler
SelfTestFreq_IRQHandler
CMP4_IRQHandler
CMP5_IRQHandler
TIM8_IRQHandler
EXTI7_IRQHandler
EXTI8_IRQHandler
EXTI9_IRQHandler
DMA_IRQHandler
                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB

                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
