/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define __MAIN_C

#include "main.h"
#include "systick.h"
/*
*********************************************************************************************************
*                                         Global macro/Structure
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             Global variable
*********************************************************************************************************
*/
CMP_Polling_InitTypeDef CMP4_Polling_InitStruct = {0};
CMP_InitTypeDef CMP4_InitStruct = {0};
/*
*********************************************************************************************************
*                                       Global Function declaration
*********************************************************************************************************
*/
void CMP4_GPIO_Init(void);
void Toggle_GPIO_Init(void);

int main(void)
{
	CMP4_GPIO_Init();
  Toggle_GPIO_Init();
  
  CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_CMP4EN);
  
  /********Initialize CMP4********/
  CMP4_InitStruct.Mode = CMP_CR1_MODE_High;
  CMP4_InitStruct.Pol = CMP_CR1_POL_Positive;
  CMP4_InitStruct.OFLT = CMP_CR1_OFLT_1;
  CMP4_InitStruct.Rhyst = CMP_CR1_RHYST_0mV;
  CMP4_InitStruct.Fhyst = CMP_CR1_FHYST_0mV;
  CMP4_InitStruct.CRV = CMP_CR2_CRV_SEL_10_20;
  CMP4_InitStruct.CRV_Src = CMP_CR2_CRV_SRC_AVCC;
  CMP4_InitStruct.CRV_En = ENABLE;
  CMP4_InitStruct.INP = CMP_CR2_INP_SEL_PC3;
  CMP4_InitStruct.INM = CMP_CR2_INM_SEL_CRV;
  CMP_Init(CMP4, &CMP4_InitStruct);
  
  /********Initialize CMP4 Polling Mode********/
  CMP4_Polling_InitStruct.Polling_INP = CMP_PLC_INP_SEL_GPIO;
  CMP4_Polling_InitStruct.Polling_INM = CMP_PLC_INM_SEL_DISABLE;
  CMP4_Polling_InitStruct.Polling_Period = CMP_PLC_Period_256_CLK;
  CMP4_Polling_InitStruct.Polling_MaskT = CMP_PLC_MaskT_32_CLK;
  CMP4_Polling_InitStruct.Polling_OFLT_Mode = CMP_PLC_OFLT_After_MaskT;
  CMP_Polling_Init(CMP4, &CMP4_Polling_InitStruct);
  
  /********Enable CMP4 Polling CH1/2/3********/
  CMP_Polling_Enable_CH1(CMP4);
  CMP_Polling_Enable_CH2(CMP4);
  CMP_Polling_Enable_CH3(CMP4);
  
  /********Set CMP4 Polling CH1/2/3 Match Value********/
  CMP_Polling_Set_Match_Data1(CMP4, CMP_OUTPUT_LEVEL_HIGH);
  CMP_Polling_Set_Match_Data2(CMP4, CMP_OUTPUT_LEVEL_HIGH);
  CMP_Polling_Set_Match_Data3(CMP4, CMP_OUTPUT_LEVEL_HIGH);
  
  /********Enable CMP4********/
  CMP_Enable(CMP4);
  
  /********Clear CMP4 Polling Match Flag********/
  CMP_Polling_Clear_Flag(CMP4, CMP_PLS_CH1_Match);
  CMP_Polling_Clear_Flag(CMP4, CMP_PLS_CH2_Match);
  CMP_Polling_Clear_Flag(CMP4, CMP_PLS_CH3_Match);
  
  /********Enable CMP4 Polling Match Interrupt********/
  CMP_Polling_Enable_IT(CMP4, CMP_PLC_Match_IT);
  NVIC_EnableIRQ(CMP4_IRQn);
  
  while (1)
  {
    WDT_FeedDog();
  }
}

void CMP4_GPIO_Init(void)
{
	GPIO_InitTypeDef CMP4_GPIO_InitStruct = {0};
	
	/*****CMP4 IO1P*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_3;
  CMP4_GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
	CMP4_GPIO_InitStruct.AF = AF_FUNCTION1;
	CMP4_GPIO_InitStruct.Pull = GPIO_FLOATING;
  GPIO_Init(GPIOC, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 IO1N*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_3;
  GPIO_Init(GPIOC, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 IO2P*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_0;
  GPIO_Init(GPIOC, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 IO2N*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_1;
  GPIO_Init(GPIOC, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 IO3P*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_7;
  GPIO_Init(GPIOB, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 IO3N*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_8;
  GPIO_Init(GPIOB, &CMP4_GPIO_InitStruct);
	
	/*****CMP4 OUT*****/
  CMP4_GPIO_InitStruct.Pin   = GPIO_PIN_4;
	CMP4_GPIO_InitStruct.AF = AF_FUNCTION3;
  CMP4_GPIO_InitStruct.Pull = GPIO_PULL_DOWN;
  GPIO_Init(GPIOA, &CMP4_GPIO_InitStruct);
}

void Toggle_GPIO_Init(void)
{
	GPIO_Set_PinMode(GPIOA, GPIO_PIN_5, GPIO_MODE);
	GPIO_Set_OutputMode(GPIOA, GPIO_PIN_5, GPIO_PUSHPULL);
	GPIO_Set_DIR(GPIOA, GPIO_PIN_5, GPIO_OUTPUT);
  GPIO_Set_HIIPM(GPIOA, GPIO_PIN_5, GPIO_HIIPM_DISABLE);
	GPIO_Clear_Output(GPIOA, GPIO_PIN_5);
  
  GPIO_Set_PinMode(GPIOA, GPIO_PIN_7, GPIO_MODE);
	GPIO_Set_OutputMode(GPIOA, GPIO_PIN_7, GPIO_PUSHPULL);
	GPIO_Set_DIR(GPIOA, GPIO_PIN_7, GPIO_OUTPUT);
  GPIO_Set_HIIPM(GPIOA, GPIO_PIN_7, GPIO_HIIPM_DISABLE);
	GPIO_Clear_Output(GPIOA, GPIO_PIN_7);
  
  GPIO_Set_PinMode(GPIOC, GPIO_PIN_6, GPIO_MODE);
	GPIO_Set_OutputMode(GPIOC, GPIO_PIN_6, GPIO_PUSHPULL);
	GPIO_Set_DIR(GPIOC, GPIO_PIN_6, GPIO_OUTPUT);
  GPIO_Set_HIIPM(GPIOC, GPIO_PIN_6, GPIO_HIIPM_DISABLE);
	GPIO_Clear_Output(GPIOC, GPIO_PIN_6);
}
