/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*    rx32s11xx
*                      Library Function
*                   All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define  __MAIN_C

#include "main.h"

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/	
/* Variables for ADC conversion data */
  __IO uint32_t aADCxConvertedData[ADC_CONVERTED_DATA_BUFFER_SIZE]; 
/*
*********************************************************************************************************
*    MAIN
*
*********************************************************************************************************
*/
/* Exported constants --------------------------------------------------------*/
  uint32_t ErrorFlag;
  uint32_t ErrorBit;
  uint32_t covert_time1;
  uint32_t covert_time2;
  uint32_t covert_number  = 0;
  uint32_t covert_number1 = 0;
  uint32_t RegAddress;
	int i;
  char    *RegName;
/* Exported functions ------------------------------------------------------- */

void clr_sram(uint32_t Start, uint32_t End)
{
	for(i=Start;i<End;)
	{
		M32(i) = 0x00000000;
		i=i+4;
	}
}

void set_sram(uint32_t Start, uint32_t End)
{
	for(i=Start;i<End;)
	{
		M32(i) = i;
		//printf("\r\n%d address %x is %x\r\n",(i-Start)/4,i,M32(i));
		i=i+4;
	}
}

void set_sram_value(uint32_t Start, uint32_t End, uint32_t value)
{
	for(i=Start;i<End;)
	{
		M32(i) = value;
//		printf("\r\n address %x is %x\r\n",i,M32(i));
		i=i+4;
	}
}

void Clock_Initial()
{
	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_DMAEN);
	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_UART1EN);
}

void UART1_Initial(void)
{
  UART_InitTypeDef UART1_InitStruct = {0};
  
  CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_UART1EN);

  UART1_InitStruct.Baudrate  = 115200;
  UART1_InitStruct.StopSel   = UART_UARTCON_STOPSEL_1Bit;
  UART1_InitStruct.LenSel    = UART_UARTCON_LENSEL_8Bit;
  UART1_InitStruct.Paritysel = UART_UARTCON_PARITYSEL_0;
  UART1_InitStruct.ParityEN  = UART_UARTCON_PARITY_Disable;
  UART_Init(UART1, &UART1_InitStruct);
  
  UART_Enable(UART1,UART_UARTCON_TXEN|UART_UARTCON_RXEN);
  
  UART_Disable_IT(UART1, UART_UARTCON_TXIE | UART_UARTCON_RXIE | UART_UARTCON_PRDIE);
  NVIC_DisableIRQ(UART1_IRQn);
}

void UART1_PIN_Initial(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  
  GPIO_InitStruct.Pin   = GPIO_PIN_8;//TX1
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
  GPIO_InitStruct.AF    = AF_FUNCTION2;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
}

int main(void)
{	
  DMA_InitTypeDef  DMA_InitStruct = {0};
	Clock_Initial();
	UART1_PIN_Initial();
  UART1_Initial();

	set_sram(0x20000800, 0x20000900);//64WORD
//  set_sram_value(0x20000800, 0x20000900,0x55555555);//64WORD
	for(i=0;i<64;i++)//move 64WORD from 0x20000800 to 0x20000A00
	{
		DMA_InitStruct.Channel           = DMA_CHANNEL_0;
		DMA_InitStruct.CHNCTL_Mode       = DMA_CHNCTL_MODE_WordTransfer;
		DMA_InitStruct.CHNCTL_Cycle      = DMA_CHNCTL_CYCLE_DISABLE;
		DMA_InitStruct.CHNCTL_Destininc  = DMA_CHNCTL_SOURCEINC_INC;
		DMA_InitStruct.CHNCTL_Sourceinc  = DMA_CHNCTL_DESTININC_INC;
		DMA_InitStruct.CHNCTL_Psize      = DMA_CHNCTL_PSIZE_Word;
		DMA_InitStruct.CHNCTL_Channel    = DMA_CHNCTL_CHANNEL_Software;
		DMA_InitStruct.SrcAddress        = 0x20000800+4*i;
		DMA_InitStruct.TarAddress        = 0x20000A00+4*i;
		DMA_InitStruct.DataLength        = 1;
		DMA_Init(DMA, &DMA_InitStruct);
		DMA_Enable_Channel(DMA, DMA_CHANNEL_0);
		DMA_SoftRequestTrig(DMA,DMA_CHANNEL_0);
	}
                                   
	for(i=0x20000800;i<0x20000C00;)//64WORD[used] + 64WORD[unused] +64WORD[used] + 64WORD[unused]
	{
		printf("\r\n%d address %x is %x\r\n",(i-0x20000800)/4,i,M32(i));
		i=i+4;
	}
	
	clr_sram(0x20000800,0x20001000);//clear last 2KB
	
  while(1)
  {
      WDT_FeedDog();
  }
}


