/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                               rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Version      : V1.0.0
* Description  : Only rx32s11xx
*********************************************************************************************************
*/
#include "main.h"

/*
*********************************************************************************************************
*                                         Local macro/Structure
*********************************************************************************************************
*/
#define Button_is_Down (GPIO_Get_PTDAT(GPIOA,GPIO_PIN_4)==0)

/*
*********************************************************************************************************
*                                             Local variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                       Local Function declaration
*********************************************************************************************************
*/
void Button_Initial(void);
void UART1_Initial(uint32_t baudrate);
void SWD_Config(void);
void EXTI_PIN_Initial(void);

/*
*********************************************************************************************************
*                                          Local Function
*********************************************************************************************************
*/
/*
* ----EXTI_PIN_LIST---- * 
    EXTI_0 -> PC6_AF3
    EXTI_1 -> PA6_AF3
    EXTI_2 -> PA7_AF3
    EXTI_3 -> PD7_AF3
    EXTI_4 -> PD8_AF3
    EXTI_5 -> PA0_AF3
    EXTI_6 -> PB8_AF3
    EXTI_7 -> PC1_AF3
    EXTI_8 -> PC4_AF3
* --------------------- * 
*/
void EXTI_PIN_Initial(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
  //EXTI GPIO Configuration
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.AF   = AF_FUNCTION3;
	GPIO_InitStruct.Pull = GPIO_FLOATING;
  //EXTI_0 -> PC6_AF3
  GPIO_InitStruct.Pin  = GPIO_PIN_6;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
  //EXTI_1 -> PA6_AF3
	GPIO_InitStruct.Pin  = GPIO_PIN_6;
	GPIO_Init(GPIOA, &GPIO_InitStruct);
  //EXTI_2 -> PA7_AF3
  GPIO_InitStruct.Pin  = GPIO_PIN_7;
	GPIO_Init(GPIOA, &GPIO_InitStruct);
  //EXTI_3 -> PD7_AF3
	GPIO_InitStruct.Pin  = GPIO_PIN_7;
	GPIO_Init(GPIOD, &GPIO_InitStruct);
  //EXTI_4 -> PD8_AF3
//  GPIO_InitStruct.Pin  = GPIO_PIN_8;
//	GPIO_Init(GPIOD, &GPIO_InitStruct);
  //EXTI_5 -> PA0_AF3
	GPIO_InitStruct.Pin  = GPIO_PIN_0;
	GPIO_Init(GPIOA, &GPIO_InitStruct);
  //EXTI_6 -> PB8_AF3
  GPIO_InitStruct.Pin  = GPIO_PIN_8;
	GPIO_Init(GPIOB, &GPIO_InitStruct);
  //EXTI_7 -> PC1_AF3
	GPIO_InitStruct.Pin  = GPIO_PIN_1;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
  //EXTI_8 -> PC4_AF3
	GPIO_InitStruct.Pin  = GPIO_PIN_4;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
}

void UART1_Initial(uint32_t baudrate)
{
  UART_InitTypeDef UART1_InitStruct = {0};
  GPIO_InitTypeDef GPIO_InitStruct  = {0};
  //Enable peripheral clock
  CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_UART1EN);
  //UART1 GPIO Configuration
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Pull  = GPIO_FLOATING;
  GPIO_InitStruct.AF    = AF_FUNCTION2;
  GPIO_InitStruct.Pin   = GPIO_PIN_8;  //PD8_AF2:TX1
  GPIO_Init(GPIOD,&GPIO_InitStruct);
  //UART1 Function Configuration
	UART1_InitStruct.Baudrate  = baudrate;
	UART1_InitStruct.StopSel   = UART_UARTCON_STOPSEL_1Bit;
	UART1_InitStruct.LenSel    = UART_UARTCON_LENSEL_8Bit;
	UART1_InitStruct.Paritysel = UART_UARTCON_PARITYSEL_0;
	UART1_InitStruct.ParityEN  = UART_UARTCON_PARITY_Disable;
	UART_Init(UART1, &UART1_InitStruct);
  //Enable UART1
  UART_Enable(UART1,UART_UARTCON_TXEN|UART_UARTCON_RXEN);
  //Enable RXIT
  UART_Enable_IT(UART1,UART_UARTCON_RXIE);
  NVIC_EnableIRQ(UART1_IRQn);
}

void SWD_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  //SWD GPIO Configuration
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Pull  = GPIO_FLOATING;
  GPIO_InitStruct.AF    = AF_FUNCTION1;
  GPIO_InitStruct.Pin   = GPIO_PIN_0;  //PA0_AF1:SWDIO
  GPIO_Init(GPIOA,&GPIO_InitStruct);
  GPIO_InitStruct.Pin   = GPIO_PIN_1;  //PA1_AF1:SWCLK
  GPIO_Init(GPIOA,&GPIO_InitStruct);
}

void Button_Initial(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  //Button GPIO Configuration
  GPIO_InitStruct.Mode  = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull  = GPIO_PULL_UP;
  GPIO_InitStruct.Pin   = GPIO_PIN_4;  //PA4:Button
  GPIO_Init(GPIOA, &GPIO_InitStruct);
}


int main(void)
{
  EXTI_InitTypeDef EXTI_InitStruct = {0};

	UART1_Initial(115200); 
  Button_Initial();
	EXTI_PIN_Initial();
  
  //EXTI Function Configuration
  EXTI_InitStruct.FltValue   = EXTI_Filter_80_ns;
  EXTI_InitStruct.TrigMode   = Rising_Edge_Trigger;
  EXTI_InitStruct.ExtiNumber = EXTI_0;
  EXTI_Init(&EXTI_InitStruct);

  
	//Enable NVIC
	NVIC_EnableIRQ(EXTI0_IRQn);
  
	printf("-------exti test-------\r\n");

	while (1)
	{
		WDT_FeedDog();
    if( Button_is_Down )
    {
      delay_ms(10);
      while( Button_is_Down );
      SWD_Config();
    }
	}
}
