/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define __MAIN_C

#include "main.h"
#include "systick.h"
/*
*********************************************************************************************************
*                                         Global macro/Structure
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                             Global variable
*********************************************************************************************************
*/
uint8_t RNum = 0;
uint8_t TNum = 0;
#define DATA_NUMBER (8)
uint32_t writeword[8] = {0x11223344, 0x22334455, 0x33445566, 0x44556677, 0x55667788, 0x66778899, 0x778899AA, 0x8899AABB};
uint32_t readword[8] = {0};
uint32_t RWord;

uint8_t writebyte[8] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};
uint8_t readbyte[8] = {0};

uint16_t writehalfword[8] = {0x1122, 0x3344, 0x5566, 0x7788, 0x99AA, 0xBBCC, 0xDDEE, 0x1010};
uint16_t readhalfword[8] = {0};
/*
*********************************************************************************************************
*                                       Global Function declaration
*********************************************************************************************************
*/

uint8_t BufferCmp(uint32_t *Buffer1, uint32_t *Buffer2, uint8_t BufferLength)
{
    while (BufferLength--)
    {
        if (*Buffer1 != *Buffer2)
        {
            return 1;
        }
        Buffer1++;
        Buffer2++;
    }
    return 0;
}

void Compare_result(uint32_t *Buffer1, uint32_t *Buffer2, uint8_t BufferLength)
{
    /* Check the datas */
    if (BufferCmp(Buffer1, Buffer2, BufferLength) == 0)
    {
        /* Expected bytes have been received */
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
    }
    else
    {
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
        GPIO_Toggle_Output(GPIOC, GPIO_PIN_0); //PC0
    }
}

int main(void)
{
	uint32_t Wbyte[DATA_NUMBER],Rbyte[DATA_NUMBER];
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  GPIO_InitStruct.Pin   = GPIO_PIN_0;
  GPIO_InitStruct.Mode  = GPIO_MODE_OUTPUT_PP;
  GPIO_Init(GPIOC, &GPIO_InitStruct);	

	
	/*          Write 8 word         */
    Flash_page_erase(8); //8*1024=0X2000
    for (uint8_t i = 0; i < 8; i++)
    {
        Flash_Wordwrite(0x2000 + i * 4, writeword[i]);
        readword[i] = Flash_WordRead(0x2000 + i * 4);			
    }
    Compare_result(writeword,readword,DATA_NUMBER);

    /*          Write 8 byte         */
    Flash_page_erase(8); //8*1024=0X2000
    Flash_Bytewrite(0x2000, writebyte, 8);
    Flash_ByteRead(readbyte, 0x2000, 8);
		for (uint8_t i = 0; i < 8; i++)
    {
        Wbyte[i] = writebyte[i];
        Rbyte[i] = readbyte[i];
    }
    Compare_result(Wbyte,Rbyte,DATA_NUMBER);
		
		/*          Write 8 halfWord         */
    Flash_page_erase(8); //8*1024=0X2000
    for (uint8_t i = 0; i < 8; i++)
    {
        Flash_HalfWordwrite(0x2000 + i * 4, writehalfword[i]);
        readhalfword[i] = Flash_HalfWordRead(0x2000 + i * 4);			
    }
		for (uint8_t i = 0; i < 8; i++)
    {
        Wbyte[i] = writehalfword[i];
        Rbyte[i] = readhalfword[i];
    }
    Compare_result(Wbyte,Rbyte,DATA_NUMBER);
		
    Flash_page_erase(8); //8*1024=0X2000
	
	
  while (1)
  {
    WDT_FeedDog();
		
		delay_ms(1000);

  }
}
