/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define __MAIN_C

#include "main.h"

/*
*********************************************************************************************************
*                                           Local macro/Structure
*********************************************************************************************************
*/

I2C_InitTypeDef I2C_Initstruct        = {0};
I2C_HandleTypeDef I2C_HandleStruct_Tr = {0};
I2C_HandleTypeDef I2C_HandleStruct_Re = {0};
GPIO_InitTypeDef I2C_GPIO_Initstruct  = {0};

/*
*********************************************************************************************************
*                                            Local variable
*********************************************************************************************************
*/
#define MASTER_BOARD
#ifdef MASTER_BOARD
  uint8_t I2C_TransmitBuffer[7] = {0x31, 0x32, 0x33, 0x34, 0x35, 0x36};
#else		
  uint8_t I2C_TransmitBuffer[7] = {0x37, 0x38, 0x39, 0x61, 0x62, 0x63};
#endif
uint8_t I2C_ReceiveBuffer[6] = {0};
uint32_t delaytime = 6000;
uint32_t j = 0;

/*
*********************************************************************************************************
*                                           Local Function declaration
*********************************************************************************************************
*/

void I2C_GPIO_Init(void);
void Button_GPIO_Init(void);

/*
*********************************************************************************************************
*                                              MAIN
*
*********************************************************************************************************
*/
void UART1_Initial(void)
{
  UART_InitTypeDef UART1_InitStruct = {0};

  UART1_InitStruct.Baudrate  = 115200;
  UART1_InitStruct.StopSel   = UART_UARTCON_STOPSEL_1Bit;
  UART1_InitStruct.LenSel    = UART_UARTCON_LENSEL_8Bit;
  UART1_InitStruct.Paritysel = UART_UARTCON_PARITYSEL_0;
  UART1_InitStruct.ParityEN  = UART_UARTCON_PARITY_Disable;
  UART_Init(UART1, &UART1_InitStruct);

  NVIC_DisableIRQ(UART1_IRQn);
  UART_Enable(UART1, UART_UARTCON_TXEN|UART_UARTCON_RXEN);
}

void UART1_PIN_Initial(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  // IO PD8 TX1
  GPIO_InitStruct.Pin   = GPIO_PIN_8;
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
  GPIO_InitStruct.AF    = AF_FUNCTION2;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
}

void delay_ms(uint16_t x) //delay 1ms
{
  uint16_t i, j;
  for (i = 0; i < x; i++)
  {
    for (j = 0; j < (3700); j++);
  }
}

int main(void)
{
	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_UART1EN);
	
	UART1_PIN_Initial();
	UART1_Initial();
	I2C_GPIO_Init();
  Button_GPIO_Init();
	
	
  CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_I2CEN);

  I2C_Initstruct.Fsys = SystemCoreClock;
  I2C_Initstruct.ClockSpeed  = 25000;
  I2C_Initstruct.OwnAddress  = 0x60;
  I2C_Initstruct.GeneralCallMode = ENABLE;
  I2C_Initstruct.Ack = ENABLE;
  I2C_Init(I2C, &I2C_Initstruct);
	I2C_Enable(I2C);

#ifdef MASTER_BOARD

  while (GPIO_Get_PTDAT(GPIOD, GPIO_PIN_7))
  {
      WDT_FeedDog();
  }
  printf("master out of feeddog\r\n");
  
  I2C_HandleStruct_Tr.DevAddress = 0x00;
  I2C_HandleStruct_Tr.pData      = I2C_TransmitBuffer;
  I2C_HandleStruct_Tr.Size       = 6;
  I2C_Master_Transmit(I2C, &I2C_HandleStruct_Tr);

  WDT_FeedDog();
  for (j = 0; j < delaytime; j++);

  I2C_HandleStruct_Re.DevAddress = 0x60;//Slave Address
  I2C_HandleStruct_Re.pData = I2C_ReceiveBuffer;
  I2C_HandleStruct_Re.Size = 6;
  I2C_Master_Receive(I2C, &I2C_HandleStruct_Re);

  WDT_FeedDog();
  for (j = 0; j < delaytime; j++);

  I2C_HandleStruct_Tr.DevAddress = 0x60;
  I2C_Master_Transmit(I2C, &I2C_HandleStruct_Tr);

  WDT_FeedDog();
  for (j = 0; j < delaytime; j++);

  I2C_Master_Receive(I2C, &I2C_HandleStruct_Re);
#else
  while (GPIO_Get_PTDAT(GPIOD, GPIO_PIN_7))
  {
      WDT_FeedDog();
  }
  printf("salve out of feeddog\r\n");
  
  I2C_HandleStruct_Re.pData = I2C_ReceiveBuffer;
  I2C_HandleStruct_Re.Size = 6;
  I2C_Slave_Receive(I2C, &I2C_HandleStruct_Re);

  WDT_FeedDog();

  I2C_HandleStruct_Tr.pData = I2C_TransmitBuffer;
  I2C_HandleStruct_Tr.Size = 6;
  I2C_Slave_Transmit(I2C, &I2C_HandleStruct_Tr);

  WDT_FeedDog();

  I2C_Slave_Receive(I2C, &I2C_HandleStruct_Re);

  WDT_FeedDog();

  I2C_Slave_Transmit(I2C, &I2C_HandleStruct_Tr);
#endif

  while (1)
  {
    WDT_FeedDog();
    delay_ms(1000);
  
#ifdef MASTER_BOARD
    printf("From slave: %s\r\n",I2C_ReceiveBuffer);
#else		
    printf("From Master: %s\r\n",I2C_ReceiveBuffer);
#endif
  }
}

void I2C_GPIO_Init(void)
{
  GPIO_InitTypeDef I2C_GPIO_InitStruct = {0};

  I2C_GPIO_InitStruct.Pin  = GPIO_PIN_0;
  I2C_GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  I2C_GPIO_InitStruct.AF   = AF_FUNCTION4;
  GPIO_Init(GPIOA, &I2C_GPIO_InitStruct);

  I2C_GPIO_InitStruct.Pin = GPIO_PIN_1;
  GPIO_Init(GPIOA, &I2C_GPIO_InitStruct);
}

void Button_GPIO_Init(void)
{
  GPIO_InitTypeDef Button_GPIO_InitStruct = {0};

  Button_GPIO_InitStruct.Pin   = GPIO_PIN_7;
  Button_GPIO_InitStruct.Mode  = GPIO_MODE_INPUT;
  Button_GPIO_InitStruct.Pull  = GPIO_PULL_UP;
  GPIO_Init(GPIOD, &Button_GPIO_InitStruct);
}
