/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#define __MAIN_C

#include "main.h"

/*
*********************************************************************************************************
*                                           Local macro/Structure
*********************************************************************************************************
*/

I2C_InitTypeDef I2C_Initstruct        = {0};
I2C_HandleTypeDef I2C_HandleStruct_Tr = {0};
I2C_HandleTypeDef I2C_HandleStruct_Re = {0};

/*
*********************************************************************************************************
*                                            Local variable
*********************************************************************************************************
*/

uint8_t I2C_TransmitBuffer[7] = {0x61,0x62,0x63,0x64,0x65,0x66};
uint8_t I2C_ReceiveBuffer[6] = {0};
uint32_t delaytime = 6000;
uint32_t j = 0;

/*
*********************************************************************************************************
*                                           Local Function declaration
*********************************************************************************************************
*/

void I2C_GPIO_Init(void);

/*
*********************************************************************************************************
*                                              MAIN
*
*********************************************************************************************************
*/
void UART1_Initial(void)
{
  UART_InitTypeDef UART1_InitStruct = {0};

  UART1_InitStruct.Baudrate  = 115200;
  UART1_InitStruct.StopSel   = UART_UARTCON_STOPSEL_1Bit;
  UART1_InitStruct.LenSel    = UART_UARTCON_LENSEL_8Bit;
  UART1_InitStruct.Paritysel = UART_UARTCON_PARITYSEL_0;
  UART1_InitStruct.ParityEN  = UART_UARTCON_PARITY_Disable;
  UART_Init(UART1, &UART1_InitStruct);

  NVIC_DisableIRQ(UART1_IRQn);
  UART_Enable(UART1, UART_UARTCON_TXEN|UART_UARTCON_RXEN);
}

void UART1_PIN_Initial(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  // IO PD8 TX1
  GPIO_InitStruct.Pin   = GPIO_PIN_8;
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
  GPIO_InitStruct.AF    = AF_FUNCTION2;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
}

void delay_ms(uint16_t x) //delay 1ms
{
  uint16_t i, j;
  for (i = 0; i < x; i++)
  {
    for (j = 0; j < (3700); j++);
  }
}

int main(void)
{
	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_UART1EN);
  UART1_PIN_Initial();
	UART1_Initial();  
	I2C_GPIO_Init();
  
  CMU_Enable_CLKCTRL0(CMU_CLKCTRL0_I2CEN);
  
  I2C_Initstruct.Fsys = SystemCoreClock;
  I2C_Initstruct.ClockSpeed  = 10000;
  I2C_Initstruct.OwnAddress  = 0x60;
  I2C_Initstruct.GeneralCallMode = DISABLE;
  I2C_Initstruct.Ack = ENABLE;
  I2C_Init(I2C, &I2C_Initstruct);
	I2C_Enable(I2C);
  
  I2C_HandleStruct_Tr.DevAddress = 0x50;
  I2C_HandleStruct_Tr.MemAddress = 0x0000;
  I2C_HandleStruct_Tr.MemAddressSize = 16;
  I2C_HandleStruct_Tr.pData = I2C_TransmitBuffer;
  I2C_HandleStruct_Tr.Size  = 6;
  I2C_Mem_Master_Write(I2C, &I2C_HandleStruct_Tr);
    
  for (j = 0; j < delaytime; j++);

  I2C_HandleStruct_Re.DevAddress = 0x50;
  I2C_HandleStruct_Re.MemAddress = 0x0000;
  I2C_HandleStruct_Re.MemAddressSize = 16;
  I2C_HandleStruct_Re.pData = I2C_ReceiveBuffer;
  I2C_HandleStruct_Re.Size  = 6;
  I2C_Mem_Master_Read(I2C, &I2C_HandleStruct_Re);

  for (j = 0; j < delaytime; j++);

  I2C_Mem_Master_Write(I2C, &I2C_HandleStruct_Tr);

  for (j = 0; j < delaytime; j++);

  I2C_Mem_Master_Read(I2C, &I2C_HandleStruct_Re);

  /*  Infinite loop  */
  while (1)
  {
    WDT_FeedDog();
		delay_ms(1000);
		printf("%s\r\n",I2C_ReceiveBuffer);
  }
}


void I2C_GPIO_Init(void)
{
  GPIO_InitTypeDef I2C_GPIO_InitStruct = {0};

  I2C_GPIO_InitStruct.Pin  = GPIO_PIN_0;
  I2C_GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
  I2C_GPIO_InitStruct.AF   = AF_FUNCTION4;
  GPIO_Init(GPIOA, &I2C_GPIO_InitStruct);

  I2C_GPIO_InitStruct.Pin = GPIO_PIN_1;
  GPIO_Init(GPIOA, &I2C_GPIO_InitStruct);
}
