/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : main.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "main.h"

/*
*********************************************************************************************************
*                                         Global macro/Structure
*********************************************************************************************************
*/

//#define WH_PIN_IO_MODE_LOW_ACTIVE
//#define WL_PIN_IO_MODE_LOW_ACTIVE

/*
*********************************************************************************************************
*                                             Global variable
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                       Global Function declaration
*********************************************************************************************************
*/
void TIM_PWM_CCNE_Init(uint16_t arr, uint16_t psc);

void CLKCTRL_Initial(void)
{
	CMU_Enable_CLKCTRL1(CMU_CLKCTRL1_TIM8EN);
}

void Delay_ms(uint32_t delay)
{
	while(delay--)
	{
		for(uint32_t cnt=0;cnt<20833;cnt++);
	}
}

/**
  * @brief  TIM1_PWM_Init
  * @param  arr psc
  AF1:
  PD0:TIM_CH2N  PD1:TIM_CH2
  PD2:TIM_CH1N  PD3:TIM_CH1
  PC9:TIM_CH3N  PC10:TIM_CH3
  AF2:
  PC6:TIM_CH4 
  AF4:
  PC6:TIM_BKIN
  */
void TIM_PWM_CCNE_Init(uint16_t arr, uint16_t psc)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	/*---------------------TIM8---------------------*/
	TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStruct = {0};
	TIM_OCInitTypeDef TIM_OCInitStruct = {0};
	TIM_BDTRInitTypeDef TIM_BDTRInitStruct = {0};
	/***************************/
	/* Time base configuration */
	/***************************/
	TIM_TimeBaseInitStruct.CounterMode       = TIM_COUNTERMODE_CENTER_UP;
	TIM_TimeBaseInitStruct.Period            = arr;
	TIM_TimeBaseInitStruct.Prescaler         = psc;
	TIM_TimeBaseInitStruct.ClockDivision     = TIM_CKD_DIV1;
	TIM_TimeBaseInitStruct.RepetitionCounter = 1;
	TIM_TimeBaseInit(TIM8, &TIM_TimeBaseInitStruct);
	/*---------------------CC1 CC2 CC3---------------------*/
	TIM_OCInitStruct.OCMode       = TIM_OCMODE_PWM2;
  TIM_OCInitStruct.OutputState  = TIM_OutputState_Enable;
  TIM_OCInitStruct.OutputNState = TIM_OutputNState_Enable;
	/*-------upper arm-------*/
	#ifdef WH_PIN_IO_MODE_LOW_ACTIVE
	TIM_OCInitStruct.OCPolarity  = TIM_OCPolarity_Low;
	TIM_OCInitStruct.OCIdleState = TIM_OCIdleState_Set;
	#else
  TIM_OCInitStruct.OCPolarity  = TIM_OCPolarity_High;
  TIM_OCInitStruct.OCIdleState = TIM_OCIdleState_Reset;
	#endif
	/*-------lower arm-------*/
	#ifdef WL_PIN_IO_MODE_LOW_ACTIVE
	TIM_OCInitStruct.OCNPolarity  = TIM_OCNPolarity_Low;
	TIM_OCInitStruct.OCNIdleState = TIM_OCNIdleState_Set;
	#else
	TIM_OCInitStruct.OCNPolarity  = TIM_OCNPolarity_High;
	TIM_OCInitStruct.OCNIdleState = TIM_OCNIdleState_Reset;
	#endif
	TIM_OCInitStruct.Pulse = 49;
	TIM_OC1Init(TIM8, &TIM_OCInitStruct);
	TIM_OC2Init(TIM8, &TIM_OCInitStruct);
	TIM_OC3Init(TIM8, &TIM_OCInitStruct);

	/*  Enable register preload for every output channels */
	TIM_Enable_OC_Preload(TIM8, TIM_CHANNEL_CH1);
	TIM_Enable_OC_Preload(TIM8, TIM_CHANNEL_CH2);
	TIM_Enable_OC_Preload(TIM8, TIM_CHANNEL_CH3);

	TIM_Enable_ARRPreload(TIM8);

	/*  BDTR */
	TIM_BDTRInitStruct.DeadTime = 2;
	TIM_BDTRInitStruct.BreakPolarity = TIM_BREAK_POLARITY_HIGH;
	TIM_BDTRInitStruct.OSSIState = TIM_OSSI_DISABLE;
	TIM_BDTRInitStruct.OSSRState = TIM_OSSR_DISABLE;
	TIM_BDTRInitStruct.AutomaticOutput = TIM_AutomaticOutput_Disable;
	TIM_BDTRInitStruct.LOCKLevel = TIM_LOCKLevel_OFF;
	TIM_BDTRInitStruct.BreakState = TIM_Break_Enable;
//	TIM_BDTRInitStruct.CMP4_BreakState = TIM_CMP4_Break_Disable;
//	TIM_BDTRInitStruct.CMP4_BreakPolarity   = TIM_CMP4OEP_POLARITY_LOW;
//	TIM_BDTRInitStruct.CMP5_BreakState = TIM_CMP5_Break_Disable;
//	TIM_BDTRInitStruct.CMP5_BreakPolarity   = TIM_CMP5OEP_POLARITY_LOW;
	TIM_BDTRInitStruct.DOE  = TIM_DOE_Enable;
	TIM_BDTRInitStruct.HDFE = TIM_HDFE_Enable;
	TIM_BDTRInit(TIM8, &TIM_BDTRInitStruct);

	/**********************************/
	/* Start output signal generation */
	/**********************************/
	/* Enable outputs OC1, OC1N, OC2, OC2N, OC3 and OC3N */
	TIM_Enable_CC_Channel(TIM8, TIM_CHANNEL_CH1 |
																	TIM_CHANNEL_CH1N |
																	TIM_CHANNEL_CH2 |
																	TIM_CHANNEL_CH2N |
																	TIM_CHANNEL_CH3 |
																	TIM_CHANNEL_CH3N); 
                                  
	/* Main output enable */
	TIM_Enable_MOE(TIM8);
	/* Enable counter */
	TIM_Enable_CEN(TIM8);
	
	TIM8->EGR = TIM_EGR_BG; // generate 1st break event to correct output idle state
  
  TIM_Disable_BRK(TIM8);
	
	/*-----------------GPIO------------------*/
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.AF   = AF_FUNCTION1;
  GPIO_InitStruct.Pull = GPIO_FLOATING;
	//TIM_CH3N
	GPIO_InitStruct.Pin = GPIO_PIN_9;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
	//TIM_CH3
	GPIO_InitStruct.Pin = GPIO_PIN_10;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
  //TIM_CH2N
	GPIO_InitStruct.Pin = GPIO_PIN_0;
	GPIO_Init(GPIOD, &GPIO_InitStruct);
	//TIM_CH2
	GPIO_InitStruct.Pin = GPIO_PIN_1;
	GPIO_Init(GPIOD, &GPIO_InitStruct);
  //TIM_CH1N
	GPIO_InitStruct.Pin = GPIO_PIN_2;
	GPIO_Init(GPIOD, &GPIO_InitStruct);
  //TIM_CH1
	GPIO_InitStruct.Pin = GPIO_PIN_3;
	GPIO_Init(GPIOD, &GPIO_InitStruct);


	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.AF   = AF_FUNCTION4;
  GPIO_InitStruct.Pull = GPIO_FLOATING;
	//TIM_BKIN
	GPIO_InitStruct.Pin = GPIO_PIN_6;
	GPIO_Init(GPIOC, &GPIO_InitStruct);  
	
	/*--------------ENABLE TIM8 OUTPUT------------------*/
  Delay_ms(2);
	TIM_Enable_MOE(TIM8);
  TIM_Enable_BRK(TIM8);
}

int main(void)
{	
  GPIO_InitTypeDef GPIO_InitStruct = {0};
  //PC4 tran BKIN PC6
	GPIO_InitStruct.Pin   = GPIO_PIN_4;
	GPIO_InitStruct.Mode  = GPIO_MODE_OUTPUT_PP;
	GPIO_Init(GPIOC, &GPIO_InitStruct);
  
	CLKCTRL_Initial();
	TIM_PWM_CCNE_Init(100,0);
  Delay_ms(5);  
  GPIO_Toggle_Output(GPIOC,GPIO_PIN_4);
	WDT_FeedDog();
  
  while (1)
  {
    WDT_FeedDog();
  }
}






